@@ -99,7 +99,7 @@ static inline double pow(int x, double y) { return std::pow((double)x, y); }
     #endif
 #endif
 
-#if !defined(__cplusplus) || (__cplusplus < 201103)
+#if !defined(__cplusplus) || __cplusplus < 201103
 # define constexpr const
 #endif
 
@@ -286,6 +286,11 @@ static inline double pow(int x, double y) { return std::pow((double)x, y); }
 
     // Use UTC for dgamelaunch servers.
     #define TIME_FN gmtime
+
+    // Outside DGL, there's a local player who can kill the game himself, so
+    // there are no false positives.
+    // (A false positive would be possible with wizmode shenanigans.)
+    #define WATCHDOG
 #endif
 
 #ifndef TIME_FN
@@ -337,6 +342,10 @@ static inline double pow(int x, double y) { return std::pow((double)x, y); }
     #endif
 #endif
 
+#ifdef DEBUG_DIAGNOSTICS
+    #define DEBUG_TESTS
+#endif
+
 // =========================================================================
 //  Lua user scripts (NOTE: this may also be enabled in your makefile!)
 // =========================================================================
