@@ -11,6 +11,7 @@
 #include <queue>
 
 #include "abyss.h"
+#include "act-iter.h"
 #include "areas.h"
 #include "artefact.h"
 #include "branch.h"
@@ -24,7 +25,6 @@
 #include "files.h"
 #include "itemprop.h"
 #include "items.h"
-#include "l_defs.h"
 #include "libutil.h"
 #include "los.h"
 #include "makeitem.h"
@@ -34,7 +34,6 @@
 #include "mgen_data.h"
 #include "misc.h"
 #include "mon-abil.h"
-#include "mon-iter.h"
 #include "mon-pathfind.h"
 #include "mon-pick.h"
 #include "mon-place.h"
@@ -73,6 +72,7 @@ static list<monster*> displaced_monsters;
 
 static void abyss_area_shift(void);
 static void _push_items(void);
+static void _push_displaced_monster(monster* mon);
 
 // If not_seen is true, don't place the feature where it can be seen from
 // the centre.  Returns the chosen location, or INVALID_COORD if it
@@ -124,6 +124,7 @@ static void _write_abyssal_features()
     if (abyssal_features.empty())
         return;
 
+    dprf(DIAG_ABYSS, "Writing a mock-up of old level.");
     const int count = abyssal_features.size();
     ASSERT(count == 213);
     const int scalar = 0xFF;
@@ -147,6 +148,10 @@ static void _write_abyssal_features()
                     {
                         grd(p) = abyssal_features[index];
                         env.level_map_mask(p) = MMT_VAULT;
+                        if (cell_is_solid(p))
+                            delete_cloud_at(p);
+                        if (monster* mon = monster_at(p))
+                            _push_displaced_monster(mon);
                     }
                 }
                 else
@@ -186,7 +191,10 @@ static void _abyss_fixup_vault(const vault_placement *vp)
         const dungeon_feature_type feat(grd(p));
         if (feat_is_stair(feat)
             && feat != DNGN_EXIT_ABYSS
-            && feat != DNGN_ENTER_PORTAL_VAULT)
+#if TAG_MAJOR_VERSION == 34
+            && feat != DNGN_ENTER_PORTAL_VAULT
+#endif
+            && !(feat >= DNGN_ENTER_FIRST_PORTAL && feat <= DNGN_ENTER_LAST_PORTAL))
         {
             grd(p) = DNGN_FLOOR;
         }
@@ -199,7 +207,7 @@ static bool _abyss_place_map(const map_def *mdef)
 {
     // This is to prevent the player position from being updated by vaults
     // until after everything is done.
-    unwind_bool gen(Generating_Level, true);
+    unwind_bool gen(crawl_state.generating_level, true);
 
     const bool did_place = dgn_safe_place_map(mdef, true, false, INVALID_COORD);
     if (did_place)
@@ -211,7 +219,7 @@ static bool _abyss_place_map(const map_def *mdef)
 static bool _abyss_place_vault_tagged(const map_bitmask &abyss_genlevel_mask,
                                       const string &tag)
 {
-    const map_def *map = random_map_for_tag(tag, false, true);
+    const map_def *map = random_map_for_tag(tag, true, true, MB_FALSE);
     if (map)
     {
         unwind_vault_placement_mask vaultmask(&abyss_genlevel_mask);
@@ -232,10 +240,15 @@ static bool _abyss_place_rune_vault(const map_bitmask &abyss_genlevel_mask)
     // Make sure we're not about to link bad items.
     debug_item_scan();
 
-    const bool result = _abyss_place_vault_tagged(abyss_genlevel_mask,
-                                                  "abyss_rune");
+    bool result = false;
+    int tries = 10;
+    do
+        result = _abyss_place_vault_tagged(abyss_genlevel_mask, "abyss_rune");
+    while (!result && --tries);
 
     // Make sure the rune is linked.
+    // XXX: I'm fairly sure this does nothing if result == false,
+    //      but leaving it alone for now. -- Grunt
     _abyss_postvault_fixup();
     return result;
 }
@@ -275,7 +288,7 @@ static bool _abyss_place_rune(const map_bitmask &abyss_genlevel_mask,
             item_colour(mitm[thing_created]);
         }
         move_item_to_grid(&thing_created, chosen_spot);
-        return (thing_created != NON_ITEM);
+        return thing_created != NON_ITEM;
     }
 
     return false;
@@ -285,10 +298,10 @@ static bool _abyss_place_rune(const map_bitmask &abyss_genlevel_mask,
 static bool _abyss_square_accepts_items(const map_bitmask &abyss_genlevel_mask,
                                         coord_def p)
 {
-    return (abyss_genlevel_mask(p)
-            && grd(p) == DNGN_FLOOR
-            && igrd(p) == NON_ITEM
-            && !map_masked(p, MMT_VAULT));
+    return abyss_genlevel_mask(p)
+           && grd(p) == DNGN_FLOOR
+           && igrd(p) == NON_ITEM
+           && !map_masked(p, MMT_VAULT);
 }
 
 static int _abyss_create_items(const map_bitmask &abyss_genlevel_mask,
@@ -401,7 +414,7 @@ static bool _abyss_check_place_feat(coord_def p,
     // Don't place features in bubbles.
     int wall_count = 0;
     for (adjacent_iterator ai(p); ai; ++ai)
-        wall_count += feat_is_solid(grd(p));
+        wall_count += cell_is_solid(p);
     if (wall_count > 6)
         return false;
 
@@ -466,7 +479,7 @@ private:
     {
         // env.map_knowledge().known() doesn't work on unmappable levels because
         // mapping flags are not set on such levels.
-        for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
+        for (radius_iterator ri(you.pos(), LOS_DEFAULT); ri; ++ri)
             if (grd(*ri) == DNGN_EXIT_ABYSS && env.map_knowledge(*ri).seen())
                 return true;
 
@@ -476,7 +489,7 @@ private:
     bool abyss_rune_nearness() const
     {
         // See above comment about env.map_knowledge().known().
-        for (radius_iterator ri(you.pos(), LOS_RADIUS); ri; ++ri)
+        for (radius_iterator ri(you.pos(), LOS_DEFAULT); ri; ++ri)
             if (env.map_knowledge(*ri).seen() && _abyssal_rune_at(*ri))
                 return true;
         return false;
@@ -574,7 +587,7 @@ static bool _pushy_feature(dungeon_feature_type feat)
     // Perhaps this should instead be merged with (the complement of)
     // _item_safe_square() in terrain.cc.  Unlike this function, that
     // one treats traps as unsafe, but closed doors as safe.
-    return (feat_is_solid(feat) || feat == DNGN_LAVA);
+    return feat_is_solid(feat) || feat == DNGN_LAVA;
 }
 
 static void _push_items()
@@ -630,6 +643,8 @@ static void _abyss_wipe_square_at(coord_def p, bool saveMonsters=false)
     // Nuke monster.
     if (monster* mon = monster_at(p))
     {
+        ASSERT(mon->alive());
+
         if (saveMonsters)
             _push_displaced_monster(mon);
         else
@@ -779,7 +794,7 @@ static void _abyss_identify_area_to_shift(coord_def source, int radius,
     mask->reset();
 
     set<int> affected_vault_indexes;
-    for (radius_iterator ri(source, radius, C_SQUARE); ri; ++ri)
+    for (rectangle_iterator ri(source, radius); ri; ++ri)
     {
         if (!map_bounds_with_margin(*ri, MAPGEN_BORDER))
             continue;
@@ -889,29 +904,6 @@ static void _abyss_generate_monsters(int nmonsters)
     }
 }
 
-static bool _abyss_teleport_within_level()
-{
-    // Try to find a good spot within the shift zone.
-    for (int i = 0; i < 100; i++)
-    {
-        const coord_def newspot =
-            dgn_random_point_in_margin(MAPGEN_BORDER
-                                       + ABYSS_AREA_SHIFT_RADIUS
-                                       + 1);
-
-        if ((grd(newspot) == DNGN_FLOOR
-             || grd(newspot) == DNGN_SHALLOW_WATER)
-            && !monster_at(newspot)
-            && env.cgrid(newspot) == EMPTY_CLOUD)
-        {
-            dprf(DIAG_ABYSS, "Abyss same-area teleport to (%d,%d).",
-                 newspot.x, newspot.y);
-            you.moveto(newspot);
-            return true;
-        }
-    }
-    return false;
-}
 
 void maybe_shift_abyss_around_player()
 {
@@ -957,18 +949,17 @@ void save_abyss_uniques()
 
 static bool _in_wastes(const coord_def &p)
 {
-    return (p.x > 0 && p.x < 0x7FFFFFF && p.y > 0 && p.y < 0x7FFFFFF);
+    return p.x > 0 && p.x < 0x7FFFFFF && p.y > 0 && p.y < 0x7FFFFFF;
 }
 
-static level_id _get_real_level()
+static level_id _get_random_level()
 {
-    rng_save_excursion rngstate(abyssal_state.seed);
     vector<level_id> levels;
-    for (int i = BRANCH_MAIN_DUNGEON; i < NUM_BRANCHES; ++i)
+    for (int i = BRANCH_DUNGEON; i < NUM_BRANCHES; ++i)
     {
-        if (i == BRANCH_SHOALS || i == BRANCH_ABYSS)
+        if (i == BRANCH_ABYSS || i == BRANCH_SHOALS)
             continue;
-        for (int j = 0; j < brdepth[i]; ++j)
+        for (int j = 1; j <= brdepth[i]; ++j)
         {
             const level_id id(static_cast<branch_type>(i), j);
             if (is_existing_level(id))
@@ -978,10 +969,10 @@ static level_id _get_real_level()
     if (levels.empty())
     {
         // Let this fail later on.
-        return level_id(static_cast<branch_type>(BRANCH_MAIN_DUNGEON), 1);
+        return level_id(static_cast<branch_type>(BRANCH_DUNGEON), 1);
     }
-    int pick = random2(levels.size());
-    return levels[pick];
+
+    return levels[hash_rand(levels.size(), abyssal_state.seed)];
 }
 
 /**************************************************************/
@@ -1032,7 +1023,7 @@ static ProceduralSample _abyss_grid(const coord_def &p)
 
     if (abyssLayout == NULL)
     {
-        const level_id lid = _get_real_level();
+        const level_id lid = _get_random_level();
         levelLayout = new LevelLayout(lid, 5, rivers);
         complex_vec[0] = levelLayout;
         complex_vec[1] = &rivers; // const
@@ -1058,7 +1049,7 @@ static cloud_type _cloud_from_feat(const dungeon_feature_type &ft)
         case DNGN_SLIMY_WALL:
         case DNGN_STONE_WALL:
         case DNGN_PERMAROCK_WALL:
-            return (coinflip() ? CLOUD_BLUE_SMOKE : CLOUD_PURPLE_SMOKE);
+            return coinflip() ? CLOUD_BLUE_SMOKE : CLOUD_PURPLE_SMOKE;
         case DNGN_CLEAR_ROCK_WALL:
         case DNGN_CLEAR_STONE_WALL:
         case DNGN_CLEAR_PERMAROCK_WALL:
@@ -1071,7 +1062,7 @@ static cloud_type _cloud_from_feat(const dungeon_feature_type &ft)
         case DNGN_DEEP_WATER:
         case DNGN_SHALLOW_WATER:
         case DNGN_FOUNTAIN_BLUE:
-            return (one_chance_in(5) ? CLOUD_RAIN : CLOUD_BLUE_SMOKE);
+            return one_chance_in(5) ? CLOUD_RAIN : CLOUD_BLUE_SMOKE;
         case DNGN_FOUNTAIN_SPARKLING:
             return CLOUD_RAIN;
         default:
@@ -1085,6 +1076,8 @@ static dungeon_feature_type _veto_dangerous_terrain(dungeon_feature_type feat)
         return DNGN_SHALLOW_WATER;
     if (feat == DNGN_LAVA)
         return DNGN_FLOOR;
+    if (feat_is_solid(feat))
+        return DNGN_FLOOR;
 
     return feat;
 }
@@ -1154,17 +1147,18 @@ static void _update_abyss_terrain(const coord_def &p,
             if (cloud != CLOUD_NONE)
                 check_place_cloud(_cloud_from_feat(currfeat), rp, cloud_life, 0, 3);
         }
+        else if (feat_is_solid(feat))
+        {
+            int cloud = env.cgrid(rp);
+            if (cloud != EMPTY_CLOUD)
+                delete_cloud(cloud);
+        }
         monster* mon = monster_at(rp);
         if (mon && !monster_habitable_grid(mon, feat))
             _push_displaced_monster(mon);
     }
 }
 
-static int _abyssal_stair_chance()
-{
-    return (you.char_direction == GDT_GAME_START ? 0 : 2800 - (200 * you.depth / 3));
-}
-
 static void _nuke_all_terrain(bool vaults)
 {
     for (rectangle_iterator ri(MAPGEN_BORDER); ri; ++ri)
@@ -1199,9 +1193,6 @@ static void _abyss_apply_terrain(const map_bitmask &abyss_genlevel_mask,
     const int exit_chance = you.runes[RUNE_ABYSSAL] ? 1250
                             : 7500 - 1250 * (you.depth - 1);
 
-    // Except for the altar on the starting position, don't place any altars.
-    const int altar_chance = you.char_direction != GDT_GAME_START ? 10000 : 0;
-
     int exits_wanted  = 0;
     int altars_wanted = 0;
     bool use_abyss_exit_map = true;
@@ -1245,22 +1236,24 @@ static void _abyss_apply_terrain(const map_bitmask &abyss_genlevel_mask,
 
         // Place abyss exits, stone arches, and altars to liven up the scene
         // (only on area creation, not on morphing).
-        (_abyss_check_place_feat(p, exit_chance,
+        _abyss_check_place_feat(p, exit_chance,
                                 &exits_wanted,
                                 &use_abyss_exit_map,
                                 DNGN_EXIT_ABYSS,
                                 abyss_genlevel_mask)
         ||
-        _abyss_check_place_feat(p, altar_chance,
-                                &altars_wanted,
-                                NULL,
-                                _abyss_pick_altar(),
-                                abyss_genlevel_mask)
+        you.char_direction != GDT_GAME_START
+        && _abyss_check_place_feat(p, 10000,
+                                   &altars_wanted,
+                                   NULL,
+                                   _abyss_pick_altar(),
+                                   abyss_genlevel_mask)
         ||
-        (level_id::current().depth < brdepth[BRANCH_ABYSS] &&
-        _abyss_check_place_feat(p, _abyssal_stair_chance(), NULL, NULL,
-                                DNGN_ABYSSAL_STAIR,
-                                abyss_genlevel_mask)));
+        you.char_direction != GDT_GAME_START
+        && level_id::current().depth < brdepth[BRANCH_ABYSS]
+        && _abyss_check_place_feat(p, 2400, NULL, NULL,
+                                   DNGN_ABYSSAL_STAIR,
+                                   abyss_genlevel_mask);
     }
     if (ii)
         dprf(DIAG_ABYSS, "Nuked %d features", ii);
@@ -1275,13 +1268,21 @@ static int _abyss_place_vaults(const map_bitmask &abyss_genlevel_mask)
 
     int vaults_placed = 0;
 
+    bool mini = false;
     const int maxvaults = 6;
     int tries = 0;
     while (vaults_placed < maxvaults)
     {
-        const map_def *map = random_map_for_tag("abyss", false, true);
+        const map_def *map = random_map_in_depth(level_id::current(), mini);
         if (!map)
+        {
+            if (!mini)
+            {
+                mini = true;
+                continue;
+            }
             break;
+        }
 
         if (!_abyss_place_map(map) || map->has_tag("extra"))
         {
@@ -1291,6 +1292,8 @@ static int _abyss_place_vaults(const map_bitmask &abyss_genlevel_mask)
             continue;
         }
 
+        mini = true;
+
         if (!one_chance_in(2 + (++vaults_placed)))
             break;
     }
@@ -1337,6 +1340,7 @@ static void _initialize_abyss_state()
     abyssal_state.phase = 0.0;
     abyssal_state.depth = random_int() & 0x7FFFFFFF;
     abyssal_state.nuke_all = false;
+    abyssal_state.level = _get_random_level();
     abyss_sample_queue = sample_queue(ProceduralSamplePQCompare());
 }
 
@@ -1503,6 +1507,7 @@ retry:
     _abyss_apply_terrain(abyss_genlevel_mask);
 
     grd(you.pos()) = _veto_dangerous_terrain(grd(you.pos()));
+    _place_displaced_monsters();
 
     for (rectangle_iterator ri(MAPGEN_BORDER); ri; ++ri)
         ASSERT(grd(*ri) > DNGN_UNSEEN);
@@ -1569,21 +1574,9 @@ void abyss_morph(double duration)
     los_changed();
 }
 
-void abyss_teleport(bool new_area)
+void abyss_teleport()
 {
     xom_abyss_feature_amusement_check xomcheck;
-    if (!new_area)
-    {
-        _abyss_teleport_within_level();
-        abyss_area_shift();
-        _initialize_abyss_state();
-        _nuke_all_terrain(true);
-        forget_map(false);
-        clear_excludes();
-        more();
-        return;
-    }
-
     dprf(DIAG_ABYSS, "New area Abyss teleport.");
     _abyss_generate_new_area();
     _write_abyssal_features();
@@ -1733,6 +1726,8 @@ static bool _is_grid_corruptible(const coord_def &c)
     {
     case DNGN_PERMAROCK_WALL:
     case DNGN_CLEAR_PERMAROCK_WALL:
+    case DNGN_OPEN_SEA:
+    case DNGN_LAVA_SEA:
         return false;
 
     case DNGN_METAL_WALL:
@@ -1785,13 +1780,20 @@ static bool _is_sealed_square(const coord_def &c)
 
 static void _corrupt_square(const corrupt_env &cenv, const coord_def &c)
 {
-    // To prevent the destruction of, say, branch entries.
-    bool preserve_feat = true;
+    // Ask dungeon_change_terrain to preserve things that are not altars.
+    // This only actually matters for malign gateways and features that happen
+    // to be on squares that happen to contain map markers (e.g. sealed doors,
+    // which come with their own markers). MAT_CORRUPTION_NEXUS is a marker,
+    // and some of those are certainly nearby, but they only appear on
+    // DNGN_FLOOR. preserve_features=true would ordinarily cause
+    // dungeon_terrain_changed to protect stairs/branch entries/portals as
+    // well, but _corrupt_square is not called on squares containing those
+    // features.
+    bool preserve_features = true;
     dungeon_feature_type feat = DNGN_UNSEEN;
     if (feat_altar_god(grd(c)) != GOD_NO_GOD)
     {
-        // altars may be safely overwritten, ha!
-        preserve_feat = false;
+        preserve_features = false;
         if (!one_chance_in(3))
             feat = DNGN_ALTAR_LUGONU;
     }
@@ -1838,7 +1840,7 @@ static void _corrupt_square(const corrupt_env &cenv, const coord_def &c)
             feat = DNGN_FLOOR;
     }
 
-    dungeon_terrain_changed(c, feat, true, preserve_feat, true);
+    dungeon_terrain_changed(c, feat, true, preserve_features, true);
     if (feat == DNGN_ROCK_WALL)
         env.grid_colours(c) = cenv.rock_colour;
     else if (feat == DNGN_FLOOR)
@@ -1946,7 +1948,7 @@ bool lugonu_corrupt_level(int power)
 
 #ifndef USE_TILE_LOCAL
     // Allow extra time for the flash to linger.
-    delay(1000);
+    scaled_delay(1000);
 #endif
 
     return true;
