@@ -3,10 +3,11 @@
  * @brief Misc abyss specific functions.
 **/
 
-
 #ifndef ABYSS_H
 #define ABYSS_H
 
+#include "externs.h"
+
 // When shifting areas in the abyss, shift the square containing player LOS
 // plus a little extra so that the player won't be disoriented by taking a
 // step backward after an abyss shift.
@@ -19,15 +20,18 @@ struct abyss_state
     uint32_t seed;
     uint32_t depth;
     double phase;
+    level_id level;
     bool nuke_all;
 };
 
+extern abyss_state abyssal_state;
+
 void abyss_morph(double duration);
 void push_features_to_abyss();
 
 void generate_abyss();
 void maybe_shift_abyss_around_player();
-void abyss_teleport(bool new_area);
+void abyss_teleport();
 void save_abyss_uniques();
 bool is_level_incorruptible(bool quiet = false);
 bool lugonu_corrupt_level(int power);
