@@ -28,11 +28,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_HUMAN,           SK_DODGING,         0),
     APT(SP_HUMAN,           SK_STEALTH,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HUMAN,           SK_STABBING,        0),
+    APT(SP_HUMAN,           SK_STABBING,      -99),
 #endif
     APT(SP_HUMAN,           SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HUMAN,           SK_TRAPS,           0),
+    APT(SP_HUMAN,           SK_TRAPS,         -99),
 #endif
     APT(SP_HUMAN,           SK_UNARMED_COMBAT,  0),
     APT(SP_HUMAN,           SK_SPELLCASTING,   -1),
@@ -67,11 +67,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_HIGH_ELF,        SK_DODGING,         1),
     APT(SP_HIGH_ELF,        SK_STEALTH,         2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HIGH_ELF,        SK_STABBING,       -1),
+    APT(SP_HIGH_ELF,        SK_STABBING,      -99),
 #endif
     APT(SP_HIGH_ELF,        SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HIGH_ELF,        SK_TRAPS,           0),
+    APT(SP_HIGH_ELF,        SK_TRAPS,         -99),
 #endif
     APT(SP_HIGH_ELF,        SK_UNARMED_COMBAT, -2),
     APT(SP_HIGH_ELF,        SK_SPELLCASTING,    1),
@@ -106,11 +106,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DEEP_ELF,        SK_DODGING,         2),
     APT(SP_DEEP_ELF,        SK_STEALTH,         3),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEEP_ELF,        SK_STABBING,        1),
+    APT(SP_DEEP_ELF,        SK_STABBING,      -99),
 #endif
     APT(SP_DEEP_ELF,        SK_SHIELDS,        -2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEEP_ELF,        SK_TRAPS,           0),
+    APT(SP_DEEP_ELF,        SK_TRAPS,         -99),
 #endif
     APT(SP_DEEP_ELF,        SK_UNARMED_COMBAT, -2),
     APT(SP_DEEP_ELF,        SK_SPELLCASTING,    3),
@@ -129,6 +129,7 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DEEP_ELF,        SK_INVOCATIONS,     1),
     APT(SP_DEEP_ELF,        SK_EVOCATIONS,      2),
 
+#if TAG_MAJOR_VERSION == 34
     // SP_SLUDGE_ELF
     APT(SP_SLUDGE_ELF,      SK_FIGHTING,        1),
     APT(SP_SLUDGE_ELF,      SK_SHORT_BLADES,   -1),
@@ -144,13 +145,9 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_SLUDGE_ELF,      SK_ARMOUR,         -2),
     APT(SP_SLUDGE_ELF,      SK_DODGING,         2),
     APT(SP_SLUDGE_ELF,      SK_STEALTH,         2),
-#if TAG_MAJOR_VERSION == 34
-    APT(SP_SLUDGE_ELF,      SK_STABBING,        0),
-#endif
+    APT(SP_SLUDGE_ELF,      SK_STABBING,      -99),
     APT(SP_SLUDGE_ELF,      SK_SHIELDS,        -2),
-#if TAG_MAJOR_VERSION == 34
-    APT(SP_SLUDGE_ELF,      SK_TRAPS,           0),
-#endif
+    APT(SP_SLUDGE_ELF,      SK_TRAPS,         -99),
     APT(SP_SLUDGE_ELF,      SK_UNARMED_COMBAT,  1),
     APT(SP_SLUDGE_ELF,      SK_SPELLCASTING,    1),
     APT(SP_SLUDGE_ELF,      SK_CONJURATIONS,   -2),
@@ -167,6 +164,7 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_SLUDGE_ELF,      SK_POISON_MAGIC,    1),
     APT(SP_SLUDGE_ELF,      SK_INVOCATIONS,     1),
     APT(SP_SLUDGE_ELF,      SK_EVOCATIONS,      1),
+#endif
 
     // SP_HALFLING
     APT(SP_HALFLING,        SK_FIGHTING,       -1),
@@ -184,11 +182,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_HALFLING,        SK_DODGING,         2),
     APT(SP_HALFLING,        SK_STEALTH,         4),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HALFLING,        SK_STABBING,        0),
+    APT(SP_HALFLING,        SK_STABBING,      -99),
 #endif
     APT(SP_HALFLING,        SK_SHIELDS,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HALFLING,        SK_TRAPS,           0),
+    APT(SP_HALFLING,        SK_TRAPS,         -99),
 #endif
     APT(SP_HALFLING,        SK_UNARMED_COMBAT, -2),
     APT(SP_HALFLING,        SK_SPELLCASTING,   -3),
@@ -223,11 +221,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_HILL_ORC,        SK_DODGING,        -2),
     APT(SP_HILL_ORC,        SK_STEALTH,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HILL_ORC,        SK_STABBING,        2),
+    APT(SP_HILL_ORC,        SK_STABBING,      -99),
 #endif
     APT(SP_HILL_ORC,        SK_SHIELDS,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_HILL_ORC,        SK_TRAPS,           0),
+    APT(SP_HILL_ORC,        SK_TRAPS,         -99),
 #endif
     APT(SP_HILL_ORC,        SK_UNARMED_COMBAT,  1),
     APT(SP_HILL_ORC,        SK_SPELLCASTING,   -3),
@@ -262,11 +260,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_KOBOLD,          SK_DODGING,         2),
     APT(SP_KOBOLD,          SK_STEALTH,         4),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_KOBOLD,          SK_STABBING,        2),
+    APT(SP_KOBOLD,          SK_STABBING,      -99),
 #endif
     APT(SP_KOBOLD,          SK_SHIELDS,        -2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_KOBOLD,          SK_TRAPS,           0),
+    APT(SP_KOBOLD,          SK_TRAPS,         -99),
 #endif
     APT(SP_KOBOLD,          SK_UNARMED_COMBAT,  0),
     APT(SP_KOBOLD,          SK_SPELLCASTING,   -1),
@@ -301,11 +299,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_MUMMY,           SK_DODGING,        -2),
     APT(SP_MUMMY,           SK_STEALTH,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MUMMY,           SK_STABBING,       -2),
+    APT(SP_MUMMY,           SK_STABBING,      -99),
 #endif
     APT(SP_MUMMY,           SK_SHIELDS,        -2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MUMMY,           SK_TRAPS,          -2),
+    APT(SP_MUMMY,           SK_TRAPS,         -99),
 #endif
     APT(SP_MUMMY,           SK_UNARMED_COMBAT, -2),
     APT(SP_MUMMY,           SK_SPELLCASTING,   -1),
@@ -340,11 +338,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_NAGA,            SK_DODGING,        -2),
     APT(SP_NAGA,            SK_STEALTH,         5),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_NAGA,            SK_STABBING,        0),
+    APT(SP_NAGA,            SK_STABBING,      -99),
 #endif
     APT(SP_NAGA,            SK_SHIELDS,        -2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_NAGA,            SK_TRAPS,           0),
+    APT(SP_NAGA,            SK_TRAPS,         -99),
 #endif
     APT(SP_NAGA,            SK_UNARMED_COMBAT,  0),
     APT(SP_NAGA,            SK_SPELLCASTING,   -1),
@@ -379,11 +377,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_OGRE,            SK_DODGING,        -1),
     APT(SP_OGRE,            SK_STEALTH,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_OGRE,            SK_STABBING,       -2),
+    APT(SP_OGRE,            SK_STABBING,      -99),
 #endif
     APT(SP_OGRE,            SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_OGRE,            SK_TRAPS,          -2),
+    APT(SP_OGRE,            SK_TRAPS,         -99),
 #endif
     APT(SP_OGRE,            SK_UNARMED_COMBAT, -1),
     APT(SP_OGRE,            SK_SPELLCASTING,    1),
@@ -418,11 +416,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_TROLL,           SK_DODGING,        -2),
     APT(SP_TROLL,           SK_STEALTH,        -4),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_TROLL,           SK_STABBING,       -2),
+    APT(SP_TROLL,           SK_STABBING,      -99),
 #endif
     APT(SP_TROLL,           SK_SHIELDS,        -2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_TROLL,           SK_TRAPS,          -4),
+    APT(SP_TROLL,           SK_TRAPS,         -99),
 #endif
     APT(SP_TROLL,           SK_UNARMED_COMBAT,  0),
     APT(SP_TROLL,           SK_SPELLCASTING,   -5),
@@ -457,11 +455,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_RED_DRACONIAN,   SK_DODGING,        -1),
     APT(SP_RED_DRACONIAN,   SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_RED_DRACONIAN,   SK_STABBING,        0),
+    APT(SP_RED_DRACONIAN,   SK_STABBING,      -99),
 #endif
     APT(SP_RED_DRACONIAN,   SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_RED_DRACONIAN,   SK_TRAPS,           0),
+    APT(SP_RED_DRACONIAN,   SK_TRAPS,         -99),
 #endif
     APT(SP_RED_DRACONIAN,   SK_UNARMED_COMBAT,  0),
     APT(SP_RED_DRACONIAN,   SK_SPELLCASTING,   -1),
@@ -496,11 +494,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_WHITE_DRACONIAN, SK_DODGING,        -1),
     APT(SP_WHITE_DRACONIAN, SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_WHITE_DRACONIAN, SK_STABBING,        0),
+    APT(SP_WHITE_DRACONIAN, SK_STABBING,      -99),
 #endif
     APT(SP_WHITE_DRACONIAN, SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_WHITE_DRACONIAN, SK_TRAPS,           0),
+    APT(SP_WHITE_DRACONIAN, SK_TRAPS,         -99),
 #endif
     APT(SP_WHITE_DRACONIAN, SK_UNARMED_COMBAT,  0),
     APT(SP_WHITE_DRACONIAN, SK_SPELLCASTING,   -1),
@@ -535,11 +533,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_GREEN_DRACONIAN, SK_DODGING,        -1),
     APT(SP_GREEN_DRACONIAN, SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GREEN_DRACONIAN, SK_STABBING,        0),
+    APT(SP_GREEN_DRACONIAN, SK_STABBING,      -99),
 #endif
     APT(SP_GREEN_DRACONIAN, SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GREEN_DRACONIAN, SK_TRAPS,           0),
+    APT(SP_GREEN_DRACONIAN, SK_TRAPS,         -99),
 #endif
     APT(SP_GREEN_DRACONIAN, SK_UNARMED_COMBAT,  0),
     APT(SP_GREEN_DRACONIAN, SK_SPELLCASTING,   -1),
@@ -574,11 +572,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_YELLOW_DRACONIAN,SK_DODGING,        -1),
     APT(SP_YELLOW_DRACONIAN,SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_YELLOW_DRACONIAN,SK_STABBING,        0),
+    APT(SP_YELLOW_DRACONIAN,SK_STABBING,      -99),
 #endif
     APT(SP_YELLOW_DRACONIAN,SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_YELLOW_DRACONIAN,SK_TRAPS,           0),
+    APT(SP_YELLOW_DRACONIAN,SK_TRAPS,         -99),
 #endif
     APT(SP_YELLOW_DRACONIAN,SK_UNARMED_COMBAT,  0),
     APT(SP_YELLOW_DRACONIAN,SK_SPELLCASTING,   -1),
@@ -613,11 +611,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_GREY_DRACONIAN,  SK_DODGING,        -1),
     APT(SP_GREY_DRACONIAN,  SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GREY_DRACONIAN,  SK_STABBING,        0),
+    APT(SP_GREY_DRACONIAN,  SK_STABBING,      -99),
 #endif
     APT(SP_GREY_DRACONIAN,  SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GREY_DRACONIAN,  SK_TRAPS,           0),
+    APT(SP_GREY_DRACONIAN,  SK_TRAPS,         -99),
 #endif
     APT(SP_GREY_DRACONIAN,  SK_UNARMED_COMBAT,  0),
     APT(SP_GREY_DRACONIAN,  SK_SPELLCASTING,   -1),
@@ -652,11 +650,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_BLACK_DRACONIAN, SK_DODGING,        -1),
     APT(SP_BLACK_DRACONIAN, SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_BLACK_DRACONIAN, SK_STABBING,        0),
+    APT(SP_BLACK_DRACONIAN, SK_STABBING,      -99),
 #endif
     APT(SP_BLACK_DRACONIAN, SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_BLACK_DRACONIAN, SK_TRAPS,           0),
+    APT(SP_BLACK_DRACONIAN, SK_TRAPS,         -99),
 #endif
     APT(SP_BLACK_DRACONIAN, SK_UNARMED_COMBAT,  0),
     APT(SP_BLACK_DRACONIAN, SK_SPELLCASTING,   -1),
@@ -691,11 +689,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_PURPLE_DRACONIAN,SK_DODGING,        -1),
     APT(SP_PURPLE_DRACONIAN,SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_PURPLE_DRACONIAN,SK_STABBING,        0),
+    APT(SP_PURPLE_DRACONIAN,SK_STABBING,      -99),
 #endif
     APT(SP_PURPLE_DRACONIAN,SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_PURPLE_DRACONIAN,SK_TRAPS,           0),
+    APT(SP_PURPLE_DRACONIAN,SK_TRAPS,         -99),
 #endif
     APT(SP_PURPLE_DRACONIAN,SK_UNARMED_COMBAT,  0),
     APT(SP_PURPLE_DRACONIAN,SK_SPELLCASTING,    1),
@@ -730,11 +728,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_MOTTLED_DRACONIAN,SK_DODGING,        -1),
     APT(SP_MOTTLED_DRACONIAN,SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MOTTLED_DRACONIAN,SK_STABBING,        0),
+    APT(SP_MOTTLED_DRACONIAN,SK_STABBING,      -99),
 #endif
     APT(SP_MOTTLED_DRACONIAN,SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MOTTLED_DRACONIAN,SK_TRAPS,           0),
+    APT(SP_MOTTLED_DRACONIAN,SK_TRAPS,         -99),
 #endif
     APT(SP_MOTTLED_DRACONIAN,SK_UNARMED_COMBAT,  0),
     APT(SP_MOTTLED_DRACONIAN,SK_SPELLCASTING,   -1),
@@ -769,11 +767,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_PALE_DRACONIAN,  SK_DODGING,        -1),
     APT(SP_PALE_DRACONIAN,  SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_PALE_DRACONIAN,  SK_STABBING,        0),
+    APT(SP_PALE_DRACONIAN,  SK_STABBING,      -99),
 #endif
     APT(SP_PALE_DRACONIAN,  SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_PALE_DRACONIAN,  SK_TRAPS,           0),
+    APT(SP_PALE_DRACONIAN,  SK_TRAPS,         -99),
 #endif
     APT(SP_PALE_DRACONIAN,  SK_UNARMED_COMBAT,  0),
     APT(SP_PALE_DRACONIAN,  SK_SPELLCASTING,   -1),
@@ -808,11 +806,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_BASE_DRACONIAN,  SK_DODGING,        -1),
     APT(SP_BASE_DRACONIAN,  SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_BASE_DRACONIAN,  SK_STABBING,        0),
+    APT(SP_BASE_DRACONIAN,  SK_STABBING,      -99),
 #endif
     APT(SP_BASE_DRACONIAN,  SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_BASE_DRACONIAN,  SK_TRAPS,           0),
+    APT(SP_BASE_DRACONIAN,  SK_TRAPS,         -99),
 #endif
     APT(SP_BASE_DRACONIAN,  SK_UNARMED_COMBAT,  0),
     APT(SP_BASE_DRACONIAN,  SK_SPELLCASTING,   -1),
@@ -847,11 +845,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_CENTAUR,         SK_DODGING,        -3),
     APT(SP_CENTAUR,         SK_STEALTH,        -3),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_CENTAUR,         SK_STABBING,       -3),
+    APT(SP_CENTAUR,         SK_STABBING,      -99),
 #endif
     APT(SP_CENTAUR,         SK_SHIELDS,        -3),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_CENTAUR,         SK_TRAPS,          -2),
+    APT(SP_CENTAUR,         SK_TRAPS,         -99),
 #endif
     APT(SP_CENTAUR,         SK_UNARMED_COMBAT,  0),
     APT(SP_CENTAUR,         SK_SPELLCASTING,   -3),
@@ -886,11 +884,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DEMIGOD,         SK_DODGING,        -1),
     APT(SP_DEMIGOD,         SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEMIGOD,         SK_STABBING,       -1),
+    APT(SP_DEMIGOD,         SK_STABBING,      -99),
 #endif
     APT(SP_DEMIGOD,         SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEMIGOD,         SK_TRAPS,          -1),
+    APT(SP_DEMIGOD,         SK_TRAPS,         -99),
 #endif
     APT(SP_DEMIGOD,         SK_UNARMED_COMBAT, -1),
     APT(SP_DEMIGOD,         SK_SPELLCASTING,   -2),
@@ -925,11 +923,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_SPRIGGAN,        SK_DODGING,         4),
     APT(SP_SPRIGGAN,        SK_STEALTH,         5),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_SPRIGGAN,        SK_STABBING,        4),
+    APT(SP_SPRIGGAN,        SK_STABBING,      -99),
 #endif
     APT(SP_SPRIGGAN,        SK_SHIELDS,        -3),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_SPRIGGAN,        SK_TRAPS,           3),
+    APT(SP_SPRIGGAN,        SK_TRAPS,         -99),
 #endif
     APT(SP_SPRIGGAN,        SK_UNARMED_COMBAT, -2),
     APT(SP_SPRIGGAN,        SK_SPELLCASTING,    2),
@@ -964,11 +962,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_MINOTAUR,        SK_DODGING,         1),
     APT(SP_MINOTAUR,        SK_STEALTH,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MINOTAUR,        SK_STABBING,       -1),
+    APT(SP_MINOTAUR,        SK_STABBING,      -99),
 #endif
     APT(SP_MINOTAUR,        SK_SHIELDS,         2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MINOTAUR,        SK_TRAPS,           0),
+    APT(SP_MINOTAUR,        SK_TRAPS,         -99),
 #endif
     APT(SP_MINOTAUR,        SK_UNARMED_COMBAT,  1),
     APT(SP_MINOTAUR,        SK_SPELLCASTING,   -4),
@@ -1003,11 +1001,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_GARGOYLE,         SK_DODGING,        -2),
     APT(SP_GARGOYLE,         SK_STEALTH,         2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GARGOYLE,         SK_STABBING,        0),
+    APT(SP_GARGOYLE,         SK_STABBING,      -99),
 #endif
     APT(SP_GARGOYLE,         SK_SHIELDS,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GARGOYLE,         SK_TRAPS,          -1),
+    APT(SP_GARGOYLE,         SK_TRAPS,         -99),
 #endif
     APT(SP_GARGOYLE,         SK_UNARMED_COMBAT,  1),
     APT(SP_GARGOYLE,         SK_SPELLCASTING,   -1),
@@ -1042,11 +1040,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DEMONSPAWN,      SK_DODGING,        -1),
     APT(SP_DEMONSPAWN,      SK_STEALTH,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEMONSPAWN,      SK_STABBING,       -1),
+    APT(SP_DEMONSPAWN,      SK_STABBING,      -99),
 #endif
     APT(SP_DEMONSPAWN,      SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEMONSPAWN,      SK_TRAPS,          -1),
+    APT(SP_DEMONSPAWN,      SK_TRAPS,         -99),
 #endif
     APT(SP_DEMONSPAWN,      SK_UNARMED_COMBAT, -1),
     APT(SP_DEMONSPAWN,      SK_SPELLCASTING,   -1),
@@ -1081,11 +1079,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_GHOUL,           SK_DODGING,        -1),
     APT(SP_GHOUL,           SK_STEALTH,         2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GHOUL,           SK_STABBING,        0),
+    APT(SP_GHOUL,           SK_STABBING,      -99),
 #endif
     APT(SP_GHOUL,           SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_GHOUL,           SK_TRAPS,          -1),
+    APT(SP_GHOUL,           SK_TRAPS,         -99),
 #endif
     APT(SP_GHOUL,           SK_UNARMED_COMBAT,  1),
     APT(SP_GHOUL,           SK_SPELLCASTING,   -2),
@@ -1120,11 +1118,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_TENGU,           SK_DODGING,         1),
     APT(SP_TENGU,           SK_STEALTH,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_TENGU,           SK_STABBING,        1),
+    APT(SP_TENGU,           SK_STABBING,      -99),
 #endif
     APT(SP_TENGU,           SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_TENGU,           SK_TRAPS,           0),
+    APT(SP_TENGU,           SK_TRAPS,         -99),
 #endif
     APT(SP_TENGU,           SK_UNARMED_COMBAT,  1),
     APT(SP_TENGU,           SK_SPELLCASTING,   -1),
@@ -1159,11 +1157,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_MERFOLK,         SK_DODGING,         3),
     APT(SP_MERFOLK,         SK_STEALTH,         2),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MERFOLK,         SK_STABBING,        2),
+    APT(SP_MERFOLK,         SK_STABBING,      -99),
 #endif
     APT(SP_MERFOLK,         SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_MERFOLK,         SK_TRAPS,          -1),
+    APT(SP_MERFOLK,         SK_TRAPS,         -99),
 #endif
     APT(SP_MERFOLK,         SK_UNARMED_COMBAT,  1),
     APT(SP_MERFOLK,         SK_SPELLCASTING,   -1),
@@ -1198,11 +1196,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_VAMPIRE,         SK_DODGING,         1),
     APT(SP_VAMPIRE,         SK_STEALTH,         5),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_VAMPIRE,         SK_STABBING,        1),
+    APT(SP_VAMPIRE,         SK_STABBING,      -99),
 #endif
     APT(SP_VAMPIRE,         SK_SHIELDS,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_VAMPIRE,         SK_TRAPS,           0),
+    APT(SP_VAMPIRE,         SK_TRAPS,         -99),
 #endif
     APT(SP_VAMPIRE,         SK_UNARMED_COMBAT,  1),
     APT(SP_VAMPIRE,         SK_SPELLCASTING,   -1),
@@ -1237,11 +1235,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DEEP_DWARF,      SK_DODGING,         1),
     APT(SP_DEEP_DWARF,      SK_STEALTH,         3),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEEP_DWARF,      SK_STABBING,       -1),
+    APT(SP_DEEP_DWARF,      SK_STABBING,      -99),
 #endif
     APT(SP_DEEP_DWARF,      SK_SHIELDS,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_DEEP_DWARF,      SK_TRAPS,           1),
+    APT(SP_DEEP_DWARF,      SK_TRAPS,         -99),
 #endif
     APT(SP_DEEP_DWARF,      SK_UNARMED_COMBAT, -1),
     APT(SP_DEEP_DWARF,      SK_SPELLCASTING,   -2),
@@ -1276,11 +1274,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_FELID,           SK_DODGING,         3),
     APT(SP_FELID,           SK_STEALTH,         4),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_FELID,           SK_STABBING,        3),
+    APT(SP_FELID,           SK_STABBING,      -99),
 #endif
     APT(SP_FELID,           SK_SHIELDS,       -99),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_FELID,           SK_TRAPS,           2),
+    APT(SP_FELID,           SK_TRAPS,         -99),
 #endif
     APT(SP_FELID,           SK_UNARMED_COMBAT,  0),
     APT(SP_FELID,           SK_SPELLCASTING,   -1),
@@ -1315,11 +1313,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_OCTOPODE,        SK_DODGING,         0),
     APT(SP_OCTOPODE,        SK_STEALTH,         4),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_OCTOPODE,        SK_STABBING,        0),
+    APT(SP_OCTOPODE,        SK_STABBING,      -99),
 #endif
     APT(SP_OCTOPODE,        SK_SHIELDS,         0),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_OCTOPODE,        SK_TRAPS,           1),
+    APT(SP_OCTOPODE,        SK_TRAPS,         -99),
 #endif
     APT(SP_OCTOPODE,        SK_UNARMED_COMBAT,  0),
     APT(SP_OCTOPODE,        SK_SPELLCASTING,   -1),
@@ -1338,6 +1336,7 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_OCTOPODE,        SK_INVOCATIONS,     1),
     APT(SP_OCTOPODE,        SK_EVOCATIONS,      2),
 
+#if TAG_MAJOR_VERSION == 34
     // SP_DJINNI
     APT(SP_DJINNI,          SK_FIGHTING,       -1),
     APT(SP_DJINNI,          SK_SHORT_BLADES,   -2),
@@ -1354,9 +1353,7 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DJINNI,          SK_DODGING,        -1),
     APT(SP_DJINNI,          SK_STEALTH,        -1),
     APT(SP_DJINNI,          SK_SHIELDS,         0),
-#if TAG_MAJOR_VERSION == 34
-    APT(SP_DJINNI,          SK_TRAPS,          -1),
-#endif
+    APT(SP_DJINNI,          SK_TRAPS,         -99),
     APT(SP_DJINNI,          SK_UNARMED_COMBAT, -1),
     APT(SP_DJINNI,          SK_SPELLCASTING,    1),
     APT(SP_DJINNI,          SK_CONJURATIONS,    1),
@@ -1371,8 +1368,9 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_DJINNI,          SK_AIR_MAGIC,       2),
     APT(SP_DJINNI,          SK_EARTH_MAGIC,    -3),
     APT(SP_DJINNI,          SK_POISON_MAGIC,   -1),
-    APT(SP_DJINNI,          SK_INVOCATIONS,     1),
+    APT(SP_DJINNI,          SK_INVOCATIONS,    -1),
     APT(SP_DJINNI,          SK_EVOCATIONS,      2),
+#endif
 
     // SP_LAVA_ORC
     APT(SP_LAVA_ORC,        SK_FIGHTING,        2),
@@ -1390,11 +1388,11 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_LAVA_ORC,        SK_DODGING,        -2),
     APT(SP_LAVA_ORC,        SK_STEALTH,        -1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_LAVA_ORC,        SK_STABBING,        2),
+    APT(SP_LAVA_ORC,        SK_STABBING,      -99),
 #endif
     APT(SP_LAVA_ORC,        SK_SHIELDS,         1),
 #if TAG_MAJOR_VERSION == 34
-    APT(SP_LAVA_ORC,        SK_TRAPS,           0),
+    APT(SP_LAVA_ORC,        SK_TRAPS,         -99),
 #endif
     APT(SP_LAVA_ORC,        SK_UNARMED_COMBAT,  1),
     APT(SP_LAVA_ORC,        SK_SPELLCASTING,   -3),
@@ -1412,6 +1410,84 @@ static const species_skill_aptitude species_skill_aptitudes[] =
     APT(SP_LAVA_ORC,        SK_POISON_MAGIC,   -1),
     APT(SP_LAVA_ORC,        SK_INVOCATIONS,     3),
     APT(SP_LAVA_ORC,        SK_EVOCATIONS,      1),
+
+    // SP_FORMICID
+    APT(SP_FORMICID,        SK_FIGHTING,        1),
+    APT(SP_FORMICID,        SK_SHORT_BLADES,    0),
+    APT(SP_FORMICID,        SK_LONG_BLADES,     0),
+    APT(SP_FORMICID,        SK_AXES,            0),
+    APT(SP_FORMICID,        SK_MACES_FLAILS,    0),
+    APT(SP_FORMICID,        SK_POLEARMS,        0),
+    APT(SP_FORMICID,        SK_STAVES,          0),
+    APT(SP_FORMICID,        SK_SLINGS,          1),
+    APT(SP_FORMICID,        SK_BOWS,           -2),
+    APT(SP_FORMICID,        SK_CROSSBOWS,       0),
+    APT(SP_FORMICID,        SK_THROWING,       -2),
+    APT(SP_FORMICID,        SK_ARMOUR,          1),
+    APT(SP_FORMICID,        SK_DODGING,        -1),
+    APT(SP_FORMICID,        SK_STEALTH,         3),
+#if TAG_MAJOR_VERSION == 34
+    APT(SP_FORMICID,        SK_STABBING,      -99),
+#endif
+    APT(SP_FORMICID,        SK_SHIELDS,         2),
+#if TAG_MAJOR_VERSION == 34
+    APT(SP_FORMICID,        SK_TRAPS,         -99),
+#endif
+    APT(SP_FORMICID,        SK_UNARMED_COMBAT,  0),
+    APT(SP_FORMICID,        SK_SPELLCASTING,    0),
+    APT(SP_FORMICID,        SK_CONJURATIONS,   -1),
+    APT(SP_FORMICID,        SK_HEXES,           2),
+    APT(SP_FORMICID,        SK_CHARMS,          0),
+    APT(SP_FORMICID,        SK_SUMMONINGS,      0),
+    APT(SP_FORMICID,        SK_NECROMANCY,      0),
+    APT(SP_FORMICID,        SK_TRANSLOCATIONS, -1),
+    APT(SP_FORMICID,        SK_TRANSMUTATIONS,  1),
+    APT(SP_FORMICID,        SK_FIRE_MAGIC,      0),
+    APT(SP_FORMICID,        SK_ICE_MAGIC,       0),
+    APT(SP_FORMICID,        SK_AIR_MAGIC,      -2),
+    APT(SP_FORMICID,        SK_EARTH_MAGIC,     2),
+    APT(SP_FORMICID,        SK_POISON_MAGIC,    3),
+    APT(SP_FORMICID,        SK_INVOCATIONS,     2),
+    APT(SP_FORMICID,        SK_EVOCATIONS,      2),
+
+    // SP_VINE_STALKER
+    APT(SP_VINE_STALKER,    SK_FIGHTING,        -1),
+    APT(SP_VINE_STALKER,    SK_SHORT_BLADES,    -1),
+    APT(SP_VINE_STALKER,    SK_LONG_BLADES,     -1),
+    APT(SP_VINE_STALKER,    SK_AXES,            -1),
+    APT(SP_VINE_STALKER,    SK_MACES_FLAILS,    -1),
+    APT(SP_VINE_STALKER,    SK_POLEARMS,        -1),
+    APT(SP_VINE_STALKER,    SK_STAVES,          -1),
+    APT(SP_VINE_STALKER,    SK_SLINGS,          -1),
+    APT(SP_VINE_STALKER,    SK_BOWS,            -1),
+    APT(SP_VINE_STALKER,    SK_CROSSBOWS,       -1),
+    APT(SP_VINE_STALKER,    SK_THROWING,        -1),
+    APT(SP_VINE_STALKER,    SK_ARMOUR,          -2),
+    APT(SP_VINE_STALKER,    SK_DODGING,         -2),
+    APT(SP_VINE_STALKER,    SK_STEALTH,         3),
+#if TAG_MAJOR_VERSION == 34
+    APT(SP_VINE_STALKER,    SK_STABBING,      -99),
+#endif
+    APT(SP_VINE_STALKER,    SK_SHIELDS,        -1),
+#if TAG_MAJOR_VERSION == 34
+    APT(SP_VINE_STALKER,    SK_TRAPS,         -99),
+#endif
+    APT(SP_VINE_STALKER,    SK_UNARMED_COMBAT,  0),
+    APT(SP_VINE_STALKER,    SK_SPELLCASTING,    0),
+    APT(SP_VINE_STALKER,    SK_CONJURATIONS,    0),
+    APT(SP_VINE_STALKER,    SK_HEXES,           0),
+    APT(SP_VINE_STALKER,    SK_CHARMS,          0),
+    APT(SP_VINE_STALKER,    SK_SUMMONINGS,      0),
+    APT(SP_VINE_STALKER,    SK_NECROMANCY,      0),
+    APT(SP_VINE_STALKER,    SK_TRANSLOCATIONS,  0),
+    APT(SP_VINE_STALKER,    SK_TRANSMUTATIONS,  0),
+    APT(SP_VINE_STALKER,    SK_FIRE_MAGIC,      0),
+    APT(SP_VINE_STALKER,    SK_ICE_MAGIC,       0),
+    APT(SP_VINE_STALKER,    SK_AIR_MAGIC,       0),
+    APT(SP_VINE_STALKER,    SK_EARTH_MAGIC,     0),
+    APT(SP_VINE_STALKER,    SK_POISON_MAGIC,    0),
+    APT(SP_VINE_STALKER,    SK_INVOCATIONS,     0),
+    APT(SP_VINE_STALKER,    SK_EVOCATIONS,      0),
 };
 
 #endif
