@@ -12,9 +12,6 @@ class AsgKISS
         uint32_t m_lcg, m_mwcm, m_mwcc, m_xorshift, m_lfsr;
 };
 
-void push_asg_state();
-void pop_asg_state();
-
-uint32_t get_uint32();
+uint32_t get_uint32(int generator = 0);
 void seed_asg(uint32_t[], int);
 #endif
