@@ -13,9 +13,9 @@
     SPELL_MAGIC_DART,
     SPELL_SEARING_RAY,
     SPELL_DAZZLING_SPRAY,
+    SPELL_FULMINANT_PRISM,
     SPELL_ISKENDERUNS_MYSTIC_BLAST,
     SPELL_BATTLESPHERE,
-    SPELL_FULMINANT_PRISM,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
@@ -48,10 +48,9 @@
     SPELL_SUMMON_ELEMENTAL,
     SPELL_SHADOW_CREATURES,
     SPELL_SUMMON_DEMON,
-    SPELL_SUMMON_UGLY_THING,
+    SPELL_MONSTROUS_MENAGERIE,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
-
 },
 
 {   // Book of Fire
@@ -67,20 +66,20 @@
 
 {   // Book of Ice
     SPELL_FREEZING_AURA,
-    SPELL_HIBERNATION,
+    SPELL_ENGLACIATION,
     SPELL_OZOCUBUS_REFRIGERATION,
     SPELL_BOLT_OF_COLD,
     SPELL_FREEZING_CLOUD,
-    SPELL_ENGLACIATION,
     SPELL_SIMULACRUM,
     SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
 },
 
 {   // Book of Spatial Translocations
     SPELL_APPORTATION,
-    SPELL_PORTAL_PROJECTILE,
     SPELL_BLINK,
     SPELL_SHROUD_OF_GOLUBRIA,
+    SPELL_PORTAL_PROJECTILE,
     SPELL_TELEPORT_OTHER,
     SPELL_GOLUBRIAS_PASSAGE,
     SPELL_NO_SPELL,
@@ -133,11 +132,11 @@
 
 {   // Book of Hinderance
     SPELL_CONFUSING_TOUCH,
-    SPELL_SLOW,
     SPELL_CONFUSE,
     SPELL_PETRIFY,
     SPELL_LEDAS_LIQUEFACTION,
     SPELL_ENGLACIATION,
+    SPELL_SUMMON_MANA_VIPER,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
@@ -155,13 +154,13 @@
 
 {   // Book of Transfigurations
     SPELL_STONESKIN,
-    SPELL_POLYMORPH,
     SPELL_ICE_FORM,
     SPELL_STATUE_FORM,
     SPELL_DRAGON_FORM,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
 },
 
 {   // Book of War Chants
@@ -181,7 +180,7 @@
     SPELL_SONG_OF_SLAYING,
     SPELL_SPECTRAL_WEAPON,
     SPELL_REGENERATION,
-    SPELL_SONG_OF_SHIELDING,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
@@ -212,11 +211,11 @@
     SPELL_SUMMON_SMALL_MAMMAL,
     SPELL_CALL_IMP,
     SPELL_CALL_CANINE_FAMILIAR,
-    SPELL_SUMMON_SCORPIONS,
+    SPELL_SUMMON_GUARDIAN_GOLEM,
+    SPELL_SUMMON_LIGHTNING_SPIRE,
     SPELL_SUMMON_ICE_BEAST,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
-    SPELL_NO_SPELL,
 },
 
 {   // Book of Maledictions
@@ -252,12 +251,11 @@
     SPELL_NO_SPELL,
 },
 
-
 {   // Book of the Warp
-    SPELL_RECALL,
     SPELL_CONTROL_TELEPORT,
     SPELL_PHASE_SHIFT,
     SPELL_WARP_BRAND,
+    SPELL_SUMMON_FOREST,
     SPELL_DISPERSAL,
     SPELL_CONTROLLED_BLINK,
     SPELL_DISJUNCTION,
@@ -267,7 +265,7 @@
 {   // Book of Envenomations
     SPELL_SPIDER_FORM,
     SPELL_POISON_WEAPON,
-    SPELL_SUMMON_SCORPIONS,
+    SPELL_INTOXICATE,
     SPELL_OLGREBS_TOXIC_RADIANCE,
     SPELL_POISONOUS_CLOUD,
     SPELL_NO_SPELL,
@@ -290,8 +288,8 @@
     SPELL_CONTROL_TELEPORT,
     SPELL_CONTROL_UNDEAD,
     SPELL_ENSLAVEMENT,
-    SPELL_MASS_CONFUSION,
     SPELL_ENGLACIATION,
+    SPELL_MASS_CONFUSION,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -304,7 +302,7 @@
     SPELL_SONG_OF_SLAYING,
     SPELL_SPECTRAL_WEAPON,
     SPELL_REGENERATION,
-    SPELL_SONG_OF_SHIELDING,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
@@ -322,32 +320,32 @@
 },
 
 {   // Book of Earth
-    SPELL_DIG,
     SPELL_LEDAS_LIQUEFACTION,
     SPELL_BOLT_OF_MAGMA,
     SPELL_STATUE_FORM,
     SPELL_IRON_SHOT,
     SPELL_SHATTER,
     SPELL_NO_SPELL,
-    SPELL_NO_SPELL
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
 },
 
 {   // Book of Wizardry
     SPELL_SUMMON_ELEMENTAL,
     SPELL_AGONY,
     SPELL_FORCE_LANCE,
-    SPELL_FIREBALL,
     SPELL_HASTE,
     SPELL_INVISIBILITY,
+    SPELL_SPELLFORGED_SERVITOR,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
 
 {   // Book of Power
+    SPELL_FULMINANT_PRISM,
     SPELL_ISKENDERUNS_MYSTIC_BLAST,
     SPELL_VENOM_BOLT,
     SPELL_BOLT_OF_MAGMA,
-    SPELL_FULMINANT_PRISM,
     SPELL_IRON_SHOT,
     SPELL_IOOD,
     SPELL_NO_SPELL,
@@ -378,9 +376,9 @@
 
 #if TAG_MAJOR_VERSION == 34
 {   // Book of Stalking
-    SPELL_PASSWALL,
-    SPELL_PETRIFY,
-    SPELL_DIG,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -405,7 +403,7 @@
     SPELL_CAUSE_FEAR,
     SPELL_BOLT_OF_FIRE,
     SPELL_DRAGON_FORM,
-    SPELL_SUMMON_DRAGON,
+    SPELL_DRAGON_CALL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -437,19 +435,19 @@
     SPELL_LETHAL_INFUSION,
     SPELL_SUBLIMATION_OF_BLOOD,
     SPELL_CONDENSATION_SHIELD,
-    SPELL_INTOXICATE,
     SPELL_PETRIFY,
     SPELL_IGNITE_POISON,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
 },
 
 {   // Book of Beasts
     SPELL_SUMMON_BUTTERFLIES,
-    SPELL_STICKS_TO_SNAKES,
     SPELL_CALL_CANINE_FAMILIAR,
-    SPELL_ICE_FORM,
     SPELL_SUMMON_ICE_BEAST,
+    SPELL_SUMMON_MANA_VIPER,
+    SPELL_MONSTROUS_MENAGERIE,
     SPELL_SUMMON_HYDRA,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -457,18 +455,18 @@
 
 {   // Book of Annihilations
     SPELL_POISON_ARROW,
-    SPELL_IOOD,
     SPELL_CHAIN_LIGHTNING,
     SPELL_LEHUDIBS_CRYSTAL_SPEAR,
-    SPELL_ICE_STORM,
+    SPELL_GLACIATE,
     SPELL_FIRE_STORM,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
 },
 
 {   // Grand Grimoire
-    SPELL_DEMONIC_HORDE,
-    SPELL_MASS_ABJURATION,
+    SPELL_AURA_OF_ABJURATION,
+    SPELL_FORCEFUL_DISMISSAL,
     SPELL_HAUNT,
     SPELL_SUMMON_GREATER_DEMON,
     SPELL_MALIGN_GATEWAY,
@@ -478,8 +476,8 @@
 },
 
 {   // Necronomicon
-    SPELL_BORGNJORS_REVIVIFICATION,
     SPELL_HAUNT,
+    SPELL_BORGNJORS_REVIVIFICATION,
     SPELL_DEATHS_DOOR,
     SPELL_NECROMUTATION,
     SPELL_NO_SPELL,
@@ -512,10 +510,10 @@
     SPELL_NO_SPELL,
 },
 
-{   // Rod of fiery destruction
-    SPELL_THROW_FLAME,
-    SPELL_FIREBALL,
-    SPELL_BOLT_OF_FIRE,
+{   // Rod of ignition
+    SPELL_EXPLOSIVE_BOLT,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -523,10 +521,10 @@
     SPELL_NO_SPELL,
 },
 
-{   // Rod of frigid destruction
-    SPELL_THROW_FROST,
-    SPELL_THROW_ICICLE,
-    SPELL_FREEZING_CLOUD,
+{   // Rod of clouds
+    SPELL_CLOUD_CONE,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -534,10 +532,10 @@
     SPELL_NO_SPELL,
 },
 
-{   // Rod of destruction (lightning, fireball, iron)
-    SPELL_LIGHTNING_BOLT,
-    SPELL_FIREBALL,
-    SPELL_IRON_SHOT,
+{   // Rod of destruction
+    SPELL_RANDOM_BOLT,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -556,21 +554,23 @@
     SPELL_NO_SPELL,
 },
 
+#if TAG_MAJOR_VERSION == 34
 {   // Rod of warding
-    SPELL_ABJURATION,
-    SPELL_CONDENSATION_SHIELD,
-    SPELL_CAUSE_FEAR,
-    SPELL_DEFLECT_MISSILES,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
+#endif
 
-{   // Rod of demonology
-    SPELL_CALL_IMP,
-    SPELL_ABJURATION,
-    SPELL_SUMMON_DEMON,
+{   // Rod of shadows
+    SPELL_WEAVE_SHADOWS,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
@@ -589,13 +589,15 @@
     SPELL_NO_SPELL,
 },
 
+#if TAG_MAJOR_VERSION == 34
 {   // Rod of venom
-    SPELL_OLGREBS_TOXIC_RADIANCE,
-    SPELL_POISONOUS_CLOUD,
-    SPELL_POISON_ARROW,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
+    SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
     SPELL_NO_SPELL,
 },
+#endif
