@@ -220,9 +220,9 @@ int CLua::execstring(const char *s, const char *context, int nresults)
 bool CLua::is_path_safe(string s, bool trusted)
 {
     lowercase(s);
-    return (s.find("..") == string::npos && shell_safe(s.c_str())
-            // loading dlua stuff would spew tons of error messages
-            && (trusted || s.find("dlua") != 0));
+    return s.find("..") == string::npos && shell_safe(s.c_str())
+           // loading dlua stuff would spew tons of error messages
+           && (trusted || s.find("dlua") != 0);
 }
 
 int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
@@ -260,7 +260,7 @@ int CLua::loadfile(lua_State *ls, const char *filename, bool trusted,
 int CLua::execfile(const char *filename, bool trusted, bool die_on_fail,
                    bool force)
 {
-    if (!force && sourced_files.find(filename) != sourced_files.end())
+    if (!force && sourced_files.count(filename))
         return 0;
 
     sourced_files.insert(filename);
@@ -529,7 +529,7 @@ bool CLua::callbooleanfn(bool def, const char *fn, const char *params, ...)
 
 bool CLua::proc_returns(const char *par) const
 {
-    return (strchr(par, '>') != NULL);
+    return strchr(par, '>') != NULL;
 }
 
 // Identical to lua_getglobal for simple names, but will look up
@@ -1017,7 +1017,7 @@ lua_call_throttle::~lua_call_throttle()
 CLua *lua_call_throttle::find_clua(lua_State *ls)
 {
     lua_clua_map::iterator i = lua_map.find(ls);
-    return (i != lua_map.end()? i->second : NULL);
+    return i != lua_map.end()? i->second : NULL;
 }
 
 // This function is a replacement for Lua's in-built pcall function. It behaves
