@@ -13,12 +13,13 @@
 
 #include "AppHdr.h"
 
-#if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_TESTS)
+#ifdef DEBUG_TESTS
 
 #include "ctest.h"
 
 #include "clua.h"
 #include "cluautil.h"
+#include "coordit.h"
 #include "dlua.h"
 #include "errors.h"
 #include "files.h"
@@ -37,9 +38,6 @@
 
 static const string test_dir = "test";
 static const string script_dir = "scripts";
-static const string test_player_name = "Superbug99";
-static const species_type test_player_species = SP_HUMAN;
-static const job_type test_player_job = JOB_FIGHTER;
 static const char *activity = "test";
 
 static int ntests = 0;
@@ -53,9 +51,9 @@ static void _reset_test_data()
     ntests = 0;
     nsuccess = 0;
     failures.clear();
-    you.your_name = test_player_name;
-    you.species = test_player_species;
-    you.char_class = test_player_job;
+    you.your_name = "Superbug99";
+    you.species = SP_HUMAN;
+    you.char_class = JOB_FIGHTER;
 }
 
 static int crawl_begin_test(lua_State *ls)
@@ -99,13 +97,20 @@ static void _init_test_bindings()
 
 static bool _is_test_selected(const string &testname)
 {
+    if (crawl_state.test_list)
+    {
+        ASSERT(ends_with(testname, ".lua"));
+        printf("%s\n", testname.substr(0, testname.length() - 4).c_str());
+        return false;
+    }
+
     if (crawl_state.tests_selected.empty() && !starts_with(testname, "big/"))
         return true;
     for (int i = 0, size = crawl_state.tests_selected.size();
          i < size; ++i)
     {
         const string &phrase(crawl_state.tests_selected[i]);
-        if (testname.find(phrase) != string::npos)
+        if (testname == phrase || testname == phrase + ".lua")
             return true;
     }
     return false;
@@ -140,6 +145,9 @@ static bool _has_test(const string& test)
 
 static void _run_test(const string &name, void (*func)(void))
 {
+    if (crawl_state.test_list)
+        return (void)printf("%s\n", name.c_str());
+
     if (!_has_test(name))
         return;
 
@@ -154,7 +162,7 @@ static void _run_test(const string &name, void (*func)(void))
 }
 
 // Assumes curses has already been initialized.
-bool run_tests(bool exit_on_complete)
+void run_tests()
 {
     if (crawl_state.script)
         activity = "script";
@@ -171,38 +179,40 @@ bool run_tests(bool exit_on_complete)
     _run_test("zotdef_wave", debug_waves);
     _run_test("mon-pick", debug_monpick);
     _run_test("mon-data", debug_mondata);
+    _run_test("coordit", coordit_tests);
 
     // Get a list of Lua files in test. Order of execution of
     // tests should be irrelevant.
-    const vector<string> tests(
-        get_dir_files_recursive(crawl_state.script? script_dir : test_dir,
-                          ".lua"));
-    for_each(tests.begin(), tests.end(), run_test);
-
-    if (failures.empty() && !ntests && crawl_state.script)
-        failures.push_back(
-            file_error(
-                "Script setup",
-                "No scripts found matching " +
-                comma_separated_line(crawl_state.tests_selected.begin(),
-                                     crawl_state.tests_selected.end(),
-                                     ", ",
-                                     ", ")));
-
-    if (exit_on_complete)
     {
-        cio_cleanup();
-        for (int i = 0, size = failures.size(); i < size; ++i)
-        {
-            const file_error &fe(failures[i]);
-            fprintf(stderr, "%s error: %s\n",
-                    activity, fe.second.c_str());
-        }
-        const int code = failures.empty() ? 0 : 1;
-        end(code, false, "%d %ss, %d succeeded, %d failed",
-            ntests, activity, nsuccess, (int)failures.size());
+        const vector<string> tests(
+            get_dir_files_recursive(crawl_state.script? script_dir : test_dir,
+                              ".lua"));
+
+        for_each(tests.begin(), tests.end(), run_test);
+
+        if (failures.empty() && !ntests && crawl_state.script)
+            failures.push_back(
+                file_error(
+                    "Script setup",
+                    "No scripts found matching " +
+                    comma_separated_line(crawl_state.tests_selected.begin(),
+                                         crawl_state.tests_selected.end(),
+                                         ", ",
+                                         ", ")));
     }
-    return failures.empty();
+
+    if (crawl_state.test_list)
+        end(0);
+    cio_cleanup();
+    for (int i = 0, size = failures.size(); i < size; ++i)
+    {
+        const file_error &fe(failures[i]);
+        fprintf(stderr, "%s error: %s\n",
+                activity, fe.second.c_str());
+    }
+    const int code = failures.empty() ? 0 : 1;
+    end(code, false, "%d %ss, %d succeeded, %d failed",
+        ntests, activity, nsuccess, (int)failures.size());
 }
 
-#endif // DEBUG_DIAGNOSTICS
+#endif // DEBUG_TESTS
