@@ -44,8 +44,8 @@ static void _dump_item(const char *name, int num, const item_def &item,
     string msg = vmake_stringf(format, args);
     va_end(args);
 
-    mpr(msg.c_str(), chan);
-    mpr(name, chan);
+    mprf(chan, "%s", msg.c_str());
+    mprf(chan, "%s", name);
 
     mprf("    item #%d:  base: %d; sub: %d; plus: %d; plus2: %d; special: %d",
          num, item.base_type, item.sub_type,
@@ -233,10 +233,10 @@ void debug_item_scan(void)
 #ifdef DEBUG_MONS_SCAN
 static void _announce_level_prob(bool warned)
 {
-    if (!warned && Generating_Level)
+    if (!warned && crawl_state.generating_level)
     {
-        mpr("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", MSGCH_ERROR);
-        mpr("mgrd problem occurred during level generation", MSGCH_ERROR);
+        mprf(MSGCH_ERROR, "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+        mprf(MSGCH_ERROR, "mgrd problem occurred during level generation");
 
         debug_dump_levgen();
     }
@@ -246,8 +246,8 @@ static bool _inside_vault(const vault_placement& place, const coord_def &pos)
 {
     const coord_def delta = pos - place.pos;
 
-    return (delta.x >= 0 && delta.y >= 0
-            && delta.x < place.size.x && delta.y < place.size.y);
+    return delta.x >= 0 && delta.y >= 0
+           && delta.x < place.size.x && delta.y < place.size.y;
 }
 
 static vector<string> _in_vaults(const coord_def &pos)
@@ -271,6 +271,31 @@ static vector<string> _in_vaults(const coord_def &pos)
     return out;
 }
 
+static string _vault_desc(const coord_def pos)
+{
+    int mi = env.level_map_ids(pos);
+    if (mi == INVALID_MAP_INDEX)
+        return "";
+
+    string out;
+
+    for (unsigned int i = 0; i < env.level_vaults.size(); ++i)
+    {
+        const vault_placement &vp = *env.level_vaults[i];
+        if (_inside_vault(vp, pos))
+        {
+            coord_def br = vp.pos + vp.size - 1;
+            out += make_stringf(" [vault: %s (%d,%d)-(%d,%d) (%dx%d)]",
+                        vp.map_name_at(pos).c_str(),
+                        vp.pos.x, vp.pos.y,
+                        br.x, br.y,
+                        vp.size.x, vp.size.y);
+        }
+    }
+
+    return out;
+}
+
 void debug_mons_scan()
 {
     vector<coord_def> bogus_pos;
@@ -306,7 +331,7 @@ void debug_mons_scan()
                      x, y, m->name(DESC_PLAIN, true).c_str(),
                      m->pos().x, m->pos().y);
                 if (!m->alive())
-                    mpr("Additionally, it isn't alive.", MSGCH_WARN);
+                    mprf(MSGCH_WARN, "Additionally, it isn't alive.");
                 warned = true;
             }
             else if (!m->alive())
@@ -384,6 +409,19 @@ void debug_mons_scan()
             }
         } // if (mgrd(m->pos()) != i)
 
+        if (feat_is_wall(grd(pos)))
+        {
+#if defined(DEBUG_FATAL)
+            // if we're going to dump, point out the culprit
+            env.pgrid(pos) |= FPROP_HIGHLIGHT;
+#endif
+            mprf(MSGCH_ERROR, "Monster %s in %s at (%d, %d)%s",
+                 m->full_name(DESC_PLAIN, true).c_str(),
+                 dungeon_feature_name(grd(pos)),
+                 pos.x, pos.y,
+                 _vault_desc(pos).c_str());
+        }
+
         for (int j = 0; j < NUM_MONSTER_SLOTS; ++j)
         {
             const int idx = m->inv[j];
@@ -444,21 +482,22 @@ void debug_mons_scan()
                 {
                     if (holder->inv[k] == idx)
                     {
-                        mpr("Other monster thinks it's holding the item, too.",
-                            MSGCH_WARN);
+                        mprf(MSGCH_WARN, "Other monster thinks it's holding the item, too.");
                         found = true;
                         break;
                     }
                 }
                 if (!found)
-                    mpr("Other monster isn't holding it, though.", MSGCH_WARN);
+                    mprf(MSGCH_WARN, "Other monster isn't holding it, though.");
             } // if (holder != m)
         } // for (int j = 0; j < NUM_MONSTER_SLOTS; j++)
 
         monster* m1 = monster_by_mid(m->mid);
         if (m1 != m)
         {
-            if (m1 && m1->mid == m->mid)
+            if (!m1)
+                die("mid cache bogosity: no monster for %d", m->mid);
+            else if (m1->mid == m->mid)
             {
                 mprf(MSGCH_ERROR,
                      "Error: monster %s(%d) has same mid as %s(%d) (%d)",
@@ -466,7 +505,7 @@ void debug_mons_scan()
                      m1->name(DESC_PLAIN, true).c_str(), m1->mindex(), m->mid);
             }
             else
-                ASSERT(monster_by_mid(m->mid) == m);
+                die("mid cache bogosity: wanted %d got %d", m->mid, m1->mid);
         }
 
         if (you.constricted_by == m->mid && (!m->constricting
@@ -500,7 +539,12 @@ void debug_mons_scan()
     {
         unsigned short idx = mc->second;
         ASSERT(!invalid_monster_index(idx));
-        ASSERT(menv[idx].mid == mc->first);
+        if (menv[idx].mid != mc->first)
+        {
+            monster &m(menv[idx]);
+            die("mid cache bogosity: mid %d points to %s mindex=%d mid=%d",
+                mc->first, m.name(DESC_PLAIN, true).c_str(), m.mindex(), m.mid);
+        }
     }
 
     if (in_bounds(you.pos()))
@@ -517,7 +561,7 @@ void debug_mons_scan()
 
     // If this wasn't the result of generating a level then there's nothing
     // more to report.
-    if (!Generating_Level)
+    if (!crawl_state.generating_level)
     {
         // Force the dev to notice problems. :P
         more();
@@ -527,7 +571,7 @@ void debug_mons_scan()
     // No vaults to report on?
     if (env.level_vaults.empty() && Temp_Vaults.empty())
     {
-        mpr("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", MSGCH_ERROR);
+        mprf(MSGCH_ERROR, "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
         // Force the dev to notice problems. :P
         more();
         return;
@@ -591,23 +635,30 @@ void debug_mons_scan()
         }
     }
 
-    mpr("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", MSGCH_ERROR);
+    mprf(MSGCH_ERROR, "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
     // Force the dev to notice problems. :P
     more();
 }
 #endif
 
+/**
+ * Check the map for validity, generating a crash if not.
+ *
+ * This checks the loaded map to make sure all dungeon features and shops are
+ * valid, all branch exits are generated, and all portals generated at fixed
+ * levels in the Depths are actually present.
+ */
 void check_map_validity()
 {
 #ifdef ASSERTS
     dungeon_feature_type portal = DNGN_UNSEEN;
-    if (you.where_are_you == BRANCH_MAIN_DUNGEON)
+    if (you.where_are_you == BRANCH_DEPTHS)
     {
-        if (you.depth == 24)
+        if (you.depth == 3)
             portal = DNGN_ENTER_PANDEMONIUM;
-        else if (you.depth == 25)
+        else if (you.depth == 4)
             portal = DNGN_ENTER_ABYSS;
-        else if (you.depth == 21)
+        else if (you.depth == 2)
             portal = DNGN_ENTER_HELL;
     }
 
@@ -643,9 +694,10 @@ void check_map_validity()
             continue;
         // no mimics below
 
-        if (feat == portal)
+        const dungeon_feature_type orig = orig_terrain(*ri);
+        if (feat == portal || orig == portal)
             portal = DNGN_UNSEEN;
-        if (feat == exit)
+        if (feat == exit || orig == exit)
             exit = DNGN_UNSEEN;
     }
 
