@@ -59,8 +59,7 @@ void get_feature_desc(const coord_def &gc, describe_info &inf);
 
 bool describe_item(item_def &item, bool allow_inscribe = false,
                    bool shopping = false);
-void get_item_desc(const item_def &item, describe_info &inf,
-                   bool terse = false);
+void get_item_desc(const item_def &item, describe_info &inf);
 void inscribe_item(item_def &item, bool msgwin);
 
 void append_weapon_stats(string &description, const item_def &item);
