@@ -35,9 +35,11 @@ static void _show_message_line(string line)
         fs.textcolor(LIGHTGREY);
         fs.cprintf("%s", line.c_str());
         formatted_mpr(fs, MSGCH_PLAIN, 0);
-        if (Options.note_chat_messages)
+        if (Options.note_dgl_messages)
+        {
             take_note(Note(NOTE_MESSAGE, MSGCH_PLAIN, 0,
                            (sender + ": " + line).c_str()));
+        }
     }
 }
 
@@ -123,7 +125,7 @@ void read_messages()
 static void _announce_messages()
 {
     // XXX: We could do a NetHack-like mail daemon here at some point.
-    mprf("Beep! Your pager goes off! Use _ to check your messages.");
+    mpr("Beep! Your pager goes off! Use _ to check your messages.");
 }
 
 void check_messages()
