@@ -3,7 +3,6 @@
  * @brief Misc stuff.
 **/
 
-
 #ifndef EFFECTS_H
 #define EFFECTS_H
 
@@ -34,11 +33,10 @@ int place_ring(vector<coord_def>& ring_points,
                int arc_occupancy,
                int& seen_count);
 
-class los_base;
-// Collect lists of points that are within LOS (under the given losgrid),
-// unoccupied, and not solid (walls/statues).
+// Collect lists of points that are within LOS, unoccupied, and not solid
+// (walls/statues).
 void collect_radius_points(vector<vector<coord_def> > &radius_points,
-                           const coord_def &origin, const los_base* los);
+                           const coord_def &origin, los_type los);
 
 void random_uselessness(int scroll_slot = -1);
 
@@ -59,10 +57,6 @@ int torment(actor *attacker, int taux, const coord_def& where);
 int torment_player(actor *attacker, int taux);
 int torment_monsters(coord_def where, actor *attacker, int taux);
 
-void immolation(int pow, immolation_source_type source, bool known = false);
-
-void conduct_electricity(coord_def where, actor *attacker);
-
 void cleansing_flame(int pow, int caster, coord_def where,
                      actor *attacker = NULL);
 
@@ -72,7 +66,7 @@ bool vitrify_area(int radius);
 void update_level(int elapsedTime);
 void handle_time();
 void recharge_rods(int aut, bool floor_only);
-void recharge_elemental_evokers(int exp);
+void recharge_xp_evokers(int exp);
 
 void slime_wall_damage(actor* act, int delay);
 
