@@ -27,6 +27,7 @@ skill_type abil_skill(ability_type abil)
     case ABIL_EVOKE_FLIGHT:
     case ABIL_EVOKE_FOG:
     case ABIL_EVOKE_TELEPORT_CONTROL:
+    case ABIL_EVOKE_JUMP:
         return SK_EVOCATIONS;
 
     case ABIL_NEMELEX_DRAW_ONE:
@@ -44,7 +45,6 @@ skill_type abil_skill(ability_type abil)
     case ABIL_ZIN_RECITE:
     case ABIL_SIF_MUNA_CHANNEL_ENERGY:
     case ABIL_OKAWARU_HEROISM:
-    case ABIL_JIYVA_CALL_JELLY:
     case ABIL_ZIN_VITALISATION:
     case ABIL_TSO_DIVINE_SHIELD:
     case ABIL_BEOGH_SMITING:
@@ -66,12 +66,10 @@ skill_type abil_skill(ability_type abil)
     case ABIL_ELYVILON_GREATER_HEALING_SELF:
     case ABIL_ELYVILON_GREATER_HEALING_OTHERS:
     case ABIL_LUGONU_BANISH:
-    case ABIL_JIYVA_SLIMIFY:
     case ABIL_TSO_CLEANSING_FLAME:
     case ABIL_OKAWARU_FINESSE:
     case ABIL_CHEIBRIADOS_SLOUCH:
     case ABIL_LUGONU_CORRUPT:
-    case ABIL_JIYVA_CURE_BAD_MUTATION:
     case ABIL_CHEIBRIADOS_TIME_STEP:
     case ABIL_ZIN_SANCTUARY:
     case ABIL_MAKHLEB_GREATER_SERVANT_OF_MAKHLEB:
@@ -80,6 +78,8 @@ skill_type abil_skill(ability_type abil)
     case ABIL_YRED_ENSLAVE_SOUL:
     case ABIL_LUGONU_ABYSS_EXIT:
     case ABIL_CHEIBRIADOS_DISTORTION:
+    case ABIL_DITHMENOS_SHADOW_STEP:
+    case ABIL_DITHMENOS_SHADOW_FORM:
         return SK_INVOCATIONS;
 
     case ABIL_KIKU_RECEIVE_CORPSES:
@@ -102,6 +102,7 @@ static int _abil_degree(ability_type abil)
     case ABIL_EVOKE_FLIGHT:
     case ABIL_EVOKE_FOG:
     case ABIL_EVOKE_TELEPORT_CONTROL:
+    case ABIL_EVOKE_JUMP:
         return 1;
 
     case ABIL_NEMELEX_DRAW_ONE:
@@ -123,7 +124,7 @@ static int _abil_degree(ability_type abil)
         return 1;
     case ABIL_SIF_MUNA_CHANNEL_ENERGY:
     case ABIL_OKAWARU_HEROISM:
-    case ABIL_JIYVA_CALL_JELLY:
+    case ABIL_DITHMENOS_SHADOW_STEP:
         return 1 + random2(3);
 
     case ABIL_ZIN_RECITE:
@@ -152,7 +153,6 @@ static int _abil_degree(ability_type abil)
     case ABIL_ELYVILON_GREATER_HEALING_SELF:
     case ABIL_ELYVILON_GREATER_HEALING_OTHERS:
     case ABIL_LUGONU_BANISH:
-    case ABIL_JIYVA_SLIMIFY:
     case ABIL_CHEIBRIADOS_DISTORTION:
         return 3 + random2(5);
     case ABIL_TSO_CLEANSING_FLAME:
@@ -166,7 +166,6 @@ static int _abil_degree(ability_type abil)
         return 4 + random2(6);
 
     case ABIL_LUGONU_CORRUPT:
-    case ABIL_JIYVA_CURE_BAD_MUTATION:
     case ABIL_CHEIBRIADOS_TIME_STEP:
     case ABIL_KIKU_TORMENT:
         return 5 + random2(5);
@@ -174,6 +173,7 @@ static int _abil_degree(ability_type abil)
         return 5 + random2(8);
 
     case ABIL_MAKHLEB_GREATER_SERVANT_OF_MAKHLEB:
+    case ABIL_DITHMENOS_SHADOW_FORM:
         return 6 + random2(6);
     case ABIL_ELYVILON_DIVINE_VIGOUR:
         return 6 + random2(10);
@@ -366,6 +366,7 @@ void practise(exer_type ex, int param1)
         switch (param1) // missile subtype
         {
         case MI_DART:
+        case MI_TOMAHAWK:
         case MI_JAVELIN:
         case MI_THROWING_NET:
             deg = 1;
