@@ -14,6 +14,7 @@
 #include "mgen_data.h"
 #include "misc.h"
 #include "mon-abil.h"
+#include "mon-cast.h"
 #include "mon-place.h"
 #include "ouch.h"
 #include "religion.h"
@@ -36,12 +37,6 @@ void final_effect::schedule()
     env.final_effects.push_back(this);
 }
 
-bool lightning_fineff::mergeable(const final_effect &fe) const
-{
-    const lightning_fineff *o = dynamic_cast<const lightning_fineff *>(&fe);
-    return o && att == o->att && posn == o->posn;
-}
-
 bool mirror_damage_fineff::mergeable(const final_effect &fe) const
 {
     const mirror_damage_fineff *o =
@@ -94,11 +89,24 @@ bool starcursed_merge_fineff::mergeable(const final_effect &fe) const
     return o && def == o->def;
 }
 
+bool shock_serpent_discharge_fineff::mergeable(const final_effect &fe) const
+{
+    const shock_serpent_discharge_fineff *o = dynamic_cast<const shock_serpent_discharge_fineff *>(&fe);
+    return o && def == o->def;
+}
+
 bool delayed_action_fineff::mergeable(const final_effect &fe) const
 {
     return false;
 }
 
+bool rakshasa_clone_fineff::mergeable(const final_effect &fe) const
+{
+    const rakshasa_clone_fineff *o =
+        dynamic_cast<const rakshasa_clone_fineff *>(&fe);
+    return o && att == o->att && def == o->def && posn == o->posn;
+}
+
 void mirror_damage_fineff::merge(const final_effect &fe)
 {
     const mirror_damage_fineff *mdfe =
@@ -134,12 +142,11 @@ void deferred_damage_fineff::merge(const final_effect &fe)
     damage += ddamfe->damage;
 }
 
-
-void lightning_fineff::fire()
+void shock_serpent_discharge_fineff::merge(const final_effect &fe)
 {
-    if (you.see_cell(posn))
-        mpr("Electricity arcs through the water!");
-    conduct_electricity(posn, attacker());
+    const shock_serpent_discharge_fineff *ssdfe =
+        dynamic_cast<const shock_serpent_discharge_fineff *>(&fe);
+    power += ssdfe->power;
 }
 
 void mirror_damage_fineff::fire()
@@ -198,7 +205,7 @@ void distortion_tele_fineff::fire()
 {
     actor *defend = defender();
     if (defend && defend->alive() && !defend->no_tele(true, false))
-        defend->teleport(true, one_chance_in(5));
+        defend->teleport(true);
 }
 
 void trj_spawn_fineff::fire()
@@ -306,9 +313,16 @@ void starcursed_merge_fineff::fire()
         starcursed_merge(defender()->as_monster(), true);
 }
 
+void shock_serpent_discharge_fineff::fire()
+{
+    actor *defend = defender();
+    shock_serpent_discharge((defend ? defend->as_monster() : NULL), position,
+                             power, attitude);
+}
+
 void delayed_action_fineff::fire()
 {
-    if (final_msg.length())
+    if (final_msg)
         mpr(final_msg);
     add_daction(action);
 }
@@ -322,6 +336,28 @@ void kirke_death_fineff::fire()
         untransform();
 }
 
+void rakshasa_clone_fineff::fire()
+{
+    actor *defend = defender();
+    if (!defend)
+        return;
+
+    monster *rakshasa = defend->as_monster();
+    ASSERT(rakshasa);
+
+    // Using SPELL_NO_SPELL to prevent overwriting normal clones
+    cast_phantom_mirror(rakshasa, rakshasa, 50, SPELL_NO_SPELL);
+    cast_phantom_mirror(rakshasa, rakshasa, 50, SPELL_NO_SPELL);
+    rakshasa->lose_energy(EUT_SPELL);
+
+    if (you.can_see(rakshasa))
+    {
+        mprf(MSGCH_MONSTER_SPELL,
+             "The injured %s weaves a defensive illusion!",
+             rakshasa->name(DESC_PLAIN).c_str());
+    }
+}
+
 // Effects that occur after all other effects, even if the monster is dead.
 // For example, explosions that would hit other creatures, but we want
 // to deal with only one creature at a time, so that's handled last.
