@@ -5,7 +5,6 @@
 #include "colour.h"
 #include "format.h"
 #include "libutil.h"
-#include "showsymb.h"
 #include "lang-fake.h"
 #include "unicode.h"
 #include "viewchar.h"
@@ -34,7 +33,7 @@ int formatted_string::get_colour(const string &tag)
         return WHITE;
 
     const int colour = str_to_colour(tag);
-    return (colour != -1? colour : LIGHTGREY);
+    return colour != -1? colour : LIGHTGREY;
 }
 
 // Display a formatted string without printing literal \n.
@@ -137,7 +136,7 @@ void formatted_string::parse_string1(const string &s, formatted_string &fs,
             continue;
         }
 
-        if (s[tag] != '<' || tag >= length - 2)
+        if (s[tag] != '<' || tag >= length - 1)
         {
             if (!masked)
                 currs += s[tag];
@@ -196,9 +195,17 @@ void formatted_string::parse_string1(const string &s, formatted_string &fs,
 
         if (revert_colour)
         {
-            colour_stack.pop_back();
-            if (colour_stack.size() < 1)
-                die("Stack underflow in string \"%s\"", s.c_str());
+            const int endcolour = get_colour(tagtext);
+
+            if (colour_stack.size() > 1 && endcolour == colour_stack.back())
+                colour_stack.pop_back();
+            else
+            {
+                // If this was the only tag, or the colour didn't match
+                // the one we are popping, display the tag as a warning.
+                fs.textcolor(LIGHTRED);
+                fs.cprintf("</%s>", tagtext.c_str());
+            }
         }
         else
             colour_stack.push_back(get_colour(tagtext));
@@ -312,7 +319,6 @@ char &formatted_string::operator [] (size_t idx)
     die("Invalid index");
 }
 
-
 string formatted_string::tostring(int s, int e) const
 {
     string st;
@@ -559,7 +565,6 @@ static int _tagged_string_printable_length(const string& s)
     return len;
 }
 
-
 // Count the length of the tags in the string.
 int tagged_string_tag_length(const string& s)
 {
