@@ -18,21 +18,21 @@ class formatted_string
 {
 public:
     formatted_string(int init_colour = 0);
-    explicit formatted_string(const std::string &s, int init_colour = 0);
+    explicit formatted_string(const string &s, int init_colour = 0);
 
-    operator std::string() const;
+    operator string() const;
     void display(int start = 0, int end = -1) const;
-    std::string tostring(int start = 0, int end = -1) const;
-    std::string to_colour_string() const;
+    string tostring(int start = 0, int end = -1) const;
+    string to_colour_string() const;
 
     void cprintf(PRINTF(1, ));
-    void cprintf(const std::string &s);
-    void add_glyph(glyph g);
+    void cprintf(const string &s);
+    void add_glyph(cglyph_t g);
     void textcolor(int color);
     formatted_string chop(int length) const;
     void del_char();
     void all_caps();
-    void capitalize();
+    void capitalise();
     void filter_lang();
 
     void clear();
@@ -41,7 +41,7 @@ public:
     void swap(formatted_string& other);
 
     int width() const;
-    std::string html_dump() const;
+    string html_dump() const;
 
     bool operator < (const formatted_string &other) const;
     const formatted_string &operator += (const formatted_string &other);
@@ -49,25 +49,22 @@ public:
 
 public:
     static formatted_string parse_string(
-            const std::string &s,
+            const string &s,
             bool  eot_ends_format = true,
-            bool (*process_tag)(const std::string &tag) = NULL,
+            bool (*process_tag)(const string &tag) = NULL,
             int main_colour = LIGHTGREY);
 
-    static void parse_string_to_multiple(
-            const std::string &s,
-            std::vector<formatted_string> &out);
+    static void parse_string_to_multiple(const string &s,
+                                         vector<formatted_string> &out);
 
-    static int get_colour(const std::string &tag);
+    static int get_colour(const string &tag);
 
 private:
     int find_last_colour() const;
 
-    static void parse_string1(
-            const std::string &s,
-            formatted_string &fs,
-            std::vector<int> &colour_stack,
-            bool (*process_tag)(const std::string &tag));
+    static void parse_string1(const string &s, formatted_string &fs,
+                              vector<int> &colour_stack,
+                              bool (*process_tag)(const string &tag));
 
 public:
     struct fs_op
@@ -75,14 +72,14 @@ public:
         fs_op_type type;
         int x, y;
         bool relative;
-        std::string text;
+        string text;
 
         fs_op(int color)
             : type(FSOP_COLOUR), x(color), y(-1), relative(false), text()
         {
         }
 
-        fs_op(const std::string &s)
+        fs_op(const string &s)
             : type(FSOP_TEXT), x(-1), y(-1), relative(false), text(s)
         {
         }
@@ -94,13 +91,13 @@ public:
         void display() const;
     };
 
-    typedef std::vector<fs_op> oplist;
+    typedef vector<fs_op> oplist;
     oplist ops;
 };
 
 int count_linebreaks(const formatted_string& fs);
 
-int tagged_string_tag_length(const std::string& s);
-void display_tagged_block(const std::string& s);
+int tagged_string_tag_length(const string& s);
+void display_tagged_block(const string& s);
 
 #endif
