@@ -7,6 +7,7 @@
 
 #include "ghost.h"
 
+#include "act-iter.h"
 #include "artefact.h"
 #include "colour.h"
 #include "database.h"
@@ -15,15 +16,14 @@
 #include "itemname.h"
 #include "itemprop.h"
 #include "libutil.h"
-#include "mon-iter.h"
 #include "ng-input.h"
 #include "random.h"
 #include "skills2.h"
 #include "spl-cast.h"
+#include "spl-util.h"
 #include "mon-util.h"
 #include "mon-transit.h"
 #include "player.h"
-#include "religion.h"
 
 #include <vector>
 
@@ -37,10 +37,11 @@ vector<ghost_demon> ghosts;
 
 // Order for looking for conjurations for the 1st & 2nd spell slots,
 // when finding spells to be remembered by a player's ghost.
-static spell_type search_order_conj[] = {
+static spell_type search_order_conj[] =
+{
     SPELL_LEHUDIBS_CRYSTAL_SPEAR,
     SPELL_FIRE_STORM,
-    SPELL_ICE_STORM,
+    SPELL_GLACIATE,
     SPELL_CHAIN_LIGHTNING,
     SPELL_BOLT_OF_DRAINING,
     SPELL_AGONY,
@@ -61,6 +62,8 @@ static spell_type search_order_conj[] = {
     SPELL_LRD,
     SPELL_STONE_ARROW,
     SPELL_FORCE_LANCE,
+    SPELL_DISCHARGE,
+    SPELL_DAZZLING_SPRAY,
     SPELL_THROW_FLAME,
     SPELL_THROW_FROST,
     SPELL_FREEZE,
@@ -77,28 +80,27 @@ static spell_type search_order_conj[] = {
 
 // Order for looking for summonings and self-enchants for the 3rd spell
 // slot.
-static spell_type search_order_third[] = {
+static spell_type search_order_third[] =
+{
     SPELL_SYMBOL_OF_TORMENT,
     SPELL_SUMMON_GREATER_DEMON,
+    SPELL_DRAGON_CALL,
     SPELL_SUMMON_HORRIBLE_THINGS,
-    SPELL_SUMMON_DRAGON,
     SPELL_HAUNT,
     SPELL_SUMMON_HYDRA,
     SPELL_SUMMON_DEMON,
-    SPELL_DEMONIC_HORDE,
     SPELL_HASTE,
     SPELL_SILENCE,
     SPELL_BATTLESPHERE,
     SPELL_SUMMON_BUTTERFLIES,
     SPELL_SUMMON_ELEMENTAL,
     SPELL_SUMMON_SWARM,
-    SPELL_SUMMON_UGLY_THING,
+    SPELL_MONSTROUS_MENAGERIE,
     SPELL_SWIFTNESS,
     SPELL_SUMMON_ICE_BEAST,
     SPELL_ANIMATE_DEAD,
     SPELL_TWISTED_RESURRECTION,
     SPELL_INVISIBILITY,
-    SPELL_SUMMON_SCORPIONS,
     SPELL_CALL_IMP,
     SPELL_SUMMON_SMALL_MAMMAL,
     SPELL_MALIGN_GATEWAY,
@@ -112,7 +114,8 @@ static spell_type search_order_third[] = {
 // Order for looking for enchants for the 4th & 5th spell slots.  If
 // this fails, go through conjurations.  Note: Dig must be in misc2
 // (5th) position to work.
-static spell_type search_order_misc[] = {
+static spell_type search_order_misc[] =
+{
     SPELL_SHATTER,
     SPELL_AGONY,
     SPELL_BANISHMENT,
@@ -181,9 +184,9 @@ void ghost_demon::init_random_demon()
 
     // Is demon a spellcaster?
     // Non-spellcasters get some boosts to their melee and speed instead.
-    spellcaster = !one_chance_in(3);
+    spellcaster = !one_chance_in(4);
 
-    see_invis = !one_chance_in(10);
+    see_invis = true;
 
     resists = 0;
 
@@ -219,14 +222,15 @@ void ghost_demon::init_random_demon()
         while (brand == SPWPN_HOLY_WRATH
 #if TAG_MAJOR_VERSION == 34
                || brand == SPWPN_ORC_SLAYING
+               || brand == SPWPN_RETURNING
+               || brand == SPWPN_REACHING
 #endif
                || brand == SPWPN_DRAGON_SLAYING
                || brand == SPWPN_PROTECTION
                || brand == SPWPN_EVASION
                || brand == SPWPN_FLAME
                || brand == SPWPN_FROST
-               || brand == SPWPN_RETURNING
-               || brand == SPWPN_REACHING);
+               );
     }
 
     // Does demon fly?
@@ -300,16 +304,18 @@ void ghost_demon::init_random_demon()
         if (one_chance_in(25))
             spells[0] = SPELL_FIRE_STORM;
         if (one_chance_in(25))
-            spells[0] = SPELL_ICE_STORM;
+            spells[0] = SPELL_GLACIATE;
         if (one_chance_in(25))
             spells[0] = SPELL_METAL_SPLINTERS;
         if (one_chance_in(25))
             spells[0] = SPELL_ENERGY_BOLT;  // eye of devastation
+        if (one_chance_in(25))
+            spells[0] = SPELL_ORB_OF_ELECTRICITY;
 
         if (one_chance_in(25))
             spells[1] = SPELL_STEAM_BALL;
         if (one_chance_in(25))
-            spells[1] = SPELL_ISKENDERUNS_MYSTIC_BLAST;
+            spells[1] = SPELL_QUICKSILVER_BOLT;
         if (one_chance_in(25))
             spells[1] = SPELL_HELLFIRE;
         if (one_chance_in(25))
@@ -352,13 +358,13 @@ static int _player_ghost_base_movement_speed()
 {
     int speed = 10;
 
-    if (player_mutation_level(MUT_FAST, false))
-        speed += player_mutation_level(MUT_FAST, false) + 1;
-    if (player_mutation_level(MUT_SLOW, false))
-        speed -= player_mutation_level(MUT_SLOW, false) + 1;
+    if (int fast = player_mutation_level(MUT_FAST, false))
+        speed += fast + 1;
+    if (int slow = player_mutation_level(MUT_SLOW, false))
+        speed -= slow + 1;
 
     if (you.wearing_ego(EQ_BOOTS, SPARM_RUNNING))
-        speed += 2;
+        speed += 1;
 
     // Cap speeds.
     if (speed < MIN_GHOST_SPEED)
@@ -499,8 +505,8 @@ static attack_flavour _very_ugly_thing_flavour_upgrade(attack_flavour u_att_flav
         u_att_flav = AF_NAPALM;
         break;
 
-    case AF_POISON_NASTY:
-        u_att_flav = AF_POISON_MEDIUM;
+    case AF_POISON:
+        u_att_flav = AF_POISON_STRONG;
         break;
 
     case AF_DISEASE:
@@ -529,7 +535,7 @@ static attack_flavour _ugly_thing_colour_to_flavour(colour_t u_colour)
         break;
 
     case GREEN:
-        u_att_flav = AF_POISON_NASTY;
+        u_att_flav = AF_POISON;
         break;
 
     case CYAN:
@@ -557,8 +563,8 @@ static attack_flavour _ugly_thing_colour_to_flavour(colour_t u_colour)
 void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
                                   colour_t force_colour)
 {
-    // Movement speed: 11, the same as in mon-data.h.
-    speed = 11;
+    // Movement speed: 10, the same as in mon-data.h.
+    speed = 10;
 
     // Midpoint: 10, as in mon-data.h.
     ev = 9 + random2(3);
@@ -643,8 +649,8 @@ static resists_t _ugly_thing_resists(bool very_ugly, attack_flavour u_att_flav)
     case AF_ACID:
         return MR_RES_ACID;
 
-    case AF_POISON_NASTY:
-    case AF_POISON_MEDIUM:
+    case AF_POISON:
+    case AF_POISON_STRONG:
         return MR_RES_POISON * (very_ugly ? 2 : 1);
 
     case AF_ELEC:
@@ -692,7 +698,7 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     // Bardiche:          speed 10, 40+20 damage, 20 AC, 40 HP, 15 EV
     // Dagger:            speed 20,  8+ 4 damage,  2 AC,  4 HP, 20 EV
     // Quick blade:       speed 23, 10+ 5 damage,  5 AC, 10 HP, 22 EV
-    // Sabre:             speed 18, 14+ 7 damage,  9 AC, 18 HP, 19 EV
+    // Cutlass:           speed 18, 14+ 7 damage,  9 AC, 18 HP, 19 EV
 
     xl = 15;
 
@@ -770,7 +776,7 @@ void ghost_demon::init_spectral_weapon(const item_def& weapon,
 
 static bool _know_spell(spell_type spell)
 {
-    return (you.has_spell(spell) && spell_fail(spell) < 50);
+    return you.has_spell(spell) && spell_fail(spell) < 50;
 }
 
 static spell_type search_first_list(int ignore_spell)
@@ -834,7 +840,7 @@ void ghost_demon::add_spells()
     spells[0] = search_first_list(SPELL_NO_SPELL);
     spells[1] = search_first_list(spells[0]);
     spells[2] = search_second_list(SPELL_NO_SPELL);
-    spells[3] = search_third_list(SPELL_DIG);
+    spells[3] = search_third_list(SPELL_NO_SPELL);
 
     if (spells[3] == SPELL_NO_SPELL)
         spells[3] = search_first_list(SPELL_NO_SPELL);
@@ -844,9 +850,6 @@ void ghost_demon::add_spells()
     if (spells[4] == SPELL_NO_SPELL)
         spells[4] = search_first_list(spells[3]);
 
-    if (_know_spell(SPELL_DIG))
-        spells[4] = SPELL_DIG;
-
     // Look for Blink or Teleport Self for the emergency slot.
     if (_know_spell(SPELL_CONTROLLED_BLINK)
         || _know_spell(SPELL_BLINK))
@@ -879,10 +882,10 @@ spell_type ghost_demon::translate_spell(spell_type spell) const
     {
     case SPELL_CONTROLLED_BLINK:
         return SPELL_BLINK;        // approximate
-    case SPELL_DEMONIC_HORDE:
-        return SPELL_SUMMON_MINOR_DEMON;
     case SPELL_DELAYED_FIREBALL:
         return SPELL_FIREBALL;
+    case SPELL_DRAGON_CALL:
+        return SPELL_SUMMON_DRAGON;
     default:
         break;
     }
@@ -966,7 +969,7 @@ int ghost_demon::n_extra_ghosts()
     if (env.absdepth0 < 10)
         return 0;
 
-    return (MAX_GHOSTS - 1);
+    return MAX_GHOSTS - 1;
 }
 
 // Sanity checks for some ghost values.
@@ -1043,3 +1046,138 @@ int ghost_level_to_rank(const int xl)
     if (xl < 27) return 6;
     return 7;
 }
+
+static spell_type servitor_spells_primary[] =
+{
+    SPELL_LEHUDIBS_CRYSTAL_SPEAR,
+    SPELL_IOOD,
+    SPELL_IRON_SHOT,
+    SPELL_BOLT_OF_FIRE,
+    SPELL_BOLT_OF_COLD,
+    SPELL_POISON_ARROW,
+    SPELL_LIGHTNING_BOLT,
+    SPELL_BOLT_OF_MAGMA,
+    SPELL_BOLT_OF_DRAINING,
+    SPELL_VENOM_BOLT,
+    SPELL_THROW_ICICLE,
+    SPELL_STONE_ARROW,
+    SPELL_ISKENDERUNS_MYSTIC_BLAST,
+    SPELL_NO_SPELL,                        // end search
+};
+
+static spell_type servitor_spells_secondary[] =
+{
+    SPELL_CONJURE_BALL_LIGHTNING,
+    SPELL_FIREBALL,
+    SPELL_AIRSTRIKE,
+    SPELL_LRD,
+    SPELL_FREEZING_CLOUD,
+    SPELL_POISONOUS_CLOUD,
+    SPELL_FORCE_LANCE,
+    SPELL_DAZZLING_SPRAY,
+    SPELL_MEPHITIC_CLOUD,
+    SPELL_NO_SPELL,                        // end search
+};
+
+static spell_type servitor_spells_fallback[] =
+{
+    SPELL_STICKY_FLAME,
+    SPELL_THROW_FLAME,
+    SPELL_THROW_FROST,
+    SPELL_FREEZE,
+    SPELL_FLAME_TONGUE,
+    SPELL_STING,
+    SPELL_SANDBLAST,
+    SPELL_MAGIC_DART,
+    SPELL_NO_SPELL,                        // end search
+};
+
+static spell_type _best_aligned_spell(vector<spell_type> spells, skill_type skill)
+{
+    for (unsigned int i = 0; i < spells.size(); ++i)
+    {
+        if (spell_typematch(spells[i], skill))
+            return spells[i];
+    }
+
+    // If we couldn't find any that match, just pick the first one
+    return spells[0];
+}
+
+// Select servitor spells based on those known to the player
+// (Primary determines whether we are populating the first 3 or next 2 slots)
+bool ghost_demon::populate_servitor_spells(spell_type* spell_list, bool primary,
+                                           skill_type primary_skill)
+{
+    vector<spell_type> candidates;
+    const unsigned int num = (primary ? 3 : 2);
+    const unsigned int offset = (primary ? 0 : 3);
+
+    int i = 0;
+    spell_type spell = SPELL_NO_SPELL;
+    while ((spell = spell_list[i++]) != SPELL_NO_SPELL)
+    {
+        if (_know_spell(spell))
+            candidates.push_back(spell);
+    }
+
+    if (candidates.size() >= num)
+    {
+        for (unsigned int j = offset; j < offset + num; ++j)
+            spells[j] = candidates[j - offset];
+    }
+    else if (candidates.size() > 0)
+    {
+        // Choose the highest-level spell best aligned with our spell
+        // skills to duplicate
+        const spell_type copy_spell = _best_aligned_spell(candidates,
+                                                            primary_skill);
+
+        for (unsigned int j = offset; j < offset + num; ++j)
+        {
+            if (candidates.size() > j - offset)
+                spells[j] = candidates[j - offset];
+            else
+                spells[j] = copy_spell;
+        }
+    }
+
+    return candidates.size() > 0;
+}
+
+void ghost_demon::init_spellforged_servitor()
+{
+    // Determine highest magic skill (used for solving some tie-breakers)
+    skill_type best_magic_skill = NUM_SKILLS;
+    int skill_level = -1;
+    for (int i = SK_FIRE_MAGIC; i <= SK_POISON_MAGIC; ++i)
+    {
+        if (you.skill((skill_type)i) >= skill_level)
+        {
+            skill_level = you.skill((skill_type)i);
+            best_magic_skill = (skill_type)i;
+        }
+    }
+
+    int pow = calc_spell_power(SPELL_SPELLFORGED_SERVITOR, true);
+
+    colour = LIGHTMAGENTA; // cf. mon-data.h
+    speed = 10;
+    ev = 10;
+    ac = 10;
+    xl = 9 + div_rand_round(pow, 14);
+    max_hp = 80;
+    spellcaster = true;
+    damage = 0;
+    att_type = AT_NONE;
+
+    // Give the servitor its spells
+    bool primary   = populate_servitor_spells(servitor_spells_primary, true,
+                                              best_magic_skill);
+    bool secondary = populate_servitor_spells(servitor_spells_secondary, false,
+                                              best_magic_skill);
+
+    if (!primary && !secondary)
+        populate_servitor_spells(servitor_spells_fallback, true, best_magic_skill);
+
+}
