@@ -4,7 +4,6 @@
 
 #include <cmath>
 
-#include "areas.h"
 #include "artefact.h"
 #include "coordit.h"
 #include "database.h"
@@ -28,8 +27,8 @@
 #include "options.h"
 #include "random.h"
 #include "religion.h"
+#include "shopping.h"
 #include "skills2.h"
-#include "stash.h"
 #include "state.h"
 #include "stuff.h"
 #include "terrain.h"
@@ -62,7 +61,7 @@ static bool _confirm_pray_sacrifice(god_type god)
 
 string god_prayer_reaction()
 {
-    string result = god_name(you.religion);
+    string result = uppercase_first(god_name(you.religion));
     if (crawl_state.player_is_dead())
         result += " was ";
     else
@@ -82,7 +81,13 @@ string god_prayer_reaction()
 
 static bool _bless_weapon(god_type god, brand_type brand, int colour)
 {
-    item_def& wpn = *you.weapon();
+    int item_slot = prompt_invent_item("Brand which weapon?", MT_INVLIST,
+                                       OSEL_BRANDABLE_WEAPON, true, true, false);
+
+    if (item_slot == PROMPT_NOTHING || item_slot == PROMPT_ABORT)
+        return false;
+
+    item_def& wpn(you.inv[item_slot]);
 
     // Only TSO allows blessing ranged weapons.
     if (!is_brandable_weapon(wpn, brand == SPWPN_HOLY_WRATH))
@@ -141,7 +146,9 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
 
     you.wield_change = true;
     you.one_time_ability_used.set(god);
-    calc_mp(); // in case the old brand was antimagic
+    calc_mp(); // in case the old brand was antimagic,
+    you.redraw_armour_class = true; // protection,
+    you.redraw_evasion = true;      // or evasion
     string desc  = old_name + " ";
             desc += (god == GOD_SHINING_ONE   ? "blessed by the Shining One" :
                      god == GOD_LUGONU        ? "corrupted by Lugonu" :
@@ -152,7 +159,7 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
               wpn.name(DESC_A).c_str(), desc.c_str()));
     wpn.flags |= ISFLAG_NOTED_ID;
 
-    mpr("Your weapon shines brightly!", MSGCH_GOD);
+    mprf(MSGCH_GOD, "Your %s shines brightly!", wpn.name(DESC_QUALNAME).c_str());
 
     flash_view(colour);
 
@@ -163,9 +170,9 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
         holy_word(100, HOLY_WORD_TSO, you.pos(), true);
 
         // Un-bloodify surrounding squares.
-        for (radius_iterator ri(you.pos(), 3, true, true); ri; ++ri)
+        for (radius_iterator ri(you.pos(), 3, C_ROUND, LOS_SOLID); ri; ++ri)
             if (is_bloodcovered(*ri))
-                env.pgrid(*ri) &= ~(FPROP_BLOODY);
+                env.pgrid(*ri) &= ~FPROP_BLOODY;
     }
 
     if (god == GOD_KIKUBAAQUDGHA)
@@ -177,7 +184,7 @@ static bool _bless_weapon(god_type god, brand_type brand, int colour)
         you.gift_timeout = 0; // protection after pain branding weapon
 
         // Bloodify surrounding squares (75% chance).
-        for (radius_iterator ri(you.pos(), 2, true, true); ri; ++ri)
+        for (radius_iterator ri(you.pos(), 2, C_ROUND, LOS_SOLID); ri; ++ri)
             if (!one_chance_in(4))
                 maybe_bloodify_square(*ri);
     }
@@ -210,15 +217,10 @@ static bool _altar_prayer()
         && you.piety >= piety_breakpoint(5)
         && !you.one_time_ability_used[GOD_SHINING_ONE])
     {
-        item_def *wpn = you.weapon();
-
-        if (wpn
-            && (get_weapon_brand(*wpn) != SPWPN_HOLY_WRATH
-                || is_blessed_convertible(*wpn)))
-        {
-            did_something = _bless_weapon(GOD_SHINING_ONE, SPWPN_HOLY_WRATH,
-                                          YELLOW);
-        }
+        simple_god_message(" will bless one of your weapons.");
+        more();
+        did_something = _bless_weapon(GOD_SHINING_ONE, SPWPN_HOLY_WRATH,
+                                      YELLOW);
     }
 
     // Lugonu blesses weapons with distortion.
@@ -226,10 +228,9 @@ static bool _altar_prayer()
         && you.piety >= piety_breakpoint(5)
         && !you.one_time_ability_used[GOD_LUGONU])
     {
-        item_def *wpn = you.weapon();
-
-        if (wpn && get_weapon_brand(*wpn) != SPWPN_DISTORTION)
-            did_something = _bless_weapon(GOD_LUGONU, SPWPN_DISTORTION, MAGENTA);
+        simple_god_message(" will brand one of your weapons with the corruption of the Abyss.");
+        more();
+        did_something = _bless_weapon(GOD_LUGONU, SPWPN_DISTORTION, MAGENTA);
     }
 
     // Kikubaaqudgha blesses weapons with pain, or gives you a Necronomicon.
@@ -242,16 +243,10 @@ static bool _altar_prayer()
             simple_god_message(
                 " will bloody your weapon with pain or grant you the Necronomicon.");
 
-            item_def *wpn = you.weapon();
+            more();
 
-            // Does the player want a pain branding?
-            if (wpn && get_weapon_brand(*wpn) != SPWPN_PAIN)
-            {
-                if (_bless_weapon(GOD_KIKUBAAQUDGHA, SPWPN_PAIN, RED))
-                    return true;
-            }
-            else
-                mpr("You have no weapon to bloody with pain.");
+            if (_bless_weapon(GOD_KIKUBAAQUDGHA, SPWPN_PAIN, RED))
+                return true;
 
             // If not, ask if the player wants a Necronomicon.
             if (!yesno("Do you wish to receive the Necronomicon?", true, 'n'))
@@ -262,8 +257,7 @@ static bool _altar_prayer()
         }
 
         int thing_created = items(1, OBJ_BOOKS, BOOK_NECRONOMICON, true, 1,
-                                  MAKE_ITEM_RANDOM_RACE,
-                                  0, 0, you.religion);
+                                  0, 0, 0, you.religion);
 
         if (thing_created == NON_ITEM)
             return false;
@@ -289,12 +283,6 @@ static bool _altar_prayer()
 
 void pray()
 {
-    if (you.cannot_speak())
-    {
-        mpr("You are unable to make a sound!");
-        return;
-    }
-
     // only successful prayer takes time
     you.turn_is_over = false;
 
@@ -332,8 +320,10 @@ void pray()
         return;
     }
 
-    mprf(MSGCH_PRAY, "You offer a prayer to %s.",
-         god_name(you.religion).c_str());
+    if (!something_happened) // If something happened, there already was a prayer
+        mprf(MSGCH_PRAY, "You offer a %sprayer to %s.",
+            you.cannot_speak() ? "silent " : "",
+            god_name(you.religion).c_str());
 
     if (you_worship(GOD_FEDHAS) && fedhas_fungal_bloom())
         something_happened = true;
@@ -342,11 +332,11 @@ void pray()
     something_happened |= _offer_items();
 
     if (you_worship(GOD_XOM))
-        mpr(getSpeakString("Xom prayer"), MSGCH_GOD);
+        mprf(MSGCH_GOD, "%s", getSpeakString("Xom prayer").c_str());
     else if (player_under_penance())
         simple_god_message(" demands penance!");
     else
-        mpr(god_prayer_reaction().c_str(), MSGCH_PRAY, you.religion);
+        mprf(MSGCH_PRAY, you.religion, "%s", god_prayer_reaction().c_str());
 
     if (something_happened)
         you.turn_is_over = true;
@@ -379,8 +369,7 @@ int zin_tithe(item_def& item, int quant, bool quiet, bool converting)
         if (item.plus == 1) // seen before worshipping Zin
         {
             tithe = 0;
-            simple_god_message(" is a bit unhappy you did not bring this "
-                               "gold earlier.");
+            simple_god_message(" ignores your late donation.");
         }
         // A single scroll can give you more than D:1-18, Lair and Orc
         // together, limit the gains.  You're still required to pay from
@@ -398,7 +387,7 @@ int zin_tithe(item_def& item, int quant, bool quiet, bool converting)
         }
         else
         {
-            if (player_in_branch(BRANCH_ORCISH_MINES) && !converting)
+            if (player_in_branch(BRANCH_ORC) && !converting)
             {
                 // Another special case: Orc gives simply too much compared to
                 // other branches.
@@ -487,52 +476,40 @@ static bool _zin_donate_gold()
     return true;
 }
 
-static int _leading_sacrifice_group()
-{
-    int weights[5];
-    get_pure_deck_weights(weights);
-    int best_i = -1, maxweight = -1;
-    for (int i = 0; i < 5; ++i)
-    {
-        if (best_i == -1 || weights[i] > maxweight)
-        {
-            maxweight = weights[i];
-            best_i = i;
-        }
-    }
-    return best_i;
-}
-
-static void _give_sac_group_feedback(int which)
-{
-    ASSERT_RANGE(which, 0, 5);
-    const char* names[] = {
-        "Escape", "Destruction", "Dungeons", "Summoning", "Wonder"
-    };
-    mprf(MSGCH_GOD, "A symbol of %s coalesces before you, then vanishes.",
-         names[which]);
-}
-
 static void _ashenzari_sac_scroll(const item_def& item)
 {
     int scr = SCR_CURSE_JEWELLERY;
-    if (you.species != SP_FELID
-        && (you.species != SP_OCTOPODE || one_chance_in(4)))
-    {
-        scr = random_choose(SCR_CURSE_WEAPON, SCR_CURSE_ARMOUR,
-                            SCR_CURSE_JEWELLERY, -1);
-    }
-    int it = items(0, OBJ_SCROLLS, scr, true, 0, MAKE_ITEM_NO_RACE,
-                   0, 0, GOD_ASHENZARI);
-    if (it == NON_ITEM)
-    {
-        mpr("You feel the world is against you.");
-        return;
-    }
+    int num = 3;
 
-    mitm[it].quantity = 1;
-    if (!move_item_to_grid(&it, you.pos(), true))
-        destroy_item(it, true); // can't happen
+    int wpn = 3;
+    int arm = 0;
+    int jwl = (you.species != SP_OCTOPODE) ? 3 : 9;
+
+    for (int i = EQ_MIN_ARMOUR; i <= EQ_MAX_ARMOUR; i++)
+        if (you_can_wear(i, true))
+            arm++;
+
+    do
+    {
+        if (you.species != SP_FELID)
+        {
+            scr = random_choose_weighted(wpn, SCR_CURSE_WEAPON,
+                                         arm, SCR_CURSE_ARMOUR,
+                                         jwl, SCR_CURSE_JEWELLERY,
+                                         0);
+        }
+        int it = items(0, OBJ_SCROLLS, scr, true, 0, 0, 0, 0, GOD_ASHENZARI);
+        if (it == NON_ITEM)
+        {
+            mpr("You feel the world is against you.");
+            return;
+        }
+
+        mitm[it].quantity = 1;
+        if (!move_item_to_grid(&it, you.pos(), true))
+            destroy_item(it, true); // can't happen
+    }
+    while (--num > 0);
 }
 
 // Is the destroyed weapon valuable enough to gain piety by doing so?
@@ -699,16 +676,6 @@ static piety_gain_t _sacrifice_one_item_noncount(const item_def& item,
         relative_piety_gain = x_chance_in_y(piety_change, piety_denom) ?
                                 is_artefact(item) ?
                                   PIETY_LOTS : PIETY_SOME : PIETY_NONE;
-
-        if (item.base_type == OBJ_FOOD && item.sub_type == FOOD_CHUNK
-            || is_blood_potion(item))
-        {
-            // Count chunks and blood potions towards decks of
-            // Summoning.
-            you.sacrifice_value[OBJ_CORPSES] += value;
-        }
-        else
-            you.sacrifice_value[item.base_type] += value;
         break;
     }
 
@@ -754,45 +721,6 @@ piety_gain_t sacrifice_item_stack(const item_def& item, int *js, int quantity)
     return relative_gain;
 }
 
-bool check_nemelex_sacrificing_item_type(const item_def& item)
-{
-    switch (item.base_type)
-    {
-    case OBJ_ARMOUR:
-        return you.nemelex_sacrificing[NEM_GIFT_ESCAPE];
-
-    case OBJ_WEAPONS:
-    case OBJ_STAVES:
-    case OBJ_RODS:
-    case OBJ_MISSILES:
-        return you.nemelex_sacrificing[NEM_GIFT_DESTRUCTION];
-
-    case OBJ_CORPSES:
-        return you.nemelex_sacrificing[NEM_GIFT_SUMMONING];
-
-    case OBJ_POTIONS:
-        if (is_blood_potion(item))
-            return you.nemelex_sacrificing[NEM_GIFT_SUMMONING];
-        return you.nemelex_sacrificing[NEM_GIFT_WONDERS];
-
-    case OBJ_FOOD:
-        if (item.sub_type == FOOD_CHUNK)
-            return you.nemelex_sacrificing[NEM_GIFT_SUMMONING];
-    // else fall through
-    case OBJ_WANDS:
-    case OBJ_SCROLLS:
-        return you.nemelex_sacrificing[NEM_GIFT_WONDERS];
-
-    case OBJ_JEWELLERY:
-    case OBJ_BOOKS:
-    case OBJ_MISCELLANY:
-        return you.nemelex_sacrificing[NEM_GIFT_DUNGEONS];
-
-    default:
-        return false;
-    }
-}
-
 static bool _offer_items()
 {
     if (!god_likes_items(you.religion))
@@ -809,8 +737,6 @@ static bool _offer_items()
     int num_disliked = 0;
     item_def *disliked_item = 0;
 
-    const int old_leading = _leading_sacrifice_group();
-
     while (i != NON_ITEM)
     {
         item_def &item(mitm[i]);
@@ -828,14 +754,6 @@ static bool _offer_items()
             continue;
         }
 
-        // Skip items you don't want to sacrifice right now.
-        if (you_worship(GOD_NEMELEX_XOBEH)
-            && !check_nemelex_sacrificing_item_type(item))
-        {
-            i = next;
-            continue;
-        }
-
         // Ignore {!D} inscribed items.
         if (!check_warning_inscriptions(item, OPER_DESTROY))
         {
@@ -845,7 +763,8 @@ static bool _offer_items()
         }
 
         if (god_likes_item(you.religion, item)
-            && (item.inscription.find("=p") != string::npos))
+            && ((item.inscription.find("=p") != string::npos)
+                || item_needs_autopickup(item)))
         {
             const string msg = "Really sacrifice " + item.name(DESC_A) + "?";
 
@@ -856,7 +775,6 @@ static bool _offer_items()
             }
         }
 
-
         piety_gain_t relative_gain = sacrifice_item_stack(item);
         print_sacrifice_message(you.religion, mitm[i], relative_gain);
         item_was_destroyed(mitm[i]);
@@ -865,18 +783,8 @@ static bool _offer_items()
         num_sacced++;
     }
 
-    if (num_sacced > 0 && you_worship(GOD_NEMELEX_XOBEH))
-    {
-        const int new_leading = _leading_sacrifice_group();
-        if (old_leading != new_leading || one_chance_in(50))
-            _give_sac_group_feedback(new_leading);
-
-#if defined(DEBUG_GIFTS) || defined(DEBUG_CARDS) || defined(DEBUG_SACRIFICE)
-        _show_pure_deck_chances();
-#endif
-    }
     // Explanatory messages if nothing the god likes is sacrificed.
-    else if (num_sacced == 0 && num_disliked > 0)
+    if (num_sacced == 0 && num_disliked > 0)
     {
         ASSERT(disliked_item);
 
@@ -884,8 +792,6 @@ static bool _offer_items()
             simple_god_message(" wants the Orb's power used on the surface!");
         else if (item_is_rune(*disliked_item))
             simple_god_message(" wants the runes to be proudly displayed.");
-        else if (item_is_horn_of_geryon(*disliked_item))
-            simple_god_message(" doesn't want your path blocked.");
         // Zin was handled above, and the other gods don't care about
         // sacrifices.
         else if (god_likes_fresh_corpses(you.religion))
@@ -906,5 +812,5 @@ static bool _offer_items()
              you.piety_max[GOD_ELYVILON] < piety_breakpoint(2) ? "" : "unholy or evil ");
     }
 
-    return (num_sacced > 0);
+    return num_sacced > 0;
 }
