@@ -61,12 +61,13 @@ InvTitle::InvTitle(Menu *mn, const string &title, invtitle_annotator tfn)
 
 string InvTitle::get_text(const bool) const
 {
-    return (titlefn ? titlefn(m, MenuEntry::get_text())
-                    : MenuEntry::get_text());
+    return titlefn ? titlefn(m, MenuEntry::get_text())
+                   : MenuEntry::get_text();
 }
 
 InvEntry::InvEntry(const item_def &i, bool show_bg)
-    : MenuEntry("", MEL_ITEM), show_background(show_bg), item(&i)
+    : MenuEntry("", MEL_ITEM), show_background(show_bg), item(&i),
+      show_weight(false)
 {
     data = const_cast<item_def *>(item);
 
@@ -111,31 +112,31 @@ const string &InvEntry::get_fullname() const
 
 bool InvEntry::is_item_cursed() const
 {
-    return (item_ident(*item, ISFLAG_KNOW_CURSE) && item->cursed());
+    return item_ident(*item, ISFLAG_KNOW_CURSE) && item->cursed();
 }
 
 bool InvEntry::is_item_glowing() const
 {
-    return (!item_ident(*item, ISFLAG_KNOW_TYPE)
-            && (get_equip_desc(*item)
-                || (is_artefact(*item)
-                    && (item->base_type == OBJ_WEAPONS
-                        || item->base_type == OBJ_ARMOUR
-                        || item->base_type == OBJ_BOOKS))));
+    return !item_ident(*item, ISFLAG_KNOW_TYPE)
+           && (get_equip_desc(*item)
+               || (is_artefact(*item)
+                   && (item->base_type == OBJ_WEAPONS
+                       || item->base_type == OBJ_ARMOUR
+                       || item->base_type == OBJ_BOOKS)));
 }
 
 bool InvEntry::is_item_ego() const
 {
-    return (item_ident(*item, ISFLAG_KNOW_TYPE) && !is_artefact(*item)
-            && item->special != 0
-            && (item->base_type == OBJ_WEAPONS
-                || item->base_type == OBJ_MISSILES
-                || item->base_type == OBJ_ARMOUR));
+    return item_ident(*item, ISFLAG_KNOW_TYPE) && !is_artefact(*item)
+           && item->special != 0
+           && (item->base_type == OBJ_WEAPONS
+               || item->base_type == OBJ_MISSILES
+               || item->base_type == OBJ_ARMOUR);
 }
 
 bool InvEntry::is_item_art() const
 {
-    return (item_ident(*item, ISFLAG_KNOW_TYPE) && is_artefact(*item));
+    return item_ident(*item, ISFLAG_KNOW_TYPE) && is_artefact(*item);
 }
 
 bool InvEntry::is_item_equipped() const
@@ -217,7 +218,7 @@ string InvEntry::get_text(bool need_cursor) const
     //estimate it by the following heuristics {kittel}.
     unsigned max_chars_in_line = get_number_of_cols() - 2;
 #ifdef USE_TILE_LOCAL
-    if (Options.tile_menu_icons && Options.show_inventory_weights)
+    if (Options.tile_menu_icons && show_weight)
         max_chars_in_line = get_number_of_cols() * 4 / 9 - 2;
 #endif
 
@@ -230,21 +231,19 @@ string InvEntry::get_text(bool need_cursor) const
         colour_tag_adjustment = colour_tag.size() * 2 + 5;
     }
 
-    if (Options.show_inventory_weights)
-    {
+    if (show_weight)
         max_chars_in_line -= 1;
-        const int w_weight = 10; //length of " (999 aum)"
-        int excess = strwidth(tstr.str()) - colour_tag_adjustment
-                     + text.size() + w_weight - max_chars_in_line;
-        if (excess > 0)
-            tstr << text.substr(0, max<int>(0, text.size() - excess - 2)) << "..";
-        else
-            tstr << text;
-    }
+
+    const int w_weight = show_weight ? 10 //length of " (999 aum)"
+                                     : 0;
+    const int excess = strwidth(tstr.str()) - colour_tag_adjustment
+                     + strwidth(text) + w_weight - max_chars_in_line;
+    if (excess > 0)
+        tstr << chop_string(text, max(0, strwidth(text) - excess - 2)) << "..";
     else
         tstr << text;
 
-    if (Options.show_inventory_weights)
+    if (show_weight)
     {
         const int mass = item_mass(*item) * item->quantity;
         // Note: If updating the " (%i aum)" format, remember to update
@@ -408,11 +407,12 @@ void InvMenu::set_title(const string &s)
         const int cap = carrying_capacity(BS_UNENCUMBERED);
 
         stitle = make_stringf(
-            "Inventory: %.0f/%.0f aum (%d%%, %d/52 slots)",
+            "Inventory: %.0f/%.0f aum (%d%%, %d/%d slots)",
             BURDEN_TO_AUM * you.burden,
             BURDEN_TO_AUM * cap,
             (you.burden * 100) / cap,
-            inv_count());
+            inv_count(),
+            ENDOFPACK);
 
         string prompt = "(_ for help)";
         stitle = stitle + string(max(0, get_number_of_cols() - strwidth(stitle)
@@ -446,6 +446,22 @@ static bool _has_tran_unwearable_armour()
     return false;
 }
 
+static bool _has_hand_evokable()
+{
+    for (int i = 0; i < ENDOFPACK; i++)
+    {
+        item_def &item(you.inv[i]);
+
+        if (item.defined()
+            && item_is_evokable(item, true, true, true, false, false)
+            && !item_is_evokable(item, true, true, true, false, true))
+        {
+            return true;
+        }
+    }
+    return false;
+}
+
 static string _no_selectables_message(int item_selector)
 {
     switch (item_selector)
@@ -494,7 +510,10 @@ static string _no_selectables_message(int item_selector)
     case OSEL_BUTCHERY:
         return "You aren't carrying any sharp implements.";
     case OSEL_EVOKABLE:
-        return "You aren't carrying any items that can be evoked.";
+        if (_has_hand_evokable())
+            return "You aren't carrying any items that can be evoked without being wielded.";
+        else
+            return "You aren't carrying any items that can be evoked.";
     case OSEL_FRUIT:
         return "You aren't carrying any fruit.";
     case OSEL_CURSED_WORN:
@@ -505,6 +524,8 @@ static string _no_selectables_message(int item_selector)
         return "You aren't wearing any piece of uncursed jewellery.";
     case OSEL_BRANDABLE_WEAPON:
         return "You aren't carrying any weapons that can be branded.";
+    case OSEL_ENCHANTABLE_WEAPON:
+        return "You aren't carrying any weapons that can be enchanted.";
     }
 
     return "You aren't carrying any such object.";
@@ -639,7 +660,7 @@ bool InvMenu::is_selectable(int index) const
 
 bool InvMenu::allow_easy_exit() const
 {
-    return (type == MT_KNOW || Menu::allow_easy_exit());
+    return type == MT_KNOW || Menu::allow_easy_exit();
 }
 
 template <const string &(InvEntry::*method)() const>
@@ -651,19 +672,19 @@ static int compare_item_str(const InvEntry *a, const InvEntry *b)
 template <typename T, T (*proc)(const InvEntry *a)>
 static int compare_item(const InvEntry *a, const InvEntry *b)
 {
-    return (int(proc(a)) - int(proc(b)));
+    return int(proc(a)) - int(proc(b));
 }
 
 template <typename T, T (InvEntry::*method)() const>
 static int compare_item(const InvEntry *a, const InvEntry *b)
 {
-    return (int((a->*method)()) - int((b->*method)()));
+    return int((a->*method)()) - int((b->*method)());
 }
 
 template <typename T, T (InvEntry::*method)() const>
 static int compare_item_rev(const InvEntry *a, const InvEntry *b)
 {
-    return (int((b->*method)()) - int((a->*method)()));
+    return int((b->*method)()) - int((a->*method)());
 }
 
 template <item_sort_fn cmp>
@@ -672,7 +693,7 @@ static int compare_reverse(const InvEntry *a, const InvEntry *b)
     return -cmp(a, b);
 }
 
-// C++ needs anonymous subs already!
+// We need C++11 already!
 // Some prototypes to prevent warnings; we can't make these static because
 // they're used as template parameters.
 int sort_item_qty(const InvEntry *a);
@@ -696,8 +717,8 @@ bool sort_item_identified(const InvEntry *a)
 
 bool sort_item_charged(const InvEntry *a)
 {
-    return (a->item->base_type != OBJ_WANDS
-            || !item_is_evokable(*(a->item), false, true));
+    return a->item->base_type != OBJ_WANDS
+           || !item_is_evokable(*(a->item), false, true);
 }
 
 static bool _compare_invmenu_items(const InvEntry *a, const InvEntry *b,
@@ -708,9 +729,9 @@ static bool _compare_invmenu_items(const InvEntry *a, const InvEntry *b,
     {
         const int cmp = i->compare(a, b);
         if (cmp)
-            return (cmp < 0);
+            return cmp < 0;
     }
-    return (a->item->link < b->item->link);
+    return a->item->link < b->item->link;
 }
 
 struct menu_entry_comparator
@@ -838,18 +859,24 @@ menu_letter InvMenu::load_items(const vector<const item_def*> &mitems,
             }
 
             add_entry(new MenuEntry(subtitle, MEL_SUBTITLE));
-            items_in_class.clear();
         }
+        items_in_class.clear();
 
         InvEntry *forced_first = NULL;
+        const bool show_weight = Options.show_inventory_weights
+                                 >= (flags & MF_DROP_PICKUP ? MB_MAYBE : MB_TRUE);
         for (int j = 0, count = mitems.size(); j < count; ++j)
         {
             if (mitems[j]->base_type != i)
                 continue;
+
+            InvEntry * const ie = new InvEntry(*mitems[j]);
+            ie->show_weight = show_weight;
+
             if (mitems[j]->sub_type == get_max_subtype(mitems[j]->base_type))
-                forced_first = new InvEntry(*mitems[j]);
+                forced_first = ie;
             else
-                items_in_class.push_back(new InvEntry(*mitems[j]));
+                items_in_class.push_back(ie);
         }
 
         sort_menu(items_in_class, cond);
@@ -909,7 +936,9 @@ bool InvMenu::process_key(int key)
 {
     if (key == CONTROL('W'))
     {
-        Options.show_inventory_weights = !Options.show_inventory_weights;
+        for (size_t i = 0; i < items.size(); i++)
+            if (InvEntry *ie = dynamic_cast<InvEntry *>(items[i]))
+                ie->show_weight = !ie->show_weight;
         draw_menu();
         return true;
     }
@@ -973,22 +1002,6 @@ bool InvMenu::process_key(int key)
         }
     }
 
-    if (items.size()
-        && type == MT_DROP
-        && (key == CONTROL('D') || key == '@'))
-    {
-        int newflag = is_set(MF_MULTISELECT) ? MF_SINGLESELECT | MF_ANYPRINTABLE
-                                             : MF_MULTISELECT;
-
-        flags &= ~(MF_SINGLESELECT | MF_MULTISELECT | MF_ANYPRINTABLE);
-        flags |= newflag;
-
-        deselect_all();
-        sel.clear();
-        draw_select_count(0, true);
-        return true;
-    }
-
     return Menu::process_key(key);
 }
 
@@ -1103,7 +1116,12 @@ vector<SelItem> select_items(const vector<const item_def*> &items,
         menu.set_title_annotator(titlefn);
         menu.set_title(title);
         if (mtype == MT_PICKUP)
+        {
             menu.set_tag("pickup");
+            // Need this before load_items.
+            menu.set_flags(menu.get_flags() | MF_DROP_PICKUP);
+        }
+
         menu.load_items(items);
         int new_flags = noselect ? MF_NOSELECT
                                  : MF_MULTISELECT | MF_ALLOW_FILTER;
@@ -1147,22 +1165,25 @@ static bool _item_class_selected(const item_def &i, int selector)
                 || is_enchantable_armour(i, true, true));
 
     case OBJ_ARMOUR:
-        return (itype == OBJ_ARMOUR && you_tran_can_wear(i));
+        return itype == OBJ_ARMOUR && you_tran_can_wear(i);
 
     case OSEL_FRUIT:
         return is_fruit(i);
 
     case OSEL_WORN_ARMOUR:
-        return (itype == OBJ_ARMOUR && item_is_equipped(i));
+        return itype == OBJ_ARMOUR && item_is_equipped(i);
 
     case OSEL_UNIDENT:
         return !fully_identified(i) || (is_deck(i) && !top_card_is_known(i));
 
     case OBJ_MISSILES:
-        return (itype == OBJ_MISSILES || itype == OBJ_WEAPONS);
+        return itype == OBJ_MISSILES || itype == OBJ_WEAPONS;
 
     case OSEL_THROWABLE:
     {
+        if (you_worship(GOD_TROG) && item_is_spellbook(i))
+            return true;
+
         if (i.base_type != OBJ_WEAPONS && i.base_type != OBJ_MISSILES)
             return false;
 
@@ -1178,11 +1199,11 @@ static bool _item_class_selected(const item_def &i, int selector)
         return item_is_wieldable(i);
 
     case OSEL_BUTCHERY:
-        return (itype == OBJ_WEAPONS && can_cut_meat(i));
+        return itype == OBJ_WEAPONS && can_cut_meat(i);
 
     case OBJ_SCROLLS:
-        return (itype == OBJ_SCROLLS
-                || (itype == OBJ_BOOKS && i.sub_type != BOOK_MANUAL));
+        return itype == OBJ_SCROLLS
+               || (itype == OBJ_BOOKS && i.sub_type != BOOK_MANUAL);
 
     case OSEL_RECHARGE:
         return item_is_rechargeable(i, true);
@@ -1197,8 +1218,8 @@ static bool _item_class_selected(const item_def &i, int selector)
         return itype == OBJ_FOOD && !is_inedible(i);
 
     case OSEL_VAMP_EAT:
-        return (itype == OBJ_CORPSES && i.sub_type == CORPSE_BODY
-                && !food_is_rotten(i) && mons_has_blood(i.mon_type));
+        return itype == OBJ_CORPSES && i.sub_type == CORPSE_BODY
+               && !food_is_rotten(i) && mons_has_blood(i.mon_type);
 
     case OSEL_DRAW_DECK:
         return is_deck(i);
@@ -1220,14 +1241,23 @@ static bool _item_class_selected(const item_def &i, int selector)
                && (&i != you.weapon() || is_weapon(i));
 
     case OSEL_UNCURSED_WORN_ARMOUR:
-        return (!i.cursed() && item_is_equipped(i) && itype == OBJ_ARMOUR);
+        return !i.cursed() && item_is_equipped(i) && itype == OBJ_ARMOUR;
 
     case OSEL_UNCURSED_WORN_JEWELLERY:
-        return (!i.cursed() && item_is_equipped(i) && itype == OBJ_JEWELLERY);
+        return !i.cursed() && item_is_equipped(i) && itype == OBJ_JEWELLERY;
 
     case OSEL_BRANDABLE_WEAPON:
         return is_brandable_weapon(i, true);
 
+    case OSEL_ENCHANTABLE_WEAPON:
+        return is_weapon(i)
+               && (itype == OBJ_WEAPONS
+                    && !is_artefact(i)
+                    && (i.plus < MAX_WPN_ENCHANT
+                        || i.plus2 < MAX_WPN_ENCHANT
+                        || !(item_ident(i, ISFLAG_KNOW_PLUSES)))
+                   || i.cursed());
+
     default:
         return false;
     }
@@ -1238,16 +1268,16 @@ static bool _userdef_item_selected(const item_def &i, int selector)
 #if defined(CLUA_BINDINGS)
     const char *luafn = selector == OSEL_WIELD ? "ch_item_wieldable"
                                                : NULL;
-    return (luafn && clua.callbooleanfn(false, luafn, "i", &i));
+    return luafn && clua.callbooleanfn(false, luafn, "i", &i);
 #else
     return false;
 #endif
 }
 
-static bool _is_item_selected(const item_def &i, int selector)
+bool is_item_selected(const item_def &i, int selector)
 {
-    return (_item_class_selected(i, selector)
-            || _userdef_item_selected(i, selector));
+    return _item_class_selected(i, selector)
+           || _userdef_item_selected(i, selector);
 }
 
 static void _get_inv_items_to_show(vector<const item_def*> &v,
@@ -1257,7 +1287,7 @@ static void _get_inv_items_to_show(vector<const item_def*> &v,
     {
         if (you.inv[i].defined()
             && you.inv[i].link != excluded_slot
-            && _is_item_selected(you.inv[i], selector))
+            && is_item_selected(you.inv[i], selector))
         {
             v.push_back(&you.inv[i]);
         }
@@ -1270,7 +1300,7 @@ bool any_items_to_select(int selector, bool msg, int excluded_slot)
     {
         if (you.inv[i].defined()
             && you.inv[i].link != excluded_slot
-            && _is_item_selected(you.inv[i], selector))
+            && is_item_selected(you.inv[i], selector))
         {
             return true;
         }
@@ -1296,6 +1326,8 @@ static unsigned char _invent_select(const char *title = NULL,
                                     Menu::selitem_tfn selitemfn = NULL,
                                     const vector<SelItem> *pre_select = NULL)
 {
+    if (type == MT_DROP || type == MT_PICKUP)
+        flags |= MF_DROP_PICKUP;
     InvMenu menu(flags | MF_ALLOW_FORMATTING);
 
     menu.set_preselect(pre_select);
@@ -1318,7 +1350,7 @@ static unsigned char _invent_select(const char *title = NULL,
     return menu.getkey();
 }
 
-unsigned char get_invent(int invent_type)
+unsigned char get_invent(int invent_type, bool redraw)
 {
     unsigned char select;
     int flags = MF_SINGLESELECT;
@@ -1342,17 +1374,12 @@ unsigned char get_invent(int invent_type)
             break;
     }
 
-    if (!crawl_state.doing_prev_cmd_again)
+    if (redraw && !crawl_state.doing_prev_cmd_again)
         redraw_screen();
 
     return select;
 }
 
-void browse_inventory()
-{
-    get_invent(OSEL_ANY);
-}
-
 // Reads in digits for a count and apprends then to val, the
 // return value is the character that stopped the reading.
 static unsigned char _get_invent_quant(unsigned char keyin, int &quant)
@@ -1394,7 +1421,7 @@ vector<SelItem> prompt_invent_items(
                         menu_type mtype,
                         int type_expect,
                         invtitle_annotator titlefn,
-                        bool allow_auto_list,
+                        bool auto_list,
                         bool allow_easy_quit,
                         const char other_valid_char,
                         vector<text_pattern> *select_filter,
@@ -1407,7 +1434,6 @@ vector<SelItem> prompt_invent_items(
     bool           need_redraw = false;
     bool           need_prompt = true;
     bool           need_getch  = true;
-    bool           auto_list   = Options.auto_list && allow_auto_list;
 
     if (auto_list)
     {
@@ -1447,21 +1473,16 @@ vector<SelItem> prompt_invent_items(
         }
         else if (keyin == '?' || keyin == '*' || keyin == ',')
         {
-            int selmode = Options.drop_mode == DM_SINGLE
-                          && (!pre_select || pre_select->empty()) ?
-                        MF_SINGLESELECT | MF_EASY_EXIT | MF_ANYPRINTABLE :
-                        MF_MULTISELECT | MF_ALLOW_FILTER;
-
             // The "view inventory listing" mode.
             const int ch = _invent_select(prompt,
                                           mtype,
                                           keyin == '*' ? OSEL_ANY : type_expect,
                                           -1,
-                                          selmode,
+                                          MF_MULTISELECT | MF_ALLOW_FILTER,
                                           titlefn, &items, select_filter, fn,
                                           pre_select);
 
-            if ((selmode & MF_SINGLESELECT) || key_is_escape(ch))
+            if (key_is_escape(ch))
             {
                 keyin       = ch;
                 need_prompt = false;
@@ -1655,8 +1676,8 @@ static string _operation_verb(operation_types oper)
     case OPER_WIELD:          return "wield";
     case OPER_QUAFF:          return "quaff";
     case OPER_DROP:           return "drop";
-    case OPER_EAT:            return (you.species == SP_VAMPIRE ?
-                                      "drain" : "eat");
+    case OPER_EAT:            return you.species == SP_VAMPIRE ?
+                                      "drain" : "eat";
     case OPER_TAKEOFF:        return "take off";
     case OPER_WEAR:           return "wear";
     case OPER_PUTON:          return "put on";
@@ -1664,7 +1685,6 @@ static string _operation_verb(operation_types oper)
     case OPER_READ:           return "read";
     case OPER_MEMORISE:       return "memorise from";
     case OPER_ZAP:            return "zap";
-    case OPER_EXAMINE:        return "examine";
     case OPER_FIRE:           return "fire";
     case OPER_PRAY:           return "sacrifice";
     case OPER_EVOKE:          return "evoke";
@@ -1676,21 +1696,38 @@ static string _operation_verb(operation_types oper)
     }
 }
 
-static bool _nasty_stasis(const item_def &item, operation_types oper)
-{
-    return (oper == OPER_PUTON
-            && item.base_type == OBJ_JEWELLERY
-            && item.sub_type == AMU_STASIS
-            && (you.duration[DUR_HASTE] || you.duration[DUR_SLOW]
-                || you.duration[DUR_TELEPORT] || you.duration[DUR_FINESSE]));
-}
-
 static bool _is_wielded(const item_def &item)
 {
     int equip = you.equip[EQ_WEAPON];
     return equip != -1 && item.link == equip;
 }
 
+static bool _is_known_no_tele_item(const item_def &item)
+{
+    if (!is_artefact(item))
+        return false;
+
+    bool known;
+    int val = artefact_wpn_property(item, ARTP_PREVENT_TELEPORTATION, known);
+
+    if (known && val)
+        return true;
+    else
+        return false;
+}
+
+static bool _nasty_stasis(const item_def &item, operation_types oper)
+{
+    return (oper == OPER_PUTON
+           && (item.base_type == OBJ_JEWELLERY
+               && item.sub_type == AMU_STASIS
+               && (you.duration[DUR_HASTE] || you.duration[DUR_SLOW]
+                   || you.duration[DUR_TELEPORT] || you.duration[DUR_FINESSE])))
+            || (oper == OPER_PUTON || oper == OPER_WEAR
+                || oper == OPER_WIELD && !_is_wielded(item))
+                && (_is_known_no_tele_item(item) && you.duration[DUR_TELEPORT]);
+}
+
 bool needs_handle_warning(const item_def &item, operation_types oper)
 {
     if (_has_warning_inscription(item, oper))
@@ -1723,6 +1760,7 @@ bool needs_handle_warning(const item_def &item, operation_types oper)
         && is_weapon(item))
     {
         if (get_weapon_brand(item) == SPWPN_DISTORTION
+            && !you_worship(GOD_LUGONU)
             && !you.duration[DUR_WEAPON_BRAND])
         {
             return true;
@@ -1819,8 +1857,8 @@ bool check_warning_inscriptions(const item_def& item,
                       + (you.duration[DUR_TELEPORT] ? "about to teleport" :
                          you.duration[DUR_SLOW] ? "slowed" : "hasted");
         prompt += "?";
-        return (yesno(prompt.c_str(), false, 'n')
-                && check_old_item_warning(item, oper));
+        return yesno(prompt.c_str(), false, 'n')
+               && check_old_item_warning(item, oper);
     }
     else
         return check_old_item_warning(item, oper);
@@ -1837,14 +1875,15 @@ bool check_warning_inscriptions(const item_def& item,
 //
 // Note: This function never checks if the item is appropriate.
 int prompt_invent_item(const char *prompt,
-                        menu_type mtype, int type_expect,
-                        bool must_exist, bool allow_auto_list,
-                        bool allow_easy_quit,
-                        const char other_valid_char,
-                        int excluded_slot,
-                        int *const count,
-                        operation_types oper,
-                        bool allow_list_known)
+                       menu_type mtype, int type_expect,
+                       bool must_exist, bool auto_list,
+                       bool allow_easy_quit,
+                       const char other_valid_char,
+                       int excluded_slot,
+                       int *const count,
+                       operation_types oper,
+                       bool allow_list_known,
+                       bool do_warning)
 {
     if (!any_items_to_select(type_expect, false, excluded_slot)
         && type_expect == OSEL_THROWABLE
@@ -1869,7 +1908,6 @@ int prompt_invent_item(const char *prompt,
     bool           need_redraw = false;
     bool           need_prompt = true;
     bool           need_getch  = true;
-    bool           auto_list   = Options.auto_list && allow_auto_list;
 
     if (auto_list)
     {
@@ -1968,7 +2006,7 @@ int prompt_invent_item(const char *prompt,
             if (res != -1)
             {
                 ret = res;
-                if (check_warning_inscriptions(you.inv[ret], oper))
+                if (!do_warning || check_warning_inscriptions(you.inv[ret], oper))
                     break;
             }
         }
@@ -1991,7 +2029,7 @@ int prompt_invent_item(const char *prompt,
 
             if (must_exist && !you.inv[ret].defined())
                 mpr("You don't have any such object.");
-            else if (check_warning_inscriptions(you.inv[ret], oper))
+            else if (!do_warning || check_warning_inscriptions(you.inv[ret], oper))
                 break;
         }
         else if (!isspace(keyin))
@@ -2009,18 +2047,13 @@ int prompt_invent_item(const char *prompt,
     return ret;
 }
 
-bool prompt_failed(int retval, string msg)
+bool prompt_failed(int retval)
 {
     if (retval != PROMPT_ABORT && retval != PROMPT_NOTHING)
         return false;
 
-    if (msg.empty())
-    {
-        if (retval == PROMPT_ABORT)
-            canned_msg(MSG_OK);
-    }
-    else
-        mpr(msg.c_str(), MSGCH_PROMPT);
+    if (retval == PROMPT_ABORT)
+        canned_msg(MSG_OK);
 
     crawl_state.cancel_cmd_repeat();
 
@@ -2032,9 +2065,9 @@ bool prompt_failed(int retval, string msg)
 bool item_is_wieldable(const item_def &item)
 {
     const int type = item.base_type;
-    return (is_weapon(item) || is_deck(item)
-            || type == OBJ_MISCELLANY
-               && item.sub_type == MISC_LANTERN_OF_SHADOWS);
+    return is_weapon(item) || is_deck(item)
+           || type == OBJ_MISCELLANY
+              && item.sub_type == MISC_LANTERN_OF_SHADOWS;
 }
 
 /*
@@ -2064,7 +2097,7 @@ bool item_is_evokable(const item_def &item, bool reach, bool known,
                 return true;
 
             if (msg)
-                mpr(error);
+                mprf("%s", error.c_str());
 
             return false;
         }
@@ -2077,13 +2110,6 @@ bool item_is_evokable(const item_def &item, bool reach, bool known,
     switch (item.base_type)
     {
     case OBJ_WANDS:
-        if (you.species == SP_FELID)
-        {
-            if (msg)
-                mpr("You cannot grasp it well enough.");
-            return false;
-        }
-
         if (all_wands)
             return true;
 
@@ -2104,7 +2130,7 @@ bool item_is_evokable(const item_def &item, bool reach, bool known,
             if (!wielded)
             {
                 if (msg)
-                    mpr(error);
+                    mprf("%s", error.c_str());
                 return false;
             }
             return true;
@@ -2118,20 +2144,20 @@ bool item_is_evokable(const item_def &item, bool reach, bool known,
         if (!wielded)
         {
             if (msg)
-                mpr(error);
+                mprf("%s", error.c_str());
             return false;
         }
         return true;
 
     case OBJ_STAVES:
-        if (!known && !item_type_known(item)
+        if (known && !item_type_known(item)
             || item.sub_type == STAFF_ENERGY
                && item_type_known(item))
         {
             if (!wielded)
             {
                 if (msg)
-                    mpr(error);
+                    mprf("%s", error.c_str());
                 return false;
             }
             return true;
@@ -2146,7 +2172,7 @@ bool item_is_evokable(const item_def &item, bool reach, bool known,
             if (!wielded)
             {
                 if (msg)
-                    mpr(error);
+                    mprf("%s", error.c_str());
                 return false;
             }
             return true;
