@@ -29,43 +29,46 @@
 #include <stdbool.h>
 #include <stddef.h>
 
-typedef enum {
-        JSON_NULL,
-        JSON_BOOL,
-        JSON_STRING,
-        JSON_NUMBER,
-        JSON_ARRAY,
-        JSON_OBJECT,
+typedef enum
+{
+    JSON_NULL,
+    JSON_BOOL,
+    JSON_STRING,
+    JSON_NUMBER,
+    JSON_ARRAY,
+    JSON_OBJECT,
 } JsonTag;
 
 typedef struct JsonNode JsonNode;
 
 struct JsonNode
 {
-        /* only if parent is an object or array (NULL otherwise) */
-        JsonNode *parent;
-        JsonNode *prev, *next;
+    /* only if parent is an object or array (NULL otherwise) */
+    JsonNode *parent;
+    JsonNode *prev, *next;
 
-        /* only if parent is an object (NULL otherwise) */
-        char *key; /* Must be valid UTF-8. */
+    /* only if parent is an object (NULL otherwise) */
+    char *key; /* Must be valid UTF-8. */
 
-        JsonTag tag;
-        union {
-                /* JSON_BOOL */
-                bool bool_;
+    JsonTag tag;
+    union
+    {
+        /* JSON_BOOL */
+        bool bool_;
 
-                /* JSON_STRING */
-                char *string_; /* Must be valid UTF-8. */
+        /* JSON_STRING */
+        char *string_; /* Must be valid UTF-8. */
 
-                /* JSON_NUMBER */
-                double number_;
+        /* JSON_NUMBER */
+        double number_;
 
-                /* JSON_ARRAY */
-                /* JSON_OBJECT */
-                struct {
-                        JsonNode *head, *tail;
-                } children;
-        };
+        /* JSON_ARRAY */
+        /* JSON_OBJECT */
+        struct
+        {
+            JsonNode *head, *tail;
+        } children;
+    };
 };
 
 /*** Encoding, decoding, and validation ***/
