@@ -125,7 +125,7 @@ static bool _coords(lua_State *ls, map_lines &lines,
     if (y2 < y1)
         swap(y1, y2);
 
-    return (x1 + border <= x2 - border && y1 + border <= y2 - border);
+    return x1 + border <= x2 - border && y1 + border <= y2 - border;
 }
 
 // Check if a given coordiante is valid for lines.
@@ -412,7 +412,6 @@ LUAFN(dgn_count_passable_neighbors)
     return 1;
 }
 
-
 LUAFN(dgn_is_valid_coord)
 {
     LINES(ls, 1, lines);
@@ -810,7 +809,7 @@ LUAFN(dgn_make_box_doors)
             lines(x, y) = door;
             for (int i = 1; i < thickness; i++)
             {
-                switch(side)
+                switch (side)
                 {
                 case 0: y++;  break;
                 case 1: y--;  break;
@@ -1045,7 +1044,7 @@ LUAFN(dgn_make_round_box)
         for (int x = 0; x < size_x; x++)
             for (int y = 0; y < size_y; y++)
             {
-                switch(new_glyphs[x][y])
+                switch (new_glyphs[x][y])
                 {
                 // leave existing glyphs on OUTSIDE
                 case WALL:  lines(x1 + x, y1 + y) = wall;  break;
@@ -1137,8 +1136,8 @@ LUAFN(dgn_remove_isolated_glyphs)
             {
                 bool do_replace = true;
                 for (radius_iterator ri(coord_def(x, y), 1,
-                                        (boxy ? C_SQUARE : C_POINTY),
-                                        NULL, true);                 ri; ++ri)
+                                        (boxy ? C_ROUND : C_POINTY),
+                                        true); ri; ++ri)
                 {
                     if (_valid_coord(ls, lines, ri->x, ri->y, false))
                         if (strchr(find, lines(*ri)))
@@ -1196,8 +1195,8 @@ LUAFN(dgn_widen_paths)
             {
                 int neighbour_count = 0;
                 for (radius_iterator ri(coord_def(x, y), 1,
-                                        (boxy ? C_SQUARE : C_POINTY),
-                                        NULL, true);                 ri; ++ri)
+                                        (boxy ? C_ROUND : C_POINTY),
+                                        true); ri; ++ri)
                 {
                     if (_valid_coord(ls, lines, ri->x, ri->y, false))
                         if (strchr(passable, lines(*ri)))
@@ -1543,7 +1542,7 @@ LUAFN(dgn_spotty_map)
                && strchr(replace, lines(x, y+2)));
 
         for (radius_iterator ai(coord_def(x, y), boxy ? 2 : 1, C_CIRCLE,
-                                NULL, false); ai; ++ai)
+                                false); ai; ++ai)
         {
             if (strchr(replace, lines(*ai)))
                 lines(*ai) = fill;
