@@ -35,24 +35,24 @@ bool key_is_escape(int key);
 // Unscales a fixed-point number, rounding up.
 static inline int unscale_round_up(int number, int scale)
 {
-    return ((number + scale - 1) / scale);
+    return (number + scale - 1) / scale;
 }
 
 // Chinese rod numerals are _not_ digits for our purposes.
 static inline bool isadigit(int c)
 {
-    return (c >= '0' && c <= '9');
+    return c >= '0' && c <= '9';
 }
 
 // 'ä' is a letter, but not a valid inv slot/etc.
 static inline bool isalower(int c)
 {
-    return (c >= 'a' && c <= 'z');
+    return c >= 'a' && c <= 'z';
 }
 
 static inline bool isaupper(int c)
 {
-    return (c >= 'A' && c <= 'Z');
+    return c >= 'A' && c <= 'Z';
 }
 
 static inline bool isaalpha(int c)
@@ -131,14 +131,14 @@ string trimmed_string(string s);
 
 static inline bool starts_with(const string &s, const string &prefix)
 {
-    return (s.rfind(prefix, 0) != string::npos);
+    return s.rfind(prefix, 0) != string::npos;
 }
 
 static inline bool ends_with(const string &s, const string &suffix)
 {
     if (s.length() < suffix.length())
         return false;
-    return (s.find(suffix, s.length() - suffix.length()) != string::npos);
+    return s.find(suffix, s.length() - suffix.length()) != string::npos;
 }
 
 // Splits string 's' on the separator 'sep'. If trim == true, trims each
@@ -198,13 +198,14 @@ int isqrt_ceil(int x);
 
 static inline bool testbits(uint64_t flags, uint64_t test)
 {
-    return ((flags & test) == test);
+    return (flags & test) == test;
 }
 
 coord_def cgetsize(GotoRegion region = GOTO_CRT);
 void cscroll(int n, GotoRegion region);
 
 string untag_tiles_console(string s);
+string colour_string(string in, int col);
 
 #ifdef TARGET_OS_WINDOWS
 enum taskbar_pos
@@ -254,4 +255,5 @@ private:
 };
 
 void init_signals();
+void release_cli_signals();
 #endif
