@@ -29,8 +29,11 @@ los_def::los_def(const los_def& los)
 
 los_def& los_def::operator=(const los_def& los)
 {
-    init(los.center, *los.opc, los.bds);
-    show = los.show;
+    if (this != &los)
+    {
+        init(los.center, *los.opc, los.bds);
+        show = los.show;
+    }
     return *this;
 }
 
@@ -87,33 +90,5 @@ bool los_def::in_bounds(const coord_def& p) const
 bool los_def::see_cell(const coord_def& p) const
 {
     const coord_def sp = p - center;
-    return (sp.rdist() <= LOS_MAX_RANGE && show(sp));
-}
-
-coord_def los_glob::get_center() const
-{
-    return center;
-}
-
-circle_def los_glob::get_bounds() const
-{
-    return circle_def(center, bds);
-}
-
-bool los_glob::in_bounds(const coord_def& p) const
-{
-    return bds.contains(p - center);
-}
-
-bool los_glob::see_cell(const coord_def& p) const
-{
-    return (in_bounds(p) && cell_see_cell(center, p, lt));
-}
-
-los_glob& los_glob::operator=(const los_glob& los)
-{
-    lt = los.lt;
-    center = los.center;
-    bds = los.bds;
-    return *this;
+    return sp.rdist() <= LOS_MAX_RANGE && show(sp);
 }
