@@ -21,8 +21,6 @@
 #include "message.h"
 #include "options.h"
 
-#ifdef WIZARD
-
 static int _incomplete(lua_State *ls, int status)
 {
     if (status == LUA_ERRSYNTAX)
@@ -100,7 +98,7 @@ static int _report(lua_State *ls, int status)
         const char *msg = lua_tostring(ls, -1);
         if (msg == NULL)
             msg = "(error object is not a string)";
-        mpr(msg, MSGCH_ERROR);
+        mprf(MSGCH_ERROR, "%s", msg);
         lua_pop(ls, 1);
     }
     return status;
@@ -157,5 +155,3 @@ void debug_terp_dlua(CLua &vm)
     }
     _run_dlua_interpreter(vm);
 }
-
-#endif
