@@ -8,7 +8,8 @@
 
 #include "externs.h"
 
-#include "abl-show.h"
+#include "ability.h"
+#include "act-iter.h"
 #include "areas.h"
 #include "coord.h"
 #include "coordit.h"
@@ -17,12 +18,10 @@
 #include "env.h"
 #include "fprop.h"
 #include "exclude.h"
-#include "itemprop.h"
 #include "losglobal.h"
 #include "macro.h"
 #include "mon-act.h"
 #include "mon-death.h"
-#include "mon-iter.h"
 #include "mon-movetarget.h"
 #include "mon-pathfind.h"
 #include "mon-speak.h"
@@ -38,8 +37,6 @@
 #include "view.h"
 #include "shout.h"
 
-static void _set_nearest_monster_foe(monster* mons);
-
 static void _guess_invis_foe_pos(monster* mon)
 {
     const actor* foe          = mon->get_foe();
@@ -49,9 +46,7 @@ static void _guess_invis_foe_pos(monster* mon)
 
     // NOTE: This depends on ignoring clouds, so that cells hidden by
     // opaque clouds are included as a possibility for the foe's location.
-    los_def los(mon->pos(), opc_fullyopaque, circle_def(guess_radius, C_ROUND));
-    los.update();
-    for (radius_iterator ri(&los); ri; ++ri)
+    for (radius_iterator ri(mon->pos(), guess_radius, C_ROUND, LOS_SOLID); ri; ++ri)
     {
         if (foe->is_habitable(*ri))
             possibilities.push_back(*ri);
@@ -67,7 +62,7 @@ static void _mon_check_foe_invalid(monster* mon)
 {
     // Assume a spectral weapon has a valid target
     // Ideally this is not outside special cased like this
-    if (mon->type == MONS_SPECTRAL_WEAPON)
+    if (mons_is_avatar(mon->type))
         return;
 
     if (mon->foe != MHITNOT && mon->foe != MHITYOU)
@@ -102,8 +97,8 @@ static bool _mon_tries_regain_los(monster* mon)
     }
 
     // Randomize it a bit to make it less predictable.
-    return (mons_intel(mon) == I_NORMAL && !one_chance_in(10)
-            || mons_intel(mon) == I_HIGH && !one_chance_in(20));
+    return mons_intel(mon) == I_NORMAL && !one_chance_in(10)
+           || mons_intel(mon) == I_HIGH && !one_chance_in(20);
 }
 
 // Monster tries to get into a firing position. Among the cells which have
@@ -122,14 +117,13 @@ static void _set_firing_pos(monster* mon, coord_def target)
     int best_distance_to_ideal_range = INT_MAX;
     coord_def best_pos(0, 0);
 
-    const los_base *los = mon->get_los();
     for (distance_iterator di(mon->pos(), true, true, LOS_RADIUS);
          di; ++di)
     {
         const coord_def p(*di);
         const int range = p.distance_from(target);
 
-        if (!los->see_cell(*di))
+        if (!mon->see_cell(*di))
             continue;
 
         if (!in_bounds(p) || range > max_range
@@ -186,14 +180,13 @@ static coord_def _furthest_aim_spot(monster* mon, coord_def target)
     int best_distance = 0;
     coord_def best_pos(0, 0);
 
-    const los_base *los = mon->get_los();
     for (distance_iterator di(mon->pos(), false, false, LOS_RADIUS);
          di; ++di)
     {
         const coord_def p(*di);
         const int range = p.distance_from(target);
 
-        if (!los->see_cell(*di))
+        if (!cell_see_cell(mon->pos(), *di, LOS_NO_TRANS))
             continue;
 
         if (!in_bounds(p) || range > LOS_RADIUS - 1
@@ -240,7 +233,7 @@ static void _set_curse_skull_lurk_pos(monster* mon)
     vector<coord_def> spots;
     for (adjacent_iterator ai(you.pos()); ai; ++ai)
     {
-        if (!feat_is_solid(grd(*ai)) || feat_is_door(grd(*ai)))
+        if (!cell_is_solid(*ai) || feat_is_door(grd(*ai)))
             spots.push_back(*ai);
     }
 
@@ -263,13 +256,13 @@ static void _set_curse_skull_lurk_pos(monster* mon)
 
 static bool _stabber_keep_distance(const monster* mon, const actor* foe)
 {
-    return (mons_class_flag(mon->type, M_STABBER)
-            && !mon->berserk_or_insane()
-            && (mons_has_incapacitating_ranged_attack(mon, foe)
-                || mons_has_incapacitating_spell(mon, foe))
-            && !foe->incapacitated()
-            && !adjacent(mon->pos(), foe->pos())
-            && !mons_aligned(mon, foe));
+    return mons_class_flag(mon->type, M_STABBER)
+           && !mon->berserk_or_insane()
+           && (mons_has_incapacitating_ranged_attack(mon, foe)
+               || mons_has_incapacitating_spell(mon, foe))
+           && !foe->incapacitated()
+           && !adjacent(mon->pos(), foe->pos())
+           && !mons_aligned(mon, foe);
 }
 
 //---------------------------------------------------------------
@@ -304,6 +297,14 @@ void handle_behaviour(monster* mon)
         }
     }
 
+    // Grand avatar targeting is handled through its triggerers
+    // and _grand_avatar_act in mon-act.cc.
+    if (mon->type == MONS_GRAND_AVATAR)
+    {
+        mon->behaviour = BEH_SEEK;
+        return;
+    }
+
     bool changed = true;
     bool isFriendly = mon->friendly();
     bool isNeutral  = mon->neutral();
@@ -334,8 +335,7 @@ void handle_behaviour(monster* mon)
             const int loss = div_rand_round(10, mon->speed);
             if (loss)
             {
-                mpr("Your flesh rots away as the Orb of Zot is desecrated.",
-                    MSGCH_DANGER);
+                mprf(MSGCH_DANGER, "Your flesh rots away as the Orb of Zot is desecrated.");
 
                 // If the rot would reduce us to <= 0 max HP, attribute the
                 // kill to the monster.
@@ -365,30 +365,20 @@ void handle_behaviour(monster* mon)
         return;
     }
 
-    // Make sure monsters are not targetting the player in arena mode.
-    ASSERT(!(crawl_state.game_is_arena() && mon->foe == MHITYOU));
-
-    if (mons_wall_shielded(mon) && cell_is_solid(mon->pos()))
-    {
-        // Monster is safe, so its behaviour can be simplified to fleeing.
-        if (mon->behaviour == BEH_CORNERED || mon->behaviour == BEH_PANIC
-            || isScared)
-        {
-            mon->behaviour = BEH_FLEE;
-        }
-    }
+    // Make sure monsters are not targeting the player in arena mode.
+    ASSERT(!crawl_state.game_is_arena() || mon->foe != MHITYOU);
 
     // Validate current target exists.
     _mon_check_foe_invalid(mon);
 
-    // The target and foe set here for a spectral weapon should never change
+    actor *owner = (mon->summoner ? actor_by_mid(mon->summoner) : nullptr);
     if (mon->type == MONS_SPECTRAL_WEAPON)
     {
         // Do nothing if we're still being placed
         if (!mon->summoner)
             return;
 
-        actor *owner = actor_by_mid(mon->summoner);
+        owner = actor_by_mid(mon->summoner);
 
         if (!owner || !owner->alive())
         {
@@ -396,22 +386,24 @@ void handle_behaviour(monster* mon)
             return;
         }
 
-        mon->target = owner->pos();
-        mon->foe = MHITNOT;
-        // Try to move towards any monsters the owner is attacking
-        if (mon->props.exists(SW_TARGET_MID))
+        // Only go after the target if it's still near the owner, and
+        // so are we.  The weapon is restricted to a leash range of 2,
+        // and things reachable within that leash range [qoala]
+        const int leash = 2;
+
+        if (grid_distance(owner->pos(), mon->pos()) > leash
+            || mon->foe == MHITNOT)
+        {
+            mon->foe = owner->mindex();
+            mon->target = owner->pos();
+        }
+        else if (mon->props.exists(SW_TARGET_MID))
         {
             actor *atarget = actor_by_mid(mon->props[SW_TARGET_MID].get_int());
 
-            // Only go after the target if it's still near the owner,
-            // and so are we.
-            // The weapon is restricted to a leash range of 2,
-            // and things reachable within that leash range [qoala]
-            const int leash = 2;
             if (atarget && atarget->alive()
                 && (grid_distance(owner->pos(), atarget->pos())
-                    <= ((mon->reach_range() == REACH_TWO) ? leash + 2 : leash + 1))
-                && (grid_distance(owner->pos(), mon->pos()) <= leash))
+                    <= ((mon->reach_range() == REACH_TWO) ? leash + 2 : leash + 1)))
             {
                 mon->target = atarget->pos();
                 mon->foe = atarget->mindex();
@@ -469,13 +461,15 @@ void handle_behaviour(monster* mon)
         && !mon->berserk_or_insane()
         && mon->behaviour != BEH_WITHDRAW
         && mon->type != MONS_GIANT_SPORE
-        && mon->type != MONS_BATTLESPHERE
-        && mon->type != MONS_SPECTRAL_WEAPON)
+        && mon->type != MONS_BALL_LIGHTNING
+        && !mons_is_avatar(mon->type))
     {
-        if  (!crawl_state.game_is_zotdef())
+        if (!crawl_state.game_is_zotdef())
         {
             if (you.pet_target != MHITNOT)
                 mon->foe = you.pet_target;
+            else if (mons_class_is_stationary(mon->type))
+                set_nearest_monster_foe(mon);
         }
         else    // Zotdef only
         {
@@ -488,9 +482,9 @@ void handle_behaviour(monster* mon)
             }
             else
             {
-               // Zotdef - this is all new, for out-of-sight friendlies to do
-               // something useful.  If no current target, get the closest one.
-                _set_nearest_monster_foe(mon);
+                // Zotdef - this is all new, for out-of-sight friendlies to do
+                // something useful.  If no current target, get the closest one.
+                set_nearest_monster_foe(mon);
             }
         }
     }
@@ -498,7 +492,9 @@ void handle_behaviour(monster* mon)
     // Instead, berserkers attack nearest monsters.
     if (mon->behaviour != BEH_SLEEP
         && (mon->has_ench(ENCH_INSANE)
-            || ((mon->berserk() || mon->type == MONS_GIANT_SPORE)
+            || ((mon->berserk()
+                 || mon->type == MONS_GIANT_SPORE
+                 || mon->type == MONS_BALL_LIGHTNING)
                 && (mon->foe == MHITNOT
                     || isFriendly && mon->foe == MHITYOU))))
     {
@@ -512,7 +508,7 @@ void handle_behaviour(monster* mon)
             mon->foe = MHITYOU;
         }
         else
-            _set_nearest_monster_foe(mon);
+            set_nearest_monster_foe(mon);
     }
 
     // Pacified monsters leaving the level prefer not to attack.
@@ -523,14 +519,13 @@ void handle_behaviour(monster* mon)
         && mon->behaviour != BEH_SLEEP
         && (proxPlayer || one_chance_in(3)))
     {
-        _set_nearest_monster_foe(mon);
+        set_nearest_monster_foe(mon);
         if (mon->foe == MHITNOT && crawl_state.game_is_zotdef())
             mon->foe = MHITYOU;
     }
 
     // Friendly summons will come back to the player if they go out of sight.
-    // Spectral weapon should keep its target even though it can't attack it
-    if (!summon_can_attack(mon) && mon->type!=MONS_SPECTRAL_WEAPON)
+    if (!summon_can_attack(mon))
         mon->target = you.pos();
 
     // Monsters do not attack themselves. {dlb}
@@ -539,7 +534,7 @@ void handle_behaviour(monster* mon)
 
     // Friendly and good neutral monsters do not attack other friendly
     // and good neutral monsters.
-    if (mon->foe != MHITNOT && mon->foe != MHITYOU
+    if (!mons_is_avatar(mon->type) && mon->foe != MHITNOT && mon->foe != MHITYOU
         && wontAttack && menv[mon->foe].wont_attack())
     {
         mon->foe = MHITNOT;
@@ -556,8 +551,9 @@ void handle_behaviour(monster* mon)
 
     // Unfriendly monsters fighting other monsters will usually
     // target the player, if they're healthy.
-    // Zotdef: 2/3 chance of retargetting changed to 1/4
+    // Zotdef: 2/3 chance of retargeting changed to 1/4
     if (!isFriendly && !isNeutral
+        && !mons_is_avatar(mon->type)
         && mon->foe != MHITYOU && mon->foe != MHITNOT
         && proxPlayer && !mon->berserk_or_insane()
         && isHealthy
@@ -636,7 +632,8 @@ void handle_behaviour(monster* mon)
                     || !proxPlayer && !isFriendly
                     || isNeutral && !mon->has_ench(ENCH_INSANE)
                     || patrolling
-                    || mon->type == MONS_GIANT_SPORE)
+                    || mon->type == MONS_GIANT_SPORE
+                    || mon->type == MONS_BALL_LIGHTNING)
                 {
                     if (mon->behaviour != BEH_LURK)
                         new_beh = BEH_WANDER;
@@ -654,7 +651,7 @@ void handle_behaviour(monster* mon)
                 && !(mon->friendly()
                      && mon->foe == MHITYOU
                      && mon->is_travelling()
-                     && mon->travel_target == MTRAV_PLAYER))
+                     && mon->travel_target == MTRAV_FOE))
             {
                 // If their foe is marked, the monster always knows exactly
                 // where they are.
@@ -674,7 +671,7 @@ void handle_behaviour(monster* mon)
                         // Having a seeking mon with a foe who's target is
                         // (0, 0) can lead to asserts, so lets try to
                         // avoid that.
-                        _set_nearest_monster_foe(mon);
+                        set_nearest_monster_foe(mon);
                         if (mon->foe == MHITNOT)
                         {
                             new_beh = BEH_WANDER;
@@ -687,13 +684,27 @@ void handle_behaviour(monster* mon)
                 if (mon->travel_target == MTRAV_SIREN)
                     mon->travel_target = MTRAV_NONE;
 
-                if (isFriendly && mon->foe != MHITYOU)
+                // Spectral weapons simply seek back to their owner if
+                // they can't see their seek target.
+                if (mons_is_avatar(mon->type))
+                {
+                    // XXX: should owner ever not be set here?
+                    new_foe = owner ? owner->mindex() : MHITNOT;
+                    mon->target = owner ? owner->pos() : mon->pos();
+                    break;
+                }
+                else if (isFriendly && mon->foe != MHITYOU)
                 {
                     if (patrolling || crawl_state.game_is_arena())
                     {
                         new_foe = MHITNOT;
                         new_beh = BEH_WANDER;
                     }
+                    // If the player can see the target location, do not reset
+                    // our target, even if this monster cannot (we'll assume
+                    // the player passes along this information to allies)
+                    else if (!foepos.origin() && you.see_cell(foepos))
+                        try_pathfind(mon);
                     else
                     {
                         new_foe = MHITYOU;
@@ -717,12 +728,15 @@ void handle_behaviour(monster* mon)
                     {   // hostiles only in Zotdef
                         if (mon->foe == MHITYOU)
                         {
+                            // infallible tracking in zotdef
                             if (crawl_state.game_is_zotdef())
-                                mon->target = PLAYER_POS;  // infallible tracking in zotdef
+                                mon->target = PLAYER_POS;
                             else
                             {
                                 if (x_chance_in_y(50, you.stealth())
-                                    || you.penance[GOD_ASHENZARI] && coinflip())
+                                    || you.penance[GOD_ASHENZARI] && coinflip()
+                                    || mons_class_flag(mon->type, M_VIGILANT)
+                                       && !one_chance_in(3))
                                 {
                                     mon->target = you.pos();
                                 }
@@ -740,7 +754,6 @@ void handle_behaviour(monster* mon)
                     }
                 }
 
-
                 if (mon->foe_memory <= 0
                     && !(mon->friendly() && mon->foe == MHITYOU))
                 {
@@ -764,6 +777,13 @@ void handle_behaviour(monster* mon)
 
                 if (!isFriendly)
                     break;
+                else if (mons_is_avatar(mon->type)
+                         && owner
+                         && !owner->is_player())
+                {
+                    mon->foe = owner->mindex();
+                    break;
+                }
             }
 
             ASSERT(proxFoe || isFriendly);
@@ -811,6 +831,8 @@ void handle_behaviour(monster* mon)
                     else
                         _set_firing_pos(mon, you.pos());
                 }
+                else if (mon->type == MONS_SIREN)
+                    find_siren_water_target(mon);
                 else if (!mon->firing_pos.zero()
                     && mon->see_cell_no_trans(mon->target))
                 {
@@ -819,13 +841,6 @@ void handle_behaviour(monster* mon)
                     mon->firing_pos.reset();
                 }
 
-                if (mon->type == MONS_SIREN
-                    && you.beheld_by(mon)
-                    && find_siren_water_target(mon))
-                {
-                    break;
-                }
-
                 if (mon->firing_pos.zero() && try_pathfind(mon))
                     break;
 
@@ -833,16 +848,7 @@ void handle_behaviour(monster* mon)
                 // (or wasn't even attempted) and we need to set our target
                 // the traditional way.
 
-                // Sometimes, your friends will wander a bit.
-                if (isFriendly && one_chance_in(8)
-                    && mon->foe == MHITYOU && proxFoe)
-                {
-                    set_random_target(mon);
-                    mon->foe = MHITNOT;
-                    new_beh  = BEH_WANDER;
-                }
-                else
-                    mon->target = PLAYER_POS;
+                mon->target = PLAYER_POS;
             }
             else
             {
@@ -851,7 +857,9 @@ void handle_behaviour(monster* mon)
                 mon->target = target->pos();
 
                 if (mons_class_flag(mon->type, M_MAINTAIN_RANGE)
-                    && !mon->berserk_or_insane())
+                    && !mon->berserk_or_insane()
+                    && !(mons_is_avatar(mon->type)
+                         && owner && mon->foe == owner->mindex()))
                 {
                     _set_firing_pos(mon, mon->target);
                 }
@@ -865,7 +873,14 @@ void handle_behaviour(monster* mon)
                     else
                         _set_firing_pos(mon, target->pos());
                 }
-
+                // Hold position if we've reached our ideal range
+                else if (mon->type == MONS_SPELLFORGED_SERVITOR
+                         && (mon->pos() - target->pos()).abs()
+                         <= dist_range(mon->props["ideal_range"].get_int())
+                         && !one_chance_in(8))
+                {
+                    mon->firing_pos = mon->pos();
+                }
             }
 
             break;
@@ -927,7 +942,8 @@ void handle_behaviour(monster* mon)
             // Creatures not currently pursuing another foe are
             // alerted by a sentinel's mark
             if (mon->foe == MHITNOT && you.duration[DUR_SENTINEL_MARK]
-                && (!isFriendly && !isNeutral && !isPacified
+                && (!isFriendly && !mons_is_avatar(mon->type) && !isNeutral
+                    && !isPacified
                     || mon->has_ench(ENCH_INSANE)))
             {
                 new_foe = MHITYOU;
@@ -942,7 +958,7 @@ void handle_behaviour(monster* mon)
             // monsters have longer memories).  Pacified monsters will
             // also eventually switch the place from which they want to
             // leave the level, in case their current choice is blocked.
-            if (!proxFoe && mon->foe != MHITNOT
+            if (!proxFoe && !mons_is_avatar(mon->type) && mon->foe != MHITNOT
                    && one_chance_in(isSmart ? 60 : 20)
                    && !mons_foe_is_marked(mon)
                 || isPacified && one_chance_in(isSmart ? 40 : 120))
@@ -994,8 +1010,6 @@ void handle_behaviour(monster* mon)
                 if (mon->foe == MHITYOU)
                     mon->target = you.pos();
             }
-            else if (mons_wall_shielded(mon) && find_wall_target(mon))
-                ; // Wall target found.
             else if (proxFoe)
             {
                 // Special-cased below so that it will flee *from* the
@@ -1005,15 +1019,13 @@ void handle_behaviour(monster* mon)
             break;
 
         case BEH_CORNERED:
-            // Plants and nonliving monsters cannot fight back.
-            if (mon->holiness() == MH_PLANT
-                || mon->holiness() == MH_NONLIVING)
-            {
-                break;
-            }
 
-            if (isHealthy)
-                new_beh = BEH_SEEK;
+            // If we were able to move since becoming cornered, resume fleeing
+            if (mon->pos() != mon->props["last_pos"].get_coord())
+            {
+                new_beh = BEH_FLEE;
+                mon->props.erase("last_pos");
+            }
 
             // Foe gone out of LOS?
             if (!proxFoe)
@@ -1131,20 +1143,10 @@ void handle_behaviour(monster* mon)
 
         mon->foe = new_foe;
     }
-
-    if (mon->travel_target == MTRAV_WALL && cell_is_solid(mon->pos()))
-    {
-        if (mon->behaviour == BEH_FLEE)
-        {
-            // Monster is safe, so stay put.
-            mon->target = mon->pos();
-            mon->foe = MHITNOT;
-        }
-    }
 }
 
 static bool _mons_check_foe(monster* mon, const coord_def& p,
-                            bool friendly, bool neutral)
+                            bool friendly, bool neutral, bool ignore_sight)
 {
     // We don't check for the player here because otherwise wandering
     // monsters will always attack you.
@@ -1156,7 +1158,7 @@ static bool _mons_check_foe(monster* mon, const coord_def& p,
         && (mon->has_ench(ENCH_INSANE)
             || foe->friendly() != friendly
             || neutral && !foe->neutral())
-        && mon->can_see(foe)
+        && (ignore_sight || mon->can_see(foe))
         && !foe->is_projectile()
         && summon_can_attack(mon, p)
         && (friendly || !is_sanctuary(p))
@@ -1173,14 +1175,13 @@ static bool _mons_check_foe(monster* mon, const coord_def& p,
 }
 
 // Choose random nearest monster as a foe.
-static void _set_nearest_monster_foe(monster* mon)
+void set_nearest_monster_foe(monster* mon, bool near_player)
 {
     // These don't look for foes.
     if (mon->good_neutral() || mon->strict_neutral()
-            || mon->behaviour == BEH_WITHDRAW
-            || mon->type == MONS_BATTLESPHERE
-            || mon->type == MONS_SPECTRAL_WEAPON
-            || mon->has_ench(ENCH_HAUNTING))
+        || mon->behaviour == BEH_WITHDRAW
+        || mons_is_avatar(mon->type)
+        || mon->has_ench(ENCH_HAUNTING))
     {
         return;
     }
@@ -1188,25 +1189,46 @@ static void _set_nearest_monster_foe(monster* mon)
     const bool friendly = mon->friendly();
     const bool neutral  = mon->neutral();
 
-    for (int k = 1; k <= LOS_RADIUS; ++k)
-    {
-        vector<coord_def> monster_pos;
-        for (int i = -k; i <= k; ++i)
-            for (int j = -k; j <= k; (abs(i) == k ? j++ : j += 2*k))
-            {
-                const coord_def p = mon->pos() + coord_def(i, j);
-                if (_mons_check_foe(mon, p, friendly, neutral))
-                    monster_pos.push_back(p);
-            }
-        if (monster_pos.empty())
-            continue;
+    coord_def center = mon->pos();
+    bool second_pass = false;
 
-        const coord_def mpos = monster_pos[random2(monster_pos.size())];
-        if (mpos == you.pos())
-            mon->foe = MHITYOU;
+    while (true)
+    {
+        for (int k = 1; k <= LOS_RADIUS; ++k)
+        {
+            vector<coord_def> monster_pos;
+            for (int i = -k; i <= k; ++i)
+                for (int j = -k; j <= k; (abs(i) == k ? j++ : j += 2*k))
+                {
+                    const coord_def p = center + coord_def(i, j);
+
+                    if (near_player && !you.see_cell(p))
+                        continue;
+
+                    if (_mons_check_foe(mon, p, friendly, neutral, second_pass))
+                        monster_pos.push_back(p);
+                }
+            if (monster_pos.empty())
+                continue;
+
+            const coord_def mpos = monster_pos[random2(monster_pos.size())];
+            if (mpos == you.pos())
+                mon->foe = MHITYOU;
+            else
+                mon->foe = env.mgrid(mpos);
+            return;
+        }
+
+        // If we're selecting a new summon's autofoe and we were unable to
+        // find a foe in los of the monster, try a second pass using the
+        // player's los instead.
+        if (near_player && !second_pass)
+        {
+            center = you.pos();
+            second_pass = true;
+        }
         else
-            mon->foe = env.mgrid(mpos);
-        return;
+            break;
     }
 }
 
@@ -1282,69 +1304,76 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
 
     case ME_WHACK:
     case ME_ANNOY:
-
         // Orders to withdraw take precedence over interruptions
-        if (mon->behaviour == BEH_WITHDRAW)
+        if (mon->behaviour == BEH_WITHDRAW && src != &you)
             break;
 
         // Will turn monster against <src>, unless they
         // are BOTH friendly or good neutral AND stupid,
         // or else fleeing anyway.  Hitting someone over
         // the head, of course, always triggers this code.
-        if (event == ME_WHACK
-            || mon->has_ench(ENCH_INSANE)
-            || ((wontAttack != sourceWontAttack || (mons_intel(mon) > I_PLANT))
-                && (!mons_is_fleeing(mon) && !mons_class_flag(mon->type, M_FLEEING))
-                && !mons_is_panicking(mon)))
+        if (event != ME_WHACK
+            && !mon->has_ench(ENCH_INSANE)
+            && (wontAttack == sourceWontAttack && mons_intel(mon) <= I_PLANT
+                || mons_is_fleeing(mon)))
         {
-            // Monster types that you can't gain experience from cannot
-            // fight back, so don't bother having them do so.  If you
-            // worship Fedhas, create a ring of friendly plants, and try
-            // to break out of the ring by killing a plant, you'll get
-            // a warning prompt and penance only once.  Without the
-            // hostility check, the plant will remain friendly until it
-            // dies, and you'll get a warning prompt and penance once
-            // *per hit*.  This may not be the best way to address the
-            // issue, though. -cao
-            if (mons_class_flag(mon->type, M_NO_EXP_GAIN)
-                && mon->attitude != ATT_FRIENDLY
-                && mon->attitude != ATT_GOOD_NEUTRAL)
-            {
-                return;
-            }
-
-            mon->foe = src_idx;
-
-            if (mon->asleep() && mons_near(mon))
-                remove_auto_exclude(mon, true);
-
-            // If the monster can't reach its target and can't attack it
-            // either, retreat.
-            try_pathfind(mon);
-            if (mons_intel(mon) > I_REPTILE && !mons_can_attack(mon)
-                && target_is_unreachable(mon))
-            {
-                mon->behaviour = BEH_RETREAT;
-            }
-            else if (!mons_is_cornered(mon) && (mon->hit_points > fleeThreshold))
-                mon->behaviour = BEH_SEEK;
-            else if (mon->asleep())
-                mon->behaviour = BEH_SEEK;
-
-            if (src == &you
-                && !mon->has_ench(ENCH_INSANE)
-                && mon->type != MONS_BATTLESPHERE
-                && mon->type != MONS_SPECTRAL_WEAPON)
-            {
-                mon->attitude = ATT_HOSTILE;
-                breakCharm    = true;
-            }
-
-            // XXX: Somewhat hacky, this being here.
-            if (mons_is_elven_twin(mon))
-                elven_twins_unpacify(mon);
+            break;
         }
 
+        // Monster types that you can't gain experience from cannot
+        // fight back, so don't bother having them do so.  If you
+        // worship Fedhas, create a ring of friendly plants, and try
+        // to break out of the ring by killing a plant, you'll get
+        // a warning prompt and penance only once.  Without the
+        // hostility check, the plant will remain friendly until it
+        // dies, and you'll get a warning prompt and penance once
+        // *per hit*.  This may not be the best way to address the
+        // issue, though. -cao
+        if (mons_class_flag(mon->type, M_NO_EXP_GAIN)
+            && mon->attitude != ATT_FRIENDLY
+            && mon->attitude != ATT_GOOD_NEUTRAL)
+        {
+            return;
+        }
+
+        mon->foe = src_idx;
+
+        if (mon->asleep() && mons_near(mon))
+            remove_auto_exclude(mon, true);
+
+        // If the monster can't reach its target and can't attack it
+        // either, retreat.
+        try_pathfind(mon);
+        if (mons_intel(mon) > I_REPTILE && !mons_can_attack(mon)
+            && target_is_unreachable(mon))
+        {
+            mon->behaviour = BEH_RETREAT;
+        }
+        else if (mon->has_ench(ENCH_FEAR))
+        {
+            if (you.can_see(mon))
+            {
+                mprf("%s attack snaps %s out of its fear.",
+                        src ? src->name(DESC_ITS).c_str() : "the",
+                        mon->name(DESC_THE).c_str());
+            }
+            mon->del_ench(ENCH_FEAR, true);
+        }
+        else if (!mons_is_fleeing(mon))
+            mon->behaviour = BEH_SEEK;
+
+        if (src == &you
+            && !mon->has_ench(ENCH_INSANE)
+            && !mons_is_avatar(mon->type))
+        {
+            mon->attitude = ATT_HOSTILE;
+            breakCharm    = true;
+        }
+
+        // XXX: Somewhat hacky, this being here.
+        if (mons_is_elven_twin(mon))
+            elven_twins_unpacify(mon);
+
         // Now set target so that monster can whack back (once) at an
         // invisible foe.
         if (event == ME_WHACK)
@@ -1365,9 +1394,13 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         if (mon->behaviour == BEH_WITHDRAW)
             break;
 
-        // Avoid moving friendly giant spores out of BEH_WANDER.
-        if (mon->friendly() && mon->type == MONS_GIANT_SPORE)
+        // Avoid moving friendly explodey things out of BEH_WANDER.
+        if (mon->friendly()
+            && (mon->type == MONS_GIANT_SPORE
+                || mon->type == MONS_BALL_LIGHTNING))
+        {
             break;
+        }
 
         // [ds] Neutral monsters don't react to your presence.
         // XXX: Neutral monsters are a tangled mess of arbitrary logic.
@@ -1386,13 +1419,8 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         // Will alert monster to <src> and turn them
         // against them, unless they have a current foe.
         // It won't turn friends hostile either.
-        if ((!mons_is_fleeing(mon) || mons_class_flag(mon->type, M_FLEEING))
-            && !mons_is_retreating(mon)
-            && !mons_is_panicking(mon)
-            && !mons_is_cornered(mon))
-        {
+        if (!mons_is_retreating(mon))
             mon->behaviour = BEH_SEEK;
-        }
 
         if (mon->foe == MHITNOT)
             mon->foe = src_idx;
@@ -1440,20 +1468,7 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         mon->foe       = src_idx;
         mon->target    = src_pos;
         if (src == &you)
-        {
-            // Friendly monsters don't become hostile if you read a
-            // scroll of fear, but enslaved ones will.
-            // Send friendlies off to a random target so they don't cling
-            // to you in fear.
-            if (mon->friendly())
-            {
-                breakCharm = true;
-                mon->foe   = MHITNOT;
-                set_random_target(mon);
-            }
-            else
                 setTarget = true;
-        }
         else if (mon->friendly() && !crawl_state.game_is_arena())
             mon->foe = MHITYOU;
 
@@ -1463,16 +1478,18 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         break;
 
     case ME_CORNERED:
-        // Some monsters can't flee.
-        if (!mons_is_retreating(mon) && !mon->has_ench(ENCH_FEAR))
+        // We only care about this event if we were actually running away
+        if (!mons_is_retreating(mon))
             break;
 
         // Pacified monsters shouldn't change their behaviour.
         if (mon->pacified())
             break;
 
-        // Just set behaviour... foe doesn't change.
-        if (!mons_is_cornered(mon) && !mon->has_ench(ENCH_WITHDRAWN))
+        // If we were already cornered last turn, give up on trying to flee
+        // and turn to fight instead. Otherwise, pause a turn in hope that
+        // an escape route will open up.
+        if (mons_is_cornered(mon))
         {
             if (mon->friendly() && !crawl_state.game_is_arena())
             {
@@ -1483,11 +1500,20 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
             {
                 msg = getSpeakString(mon->name(DESC_PLAIN) + " cornered");
                 if (msg.empty())
-                    msg = "PLAIN:@The_monster@ turns to fight!";
+                    msg = "PLAIN:Cornered, @The_monster@ turns to fight!";
             }
+            mon->del_ench(ENCH_FEAR, true);
+            mon->behaviour = BEH_SEEK;
         }
-
-        mon->behaviour = BEH_CORNERED;
+        else if (mons_is_fleeing(mon))
+        {
+            // Save their current position so we know if they manage to move
+            // on the following turn (and thus resume BEH_FLEE)
+            mon->props["last_pos"].get_coord() = mon->pos();
+            mon->behaviour = BEH_CORNERED;
+        }
+        else
+            mon->behaviour = BEH_SEEK;
         break;
 
     case ME_HURT:
@@ -1503,7 +1529,6 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         //   at 10 hp: 50% chance of fleeing
         //   (chance increases by 5% for every hp lost.)
         if (mons_class_flag(mon->type, M_FLEES)
-            && !mons_is_cornered(mon)
             && !mon->berserk_or_insane()
             && x_chance_in_y(fleeThreshold - mon->hit_points, fleeThreshold))
         {
@@ -1520,8 +1545,7 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         mon->target = src_pos;
         if (src->is_player()
             && !mon->has_ench(ENCH_INSANE)
-            && mon->type != MONS_BATTLESPHERE
-            && mon->type != MONS_SPECTRAL_WEAPON)
+            && !mons_is_avatar(mon->type))
         {
             // Why only attacks by the player change attitude? -- 1KB
             mon->attitude = ATT_HOSTILE;
@@ -1585,9 +1609,9 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
             {
                 ASSERT_RANGE(get_talent(ABIL_CONVERT_TO_BEOGH, false).hotkey,
                              'A', 'z' + 1);
-                mprf("(press <white>%s %c</white> to convert to Beogh)",
-                     command_to_string(CMD_USE_ABILITY).c_str(),
-                     get_talent(ABIL_CONVERT_TO_BEOGH, false).hotkey);
+                mprf("(press <w>%c</w> on the <w>%s</w>bility menu to convert to Beogh)",
+                     get_talent(ABIL_CONVERT_TO_BEOGH, false).hotkey,
+                     command_to_string(CMD_USE_ABILITY).c_str());
                 you.attribute[ATTR_SEEN_BEOGH] = 1;
             }
         }
