@@ -3,7 +3,6 @@
  * @brief Functions used when placing monsters in the dungeon.
 **/
 
-
 #ifndef MONPLACE_H
 #define MONPLACE_H
 
@@ -27,6 +26,11 @@ monster* create_monster(mgen_data mg, bool fail_msg = true);
  * *********************************************************************** */
 monster* mons_place(mgen_data mg);
 
+bool needs_resolution(monster_type mon_type);
+
+const monster_type fixup_zombie_type(const monster_type cls,
+                                     const monster_type base_type);
+
 /* ***********************************************************************
  * This isn't really meant to be a public function.  It is a low level
  * monster placement function used by dungeon building routines and
@@ -65,7 +69,8 @@ conduct_type player_will_anger_monster(monster* mon);
 bool player_angers_monster(monster* mon);
 
 bool find_habitable_spot_near(const coord_def& where, monster_type mon_type,
-                              int radius, bool allow_centre, coord_def& empty);
+                              int radius, bool allow_centre, coord_def& empty,
+                              const monster* viable_mon = nullptr);
 
 monster_type summon_any_demon(monster_type dct);
 
@@ -87,7 +92,8 @@ bool monster_habitable_grid(
     int flies = -1,
     bool paralysed = false);
 bool monster_can_submerge(const monster* mon, dungeon_feature_type grid);
-coord_def find_newmons_square(monster_type mons_class, const coord_def &p);
+coord_def find_newmons_square(monster_type mons_class, const coord_def &p,
+                              const monster* viable_mon = nullptr);
 coord_def find_newmons_square_contiguous(monster_type mons_class,
                                          const coord_def &start,
                                          int maxdistance = 3);
