@@ -3,7 +3,6 @@
  * @brief Misc monster related functions.
 **/
 
-
 #ifndef MONUTIL_H
 #define MONUTIL_H
 
@@ -11,7 +10,6 @@
 #include "enum.h"
 #include "mon-enum.h"
 #include "player.h"
-#include "mon-mst.h"
 
 struct bolt;
 
@@ -90,21 +88,25 @@ struct mon_energy_usage
         me.pickup_percent = combine(pickup_percent, o.pickup_percent, 100);
         return me;
     }
+
+    bool operator == (const mon_energy_usage &rvalue) const
+    {
+        return move == rvalue.move
+               && swim == rvalue.swim
+               && attack == rvalue.attack
+               && missile == rvalue.missile
+               && spell == rvalue.spell
+               && special == rvalue.special
+               && item == rvalue.item
+               && pickup_percent == rvalue.pickup_percent;
+    }
 private:
     static uint8_t combine(uint8_t a, uint8_t b, uint8_t def = 10)
     {
-        return (b != def? b : a);
+        return b != def? b : a;
     }
 };
 
-struct mon_spellbook
-{
-    mon_spellbook_type type;
-    spell_type spells[NUM_MONSTER_SPELL_SLOTS];
-};
-
-extern const mon_spellbook mspell_list[];
-
 struct monsterentry
 {
     short mc;            // monster number
@@ -143,7 +145,7 @@ struct monsterentry
 
     int8_t AC; // armour class
     int8_t ev; // evasion
-    mon_spellbook_type sec;
+    int sec;   // actually mon_spellbook_type
     corpse_effect_type corpse_thingy;
     zombie_size_type   zombie_size;
     shout_type         shouts;
@@ -202,7 +204,6 @@ flight_type mons_flies(const monster* mon, bool temp = true);
 
 bool mons_flattens_trees(const monster* mon);
 int mons_class_res_wind(monster_type mc);
-bool mons_wall_shielded(const monster* mon);
 
 mon_itemuse_type mons_class_itemuse(monster_type mc);
 mon_itemuse_type mons_itemuse(const monster* mon);
@@ -245,8 +246,10 @@ void discover_shifter(monster* shifter);
 bool mons_is_statue(monster_type mc, bool allow_disintegrate = false);
 bool mons_is_demon(monster_type mc);
 bool mons_is_draconian(monster_type mc);
+bool mons_is_demonspawn(monster_type mc);
 bool mons_is_conjured(monster_type mc);
 bool mons_is_beast(monster_type mc);
+bool mons_is_avatar(monster_type mc);
 int mons_demon_tier(monster_type mc);
 
 bool mons_class_wields_two_weapons(monster_type mc);
@@ -271,11 +274,13 @@ bool mons_zombifiable(monster_type mc);
 
 int mons_weight(monster_type mc);
 int mons_class_base_speed(monster_type mc);
+mon_energy_usage mons_class_energy(monster_type mc);
 int mons_class_zombie_base_speed(monster_type zombie_base_mc);
 int mons_base_speed(const monster* mon);
 
 bool mons_class_can_regenerate(monster_type mc);
 bool mons_can_regenerate(const monster* mon);
+bool mons_class_fast_regen(monster_type mc);
 bool mons_class_can_display_wounds(monster_type mc);
 bool mons_can_display_wounds(const monster* mon);
 int mons_zombie_size(monster_type mc);
@@ -283,12 +288,10 @@ monster_type mons_zombie_base(const monster* mon);
 bool mons_class_is_zombified(monster_type mc);
 bool mons_class_is_hybrid(monster_type mc);
 bool mons_class_is_chimeric(monster_type mc);
-bool mons_class_is_jumpy(monster_type mc);
-bool mons_class_is_clingy(monster_type mc);
 bool mons_class_is_animated_weapon(monster_type type);
-bool mons_class_has_base_type(monster_type mc);
 monster_type mons_base_type(const monster* mon);
 bool mons_class_can_leave_corpse(monster_type mc);
+bool mons_class_leaves_hide(monster_type mc);
 bool mons_is_zombified(const monster* mons);
 bool mons_class_can_be_zombified(monster_type mc);
 bool mons_can_be_zombified(const monster* mon);
@@ -308,6 +311,7 @@ int mons_class_colour(monster_type mc);
 
 monster_type royal_jelly_ejectable_monster();
 monster_type random_draconian_monster_species();
+monster_type random_demonspawn_monster_species();
 
 bool init_abomination(monster* mon, int hd);
 void define_monster(monster* mons);
@@ -326,8 +330,6 @@ bool mons_can_attack(const monster* mon);
 bool mons_has_incapacitating_spell(const monster* mon, const actor* foe);
 bool mons_has_incapacitating_ranged_attack(const monster* mon, const actor* foe);
 
-vector<mon_spellbook_type> mons_spellbook_list(monster_type mon);
-
 const char *mons_pronoun(monster_type mon_type, pronoun_type variant,
                          bool visible = true);
 
@@ -351,22 +353,21 @@ bool mons_is_wandering(const monster* m);
 bool mons_is_seeking(const monster* m);
 bool mons_is_fleeing(const monster* m);
 bool mons_is_retreating(const monster* m);
-bool mons_is_panicking(const monster* m);
 bool mons_is_cornered(const monster* m);
 bool mons_is_lurking(const monster* m);
 bool mons_is_batty(const monster* m);
 bool mons_is_influenced_by_sanctuary(const monster* m);
 bool mons_is_fleeing_sanctuary(const monster* m);
+bool mons_just_slept(const monster* m);
 bool mons_class_is_slime(monster_type mc);
 bool mons_is_slime(const monster* mon);
 bool mons_class_is_plant(monster_type mc);
 bool mons_is_plant(const monster* mon);
 bool mons_eats_items(const monster* mon);
 bool mons_eats_corpses(const monster* mon);
-bool mons_eats_food(const monster* mon);
 monster_type mons_genus(monster_type mc);
 monster_type mons_species(monster_type mc);
-monster_type draco_subspecies(const monster* mon);
+monster_type draco_or_demonspawn_subspecies(const monster* mon);
 monster_type mons_detected_base(monster_type mt);
 
 bool mons_looks_stabbable(const monster* m);
@@ -389,10 +390,11 @@ bool herd_monster(const monster* mon);
 
 int cheibriados_monster_player_speed_delta(const monster* mon);
 bool cheibriados_thinks_mons_is_fast(const monster* mon);
+bool mons_is_illuminating(const monster* mon);
+bool mons_is_fiery(const monster* mon);
 bool mons_is_projectile(monster_type mc);
 bool mons_is_projectile(const monster* mon);
 bool mons_is_boulder(const monster* mon);
-bool mons_is_jumpy(const monster* mon);
 bool mons_can_cling_to_walls(const monster* mon);
 bool mons_is_object(monster_type mc);
 bool mons_has_blood(monster_type mc);
@@ -417,6 +419,8 @@ colour_t random_monster_colour();
 int ugly_thing_colour_offset(colour_t colour);
 string  draconian_colour_name(monster_type mon_type);
 monster_type draconian_colour_by_name(const string &colour);
+string  demonspawn_base_name(monster_type mon_type);
+monster_type demonspawn_base_by_name(const string &colour);
 
 monster_type random_monster_at_grid(const coord_def& p, bool species = false);
 
@@ -435,6 +439,7 @@ bool mons_class_can_pass(monster_type mc, const dungeon_feature_type grid);
 bool mons_can_open_door(const monster* mon, const coord_def& pos);
 bool mons_can_eat_door(const monster* mon, const coord_def& pos);
 bool mons_can_traverse(const monster* mon, const coord_def& pos,
+                       bool only_in_sight = false,
                        bool checktraps = true);
 
 mon_inv_type equip_slot_to_mslot(equipment_type eq);
@@ -470,6 +475,9 @@ vector<monster* > get_on_level_followers();
 
 bool mons_stores_tracking_data(const monster* mons);
 
+bool mons_is_player_shadow(const monster* mon);
+bool mons_antimagic_affected(const monster* mons);
+
 void reset_all_monsters();
 void debug_mondata();
 #endif
