@@ -30,7 +30,7 @@
 },
 
 { MUT_STRONG,                         7, 2, false,  true, false,
-  NULL,
+  "strong",
 
   {"Your muscles are strong. (Str +2)",
    "Your muscles are very strong. (Str +4)", ""},
@@ -41,7 +41,7 @@
 },
 
 { MUT_CLEVER,                         7, 2, false,  true, false,
-  NULL,
+  "clever",
 
   {"Your mind is acute. (Int +2)",
    "Your mind is very acute. (Int +4)", ""},
@@ -52,7 +52,7 @@
 },
 
 { MUT_AGILE,                          7, 2, false,  true, false,
-  NULL,
+  "agile",
 
   {"You are agile. (Dex +2)",
    "You are very agile. (Dex +4)", ""},
@@ -144,6 +144,42 @@
   "cold resistance"
 },
 
+{ MUT_HEAT_VULNERABILITY,             0,  3, true, false,  true,
+  "heat vulnerability",
+
+  {"You are vulnerable to heat.",
+   "You are very vulnerable to heat.",
+   "You are extremely vulnerable to heat."},
+
+  {"You feel hot for a moment.",
+   "You feel hot for a moment.",
+   "You feel hot for a moment."},
+
+  {"You feel a sudden chill.",
+   "You feel a sudden chill.",
+   "You feel a sudden chill."},
+
+  "heat vulnerability"
+},
+
+{ MUT_COLD_VULNERABILITY,             0,  3, true, false,  true,
+  "cold vulnerability",
+
+  {"You are vulnerable to cold.",
+   "You are very vulnerable to cold.",
+   "You are extremely vulnerable to cold."},
+
+  {"You feel a sudden chill.",
+   "You feel a sudden chill.",
+   "You feel a sudden chill."},
+
+  {"You feel hot for a moment.",
+   "You feel hot for a moment.",
+   "You feel hot for a moment."},
+
+  "cold vulnerability"
+},
+
 { MUT_DEMONIC_GUARDIAN,               0,  3, false, false, false,
   "demonic guardian",
 
@@ -193,17 +229,17 @@
 { MUT_SLOW_HEALING,                   3,  3,  true, false, false,
   "slow healing",
 
-  {"You heal slowly.",
-   "You heal very slowly.",
+  {"You heal slowly when monsters are visible.",
+   "You do not heal naturally when monsters are visible.",
    "You do not heal naturally."},
 
-  {"You begin to heal more slowly.",
-   "You begin to heal more slowly.",
+  {"Your natural healing is weakened.",
+   "Your natural healing is weakened.",
    "You stop healing."},
 
-  {"Your rate of healing increases.",
-   "Your rate of healing increases.",
-   "Your rate of healing increases."},
+  {"Your natural healing is strengthened.",
+   "Your natural healing is strengthened.",
+   "Your natural healing is strengthened."},
 
   "slow healing"
 },
@@ -245,7 +281,7 @@
 },
 
 { MUT_WEAK,                          8, 2,  true,  true, false,
-  NULL,
+  "weak",
   {"You are weak. (Str -2)",
    "You are very weak. (Str -4)", ""},
   {"", "", ""},
@@ -254,7 +290,7 @@
 },
 
 { MUT_DOPEY,                         8, 2,  true,  true, false,
-  NULL,
+  "dopey",
   {"You are dopey. (Int -2)",
    "You are very dopey. (Int -4)", ""},
   {"", "", ""},
@@ -263,7 +299,7 @@
 },
 
 { MUT_CLUMSY,                        8, 2,  true,  true, false,
-  NULL,
+  "clumsy",
   {"You are clumsy. (Dex -2)",
    "You are very clumsy. (Dex -4)", ""},
   {"", "", ""},
@@ -319,7 +355,7 @@
   "magic resistance"
 },
 
-{ MUT_FAST,                           1,  3, false, false,  true,
+{ MUT_FAST,                           0,  3, false, false,  true,
   "speed",
 
   {"You cover ground quickly.",
@@ -337,7 +373,7 @@
   "fast"
 },
 
-{ MUT_SLOW,                           3,  3,  true, false,  true,
+{ MUT_SLOW,                           0,  3,  true, false,  true,
   "slowness",
 
   {"You cover ground slowly.",
@@ -417,6 +453,24 @@
   "breathe flames"
 },
 
+{ MUT_JUMP,                 0,  3, false, false, false,
+  "jump",
+
+  {"You can jump attack at a short distance.",
+   "You can jump attack at a medium distance.",
+   "You can jump attack at a long distance."},
+
+  {"You feel more sure on your feet.",
+   "You feel more sure on your feet.",
+   "You feel more sure on your feet."},
+
+  {"You feel less sure on your feet.",
+   "You feel less sure on your feet.",
+   "You feel less sure on your feet."},
+
+  "jump"
+},
+
 { MUT_BLINK,                          3,  3, false, false, false,
   "blink",
 
@@ -437,7 +491,7 @@
 
 #if TAG_MAJOR_VERSION == 34
 { MUT_STRONG_STIFF,                  0,  3, false,  true, false,
-  NULL,
+  "strong stiff",
 
   {"Your muscles are strong, but stiff (Str +1, Dex -1).",
    "Your muscles are very strong, but stiff (Str +2, Dex -2).",
@@ -455,7 +509,7 @@
 },
 
 { MUT_FLEXIBLE_WEAK,                 0,  3, false,  true, false,
-  NULL,
+  "flexible weak",
 
   {"Your muscles are flexible, but weak (Str -1, Dex +1).",
    "Your muscles are very flexible, but weak (Str -2, Dex +2).",
@@ -592,7 +646,7 @@
 },
 
 { MUT_FRAIL,                         10,  3,  true,  true, false,
-  NULL,
+  "frail",
 
   {"You are frail (-10% HP).",
    "You are very frail (-20% HP).",
@@ -610,7 +664,7 @@
 },
 
 { MUT_ROBUST,                         5,  3, false,  true, false,
-  NULL,
+  "robust",
 
   {"You are robust (+10% HP).",
    "You are very robust (+20% HP).",
@@ -729,7 +783,7 @@
 
   {"Your teeth lengthen and sharpen.",
    "Your teeth lengthen and sharpen some more.",
-   "Your teeth are very long and razor-sharp."},
+   "Your teeth grow very long and razor-sharp."},
 
   {"Your teeth shrink to normal size.",
    "Your teeth shrink and become duller.",
@@ -794,7 +848,7 @@
 
 // Octopode only
 { MUT_TENTACLE_SPIKE,                10,  3, false,  true,  true,
-  "spike",
+  "tentacle spike",
 
   {"One of your tentacles bears a spike.",
    "One of your tentacles bears a nasty spike.",
@@ -811,8 +865,8 @@
   "tentacle spike"
 },
 
-// Naga only; getting it is special-cased.
-{ MUT_BREATHE_POISON,                 0,  1, false, false,  false,
+// Naga only
+{ MUT_BREATHE_POISON,                 4,  1, false, false,  false,
   "breathe poison",
 
   {"You can exhale a cloud of poison.", "", ""},
@@ -880,7 +934,7 @@
 },
 
 { MUT_SHAGGY_FUR,                     2,  3, false,  true,  true,
-  NULL,
+  "shaggy fur",
 
   {"You are covered in fur (AC +1).",
    "You are covered in thick fur (AC +2).",
@@ -898,7 +952,7 @@
 },
 
 { MUT_HIGH_MAGIC,                     2,  3, false, false, false,
-  NULL,
+  "high mp",
 
   {"You have an increased reservoir of magic (+10% MP).",
    "You have a considerably increased reservoir of magic (+20% MP).",
@@ -916,7 +970,7 @@
 },
 
 { MUT_LOW_MAGIC,                      9,  3,  true, false, false,
-  NULL,
+  "low mp",
 
   {"Your magical capacity is low (-10% MP).",
    "Your magical capacity is very low (-20% MP).",
@@ -980,7 +1034,7 @@
 },
 
 { MUT_ICEMAIL,                        0,  1, false, false, false,
-  NULL,
+  "icemail",
 
   {"A meltable icy envelope protects you from harm and freezing vapours (AC +", "", ""},
   {"An icy envelope takes form around you.", "", ""},
@@ -1136,7 +1190,7 @@
 
 // Jiyva only mutations
 { MUT_GELATINOUS_BODY,                0,  3, false,  true,  true,
-  NULL,
+  "gelatinous body",
 
   {"Your rubbery body absorbs attacks (AC +1).",
    "Your pliable body absorbs attacks (AC +1, EV +1).",
@@ -1154,7 +1208,7 @@
 },
 
 { MUT_EYEBALLS,                       0,  3, false,  true,  true,
-  NULL,
+  "eyeballs",
 
   {"Your body is partially covered in golden eyeballs (Acc +3).",
    "Your body is mostly covered in golden eyeballs (Acc +5).",
@@ -1229,9 +1283,29 @@
   "acidic bite"
 },
 
+{ MUT_ANTIMAGIC_BITE,                 0,  1, false,  true,  true,
+  "antimagic bite",
+
+  {"Your bite disrupts and absorbs the magic of your enemies.", "", ""},
+  {"You feel a sudden thirst for magic.", "", ""},
+  {"Your magical appetite wanes.", "", ""},
+
+  "antimagic bite"
+},
+
+{ MUT_NO_DEVICE_HEAL,                 0,  1, true,  false,  false,
+  "no device heal",
+
+  {"Potions and wands cannot restore your HP.", "", ""},
+  {"Your system rejects artificial healing.", "", ""},
+  {"Your system accepts artificial healing.", "", ""},
+
+  "no device heal"
+},
+
 // Scale mutations
 { MUT_DISTORTION_FIELD,               2,  3, false, false, false,
-  NULL,
+  "repulsion field",
 
   {"You are surrounded by a mild repulsion field (EV +2).",
    "You are surrounded by a moderate repulsion field (EV +3).",
@@ -1249,7 +1323,7 @@
 },
 
 { MUT_ICY_BLUE_SCALES,                2,  3, false,  true,  true,
-  NULL,
+  "icy blue scales",
 
   {"You are partially covered in icy blue scales (AC +1).",
    "You are mostly covered in icy blue scales (AC +3, EV -1).",
@@ -1267,7 +1341,7 @@
 },
 
 { MUT_IRIDESCENT_SCALES,              2,  3, false,  true,  true,
-  NULL,
+  "iridescent scales",
 
   {"You are partially covered in iridescent scales (AC +4).",
    "You are mostly covered in iridescent scales (AC +6).",
@@ -1285,7 +1359,7 @@
 },
 
 { MUT_LARGE_BONE_PLATES,              2,  3, false,  true,  true,
-  NULL,
+  "large bone plates",
 
   {"You are partially covered in large bone plates (AC +2, SH +2).",
    "You are mostly covered in large bone plates (AC +3, SH +4).",
@@ -1303,7 +1377,7 @@
 },
 
 { MUT_MOLTEN_SCALES,                  2,  3, false,  true,  true,
-  NULL,
+  "molten scales",
 
   {"You are partially covered in molten scales (AC +1).",
    "You are mostly covered in molten scales (AC +3, EV -1).",
@@ -1321,7 +1395,7 @@
 },
 
 { MUT_ROUGH_BLACK_SCALES,             2,  3, false,  true,  true,
-  NULL,
+  "rough black scales",
 
   {"You are partially covered in rough black scales (AC +4, Dex -1).",
    "You are mostly covered in rough black scales (AC +7, Dex -2).",
@@ -1339,7 +1413,7 @@
 },
 
 { MUT_RUGGED_BROWN_SCALES,            2,  3, false,  true,  true,
-  NULL,
+  "rugged brown scales",
 
   {"You are partially covered in rugged brown scales (AC +1, +3% HP).",
    "You are mostly covered in rugged brown scales (AC +2, +5% HP).",
@@ -1357,7 +1431,7 @@
 },
 
 { MUT_SLIMY_GREEN_SCALES,             2,  3, false,  true,  true,
-  NULL,
+  "slimy green scales",
 
   {"You are partially covered in slimy green scales (AC +2).",
    "You are mostly covered in slimy green scales (AC +3).",
@@ -1375,7 +1449,7 @@
 },
 
 { MUT_THIN_METALLIC_SCALES,           2,  3, false,  true,  true,
-  NULL,
+  "thin metallic scales",
 
   {"You are partially covered in thin metallic scales (AC +2).",
    "You are mostly covered in thin metallic scales (AC +3).",
@@ -1393,11 +1467,11 @@
 },
 
 { MUT_THIN_SKELETAL_STRUCTURE,        2,  3, false,  true, false,
-  NULL,
+  "thin skeletal structure",
 
-  {"You have a somewhat thin skeletal structure (Dex +2, Stlth).",
-   "You have a moderately thin skeletal structure (Dex +4, Str -1, Stlth).",
-   "You have an unnaturally thin skeletal structure (Dex +6, Str -2, Stlth)."},
+  {"You have a somewhat thin skeletal structure (Dex +2, Stealth).",
+   "You have a moderately thin skeletal structure (Dex +4, Stealth+).",
+   "You have an unnaturally thin skeletal structure (Dex +6, Stealth++)."},
 
   {"Your bones become slightly less dense.",
    "Your bones become somewhat less dense.",
@@ -1411,7 +1485,7 @@
 },
 
 { MUT_YELLOW_SCALES,                  2,  3, false,  true,  true,
-  NULL,
+  "yellow scales",
 
   {"You are partially covered in yellow scales (AC +2).",
    "You are mostly covered in yellow scales (AC +3).",
@@ -1483,7 +1557,7 @@
 },
 
 { MUT_JELLY_GROWTH,                       0,  1, false,  true, true,
-  "a jelly is attached to you",
+  "jelly sensing items",
 
   {"You have a small jelly attached to you that senses nearby items.", "", ""},
   {"Your body partially splits into a small jelly.", "", ""},
@@ -1493,7 +1567,7 @@
 },
 
 { MUT_JELLY_MISSILE,                       0,  1, false,  true, true,
-  "absorbing missiles",
+  "jelly absorbing missiles",
 
   {"You have a small jelly attached to you that may absorb incoming projectiles.", "", ""},
   {"Your body partially splits into a small jelly.", "", ""},
@@ -1532,7 +1606,24 @@
 
   "cling"
 },
-#endif
+
+{ MUT_EXOSKELETON,                         0,  2, false,  true,  true,
+  "exoskeleton",
+
+  {"Your body is surrounded by an exoskeleton. (buggy)",
+   "Your body is surrounded by a tough exoskeleton. (buggy)",
+   ""},
+
+  {"Your exoskeleton hardens.",
+   "Your exoskeleton becomes even harder.",
+   ""},
+
+  {"Your exoskeleton softens.",
+   "Your exoskeleton softens.",
+   ""},
+
+  "exoskeleton"
+},
 
 { MUT_FUMES,            0,  2, false, false, false,
   "fuming",
@@ -1542,4 +1633,15 @@
   {"You stop fuming.", "You fume less.", ""},
 
   "fumes"
+},
+#endif
+
+{ MUT_BLACK_MARK,                  0,  1, false, false, false,
+  "black mark",
+
+  {"Your melee attacks sometimes drain vitality from your foes.", "", ""},
+  {"An ominous black mark forms on your body.", "", ""},
+  {"", "", ""},
+
+  "black mark"
 }
