@@ -53,15 +53,36 @@ static uint8_t _random_potion_description()
 // Determine starting depths of branches.
 void initialise_branch_depths()
 {
-    root_branch = BRANCH_MAIN_DUNGEON;
+    root_branch = BRANCH_DUNGEON;
 
-    for (int branch = BRANCH_ECUMENICAL_TEMPLE; branch < NUM_BRANCHES; ++branch)
+    for (int br = 0; br < NUM_BRANCHES; ++br)
+        brentry[br].clear();
+
+    if (crawl_state.game_is_sprint())
+    {
+        brdepth.init(-1);
+        brdepth[BRANCH_DUNGEON] = 1;
+        return;
+    }
+
+    if (crawl_state.game_is_zotdef())
+    {
+        root_branch = BRANCH_ZOT;
+        brdepth.init(-1);
+        brdepth[BRANCH_ZOT] = 1;
+        brdepth[BRANCH_BAZAAR] = 1;
+        return;
+    }
+
+    for (int branch = 0; branch < NUM_BRANCHES; ++branch)
     {
         const Branch *b = &branches[branch];
-        if (branch_is_unfinished(b->id))
-            startdepth[branch] = -1;
-        else
-            startdepth[branch] = random_range(b->mindepth, b->maxdepth);
+        ASSERT(b->id == branch);
+        if (!branch_is_unfinished(b->id) && b->parent_branch != NUM_BRANCHES)
+        {
+            brentry[branch] = level_id(b->parent_branch,
+                                       random_range(b->mindepth, b->maxdepth));
+        }
     }
 
     // You will get one of Shoals/Swamp and one of Spider/Snake.
@@ -69,32 +90,13 @@ void initialise_branch_depths()
     branch_type disabled_branch[] =
     {
         random_choose(BRANCH_SWAMP, BRANCH_SHOALS, -1),
-        random_choose(BRANCH_SNAKE_PIT, BRANCH_SPIDER_NEST, -1),
-        random_choose(BRANCH_CRYPT, BRANCH_FOREST, -1),
+        random_choose(BRANCH_SNAKE, BRANCH_SPIDER, -1),
     };
-    if (Version::ReleaseType != VER_ALPHA)
-        disabled_branch[2] = BRANCH_FOREST;
 
     for (unsigned int i = 0; i < ARRAYSZ(disabled_branch); ++i)
     {
         dprf("Disabling branch: %s", branches[disabled_branch[i]].shortname);
-        startdepth[disabled_branch[i]] = -1;
-    }
-
-    if (crawl_state.game_is_sprint())
-    {
-        brdepth.init(-1);
-        brdepth[BRANCH_MAIN_DUNGEON] = 1;
-        return;
-    }
-
-    if (crawl_state.game_is_zotdef())
-    {
-        root_branch = BRANCH_HALL_OF_ZOT;
-        brdepth.init(-1);
-        brdepth[BRANCH_HALL_OF_ZOT] = 1;
-        brdepth[BRANCH_BAZAAR] = 1;
-        return;
+        brentry[disabled_branch[i]].clear();
     }
 
     for (int i = 0; i < NUM_BRANCHES; i++)
@@ -129,7 +131,7 @@ void initialise_temples()
 {
     //////////////////////////////////////////
     // First determine main temple map to use.
-    level_id ecumenical(BRANCH_ECUMENICAL_TEMPLE, 1);
+    level_id ecumenical(BRANCH_TEMPLE, 1);
 
     map_def *main_temple = NULL;
     for (int i = 0; i < 10; i++)
@@ -230,7 +232,7 @@ void initialise_temples()
                  map != maps.end(); map++)
             {
                 // XXX: this should handle level depth better
-                chance += (*map)->weight(level_id(BRANCH_MAIN_DUNGEON,
+                chance += (*map)->weight(level_id(BRANCH_DUNGEON,
                                                   MAX_OVERFLOW_LEVEL));
             }
             overflow_weights[i] = chance;
@@ -280,7 +282,7 @@ multi_overflow:
             for (mapref_vector::iterator map = maps.begin(); map != maps.end();
                  map++)
             {
-                chance += (*map)->weight(level_id(BRANCH_MAIN_DUNGEON,
+                chance += (*map)->weight(level_id(BRANCH_DUNGEON,
                                                   MAX_OVERFLOW_LEVEL));
             }
             if (!x_chance_in_y(chance, overflow_weights[num] + chance))
