@@ -13,7 +13,7 @@
 #include "species.h"
 #include "jobs.h"
 
-char_choice_restriction job_allowed(species_type speci, job_type job)
+char_choice_restriction species_allowed(job_type job, species_type speci)
 {
     if (!is_species_valid_choice(speci) || !is_job_valid_choice(job))
         return CC_BANNED;
@@ -23,21 +23,15 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
     case JOB_FIGHTER:
         switch (speci)
         {
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_MERFOLK:
-        case SP_KOBOLD:
-        case SP_SPRIGGAN:
-        case SP_NAGA:
-        case SP_OGRE:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_DWARF:
+        case SP_HILL_ORC:
+        case SP_TROLL:
+        case SP_MINOTAUR:
+        case SP_GARGOYLE:
+        case SP_CENTAUR:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_GLADIATOR:
@@ -45,42 +39,30 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_FELID:
             return CC_BANNED;
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_SPRIGGAN:
-        case SP_NAGA:
-        case SP_OGRE:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_DWARF:
+        case SP_HILL_ORC:
+        case SP_MERFOLK:
+        case SP_MINOTAUR:
+        case SP_GARGOYLE:
+        case SP_CENTAUR:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_MONK:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_SPRIGGAN:
-        case SP_NAGA:
-        case SP_OGRE:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_OCTOPODE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_HILL_ORC:
+        case SP_TROLL:
+        case SP_MINOTAUR:
+        case SP_GARGOYLE:
+        case SP_GHOUL:
+        case SP_MERFOLK:
+        case SP_VAMPIRE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_BERSERKER:
@@ -88,18 +70,16 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_DEMIGOD:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_SPRIGGAN:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_HILL_ORC:
+        case SP_HALFLING:
+        case SP_OGRE:
+        case SP_MERFOLK:
+        case SP_MINOTAUR:
+        case SP_GARGOYLE:
+        case SP_DEMONSPAWN:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_CHAOS_KNIGHT:
@@ -107,6 +87,14 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_DEMIGOD:
             return CC_BANNED;
+        case SP_HILL_ORC:
+        case SP_TROLL:
+        case SP_CENTAUR:
+        case SP_MERFOLK:
+        case SP_MINOTAUR:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMONSPAWN:
+            return CC_UNRESTRICTED;
         default:
             return CC_RESTRICTED;
         }
@@ -117,22 +105,15 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case SP_DEMIGOD:
         case SP_GARGOYLE:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_SPRIGGAN:
-        case SP_OGRE:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_FELID:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_HUMAN:
+        case SP_HILL_ORC:
+        case SP_TROLL:
+        case SP_MERFOLK:
+        case SP_MINOTAUR:
+        case SP_DEMONSPAWN:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ABYSSAL_KNIGHT:
@@ -140,24 +121,15 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_DEMIGOD:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_DEEP_DWARF:
-        case SP_KOBOLD:
-        case SP_OGRE:
+        case SP_HILL_ORC:
+        case SP_SPRIGGAN:
         case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_FELID:
-        case SP_OCTOPODE:
-        case SP_GARGOYLE:
-            return CC_RESTRICTED;
-        default:
+        case SP_MERFOLK:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMONSPAWN:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_HEALER:
@@ -169,169 +141,114 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case SP_GHOUL:
         case SP_VAMPIRE:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_HALFLING:
-        case SP_SPRIGGAN:
-        case SP_OGRE:
+        case SP_NAGA:
+        case SP_DEEP_DWARF:
+        case SP_HILL_ORC:
         case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_FELID:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_BASE_DRACONIAN:
+        case SP_GARGOYLE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_SKALD:
         switch (speci)
         {
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_DEEP_DWARF:
-        case SP_KOBOLD:
-        case SP_SPRIGGAN:
+        case SP_HIGH_ELF:
+        case SP_HALFLING:
         case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_FELID:
-        case SP_OCTOPODE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_MERFOLK:
+        case SP_BASE_DRACONIAN:
+        case SP_VAMPIRE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_WARPER:
         switch (speci)
         {
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_KOBOLD:
+        case SP_HALFLING:
+        case SP_HIGH_ELF:
+        case SP_DEEP_DWARF:
+        case SP_SPRIGGAN:
         case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
         case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_WIZARD:
         switch (speci)
         {
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_CENTAUR:
-        case SP_MINOTAUR:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_FELID:
-            return CC_RESTRICTED;
-        default:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_NAGA:
+        case SP_BASE_DRACONIAN:
+        case SP_OCTOPODE:
+        case SP_HUMAN:
+        case SP_MUMMY:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_CONJURER:
         switch (speci)
         {
-        case SP_SLUDGE_ELF:
-        case SP_DEEP_DWARF:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-            return CC_RESTRICTED;
-        default:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_NAGA:
+        case SP_TENGU:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMIGOD:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ENCHANTER:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_DEMONSPAWN:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_FELID:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_ELF:
+        case SP_HALFLING:
+        case SP_KOBOLD:
+        case SP_SPRIGGAN:
+        case SP_NAGA:
+        case SP_VAMPIRE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_SUMMONER:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_DEEP_DWARF:
+        case SP_DEEP_ELF:
         case SP_HILL_ORC:
-        case SP_LAVA_ORC:
+        case SP_VINE_STALKER:
         case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_SPRIGGAN:
-        case SP_NAGA:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_MUMMY:
-        case SP_GHOUL:
-            return CC_RESTRICTED;
-        default:
+        case SP_TENGU:
+        case SP_VAMPIRE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_NECROMANCER:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_DJINNI:
-        case SP_GARGOYLE:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_ELF:
+        case SP_DEEP_DWARF:
+        case SP_HILL_ORC:
+        case SP_DEMONSPAWN:
+        case SP_MUMMY:
+        case SP_VAMPIRE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_TRANSMUTER:
@@ -340,124 +257,89 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         case SP_MUMMY:
         case SP_GHOUL:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-        case SP_OGRE:
+        case SP_NAGA:
+        case SP_MERFOLK:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMIGOD:
+        case SP_DEMONSPAWN:
         case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_VAMPIRE:
-        case SP_DJINNI:
-        case SP_GARGOYLE:
-            return CC_RESTRICTED;
-        default:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_FIRE_ELEMENTALIST:
         switch (speci)
         {
-        case SP_DEEP_DWARF:
-        case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_SPRIGGAN:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_FELID:
-            return CC_RESTRICTED;
-        default:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_HILL_ORC:
+        case SP_NAGA:
+        case SP_TENGU:
+        case SP_DEMIGOD:
+        case SP_GARGOYLE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ICE_ELEMENTALIST:
         switch (speci)
         {
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_HALFLING:
-        case SP_SPRIGGAN:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_FELID:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_MERFOLK:
+        case SP_NAGA:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMIGOD:
+        case SP_GARGOYLE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_AIR_ELEMENTALIST:
         switch (speci)
         {
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_FELID:
-        case SP_GARGOYLE:
-            return CC_RESTRICTED;
-        default:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_TENGU:
+        case SP_BASE_DRACONIAN:
+        case SP_NAGA:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_EARTH_ELEMENTALIST:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_MERFOLK:
-        case SP_HALFLING:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_MUMMY:
-        case SP_VAMPIRE:
-        case SP_FELID:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_ELF:
+        case SP_DEEP_DWARF:
+        case SP_SPRIGGAN:
+        case SP_GARGOYLE:
+        case SP_DEMIGOD:
+        case SP_GHOUL:
+        case SP_OCTOPODE:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_VENOM_MAGE:
         switch (speci)
         {
-        case SP_HIGH_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_HALFLING:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_ELF:
+        case SP_SPRIGGAN:
+        case SP_NAGA:
+        case SP_MERFOLK:
+        case SP_TENGU:
+        case SP_FELID:
+        case SP_DEMONSPAWN:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ASSASSIN:
@@ -465,26 +347,15 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_FELID:
             return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_NAGA:
-        case SP_CENTAUR:
-        case SP_OGRE:
         case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
+        case SP_HALFLING:
+        case SP_SPRIGGAN:
         case SP_DEMONSPAWN:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_OCTOPODE:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        default:
+        case SP_VAMPIRE:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_HUNTER:
@@ -492,46 +363,30 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_FELID:
             return CC_BANNED;
-        case SP_DEEP_ELF:
+        case SP_HIGH_ELF:
         case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_SPRIGGAN:
-        case SP_NAGA:
-        case SP_TENGU:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_HALFLING:
+        case SP_KOBOLD:
+        case SP_OGRE:
+        case SP_TROLL:
+        case SP_CENTAUR:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ARTIFICER:
         switch (speci)
         {
-        case SP_FELID:
-            return CC_BANNED;
-        case SP_HIGH_ELF:
-        case SP_DEEP_ELF:
-        case SP_SLUDGE_ELF:
-        case SP_NAGA:
-        case SP_CENTAUR:
-        case SP_OGRE:
-        case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_DEEP_DWARF:
+        case SP_HALFLING:
+        case SP_KOBOLD:
+        case SP_SPRIGGAN:
+        case SP_BASE_DRACONIAN:
+        case SP_DEMONSPAWN:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
 
     case JOB_ARCANE_MARKSMAN:
@@ -539,40 +394,461 @@ char_choice_restriction job_allowed(species_type speci, job_type job)
         {
         case SP_FELID:
             return CC_BANNED;
-        case SP_DEEP_DWARF:
-        case SP_HILL_ORC:
-        case SP_LAVA_ORC:
-        case SP_MERFOLK:
-        case SP_NAGA:
+        case SP_HIGH_ELF:
+        case SP_DEEP_ELF:
+        case SP_KOBOLD:
+        case SP_SPRIGGAN:
         case SP_TROLL:
-        case SP_MINOTAUR:
-        case SP_TENGU:
-        case SP_BASE_DRACONIAN:
-        case SP_DEMIGOD:
-        case SP_MUMMY:
-        case SP_GHOUL:
-        case SP_OCTOPODE:
-            return CC_RESTRICTED;
-        default:
+        case SP_CENTAUR:
             return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
         }
-
     case JOB_WANDERER:
-        return CC_RESTRICTED;
+        switch (speci)
+        {
+        case SP_HILL_ORC:
+        case SP_SPRIGGAN:
+        case SP_CENTAUR:
+        case SP_MERFOLK:
+        case SP_BASE_DRACONIAN:
+        case SP_HUMAN:
+        case SP_DEMONSPAWN:
+            return CC_UNRESTRICTED;
+        default:
+            return CC_RESTRICTED;
+        }
 
     default:
         return CC_BANNED;
     }
 }
 
-bool is_good_combination(species_type spc, job_type job, bool good)
+char_choice_restriction job_allowed(species_type speci, job_type job)
 {
-    const char_choice_restriction restrict = job_allowed(spc, job);
+    if (!is_species_valid_choice(speci) || !is_job_valid_choice(job))
+        return CC_BANNED;
+
+    switch (speci)
+    {
+    case SP_HUMAN:
+        switch (job)
+        {
+            case JOB_BERSERKER:
+            case JOB_DEATH_KNIGHT:
+            case JOB_CONJURER:
+            case JOB_NECROMANCER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_HIGH_ELF:
+        switch (job)
+        {
+            case JOB_HUNTER:
+            case JOB_SKALD:
+            case JOB_WIZARD:
+            case JOB_CONJURER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_AIR_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_DEEP_ELF:
+        switch (job)
+        {
+            case JOB_WIZARD:
+            case JOB_CONJURER:
+            case JOB_SUMMONER:
+            case JOB_NECROMANCER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_HALFLING:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_HUNTER:
+            case JOB_ASSASSIN:
+            case JOB_BERSERKER:
+            case JOB_ENCHANTER:
+            case JOB_AIR_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_HILL_ORC:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_GLADIATOR:
+            case JOB_BERSERKER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_DEATH_KNIGHT:
+            case JOB_HEALER:
+            case JOB_NECROMANCER:
+            case JOB_FIRE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_KOBOLD:
+        switch (job)
+        {
+            case JOB_HUNTER:
+            case JOB_ASSASSIN:
+            case JOB_BERSERKER:
+            case JOB_ARCANE_MARKSMAN:
+            case JOB_ENCHANTER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_MUMMY:
+        switch (job)
+        {
+            case JOB_HEALER:
+            case JOB_TRANSMUTER:
+                return CC_BANNED;
+            case JOB_WIZARD:
+            case JOB_CONJURER:
+            case JOB_NECROMANCER:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_SUMMONER:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_NAGA:
+        switch (job)
+        {
+            case JOB_BERSERKER:
+            case JOB_TRANSMUTER:
+            case JOB_ENCHANTER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_WARPER:
+            case JOB_WIZARD:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_OGRE:
+        switch (job)
+        {
+            case JOB_HUNTER:
+            case JOB_BERSERKER:
+            case JOB_ARCANE_MARKSMAN:
+            case JOB_WIZARD:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_DEATH_KNIGHT:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_TROLL:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_MONK:
+            case JOB_HUNTER:
+            case JOB_BERSERKER:
+            case JOB_WARPER:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_WIZARD:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_BASE_DRACONIAN:
+        switch (job)
+        {
+            case JOB_BERSERKER:
+            case JOB_TRANSMUTER:
+            case JOB_CONJURER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_CENTAUR:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_GLADIATOR:
+            case JOB_HUNTER:
+            case JOB_DEATH_KNIGHT:
+            case JOB_WARPER:
+            case JOB_ARCANE_MARKSMAN:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_DEMIGOD:
+        switch (job)
+        {
+            case JOB_BERSERKER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_CHAOS_KNIGHT:
+            case JOB_DEATH_KNIGHT:
+            case JOB_HEALER:
+                return CC_BANNED;
+            case JOB_TRANSMUTER:
+            case JOB_CONJURER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_SPRIGGAN:
+        switch (job)
+        {
+            case JOB_ASSASSIN:
+            case JOB_ARTIFICER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_WARPER:
+            case JOB_ENCHANTER:
+            case JOB_CONJURER:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_MINOTAUR:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_GLADIATOR:
+            case JOB_MONK:
+            case JOB_HUNTER:
+            case JOB_BERSERKER:
+            case JOB_DEATH_KNIGHT:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_DEMONSPAWN:
+        switch (job)
+        {
+            case JOB_HEALER:
+                return CC_BANNED;
+            case JOB_GLADIATOR:
+            case JOB_BERSERKER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_WIZARD:
+            case JOB_NECROMANCER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_GHOUL:
+        switch (job)
+        {
+            case JOB_HEALER:
+            case JOB_TRANSMUTER:
+                return CC_BANNED;
+            case JOB_WARPER:
+            case JOB_GLADIATOR:
+            case JOB_MONK:
+            case JOB_NECROMANCER:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_TENGU:
+        switch (job)
+        {
+            case JOB_BERSERKER:
+            case JOB_WIZARD:
+            case JOB_CONJURER:
+            case JOB_SUMMONER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_MERFOLK:
+        switch (job)
+        {
+            case JOB_GLADIATOR:
+            case JOB_BERSERKER:
+            case JOB_DEATH_KNIGHT:
+            case JOB_SKALD:
+            case JOB_TRANSMUTER:
+            case JOB_SUMMONER:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_VAMPIRE:
+        switch (job)
+        {
+            case JOB_HEALER:
+                return CC_BANNED;
+            case JOB_MONK:
+            case JOB_ASSASSIN:
+            case JOB_ENCHANTER:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_NECROMANCER:
+            case JOB_ICE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_DEEP_DWARF:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_HUNTER:
+            case JOB_BERSERKER:
+            case JOB_HEALER:
+            case JOB_NECROMANCER:
+            case JOB_EARTH_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_FELID:
+        switch (job)
+        {
+            case JOB_GLADIATOR:
+            case JOB_HUNTER:
+            case JOB_ASSASSIN:
+            case JOB_ARCANE_MARKSMAN:
+                return CC_BANNED;
+            case JOB_BERSERKER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_TRANSMUTER:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_CONJURER:
+            case JOB_SUMMONER:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_OCTOPODE:
+        switch (job)
+        {
+            case JOB_TRANSMUTER:
+            case JOB_WIZARD:
+            case JOB_CONJURER:
+            case JOB_ASSASSIN:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_LAVA_ORC:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_GLADIATOR:
+            case JOB_BERSERKER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_DEATH_KNIGHT:
+            case JOB_HEALER:
+            case JOB_NECROMANCER:
+            case JOB_FIRE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_GARGOYLE:
+        switch (job)
+        {
+            case JOB_DEATH_KNIGHT:
+                return CC_BANNED;
+            case JOB_FIGHTER:
+            case JOB_GLADIATOR:
+            case JOB_MONK:
+            case JOB_BERSERKER:
+            case JOB_FIRE_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_FORMICID:
+        switch (job)
+        {
+            case JOB_FIGHTER:
+            case JOB_HUNTER:
+            case JOB_ABYSSAL_KNIGHT:
+            case JOB_HEALER:
+            case JOB_ARCANE_MARKSMAN:
+            case JOB_EARTH_ELEMENTALIST:
+            case JOB_VENOM_MAGE:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    case SP_VINE_STALKER:
+        switch (job)
+        {
+            case JOB_MONK:
+            case JOB_ASSASSIN:
+            case JOB_ENCHANTER:
+            case JOB_CONJURER:
+            case JOB_NECROMANCER:
+            case JOB_AIR_ELEMENTALIST:
+            case JOB_ICE_ELEMENTALIST:
+                return CC_UNRESTRICTED;
+            default:
+                return CC_RESTRICTED;
+        }
+    default:
+        return CC_BANNED;
+    }
+}
+
+bool is_good_combination(species_type spc, job_type job, bool species_first,
+                         bool good)
+{
+    const char_choice_restriction restrict =
+        species_first ? job_allowed(spc, job) : species_allowed(job, spc);
 
     if (good)
-        return (restrict == CC_UNRESTRICTED);
+        return restrict == CC_UNRESTRICTED;
 
-    return (restrict != CC_BANNED);
+    return restrict != CC_BANNED;
 }
 
 // Is the given god restricted for the character defined by ng?
@@ -592,6 +868,7 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         return CC_BANNED;
 
     case WPN_SHORT_SWORD:
+    case WPN_CUTLASS:
         switch (ng.species)
         {
         case SP_NAGA:
@@ -602,24 +879,22 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             // else fall through
         case SP_HIGH_ELF:
         case SP_DEEP_ELF:
-            // Sludge elves have bad aptitudes with short swords (-1), but
-            // are still better with them than any other starting weapon.
-        case SP_SLUDGE_ELF:
         case SP_HALFLING:
         case SP_KOBOLD:
         case SP_SPRIGGAN:
+        case SP_FORMICID:
             return CC_UNRESTRICTED;
 
         default:
             return CC_RESTRICTED;
         }
 
-        // Maces and hand axes usually share the same restrictions.
+        // Maces and axes usually share the same restrictions.
     case WPN_MACE:
+    case WPN_FLAIL:
         if (ng.species == SP_TROLL
             || ng.species == SP_OGRE
-            || ng.species == SP_GARGOYLE
-            || ng.species == SP_DJINNI)
+            || ng.species == SP_GARGOYLE)
         {
             return CC_UNRESTRICTED;
         }
@@ -627,6 +902,7 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             return CC_RESTRICTED;
         // else fall-through
     case WPN_HAND_AXE:
+    case WPN_WAR_AXE:
         switch (ng.species)
         {
         case SP_HUMAN:
@@ -643,6 +919,8 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_VAMPIRE:
         case SP_OCTOPODE:
         case SP_BASE_DRACONIAN:
+        case SP_FORMICID:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
 
         default:
@@ -650,6 +928,7 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         }
 
     case WPN_SPEAR:
+    case WPN_TRIDENT:
         switch (ng.species)
         {
         case SP_HUMAN:
@@ -657,7 +936,6 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_LAVA_ORC:
         case SP_MERFOLK:
         case SP_NAGA:
-        case SP_OGRE:
         case SP_CENTAUR:
         case SP_MINOTAUR:
         case SP_TENGU:
@@ -666,22 +944,20 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_MUMMY:
         case SP_OCTOPODE:
         case SP_BASE_DRACONIAN:
-        case SP_DJINNI:
+        case SP_FORMICID:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
 
         case SP_SPRIGGAN:
-        case SP_HALFLING:
-        case SP_KOBOLD:
-            // Can't use them with a shield, terrible upgrade path;
-            // jobs with other benefits like Warper, Skald or godlies
-            // are better with two-handers.
-            if (ng.job == JOB_FIGHTER || ng.job == JOB_GLADIATOR)
+            // Can't use them with a shield.
+            if (ng.job == JOB_FIGHTER)
                 return CC_BANNED;
 
         default:
             return CC_RESTRICTED;
         }
     case WPN_FALCHION:
+    case WPN_LONG_SWORD:
         switch (ng.species)
         {
         case SP_HUMAN:
@@ -692,7 +968,6 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_CENTAUR:
         case SP_MINOTAUR:
         case SP_HIGH_ELF:
-        case SP_SLUDGE_ELF:
         case SP_DEEP_DWARF:
         case SP_TENGU:
         case SP_DEMIGOD:
@@ -701,35 +976,14 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_VAMPIRE:
         case SP_OCTOPODE:
         case SP_BASE_DRACONIAN:
-        case SP_DJINNI:
+        case SP_FORMICID:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
 
         default:
             return CC_RESTRICTED;
         }
 
-    case WPN_TRIDENT:
-        if (ng.job != JOB_FIGHTER && ng.job != JOB_GLADIATOR
-            || species_size(ng.species, PSIZE_BODY) < SIZE_MEDIUM)
-        {
-            return CC_BANNED;
-        }
-
-        // Tridents are strictly better than spears, so unrestrict them
-        // for some species whose Polearm aptitudes are not too bad.
-        switch (ng.species)
-        {
-        case SP_DEEP_DWARF:
-        case SP_GHOUL:
-        case SP_VAMPIRE:
-            return CC_UNRESTRICTED;
-        default:
-            break;
-        }
-
-        // Both are polearms, right?
-        return weapon_restriction(WPN_SPEAR, ng);
-
     case WPN_QUARTERSTAFF:
         if (ng.job != JOB_GLADIATOR)
             return CC_BANNED;
@@ -746,10 +1000,10 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_MINOTAUR:
         case SP_MUMMY:
         case SP_OCTOPODE:
-        case SP_SLUDGE_ELF:
         case SP_BASE_DRACONIAN:
-        case SP_DJINNI:
+        case SP_FORMICID:
         case SP_GARGOYLE:
+        case SP_VINE_STALKER:
             return CC_UNRESTRICTED;
 
         default:
@@ -768,7 +1022,6 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_LAVA_ORC:
         case SP_SPRIGGAN:
         case SP_TROLL:
-        case SP_SLUDGE_ELF:
             return CC_RESTRICTED;
         case SP_FELID:
             return CC_BANNED;
@@ -786,8 +1039,8 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         case SP_TROLL:
         case SP_HILL_ORC:
         case SP_LAVA_ORC:
-        case SP_SLUDGE_ELF:
-        case SP_DJINNI:
+        case SP_FORMICID:
+        case SP_SPRIGGAN:
             return CC_RESTRICTED;
         case SP_FELID:
             return CC_BANNED;
@@ -800,7 +1053,6 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
         {
         case SP_DEEP_ELF:
         case SP_HIGH_ELF:
-        case SP_SLUDGE_ELF:
         case SP_TENGU:
         case SP_MERFOLK:
         case SP_OGRE:
@@ -815,34 +1067,10 @@ char_choice_restriction weapon_restriction(weapon_type wpn,
             return CC_UNRESTRICTED;
         }
 
-    case WPN_ROCKS:
-        switch (ng.species)
-        {
-        case SP_OGRE:
-        case SP_TROLL:
-            return CC_UNRESTRICTED;
-        default:
-            return CC_BANNED;
-        }
-
-    case WPN_JAVELINS:
+    case WPN_THROWN:
         switch (ng.species)
         {
         case SP_DEEP_DWARF:
-        case SP_DJINNI:
-            return CC_RESTRICTED;
-        case SP_SPRIGGAN:
-        case SP_FELID:
-            return CC_BANNED;
-        default:
-            return CC_UNRESTRICTED;
-        }
-
-    case WPN_DARTS:
-        switch (ng.species)
-        {
-        case SP_DEEP_DWARF:
-        case SP_DJINNI:
             return CC_RESTRICTED;
         case SP_FELID:
             return CC_BANNED;
