@@ -409,7 +409,7 @@ static void _good_potion_or_scroll(int & slot)
 
     case 3:
         you.inv[slot].base_type = OBJ_POTIONS;
-        you.inv[slot].sub_type  = POT_SPEED;
+        you.inv[slot].sub_type  = POT_HASTE;
         break;
 
     case 4:
@@ -421,29 +421,6 @@ static void _good_potion_or_scroll(int & slot)
     slot++;
 }
 
-// Make n random attempts at identifying curing or teleportation.
-static void _curing_or_teleport(int n)
-{
-    int temp_rand = 2;
-
-    // No potions for mummies.
-    if (you.is_undead == US_UNDEAD)
-        temp_rand--;
-
-    for (int i = 0; i < n; ++i)
-    {
-        switch (random2(temp_rand))
-        {
-        case 0:
-            set_ident_type(OBJ_SCROLLS, SCR_TELEPORTATION, ID_KNOWN_TYPE);
-            break;
-        case 1:
-            set_ident_type(OBJ_POTIONS, POT_CURING, ID_KNOWN_TYPE);
-            break;
-        }
-    }
-}
-
 // Create a random wand in the inventory.
 static void _wanderer_random_evokable(int & slot)
 {
@@ -556,9 +533,8 @@ static void _wanderer_good_equipment(skill_type & skill, int & slot)
     case SK_UNARMED_COMBAT:
     case SK_INVOCATIONS:
     {
-        // Random consumables: 2x attempts to ID curing/teleportation
-        // 1 good potion/scroll.
-        _curing_or_teleport(2);
+        // 2 random good potions/scrolls
+        _good_potion_or_scroll(slot);
         _good_potion_or_scroll(slot);
         break;
     }
@@ -646,7 +622,7 @@ static void _wanderer_decent_equipment(skill_type & skill,
     // If we already gave an item for this type, just give the player
     // a consumable.
     if ((skill == SK_DODGING || skill == SK_STEALTH)
-        && gift_skills.find(SK_ARMOUR) != gift_skills.end())
+        && gift_skills.count(SK_ARMOUR))
     {
         skill = SK_NONE;
     }
@@ -685,7 +661,7 @@ static void _wanderer_decent_equipment(skill_type & skill,
 
     // Don't give a gift from the same skill twice; just default to
     // a curing potion/teleportation scroll.
-    if (gift_skills.find(skill) != gift_skills.end())
+    if (gift_skills.count(skill))
         skill = SK_NONE;
 
     switch ((int)skill)
@@ -738,7 +714,6 @@ static void _wanderer_decent_equipment(skill_type & skill,
     case SK_INVOCATIONS:
     case SK_EVOCATIONS:
     case SK_NONE:
-        _curing_or_teleport(1);
         break;
     }
 }
@@ -865,14 +840,6 @@ void create_wanderer(void)
     you.skills[util_skill1]++;
     you.skills[util_skill2]++;
 
-    // Caster types maybe need more MP?
-    int mp_adjust = 0;
-    if (primary_role == STAT_INT)
-        mp_adjust++;
-    if (secondary_role == STAT_INT)
-        mp_adjust++;
-    you.mp_max_perm += mp_adjust;
-
     // Keep track of what skills we got items from, mostly to prevent
     // giving a good and then a normal version of the same weapon.
     set<skill_type> gift_skills;
