@@ -51,6 +51,7 @@ enum NOTE_TYPES
     NOTE_NAMED_ALLY,            /* needs: ally name (string) */
     NOTE_ALLY_DEATH,            /* needs: ally name (string) */
     NOTE_FEAT_MIMIC,            /* needs: mimiced feature (string) */
+    NOTE_OFFERED_SPELL,         /* needs: spell idx */
     NOTE_NUM_TYPES
 };
 
@@ -61,8 +62,7 @@ struct Note
           const char* d = 0);
     void save(writer& outf) const;
     void load(reader& inf);
-    std::string describe(bool when = true, bool where = true,
-                          bool what = true) const;
+    string describe(bool when = true, bool where = true, bool what = true) const;
     void check_milestone() const;
 
     NOTE_TYPES type;
@@ -70,11 +70,11 @@ struct Note
     int turn;
     unsigned short packed_place;
 
-    std::string name;
-    std::string desc;
+    string name;
+    string desc;
 };
 
-extern std::vector<Note> note_list;
+extern vector<Note> note_list;
 void activate_notes(bool active);
 bool notes_are_active();
 void take_note(const Note& note, bool force = false);
