@@ -60,6 +60,7 @@
 #include "religion.h"
 #include "shopping.h"
 #include "skills2.h"
+#include "spl-clouds.h"
 #include "spl-selfench.h"
 #include "spl-other.h"
 #include "state.h"
@@ -89,7 +90,7 @@ static void _maybe_melt_player_enchantments(beam_type flavour, int damage)
 
         if (you.mutation[MUT_ICEMAIL])
         {
-            mpr("Your icy envelope dissipates!", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "Your icy envelope dissipates!");
             you.duration[DUR_ICEMAIL_DEPLETED] = ICEMAIL_TIME;
             you.redraw_armour_class = true;
         }
@@ -100,7 +101,7 @@ static void _maybe_melt_player_enchantments(beam_type flavour, int damage)
             if (you.duration[DUR_ICY_ARMOUR] <= 0)
                 remove_ice_armour();
             else
-                you.props["melt_armour"] = true;
+                you.props[MELT_ARMOUR_KEY] = true;
         }
     }
 }
@@ -163,12 +164,14 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_HELLFIRE:
+#if TAG_MAJOR_VERSION == 34
         if (you.species == SP_DJINNI)
         {
             hurted = 0;
             if (doEffects)
                 mpr("You resist completely.");
         }
+#endif
         // Inconsistency: no penalty for rF-, unlike monsters.  That's
         // probably good, and monsters should be changed.
         break;
@@ -197,7 +200,10 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     case BEAM_POISON:
         if (doEffects)
         {
-            resist = poison_player(coinflip() ? 2 : 1, source, kaux) ? 0 : 1;
+            int pois = div_rand_round(beam->damage.num * beam->damage.size, 3);
+            pois = 3 + random_range(pois * 2 / 3, pois * 4 / 3);
+
+            resist = poison_player(pois, source, kaux) ? 0 : 1;
 
             hurted = resist_adjust_damage(&you, flavour, resist,
                                           hurted, true);
@@ -220,9 +226,10 @@ int check_your_resists(int hurted, beam_type flavour, string source,
 
         if (doEffects)
         {
-            int poison_amount = 2 + random2(3);
-            poison_amount += (resist ? 0 : 2);
-            poison_player(poison_amount, source, kaux, true);
+            int pois = div_rand_round(beam->damage.num * beam->damage.size, 3);
+            pois = 3 + random_range(pois * 2 / 3, pois * 4 / 3);
+
+            poison_player((resist ? pois / 2 : pois), source, kaux, true);
         }
 
         hurted = resist_adjust_damage(&you, flavour, resist, hurted);
@@ -291,7 +298,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         else if (rhe == 0)
             hurted /= 2;
         else if (rhe < -1)
-            hurted = (hurted * 3) / 2;
+            hurted = hurted * 3 / 2;
 
         if (hurted == 0 && doEffects)
             canned_msg(MSG_YOU_RESIST);
@@ -335,17 +342,6 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
     }
 
-    case BEAM_LIGHT:
-        if (you.species == SP_VAMPIRE)
-            hurted += hurted / 2;
-
-        if (hurted > original && doEffects)
-        {
-            mpr("The light scorches you terribly!");
-            xom_is_stimulated(200);
-        }
-        break;
-
     case BEAM_AIR:
     {
         // Airstrike.
@@ -381,13 +377,14 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
     }                           // end switch
 
-    if (doEffects && hurted != original)
+    if (doEffects)
         maybe_id_resist(flavour);
 
     return hurted;
 }
 
-void splash_with_acid(int acid_strength, int death_source, bool corrode_items, string hurt_message)
+void splash_with_acid(int acid_strength, int death_source, bool corrode_items,
+                      const char* hurt_msg)
 {
     int dam = 0;
     const bool wearing_cloak = player_wearing_slot(EQ_CLOAK);
@@ -425,10 +422,14 @@ void splash_with_acid(int acid_strength, int death_source, bool corrode_items, s
 
     if (post_res_dam > 0)
     {
-        mpr(hurt_message.empty() ? "The acid burns!" : hurt_message);
+        mpr(hurt_msg ? hurt_msg : "The acid burns!");
 
         if (post_res_dam < dam)
+        {
             canned_msg(MSG_YOU_RESIST);
+            if (!player_res_acid(false, true))
+                maybe_id_resist(BEAM_ACID);
+        }
 
         ouch(post_res_dam, death_source, KILLED_BY_ACID);
     }
@@ -523,6 +524,13 @@ static bool _expose_invent_to_element(beam_type flavour, int strength)
                 continue;
             }
 
+            if (target_class == OBJ_SCROLLS
+                && player_equip_unrand(UNRAND_FIRESTARTER)
+                && !one_chance_in(10))
+            {
+                continue;
+            }
+
             if (target_class == OBJ_POTIONS
                 && you.mutation[MUT_CONSERVE_POTIONS]
                 && !one_chance_in(10))
@@ -622,80 +630,6 @@ static bool _expose_invent_to_element(beam_type flavour, int strength)
     return true;
 }
 
-bool expose_items_to_element(beam_type flavour, const coord_def& where,
-                             int strength)
-{
-    int num_dest = 0;
-
-    const int target_class = _get_target_class(flavour);
-    if (target_class == OBJ_UNASSIGNED)
-        return false;
-
-    // Beams fly *over* water and lava.
-    if (grd(where) == DNGN_LAVA || grd(where) == DNGN_DEEP_WATER)
-        return false;
-
-    for (stack_iterator si(where); si; ++si)
-    {
-        if (!si->defined())
-            continue;
-
-        if (si->base_type == target_class
-            || target_class == OBJ_FOOD && si->base_type == OBJ_CORPSES)
-        {
-            for (int j = 0; j < si->quantity; ++j)
-            {
-                if (x_chance_in_y(strength, 100))
-                {
-                    num_dest++;
-                    if (!dec_mitm_item_quantity(si->index(), 1)
-                        && is_blood_potion(*si))
-                    {
-                       remove_oldest_blood_potion(*si);
-                    }
-                }
-            }
-        }
-    }
-
-    if (!num_dest)
-        return false;
-
-    if (flavour == BEAM_DEVOUR_FOOD)
-        return true;
-
-    if (you.see_cell(where))
-    {
-        switch (target_class)
-        {
-        case OBJ_SCROLLS:
-            mprf("You see %s of smoke.",
-                 (num_dest > 1) ? "some puffs" : "a puff");
-            break;
-
-        case OBJ_POTIONS:
-            mprf("You see %s shatter.",
-                 (num_dest > 1) ? "some glass" : "glass");
-            break;
-
-        case OBJ_FOOD:
-            mprf("You see %s of spores.",
-                 (num_dest > 1) ? "some clouds" : "a cloud");
-            break;
-
-        default:
-            mprf("%s on the floor %s destroyed!",
-                 (num_dest > 1) ? "Some items" : "An item",
-                 (num_dest > 1) ? "were" : "was");
-            break;
-        }
-    }
-
-    xom_is_stimulated((num_dest > 1) ? 25 : 12);
-
-    return true;
-}
-
 // Handle side-effects for exposure to element other than damage.  This
 // function exists because some code calculates its own damage instead
 // of using check_your_resists() and we want to isolate all the special
@@ -717,6 +651,14 @@ bool expose_player_to_element(beam_type flavour, int strength,
         you.slow_down(0, strength);
     }
 
+    if (flavour == BEAM_WATER && you.duration[DUR_LIQUID_FLAMES])
+    {
+        mprf(MSGCH_WARN, "The flames go out!");
+        you.duration[DUR_LIQUID_FLAMES] = 0;
+        you.props.erase("napalmer");
+        you.props.erase("napalm_aux");
+    }
+
     if (strength <= 0 || !damage_inventory)
         return false;
 
@@ -731,7 +673,7 @@ static void _lose_level_abilities()
     {
         you.increase_duration(DUR_FLIGHT, 50, 100);
         you.attribute[ATTR_PERM_FLIGHT] = 0;
-        mpr("You feel your flight won't last long.", MSGCH_WARN);
+        mprf(MSGCH_WARN, "You feel your flight won't last long.");
     }
 }
 
@@ -778,11 +720,11 @@ void lose_level(int death_source, const char *aux)
     ouch(0, death_source, KILLED_BY_DRAINING, aux);
 }
 
-bool drain_exp(bool announce_full, int power)
+bool drain_exp(bool announce_full, int power, bool ignore_protection)
 {
     const int protection = player_prot_life();
 
-    if (protection == 3)
+    if (protection == 3 && !ignore_protection)
     {
         if (announce_full)
             canned_msg(MSG_YOU_RESIST);
@@ -790,7 +732,7 @@ bool drain_exp(bool announce_full, int power)
         return false;
     }
 
-    if (protection > 0)
+    if (protection > 0 && !ignore_protection)
     {
         canned_msg(MSG_YOU_PARTIALLY_RESIST);
         power /= (protection * 2);
@@ -802,6 +744,9 @@ bool drain_exp(bool announce_full, int power)
         xom_is_stimulated(15);
 
         you.attribute[ATTR_XP_DRAIN] += power;
+        // Losing skills may affect AC/EV.
+        you.redraw_armour_class = true;
+        you.redraw_evasion = true;
 
         dprf("Drained by %d points (%d total)", power, you.attribute[ATTR_XP_DRAIN]);
 
@@ -816,7 +761,7 @@ static void _xom_checks_damage(kill_method_type death_type,
 {
     if (you_worship(GOD_XOM))
     {
-        if (death_type == KILLED_BY_TARGETTING
+        if (death_type == KILLED_BY_TARGETING
             || death_type == KILLED_BY_BOUNCE
             || death_type == KILLED_BY_REFLECTION
             || death_type == KILLED_BY_SELF_AIMED
@@ -938,13 +883,6 @@ static void _maybe_spawn_jellies(int dam, const char* aux,
 
         if (how_many > 0)
         {
-            if (x_chance_in_y(how_many, 8)
-                && !lose_stat(STAT_STR, 1, true, "spawning slimes"))
-            {
-                canned_msg(MSG_NOTHING_HAPPENS);
-                return;
-            }
-
             int count_created = 0;
             for (int i = 0; i < how_many; ++i)
             {
@@ -1004,6 +942,25 @@ static void _powered_by_pain(int dam)
     }
 }
 
+static void _maybe_fog(int dam)
+{
+    const int upper_threshold = you.hp_max / 2;
+    const int lower_threshold = upper_threshold
+                                - upper_threshold
+                                  * (you.piety - piety_breakpoint(2))
+                                  / (MAX_PIETY - piety_breakpoint(2));
+    if (you_worship(GOD_DITHMENOS)
+        && you.piety >= piety_breakpoint(2)
+        && (dam > 0 && you.form == TRAN_SHADOW
+            || dam >= lower_threshold
+               && x_chance_in_y(dam - lower_threshold,
+                                upper_threshold - lower_threshold)))
+    {
+        mpr("You emit a cloud of dark smoke.");
+        big_cloud(CLOUD_BLACK_SMOKE, &you, you.pos(), 50, 4 + random2(5));
+    }
+}
+
 static void _place_player_corpse(bool explode)
 {
     if (!in_bounds(you.pos()))
@@ -1035,12 +992,13 @@ static void _place_player_corpse(bool explode)
     move_item_to_grid(&o, you.pos(), !you.in_water());
 }
 
-
 #if defined(WIZARD) || defined(DEBUG)
 static void _wizard_restore_life()
 {
     if (you.hp_max <= 0)
         unrot_hp(9999);
+    while (you.hp_max <= 0)
+        you.hp_max_perm++, calc_hp();
     if (you.hp <= 0)
         set_hp(you.hp_max);
 }
@@ -1065,6 +1023,8 @@ void ouch(int dam, int death_source, kill_method_type death_type,
     if (you.dead) // ... but eligible for revival
         return;
 
+    int drain_amount = 0;
+
     if (attacker_effects && dam != INSTANT_DEATH
         && !invalid_monster_index(death_source)
         && menv[death_source].has_ench(ENCH_WRETCHED))
@@ -1077,7 +1037,8 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         dam = div_rand_round(dam * (10 - min(degree, 5)), 10);
     }
 
-    if (dam != INSTANT_DEATH && you.species == SP_DEEP_DWARF)
+    if (you.species == SP_DEEP_DWARF && dam != INSTANT_DEATH
+                                     && death_type != KILLED_BY_POISON)
     {
         // Deep Dwarves get to shave any hp loss.
         int shave = 1 + random2(2 + random2(1 + you.experience_level / 3));
@@ -1094,6 +1055,11 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
     if (dam != INSTANT_DEATH)
     {
+        if (you.form == TRAN_SHADOW)
+        {
+            drain_amount = (dam - (dam / 2));
+            dam /= 2;
+        }
         if (you.petrified())
             dam /= 2;
         else if (you.petrifying())
@@ -1117,20 +1083,6 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
     if (dam != INSTANT_DEATH)
     {
-        if (you.duration[DUR_SONG_OF_SHIELDING] && you.magic_points > 0)
-        {
-            if (dam > you.magic_points)
-            {
-                dam = dam - you.magic_points;
-                dec_mp(you.magic_points);
-            }
-            else
-            {
-                dec_mp(dam);
-                dam = 0;
-                return;
-            }
-        }
         if (you.spirit_shield() && death_type != KILLED_BY_POISON
             && !(aux && strstr(aux, "flay_damage")))
         {
@@ -1143,7 +1095,7 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
             dam -= mp;
             dec_mp(mp);
-            if (dam <= 0)
+            if (dam <= 0 && you.hp > 0)
                 return;
         }
 
@@ -1166,14 +1118,15 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         // Even if we have low HP messages off, we'll still give a
         // big hit warning (in this case, a hit for half our HPs) -- bwr
         if (dam > 0 && you.hp_max <= dam * 2)
-            mpr("Ouch! That really hurt!", MSGCH_DANGER);
+            mprf(MSGCH_DANGER, "Ouch! That really hurt!");
 
-        if (you.hp > 0)
+        if (you.hp > 0 && dam > 0)
         {
             if (Options.hp_warning
-                && you.hp <= (you.hp_max * Options.hp_warning) / 100)
+                && you.hp <= (you.hp_max * Options.hp_warning) / 100
+                && (death_type != KILLED_BY_POISON || poison_is_lethal()))
             {
-                mpr("* * * LOW HITPOINT WARNING * * *", MSGCH_DANGER);
+                mprf(MSGCH_DANGER, "* * * LOW HITPOINT WARNING * * *");
                 dungeon_events.fire_event(DET_HP_WARNING);
             }
 
@@ -1197,10 +1150,13 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
             _yred_mirrors_injury(dam, death_source);
             _maybe_spawn_jellies(dam, aux, death_type, death_source);
+            _maybe_fog(dam);
             _powered_by_pain(dam);
-
-            return;
-        } // else hp <= 0
+            if (drain_amount > 0)
+                drain_exp(true, drain_amount, true);
+        }
+        if (you.hp > 0)
+          return;
     }
 
     // Is the player being killed by a direct act of Xom?
@@ -1240,11 +1196,8 @@ void ouch(int dam, int death_source, kill_method_type death_type,
             death_type = KILLED_BY_XOM;
     }
     // Xom may still try to save your life.
-    else if (xom_saves_your_life(dam, death_source, death_type, aux,
-                                 see_source))
-    {
+    else if (xom_saves_your_life(death_type, aux))
         return;
-    }
 
 #if defined(WIZARD) || defined(DEBUG)
     if (!non_death && crawl_state.disables[DIS_DEATH])
@@ -1310,7 +1263,7 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         if (!crawl_state.disables[DIS_SAVE_CHECKPOINTS])
             save_game(false);
 
-        mprnojoin("You die...");
+        canned_msg(MSG_YOU_DIE);
         xom_death_message((kill_method_type) se.get_death_type());
         more();
 
@@ -1400,7 +1353,7 @@ void _end_game(scorefile_entry &se)
         && se.get_death_type() != KILLED_BY_QUITTING
         && se.get_death_type() != KILLED_BY_WINNING)
     {
-        mprnojoin("You die...");      // insert player name here? {dlb}
+        canned_msg(MSG_YOU_DIE);
         xom_death_message((kill_method_type) se.get_death_type());
 
         switch (you.religion)
@@ -1437,8 +1390,7 @@ void _end_game(scorefile_entry &se)
             else if (se.get_death_type() != KILLED_BY_DISINT
                      && se.get_death_type() != KILLED_BY_LAVA)
             {
-                mpr("Your body rises from the dead as a mindless zombie.",
-                    MSGCH_GOD);
+                mprf(MSGCH_GOD, "Your body rises from the dead as a mindless zombie.");
             }
             // No message if you're not undead and your corpse is lost.
             break;
@@ -1454,27 +1406,34 @@ void _end_game(scorefile_entry &se)
             hints_death_screen();
     }
 
-    if (!dump_char(morgue_name(you.your_name, se.get_death_time()),
-                   true, true, &se))
+    string fname = morgue_name(you.your_name, se.get_death_time());
+    if (!dump_char(fname, true, true, &se))
     {
         mpr("Char dump unsuccessful! Sorry about that.");
         if (!crawl_state.seen_hups)
             more();
         clrscr();
     }
+#ifdef USE_TILE_WEB
+    else
+        tiles.send_dump_info("morgue", fname);
+#endif
 
+#if defined(DGL_WHEREIS) || defined(USE_TILE_WEB)
+    string reason = se.get_death_type() == KILLED_BY_QUITTING? "quit" :
+                    se.get_death_type() == KILLED_BY_WINNING ? "won"  :
+                    se.get_death_type() == KILLED_BY_LEAVING ? "bailed out"
+                                                             : "dead";
 #ifdef DGL_WHEREIS
-    whereis_record(se.get_death_type() == KILLED_BY_QUITTING? "quit" :
-                   se.get_death_type() == KILLED_BY_WINNING ? "won"  :
-                   se.get_death_type() == KILLED_BY_LEAVING ? "bailed out"
-                                                            : "dead");
+    whereis_record(reason.c_str());
+#endif
 #endif
 
     if (!crawl_state.seen_hups)
         more();
 
     if (!crawl_state.disables[DIS_CONFIRMATIONS])
-        browse_inventory();
+        get_invent(OSEL_ANY);
     textcolor(LIGHTGREY);
 
     // Prompt for saving macros.
@@ -1509,6 +1468,10 @@ void _end_game(scorefile_entry &se)
     if (se.get_death_type() == KILLED_BY_WINNING)
         crawl_state.last_game_won = true;
 
+#ifdef USE_TILE_WEB
+    tiles.send_exit_reason(reason, hiscore);
+#endif
+
     game_ended();
 }
 
@@ -1517,7 +1480,7 @@ int actor_to_death_source(const actor* agent)
     if (agent->is_player())
         return NON_MONSTER;
     else if (agent->is_monster())
-        return (agent->as_monster()->mindex());
+        return agent->as_monster()->mindex();
     else
         return NON_MONSTER;
 }
