@@ -15,7 +15,7 @@
 #include "options.h"
 #include "species.h"
 
-#include "abl-show.h"
+#include "ability.h"
 #include "areas.h"
 #include "artefact.h"
 #include "branch.h"
@@ -29,6 +29,7 @@
 #include "godabil.h"
 #include "initfile.h"
 #include "itemname.h"
+#include "items.h"
 #include "lang-fake.h"
 #include "libutil.h"
 #include "menu.h"
@@ -45,14 +46,12 @@
 #include "state.h"
 #include "status.h"
 #include "stuff.h"
-#include "tagstring.h"
 #include "throw.h"
 #include "transform.h"
 #include "travel.h"
 #include "viewchar.h"
 #include "viewgeom.h"
 #include "showsymb.h"
-#include "spl-transloc.h"
 
 #ifndef USE_TILE_LOCAL
 #include "directn.h"
@@ -288,7 +287,6 @@ static void _nowrap_eol_cprintf_touchui(const char *format, ...)
 #define NOWRAP_EOL_CPRINTF nowrap_eol_cprintf
 #endif
 
-
 static string _god_powers(bool simple = false);
 
 // Color for captions like 'Health:', 'Str:', etc.
@@ -325,11 +323,11 @@ class colour_bar
 
     bool wants_redraw() const
     {
-        return (m_request_redraw_after
-                && you.num_turns >= m_request_redraw_after);
+        return m_request_redraw_after
+               && you.num_turns >= m_request_redraw_after;
     }
 
-    void draw(int ox, int oy, int val, int max_val, bool temp = false)
+    void draw(int ox, int oy, int val, int max_val, bool temp = false, int sub_val = 0)
     {
         ASSERT(val <= max_val);
         if (max_val <= 0)
@@ -340,9 +338,14 @@ class colour_bar
 
         const colour_t temp_colour = temperature_colour(temperature());
         const int width = crawl_view.hudsz.x - (ox - 1);
-        const int disp  = width * val / max_val;
-        const int old_disp = (m_old_disp < 0) ? disp : m_old_disp;
-        m_old_disp = disp;
+        const int sub_disp = (width * val / max_val);
+        int disp  = width * max(0, val - sub_val) / max_val;
+        const int old_disp = (m_old_disp < 0) ? sub_disp : m_old_disp;
+        m_old_disp = sub_disp;
+
+        // Always show at least one sliver of the sub-bar, if it exists
+        if (sub_val)
+            disp = max(0, min(sub_disp - 1, disp));
 
         CGOTOXY(ox, oy, GOTO_STAT);
 
@@ -355,7 +358,9 @@ class colour_bar
 
             if (cx < disp)
                 textcolor(BLACK + (temp) ? temp_colour * 16 : m_default * 16);
-            else if (old_disp > disp && cx < old_disp)
+            else if (cx < sub_disp)
+                textcolor(BLACK + YELLOW * 16);
+            else if (old_disp >= sub_disp && cx < old_disp)
                 textcolor(BLACK + m_change_neg * 16);
             putwch(' ');
 #else
@@ -369,6 +374,11 @@ class colour_bar
                 textcolor(m_change_pos);
                 putwch('=');
             }
+            else if (cx < sub_disp)
+            {
+                textcolor(YELLOW);
+                putwch('=');
+            }
             else if (cx < old_disp)
             {
                 textcolor(m_change_neg);
@@ -499,10 +509,10 @@ void update_turn_count()
     textcolor(HUD_VALUE_COLOUR);
     if (Options.show_game_turns)
     {
-       CPRINTF("%.1f (%.1f)%s", you.elapsed_time / 10.0,
-               (you.elapsed_time - you.elapsed_time_at_last_input) / 10.0,
-               // extra spaces to erase excess if previous output was longer
-               "    ");
+        CPRINTF("%.1f (%.1f)%s", you.elapsed_time / 10.0,
+                (you.elapsed_time - you.elapsed_time_at_last_input) / 10.0,
+                // extra spaces to erase excess if previous output was longer
+                "    ");
     }
     else
         CPRINTF("%d", you.num_turns);
@@ -522,7 +532,6 @@ static int _count_digits(int val)
 
 static void _print_stats_temperature(int x, int y)
 {
-
     cgotoxy(x, y, GOTO_STAT);
     textcolor(HUD_CAPTION_COLOUR);
     cprintf("Temperature: ");
@@ -532,9 +541,11 @@ static void _print_stats_temperature(int x, int y)
 
 static void _print_stats_mp(int x, int y)
 {
+#if TAG_MAJOR_VERSION == 34
     if (you.species == SP_DJINNI)
         return;
 
+#endif
     // Calculate colour
     short mp_colour = HUD_VALUE_COLOUR;
 
@@ -575,32 +586,36 @@ static void _print_stats_mp(int x, int y)
     MP_Bar.draw(19, y, you.magic_points, you.max_magic_points);
 }
 
+#if TAG_MAJOR_VERSION == 34
 static void _print_stats_contam(int x, int y)
 {
     if (you.species != SP_DJINNI)
         return;
 
-    const int max_contam = 16000;
+    const int max_contam = 8000;
     int contam = min(you.magic_contamination, max_contam);
 
     // Calculate colour
-    int contam_level = get_contamination_level();
-
-    if (contam_level > 3)
+    if (you.magic_contamination > 15000)
     {
         Contam_Bar.m_default = RED;
         Contam_Bar.m_change_pos = Contam_Bar.m_change_neg = RED;
     }
-    else if (contam_level > 2)
+    else if (you.magic_contamination > 5000) // harmful
     {
         Contam_Bar.m_default = LIGHTRED;
         Contam_Bar.m_change_pos = Contam_Bar.m_change_neg = RED;
     }
-    else if (contam_level > 1)
+    else if (you.magic_contamination > 3333)
     {
         Contam_Bar.m_default = YELLOW;
         Contam_Bar.m_change_pos = Contam_Bar.m_change_neg = BROWN;
     }
+    else if (you.magic_contamination > 1666)
+    {
+        Contam_Bar.m_default = LIGHTGREY;
+        Contam_Bar.m_change_pos = Contam_Bar.m_change_neg = DARKGREY;
+    }
     else
     {
 #ifdef USE_TILE_LOCAL
@@ -618,12 +633,14 @@ static void _print_stats_contam(int x, int y)
 #endif
     Contam_Bar.draw(19, y, contam, max_contam);
 }
-
+#endif
 static void _print_stats_hp(int x, int y)
 {
     int max_max_hp = get_real_hp(true, true);
+#if TAG_MAJOR_VERSION == 34
     if (you.species == SP_DJINNI)
         max_max_hp += get_real_mp(true);
+#endif
 
     // Calculate colour
     short hp_colour = HUD_VALUE_COLOUR;
@@ -636,7 +653,7 @@ static void _print_stats_hp(int x, int y)
     else
     {
         const int hp_percent =
-            (you.hp * 100) / (max_max_hp ? max_max_hp : you.hp);
+            (you.hp * 100) / get_real_hp(true, false);
 
         for (unsigned int i = 0; i < Options.hp_colour.size(); ++i)
             if (hp_percent <= Options.hp_colour[i].first)
@@ -647,10 +664,12 @@ static void _print_stats_hp(int x, int y)
     // Health: xxx/yyy (zzz)
     CGOTOXY(x, y, GOTO_STAT);
     textcolor(HUD_CAPTION_COLOUR);
+#if TAG_MAJOR_VERSION == 34
     if (you.species == SP_DJINNI)
         CPRINTF(max_max_hp != you.hp_max ? "EP: " : "Essence: ");
     else
-        CPRINTF(max_max_hp != you.hp_max ? "HP: " : "Health: ");
+#endif
+    CPRINTF(max_max_hp != you.hp_max ? "HP: " : "Health: ");
     textcolor(hp_colour);
     CPRINTF("%d", you.hp);
     if (!boosted)
@@ -668,17 +687,21 @@ static void _print_stats_hp(int x, int y)
 #ifdef USE_TILE_LOCAL
     if (tiles.is_using_small_layout())
     {
-        if (you.species != SP_DJINNI)
-            HP_Bar.vdraw(2, 10, you.hp, you.hp_max);
-        else
+#if TAG_MAJOR_VERSION == 34
+        if (you.species == SP_DJINNI)
             EP_Bar.vdraw(2, 10, you.hp, you.hp_max);
+        else
+#endif
+        HP_Bar.vdraw(2, 10, you.hp, you.hp_max);
     }
     else
 #endif
-    if (you.species != SP_DJINNI)
-        HP_Bar.draw(19, y, you.hp, you.hp_max);
-    else
+#if TAG_MAJOR_VERSION == 34
+    if (you.species == SP_DJINNI)
         EP_Bar.draw(19, y, you.hp, you.hp_max);
+    else
+#endif
+        HP_Bar.draw(19, y, you.hp, you.hp_max, false, you.hp - max(0, poison_survival()));
 }
 
 static short _get_stat_colour(stat_type stat)
@@ -689,7 +712,7 @@ static short _get_stat_colour(stat_type stat)
     // Check the stat_colour option for warning thresholds.
     for (unsigned int i = 0; i < Options.stat_colour.size(); ++i)
         if (you.stat(stat) <= Options.stat_colour[i].first)
-            return (Options.stat_colour[i].second);
+            return Options.stat_colour[i].second;
 
     // Stat is magically increased.
     if (you.duration[DUR_DIVINE_STAMINA]
@@ -725,10 +748,8 @@ static void _print_stats_ac(int x, int y)
 {
     // AC:
     CGOTOXY(x+4, y, GOTO_STAT);
-    if (you.duration[DUR_ICY_ARMOUR] || player_stoneskin())
+    if (you.duration[DUR_ICY_ARMOUR] || player_stoneskin() || player_icemail_armour_class())
         textcolor(LIGHTBLUE);
-    else if (you.duration[DUR_ICEMAIL_DEPLETED] > ICEMAIL_TIME / ICEMAIL_MAX)
-        textcolor(RED);
     else
         textcolor(HUD_VALUE_COLOUR);
     string ac = make_stringf("%2d ", you.armour_class());
@@ -803,6 +824,7 @@ static void _print_stats_wp(int y)
             col = GREEN;
             break;
         case TRAN_LICH:
+        case TRAN_SHADOW:
             col = MAGENTA;
             break;
         case TRAN_BAT:
@@ -902,7 +924,7 @@ struct status_light
 //
 // Prints burden, hunger,
 // pray, holy, teleport, regen, fly/lev, invis, silence,
-//   conf. touch, bargain, sage
+//   conf. touch, sage
 // confused, mesmerised, fire, poison, disease, rot, held, glow, swift,
 //   fast, slow, breath
 //
@@ -927,7 +949,8 @@ static void _get_status_lights(vector<status_light>& out)
     }
 #endif
 
-    const int statuses[] = {
+    const int statuses[] =
+    {
         STATUS_STR_ZERO, STATUS_INT_ZERO, STATUS_DEX_ZERO,
         STATUS_BURDEN,
         STATUS_HUNGER,
@@ -942,14 +965,14 @@ static void _get_status_lights(vector<status_light>& out)
         STATUS_MISSILES,
         STATUS_REGENERATION,
         DUR_BERSERK,
+        STATUS_DIG,
         DUR_RESISTANCE,
         STATUS_AIRBORNE,
-        DUR_INVIS,
+        STATUS_INVISIBLE,
         DUR_CONTROL_TELEPORT,
         DUR_DISJUNCTION,
         DUR_SILENCE,
         DUR_CONFUSING_TOUCH,
-        DUR_BARGAIN,
         STATUS_SAGE,
         DUR_FIRE_SHIELD,
         DUR_SLIMIFY,
@@ -958,7 +981,6 @@ static void _get_status_lights(vector<status_light>& out)
         DUR_LOWERED_MR,
         STATUS_BEHELD,
         DUR_LIQUID_FLAMES,
-        DUR_MISLED,
         DUR_POISONING,
         STATUS_SICK,
         STATUS_ROT,
@@ -977,7 +999,6 @@ static void _get_status_lights(vector<status_light>& out)
         DUR_MIRROR_DAMAGE,
         DUR_SCRYING,
         STATUS_CLINGING,
-        STATUS_HOVER,
         DUR_TORNADO,
         DUR_LIQUEFYING,
         DUR_HEROISM,
@@ -992,7 +1013,6 @@ static void _get_status_lights(vector<status_light>& out)
         STATUS_CONSTRICTED,
         DUR_DIVINE_STAMINA,
         STATUS_AUGMENTED,
-        STATUS_SUPPRESSED,
         STATUS_TERRAIN,
         STATUS_SILENCE,
         STATUS_NO_CTELE,
@@ -1005,16 +1025,25 @@ static void _get_status_lights(vector<status_light>& out)
         DUR_WEAK,
         DUR_DIMENSION_ANCHOR,
         STATUS_BEOGH,
-        DUR_SPIRIT_HOWL,
         DUR_INFUSION,
         DUR_SONG_OF_SLAYING,
-        DUR_SONG_OF_SHIELDING,
         STATUS_DRAINED,
         DUR_TOXIC_RADIANCE,
         STATUS_RAY,
         DUR_RECITE,
         DUR_GRASPING_ROOTS,
         DUR_FIRE_VULN,
+        DUR_FROZEN,
+        DUR_SAP_MAGIC,
+        STATUS_MAGIC_SAPPED,
+        STATUS_ELIXIR,
+        DUR_BARBS,
+        DUR_POISON_VULN,
+        DUR_PORTAL_PROJECTILE,
+        DUR_FORESTED,
+        DUR_DRAGON_CALL,
+        DUR_DRAGON_CALL_COOLDOWN,
+        DUR_ABJURATION_AURA,
     };
 
     status_info inf;
@@ -1241,6 +1270,13 @@ void print_stats(void)
     if (Temp_Bar.wants_redraw() && you.species == SP_LAVA_ORC)
         you.redraw_temperature = true;
 
+    // Poison display depends on regen rate, so should be redrawn every turn.
+    if (you.duration[DUR_POISONING])
+    {
+        you.redraw_hit_points = true;
+        you.redraw_status_flags |= REDRAW_POISONED;
+    }
+
 #ifdef USE_TILE_LOCAL
     bool has_changed = _need_stats_printed();
 #endif
@@ -1252,7 +1288,9 @@ void print_stats(void)
     }
     if (you.redraw_hit_points)   { you.redraw_hit_points = false;   _print_stats_hp (1, 3); }
     if (you.redraw_magic_points) { you.redraw_magic_points = false; _print_stats_mp (1, 4); }
+#if TAG_MAJOR_VERSION == 34
     _print_stats_contam(1, 4);
+#endif
     if (you.redraw_temperature)  { you.redraw_temperature = false;  _print_stats_temperature (1, temp_pos); }
     if (you.redraw_armour_class) { you.redraw_armour_class = false; _print_stats_ac (1, ac_pos); }
     if (you.redraw_evasion)      { you.redraw_evasion = false;      _print_stats_ev (1, ev_pos); }
@@ -1399,10 +1437,12 @@ void draw_border(void)
 
     //CGOTOXY(1, 3, GOTO_STAT); CPRINTF("Hp:");
     CGOTOXY(1, mp_pos, GOTO_STAT);
+#if TAG_MAJOR_VERSION == 34
     if (you.species == SP_DJINNI)
         CPRINTF("Contam:");
     else
-        CPRINTF("Magic:");
+#endif
+    CPRINTF("Magic:");
     CGOTOXY(1, ac_pos, GOTO_STAT); CPRINTF("AC:");
     CGOTOXY(1, ev_pos, GOTO_STAT); CPRINTF("EV:");
     CGOTOXY(1, sh_pos, GOTO_STAT); CPRINTF("SH:");
@@ -1514,7 +1554,7 @@ static void _print_next_monster_desc(const vector<monster_info>& mons,
     {
         int printed = 0;
 
-        // for targetting
+        // for targeting
         if (idx >= 0)
         {
             textcolor(WHITE);
@@ -1634,7 +1674,7 @@ int update_monster_pane()
         // i_mons is incremented by _print_next_monster_desc
         if (i_print >= skip_lines && i_mons < (int) mons.size())
         {
-             int idx = crawl_state.mlist_targetting ? i_print - skip_lines : -1;
+             int idx = crawl_state.mlist_targeting ? i_print - skip_lines : -1;
              _print_next_monster_desc(mons, i_mons, full_info, idx);
         }
         else
@@ -1661,29 +1701,40 @@ int update_monster_pane()
 }
 #endif
 
-static const char* _itosym1(int stat)
+// Converts a numeric resistance to its symbolic counterpart.
+// Can handle any maximum level. The default is for single level resistances
+// (the most common case). Negative resistances are allowed.
+// Resistances with a maximum of up to 4 are spaced (arbitrary choice), and
+// starting at 5 levels, they are continuous.
+// params:
+//  level : actual resistance level
+//  max : maximum number of levels of the resistance
+static string _itosym(int level, int max = 1)
 {
-    return ((stat >= 1) ? "+  " :
-            (stat == 0) ? ".  " :
-                          "x  ");
-}
+    if (max < 1)
+        return "";
 
-static const char* _itosym2(int stat)
-{
-    return ((stat >= 2) ? "+ +" :
-            (stat == 1) ? "+ ." :
-                          ". .");
-}
+    string sym;
+    bool spacing = (max >= 5) ? false : true;
 
-static const char* _itosym3(int stat)
-{
-    return ((stat >=  3) ? "+ + +" :
-            (stat ==  2) ? "+ + ." :
-            (stat ==  1) ? "+ . ." :
-            (stat ==  0) ? ". . ." :
-            (stat == -1) ? "x . ." :
-            (stat == -2) ? "x x ." :
-                           "x x x");
+    while (max > 0)
+    {
+        if (level == 0)
+            sym += ".";
+        else if (level > 0)
+        {
+            sym += "+";
+            --level;
+        }
+        else // negative resistance
+        {
+            sym += "x";
+            ++level;
+        }
+        sym += (spacing) ? " " : "";
+        --max;
+    }
+    return sym;
 }
 
 static const char *s_equip_slot_names[] =
@@ -1692,13 +1743,17 @@ static const char *s_equip_slot_names[] =
     "Shield", "Armour", "Left Ring", "Right Ring", "Amulet",
     "First Ring", "Second Ring", "Third Ring", "Fourth Ring",
     "Fifth Ring", "Sixth Ring", "Seventh Ring", "Eighth Ring",
+    "Amulet Ring"
 };
 
 const char *equip_slot_to_name(int equip)
 {
+    COMPILE_CHECK(ARRAYSZ(s_equip_slot_names) == NUM_EQUIP);
+
     if (equip == EQ_RINGS
         || equip == EQ_LEFT_RING || equip == EQ_RIGHT_RING
-        || equip >= EQ_RING_ONE && equip <= EQ_RING_EIGHT)
+        || equip >= EQ_RING_ONE && equip <= EQ_RING_EIGHT
+        || equip == EQ_RING_AMULET)
     {
         return "Ring";
     }
@@ -1728,6 +1783,10 @@ int equip_name_to_slot(const char *s)
 // Take maximum possible level into account.
 static const char* _determine_colour_string(int level, int max_level)
 {
+    // No colouring for larger bars.
+    if (max_level > 3)
+        return "<lightgrey>";
+
     switch (level)
     {
     case 3:
@@ -1762,6 +1821,7 @@ static void _print_overview_screen_equip(column_composer& cols,
         EQ_GLOVES, EQ_BOOTS, EQ_AMULET, EQ_RIGHT_RING, EQ_LEFT_RING,
         EQ_RING_ONE, EQ_RING_TWO, EQ_RING_THREE, EQ_RING_FOUR,
         EQ_RING_FIVE, EQ_RING_SIX, EQ_RING_SEVEN, EQ_RING_EIGHT,
+        EQ_RING_AMULET,
     };
 
     sw = min(max(sw, 79), 640);
@@ -1784,7 +1844,13 @@ static void _print_overview_screen_equip(column_composer& cols,
             continue;
         }
 
-        if (you.species != SP_OCTOPODE && eqslot > EQ_AMULET)
+        if (you.species != SP_OCTOPODE
+            && eqslot >= EQ_RING_ONE && eqslot <= EQ_RING_EIGHT)
+        {
+            continue;
+        }
+
+        if (eqslot == EQ_RING_AMULET && !you_can_wear(eqslot))
             continue;
 
         const string slot_name_lwr = lowercase_string(equip_slot_to_name(eqslot));
@@ -1815,7 +1881,8 @@ static void _print_overview_screen_equip(column_composer& cols,
                      equip_char,
                      colname,
                      melded ? "melded " : "",
-                     chop_string(item.name(DESC_PLAIN, true), sw - 38, false).c_str(),
+                     chop_string(item.name(DESC_PLAIN, true),
+                                 melded ? sw - 38 : sw - 31, false).c_str(),
                      colname);
             equip_chars.push_back(equip_char);
         }
@@ -1885,17 +1952,17 @@ static string _overview_screen_title(int sw)
     {
         switch (count)
         {
-          case 0:
-              snprintf(species_job, sizeof species_job,
-                       "(%s%s)",
-                       get_species_abbrev(you.species),
-                       get_job_abbrev(you.char_class));
-              break;
-          case 1:
-              strcpy(title, "");
-              break;
-          default:
-              break;
+        case 0:
+            snprintf(species_job, sizeof species_job,
+                     "(%s%s)",
+                     get_species_abbrev(you.species),
+                     get_job_abbrev(you.char_class));
+            break;
+        case 1:
+            strcpy(title, "");
+            break;
+        default:
+            break;
         }
         linelength = strwidth(you.your_name) + strwidth(title)
                      + strwidth(species_job) + strwidth(time_turns);
@@ -1921,9 +1988,9 @@ static string _overview_screen_title(int sw)
 static string _wiz_god_powers()
 {
     string godpowers = god_name(you.religion);
-    return (make_stringf("%s %d (%d)", god_name(you.religion).c_str(),
-                                       you.piety,
-                                       you.duration[DUR_PIETY_POOL]));
+    return make_stringf("%s %d (%d)", god_name(you.religion).c_str(),
+                                      you.piety,
+                                      you.duration[DUR_PIETY_POOL]);
 }
 #endif
 
@@ -1936,13 +2003,13 @@ static string _god_powers(bool simple)
             godpowers += simple ? "- BORED" : " - BORED";
         else if (you.gift_timeout == 1)
             godpowers += simple ? "- getting BORED" : " - getting BORED";
-        return (simple ? godpowers
-                       : colour_string(godpowers, god_colour(you.religion)));
+        return simple ? godpowers
+                      : colour_string(godpowers, god_colour(you.religion));
     }
     else if (!you_worship(GOD_NO_GOD))
     {
         if (player_under_penance())
-            return (simple ? "*" : colour_string("*" + godpowers, RED));
+            return simple ? "*" : colour_string("*" + godpowers, RED);
         else
         {
             // piety rankings
@@ -2157,113 +2224,115 @@ static vector<formatted_string> _get_overview_stats()
     return cols1.formatted_lines();
 }
 
+// generator of resistance strings:
+// params :
+//      name : name of the resist, correct spacing is handled here
+//      spacing : width of the name column
+//      value : actual value of the resistance (can be negative)
+//      max : maximum value of the resistance (for color AND representation),
+//          default is the most common case (1)
+//      pos_resist : false for "bad" resistances (no tele, random tele, *Rage),
+//          inverts the value for the colour choice
+static string _resist_composer(
+    const char * name, int spacing, int value, int max = 1, bool pos_resist = true)
+{
+    string out;
+    out += _determine_colour_string(pos_resist ? value : -value, max);
+    out += chop_string(name, spacing);
+    out += _itosym(value, max);
+
+    return out;
+}
+
 static vector<formatted_string> _get_overview_resistances(
     vector<char> &equip_chars, bool calc_unid, int sw)
 {
-    char buf[1000];
-
-    // 3 columns, splits at columns 21, 38
-    column_composer cols(3, 21, 38);
+    // 3 columns, splits at columns 18, 33
+    column_composer cols(3, 18, 33);
+    // First column, resist name is 7 chars
+    int cwidth = 7;
+    string out;
 
     const int rfire = player_res_fire(calc_unid);
+    out += _resist_composer("rFire", cwidth, rfire, 3) + "\n";
+
     const int rcold = player_res_cold(calc_unid);
+    out += _resist_composer("rCold", cwidth, rcold, 3) + "\n";
+
     const int rlife = player_prot_life(calc_unid);
+    out += _resist_composer("rNeg", cwidth, rlife, 3) + "\n";
+
     const int rpois = player_res_poison(calc_unid);
+    out += _resist_composer("rPois", cwidth, rpois) + "\n";
+
     const int relec = player_res_electricity(calc_unid);
+    out += _resist_composer("rElec", cwidth, relec) + "\n";
+
     const int rsust = player_sust_abil(calc_unid);
+    out += _resist_composer("SustAb", cwidth, rsust, 2) + "\n";
+
     const int rmuta = (you.rmut_from_item(calc_unid)
                        || player_mutation_level(MUT_MUTATION_RESISTANCE) == 3);
-    const int rrott = you.res_rotting();
+    out += _resist_composer("rMut", cwidth, rmuta) + "\n";
 
-    snprintf(buf, sizeof buf,
-             "%sRes.Fire  : %s\n"
-             "%sRes.Cold  : %s\n"
-             "%sLife Prot.: %s\n"
-             "%sRes.Poison: %s\n"
-             "%sRes.Elec. : %s\n"
-             "%sSust.Abil.: %s\n"
-             "%sRes.Mut.  : %s\n"
-             "%sRes.Rott. : %s\n",
-             _determine_colour_string(rfire, 3), _itosym3(rfire),
-             _determine_colour_string(rcold, 3), _itosym3(rcold),
-             _determine_colour_string(rlife, 3), _itosym3(rlife),
-             _determine_colour_string(rpois, 1), _itosym1(rpois),
-             _determine_colour_string(relec, 1), _itosym1(relec),
-             _determine_colour_string(rsust, 2), _itosym2(rsust),
-             _determine_colour_string(rmuta, 1), _itosym1(rmuta),
-             _determine_colour_string(rrott, 1), _itosym1(rrott));
-    cols.add_formatted(0, buf, false);
+    const int saplevel = player_mutation_level(MUT_SAPROVOROUS);
+    const bool show_gourm = (you.species != SP_MUMMY
+                             && you.species != SP_VAMPIRE
+                             && player_mutation_level(MUT_HERBIVOROUS) < 3
+                             && you.gourmand());
+    out += _resist_composer(show_gourm ? "Gourm" : "Saprov",
+                            cwidth,
+                            show_gourm ? 1 : saplevel,
+                            show_gourm ? 1 : 3) + "\n";
 
-    int saplevel = player_mutation_level(MUT_SAPROVOROUS);
-    const char* pregourmand;
-    const char* postgourmand;
-
-    if (you.species != SP_MUMMY
-        && you.species != SP_VAMPIRE
-        && player_mutation_level(MUT_HERBIVOROUS) < 3
-        && you.gourmand())
-    {
-        pregourmand = "Gourmand  : ";
-        postgourmand = _itosym1(1);
-        saplevel = 1;
-    }
-    else
-    {
-        pregourmand = "Saprovore : ";
-        postgourmand = _itosym3(saplevel);
-    }
-    snprintf(buf, sizeof buf, "%s%s%s",
-             _determine_colour_string(saplevel, 3), pregourmand, postgourmand);
-    cols.add_formatted(0, buf, false);
+    const int rmagi = player_res_magic(calc_unid) / 40;
+    out += _resist_composer("MR", cwidth, rmagi, 5) + "\n";
 
+    cols.add_formatted(0, out, false);
 
+    // Second column, resist name is 9 chars
+    out.clear();
+    cwidth = 9;
     const int rinvi = you.can_see_invisible(calc_unid);
-    const int rward = you.warding(calc_unid);
-    const int rcons = you.conservation(calc_unid);
-    const int rcorr = you.res_corr(calc_unid);
-    const int rclar = you.clarity(calc_unid);
-    const int rspir = you.spirit_shield(calc_unid);
-    snprintf(buf, sizeof buf,
-             "%sSee Invis. : %s\n"
-             "%sWarding    : %s\n"
-             "%sConserve   : %s\n"
-             "%sRes.Corr.  : %s\n"
-             "%sClarity    : %s\n"
-             "%sSpirit.Shd : %s\n"
-             ,
-             _determine_colour_string(rinvi, 1), _itosym1(rinvi),
-             _determine_colour_string(rward, 1), _itosym1(rward),
-             _determine_colour_string(rcons, 1), _itosym1(rcons),
-             _determine_colour_string(rcorr, 1), _itosym1(rcorr),
-             _determine_colour_string(rclar, 1), _itosym1(rclar),
-             _determine_colour_string(rspir, 1), _itosym1(rspir));
-    cols.add_formatted(1, buf, false);
+    out += _resist_composer("SeeInvis", cwidth, rinvi) + "\n";
 
+    const int rclar = you.clarity(calc_unid);
     const int stasis = you.stasis(calc_unid);
+    // TODO: what about different levels of anger/berserkitis?
+    const bool show_angry = (you.angry(calc_unid)
+                             || player_mutation_level(MUT_BERSERK))
+                            && !rclar && !stasis;
+    out += show_angry ? _resist_composer("Rnd*Rage", cwidth, 1, 1, false) + "\n"
+                      : _resist_composer("Clarity", cwidth, rclar) + "\n";
+
+    const int rcons = you.conservation(calc_unid);
+    out += _resist_composer("Conserve", cwidth, rcons) + "\n";
+    const int rcorr = you.res_corr(calc_unid);
+    out += _resist_composer("rCorr", cwidth, rcorr) + "\n";
+    const int rrott = you.res_rotting();
+    out += _resist_composer("rRot", cwidth, rrott) + "\n";
+    const int rspir = you.spirit_shield(calc_unid);
+    out += _resist_composer("Spirit", cwidth, rspir) + "\n";
+    const int rward = you.warding(calc_unid);
+    out += _resist_composer("Warding", cwidth, rward) + "\n";
+
     const int notele = you.no_tele(calc_unid);
     const int rrtel = !!player_teleport(calc_unid);
     if (notele && !stasis)
-    {
-        snprintf(buf, sizeof buf, "%sPrev.Telep.: %s",
-                 _determine_colour_string(-1, 1), _itosym1(1));
-    }
+        out += _resist_composer("NoTele", cwidth, 1, 1, false) + "\n";
     else if (rrtel && !stasis)
-    {
-        snprintf(buf, sizeof buf, "%sRnd.Telep. : %s",
-                 _determine_colour_string(-1, 1), _itosym1(1));
-    }
+        out += _resist_composer("Rnd*Tele", cwidth, 1, 1, false) + "\n";
     else
-    {
-        snprintf(buf, sizeof buf, "%sStasis     : %s",
-                 _determine_colour_string(stasis, 1), _itosym1(stasis));
-    }
-    cols.add_formatted(1, buf, false);
+        out += _resist_composer("Stasis", cwidth, stasis) + "\n";
+    cols.add_formatted(1, out, false);
 
-    const int rflyi = you.airborne();
-    snprintf(buf, sizeof buf,
-             "%sFlight     : %s\n",
-             _determine_colour_string(rflyi, 1), _itosym1(rflyi));
-    cols.add_formatted(1, buf, false);
+    const int no_cast = you.no_cast(calc_unid);
+    if (no_cast)
+    {
+        out.clear();
+        out += _resist_composer("NoCast", cwidth, 1, 1, false);
+        cols.add_formatted(1, out, false);
+    }
 
     _print_overview_screen_equip(cols, equip_chars, sw);
 
@@ -2369,7 +2438,7 @@ string stealth_desc(int stealth)
          (stealth < 400) ? "extraordinarily " :
          (stealth < 520) ? "incredibly "
                          : "uncannily ";
-    return (prefix + "stealthy");
+    return prefix + "stealthy";
 }
 
 string magic_res_adjective(int mr)
@@ -2378,15 +2447,11 @@ string magic_res_adjective(int mr)
         return "immune";
 
     string prefix =
-            (mr <  10) ? "not" :
-            (mr <  30) ? "slightly" :
-            (mr <  60) ? "somewhat" :
-            (mr <  90) ? "quite" :
+            (mr <  40) ? "not" :
+            (mr <  80) ? "somewhat" :
             (mr < 120) ? "very" :
-            (mr < 150) ? "extremely" :
-            (mr < 190) ? "extraordinarily" :
-            (mr < 240) ? "incredibly" :
-            (mr < 300) ? "uncannily"
+            (mr < 160) ? "extremely" :
+            (mr < 200) ? "incredibly"
                        : "almost entirely";
     return prefix + " resistant";
 }
@@ -2394,7 +2459,7 @@ string magic_res_adjective(int mr)
 static string _annotate_form_based(string desc, bool suppressed)
 {
     if (suppressed)
-        return ("<darkgrey>(" + desc + ")</darkgrey>");
+        return "<darkgrey>(" + desc + ")</darkgrey>";
     else
         return desc;
 }
@@ -2415,7 +2480,8 @@ static string _status_mut_abilities(int sw)
     string text = "<w>@:</w> ";
     vector<string> status;
 
-    const int statuses[] = {
+    const int statuses[] =
+    {
         DUR_TRANSFORMATION,
         DUR_PARALYSIS,
         DUR_PETRIFIED,
@@ -2438,7 +2504,7 @@ static string _status_mut_abilities(int sw)
         DUR_DEATH_CHANNEL,
         DUR_PHASE_SHIFT,
         DUR_SILENCE,
-        DUR_INVIS,
+        STATUS_INVISIBLE,
         DUR_CONF,
         DUR_EXHAUSTED,
         DUR_MIGHT,
@@ -2448,8 +2514,6 @@ static string _status_mut_abilities(int sw)
         DUR_DIVINE_STAMINA,
         DUR_BERSERK,
         STATUS_AIRBORNE,
-        DUR_BARGAIN,
-        DUR_SLAYING,
         STATUS_MANUAL,
         STATUS_SAGE,
         DUR_MAGIC_SHIELD,
@@ -2478,19 +2542,24 @@ static string _status_mut_abilities(int sw)
         STATUS_UMBRA,
         STATUS_CONSTRICTED,
         STATUS_AUGMENTED,
-        STATUS_SUPPRESSED,
         STATUS_RECALL,
         STATUS_LIQUEFIED,
         DUR_FLAYED,
         DUR_RETCHING,
         DUR_WEAK,
         DUR_DIMENSION_ANCHOR,
-        DUR_SPIRIT_HOWL,
         STATUS_DRAINED,
         DUR_TOXIC_RADIANCE,
         DUR_RECITE,
         DUR_GRASPING_ROOTS,
         DUR_FIRE_VULN,
+        DUR_POISON_VULN,
+        DUR_FROZEN,
+        DUR_SAP_MAGIC,
+        STATUS_MAGIC_SAPPED,
+        DUR_FORESTED,
+        DUR_DRAGON_CALL,
+        DUR_ABJURATION_AURA,
     };
 
     status_info inf;
@@ -2526,11 +2595,6 @@ static string _status_mut_abilities(int sw)
     text += "<w>A:</w> ";
 
     int AC_change  = 0;
-    int EV_change  = 0;
-    int SH_change  = 0;
-    int Str_change = 0;
-    int Int_change = 0;
-    int Dex_change = 0;
 
     vector<string> mutations;
 
@@ -2650,16 +2714,26 @@ static string _status_mut_abilities(int sw)
         mutations.push_back(_dragon_abil("breathe steam"));
         break;
 
+    case SP_FORMICID:
+        mutations.push_back("permanent stasis");
+        mutations.push_back("dig shafts and tunnels");
+        mutations.push_back("four strong arms");
+        break;
+
     case SP_GARGOYLE:
+        if (you.experience_level >= 14)
+            mutations.push_back("able to fly continuously");
         AC_change += 2 + you.experience_level * 2 / 5
                        + max(0, you.experience_level - 7) * 2 / 5;
         break;
 
+#if TAG_MAJOR_VERSION == 34
     case SP_DJINNI:
         mutations.push_back("fire immunity");
         mutations.push_back("cold vulnerability");
         break;
 
+#endif
     default:
         break;
     }                           //end switch - innate abilities
@@ -2680,7 +2754,7 @@ static string _status_mut_abilities(int sw)
     if (you.species == SP_FELID)
     {
         mutations.push_back("no armour");
-        mutations.push_back("no advanced items");
+        mutations.push_back("no weapons or thrown items");
     }
 
     if (you.species == SP_OCTOPODE)
@@ -2711,128 +2785,14 @@ static string _status_mut_abilities(int sw)
 
         current = "";
 
-        if (mdef.short_desc)
+        current += mdef.short_desc;
+
+        if (mdef.levels > 1)
         {
-            current += mdef.short_desc;
+            ostringstream ostr;
+            ostr << ' ' << level;
 
-            if (mdef.levels > 1)
-            {
-                ostringstream ostr;
-                ostr << ' ' << level;
-
-                current += ostr.str();
-            }
-        }
-        else if (level)
-        {
-            switch (i)
-            {
-            case MUT_TOUGH_SKIN:
-                AC_change += level;
-                break;
-            case MUT_SHAGGY_FUR:
-                AC_change += level;
-                break;
-            case MUT_STRONG:
-                Str_change += level * 2;
-                break;
-            case MUT_CLEVER:
-                Int_change += level * 2;
-                break;
-            case MUT_AGILE:
-                Dex_change += level * 2;
-                break;
-            case MUT_WEAK:
-                Str_change -= level * 2;
-                break;
-            case MUT_DOPEY:
-                Int_change -= level * 2;
-                break;
-            case MUT_CLUMSY:
-                Dex_change -= level * 2;
-                break;
-#if TAG_MAJOR_VERSION == 34
-            case MUT_STRONG_STIFF:
-                Str_change += level;
-                Dex_change -= level;
-                break;
-            case MUT_FLEXIBLE_WEAK:
-                Str_change -= level;
-                Dex_change += level;
-                break;
-#endif
-            case MUT_FRAIL:
-                snprintf(info, INFO_SIZE, "-%d%% hp", level*10);
-                current = info;
-                break;
-            case MUT_ROBUST:
-                snprintf(info, INFO_SIZE, "+%d%% hp", level*10);
-                current = info;
-                break;
-            case MUT_LOW_MAGIC:
-                snprintf(info, INFO_SIZE, "-%d%% mp", level*10);
-                current = info;
-                break;
-            case MUT_HIGH_MAGIC:
-                snprintf(info, INFO_SIZE, "+%d%% mp", level*10);
-                current = info;
-                break;
-            case MUT_ICEMAIL:
-                AC_change += player_icemail_armour_class();
-                break;
-            case MUT_EYEBALLS:
-                 snprintf(info, INFO_SIZE, "+%d accuracy", level*2+1);
-                 current = info;
-                 break;
-
-            // scales -> calculate sum of AC bonus
-            case MUT_DISTORTION_FIELD:
-                EV_change += level + 1;
-                if (level == 3)
-                    current = "repel missiles";
-                break;
-            case MUT_ICY_BLUE_SCALES:
-                AC_change += level + (level > 1 ? 1 : 0);
-                EV_change -= level > 1 ? 1 : 0;
-                break;
-            case MUT_IRIDESCENT_SCALES:
-                AC_change += 2*level+2;
-                break;
-            case MUT_LARGE_BONE_PLATES:
-                AC_change += level + 1;
-                SH_change += level * 2;
-                break;
-            case MUT_MOLTEN_SCALES:
-                AC_change += level + (level > 1 ? 1 : 0);
-                EV_change -= level > 1 ? 1 : 0;
-                break;
-            case MUT_ROUGH_BLACK_SCALES:
-                AC_change  += 3*level+1;
-                Dex_change -= level;
-                break;
-            case MUT_RUGGED_BROWN_SCALES:
-                AC_change += level;
-                break;
-            case MUT_SLIMY_GREEN_SCALES:
-                AC_change += level + 1;
-                break;
-            case MUT_THIN_METALLIC_SCALES:
-                AC_change += level + 1;
-                break;
-            case MUT_THIN_SKELETAL_STRUCTURE:
-                Dex_change += 2 * level;
-                Str_change -= (level - 1);
-                break;
-            case MUT_YELLOW_SCALES:
-                AC_change += level + 1;
-                break;
-            case MUT_GELATINOUS_BODY:
-                AC_change += (level == 3) ? 2 : 1;
-                EV_change += level - 1;
-                break;
-            default:
-                die("mutation without a short desc: %d", i);
-            }
+            current += ostr.str();
         }
 
         if (!current.empty())
@@ -2852,31 +2812,6 @@ static string _status_mut_abilities(int sw)
         snprintf(info, INFO_SIZE, "AC %s%d", (AC_change > 0 ? "+" : ""), AC_change);
         mutations.push_back(info);
     }
-    if (EV_change)
-    {
-        snprintf(info, INFO_SIZE, "EV %s%d", (EV_change > 0 ? "+" : ""), EV_change);
-        mutations.push_back(info);
-    }
-    if (SH_change)
-    {
-        snprintf(info, INFO_SIZE, "SH %s%d", (SH_change > 0 ? "+" : ""), SH_change);
-        mutations.push_back(info);
-    }
-    if (Str_change)
-    {
-        snprintf(info, INFO_SIZE, "Str %s%d", (Str_change > 0 ? "+" : ""), Str_change);
-        mutations.push_back(info);
-    }
-    if (Int_change)
-    {
-        snprintf(info, INFO_SIZE, "Int %s%d", (Int_change > 0 ? "+" : ""), Int_change);
-        mutations.push_back(info);
-    }
-    if (Dex_change)
-    {
-        snprintf(info, INFO_SIZE, "Dex %s%d", (Dex_change > 0 ? "+" : ""), Dex_change);
-        mutations.push_back(info);
-    }
 
     if (mutations.empty())
         text +=  "no striking features";
@@ -2892,6 +2827,12 @@ static string _status_mut_abilities(int sw)
 
     text += print_abilities();
 
+    //--------------
+    // print the Orb
+    //--------------
+    if (player_has_orb())
+        text += "\n<w>0:</w> Orb of Zot";
+
     //--------------
     // print runes
     //--------------
