@@ -18,7 +18,6 @@
 #include "food.h"
 #include "goditem.h"
 #include "hints.h"
-#include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
 #include "libutil.h"
@@ -26,6 +25,7 @@
 #include "monster.h"
 #include "spl-damage.h"
 #include "state.h"
+#include "stuff.h"
 #include "terrain.h"
 #include "transform.h"
 #include "traps.h"
@@ -135,12 +135,13 @@ bool player::floundering() const
 bool player::extra_balanced() const
 {
     const dungeon_feature_type grid = grd(pos());
-    return (species == SP_GREY_DRACONIAN
-               || grid == DNGN_SHALLOW_WATER
-                   && (species == SP_NAGA // tails, not feet
-                       || body_size(PSIZE_BODY) >= SIZE_LARGE)
-                   && (form == TRAN_LICH || form == TRAN_STATUE
-                       || !form_changed_physiology()));
+    return species == SP_GREY_DRACONIAN
+              || grid == DNGN_SHALLOW_WATER
+                  && (species == SP_NAGA // tails, not feet
+                      || body_size(PSIZE_BODY) >= SIZE_LARGE)
+                  && (form == TRAN_LICH || form == TRAN_STATUE
+                      || form == TRAN_SHADOW
+                      || !form_changed_physiology());
 }
 
 int player::get_experience_level() const
@@ -158,10 +159,14 @@ bool player::is_habitable_feat(dungeon_feature_type actual_grid) const
     if (!can_pass_through_feat(actual_grid))
         return false;
 
-    if (airborne() || species == SP_DJINNI)
+    if (airborne()
+#if TAG_MAJOR_VERSION == 34
+            || species == SP_DJINNI
+#endif
+            )
         return true;
 
-    if (actual_grid == DNGN_LAVA
+    if (actual_grid == DNGN_LAVA && species != SP_LAVA_ORC
         || actual_grid == DNGN_DEEP_WATER && !can_swim())
     {
         return false;
@@ -177,8 +182,8 @@ size_type player::body_size(size_part_type psize, bool base) const
     else
     {
         size_type tf_size = transform_size(form, psize);
-        return (tf_size == SIZE_CHARACTER ? species_size(species, psize)
-                                          : tf_size);
+        return tf_size == SIZE_CHARACTER ? species_size(species, psize)
+                                         : tf_size;
     }
 }
 
@@ -195,6 +200,7 @@ int player::body_weight(bool base) const
         weight *= 2;
         break;
     case TRAN_LICH:
+    case TRAN_SHADOW:
         weight /= 2;
         break;
     default:
@@ -206,7 +212,7 @@ int player::body_weight(bool base) const
 
 int player::total_weight() const
 {
-    return (body_weight() + burden);
+    return body_weight() + burden;
 }
 
 int player::damage_type(int)
@@ -230,7 +236,7 @@ brand_type player::damage_brand(int)
 
     if (wpn != -1 && !melded[EQ_WEAPON])
     {
-        if (!is_range_weapon(inv[wpn]) && !suppressed())
+        if (!is_range_weapon(inv[wpn]))
             ret = get_weapon_brand(inv[wpn]);
     }
     else if (duration[DUR_CONFUSING_TOUCH])
@@ -269,7 +275,7 @@ brand_type player::damage_brand(int)
 }
 
 // Returns the item in the given equipment slot, NULL if the slot is empty.
-// eq must be in [EQ_WEAPON, EQ_RING_EIGHT], or bad things will happen.
+// eq must be in [EQ_WEAPON, EQ_RING_AMULET], or bad things will happen.
 item_def *player::slot_item(equipment_type eq, bool include_melded) const
 {
     ASSERT_RANGE(eq, EQ_WEAPON, NUM_EQUIP);
@@ -277,7 +283,7 @@ item_def *player::slot_item(equipment_type eq, bool include_melded) const
     const int item = equip[eq];
     if (item == -1 || !include_melded && melded[eq])
         return NULL;
-    return (const_cast<item_def *>(&inv[item]));
+    return const_cast<item_def *>(&inv[item]);
 }
 
 // Returns the item in the player's weapon slot.
@@ -289,6 +295,15 @@ item_def *player::weapon(int /* which_attack */) const
     return slot_item(EQ_WEAPON, false);
 }
 
+// Give hands required to wield weapon.
+hands_reqd_type player::hands_reqd(const item_def &item) const
+{
+    if (species == SP_FORMICID)
+        return HANDS_ONE;
+    else
+        return actor::hands_reqd(item);
+}
+
 bool player::can_wield(const item_def& item, bool ignore_curse,
                        bool ignore_brand, bool ignore_shield,
                        bool ignore_transform) const
@@ -301,7 +316,7 @@ bool player::can_wield(const item_def& item, bool ignore_curse,
 
     // Unassigned means unarmed combat.
     const bool two_handed = item.base_type == OBJ_UNASSIGNED
-                            || hands_reqd(item, body_size()) == HANDS_TWO;
+                            || hands_reqd(item) == HANDS_TWO;
 
     if (two_handed && !ignore_shield && player_wearing_slot(EQ_SHIELD))
         return false;
@@ -314,6 +329,7 @@ bool player::could_wield(const item_def &item, bool ignore_brand,
 {
     if (species == SP_FELID)
         return false;
+
     if (body_size(PSIZE_TORSO, ignore_transform) < SIZE_LARGE
             && (item_mass(item) >= 500
                 || item.base_type == OBJ_WEAPONS
@@ -415,6 +431,7 @@ string player::hand_name(bool plural, bool *can_plural) const
     else if (form == TRAN_JELLY)
         str = "bump"; // not even pseudopods...
     else if (form == TRAN_LICH || form == TRAN_STATUE
+             || form == TRAN_SHADOW
              || !form_changed_physiology())
     {
         if (species == SP_FELID)
@@ -426,7 +443,7 @@ string player::hand_name(bool plural, bool *can_plural) const
     }
 
     if (str.empty())
-        return (plural ? "hands" : "hand");
+        return plural ? "hands" : "hand";
 
     if (plural && *can_plural)
         str = pluralise(str);
@@ -452,6 +469,7 @@ string player::foot_name(bool plural, bool *can_plural) const
     else if (form == TRAN_JELLY)
         str = "underside", *can_plural = false;
     else if (form == TRAN_LICH || form == TRAN_STATUE
+             || form == TRAN_SHADOW
              || !form_changed_physiology())
     {
         if (player_mutation_level(MUT_HOOVES) >= 3)
@@ -470,6 +488,10 @@ string player::foot_name(bool plural, bool *can_plural) const
         }
         else if (species == SP_FELID)
             str = "paw";
+#if TAG_MAJOR_VERSION == 34
+        else if (species == SP_DJINNI)
+            str = "underside", *can_plural = false;
+#endif
         else if (fishtail)
         {
             str         = "tail";
@@ -478,7 +500,7 @@ string player::foot_name(bool plural, bool *can_plural) const
     }
 
     if (str.empty())
-        return (plural ? "feet" : "foot");
+        return plural ? "feet" : "foot";
 
     if (plural && *can_plural)
         str = pluralise(str);
@@ -508,6 +530,8 @@ string player::arm_name(bool plural, bool *can_plural) const
 
     if (form == TRAN_LICH)
         adj = "bony";
+    else if (form == TRAN_SHADOW)
+        adj = "shadowy";
 
     if (!adj.empty())
         str = adj + " " + str;
@@ -653,7 +677,7 @@ bool player::can_go_berserk(bool intentional, bool potion, bool quiet) const
         return false;
     }
 
-    if (beheld() && !player_equip_unrand_effect(UNRAND_DEMON_AXE))
+    if (beheld() && !player_equip_unrand(UNRAND_DEMON_AXE))
     {
         if (verbose)
             mpr("You are too mesmerised to rage.");
@@ -669,6 +693,16 @@ bool player::can_go_berserk(bool intentional, bool potion, bool quiet) const
         return false;
     }
 
+#if TAG_MAJOR_VERSION == 34
+    if (you.species == SP_DJINNI)
+    {
+        if (verbose)
+            mpr("Only creatures of flesh and blood can berserk.");
+
+        return false;
+    }
+
+#endif
     if (is_lifeless_undead())
     {
         if (verbose)
@@ -684,11 +718,7 @@ bool player::can_go_berserk(bool intentional, bool potion, bool quiet) const
     if (stasis(false))
     {
         if (verbose)
-        {
-            const item_def *amulet = slot_item(EQ_AMULET, false);
-            mprf("You cannot go berserk with %s on.",
-                 amulet? amulet->name(DESC_YOUR).c_str() : "your amulet");
-        }
+            mpr("You cannot go berserk while under stasis.");
         return false;
     }
 
@@ -704,8 +734,8 @@ bool player::can_go_berserk(bool intentional, bool potion, bool quiet) const
         return false;
     }
 
-    ASSERT(HUNGER_STARVING - 100 + BERSERK_NUTRITION < 2066);
-    if (hunger <= 2066)
+    COMPILE_CHECK(HUNGER_STARVING - 100 + BERSERK_NUTRITION < HUNGER_VERY_HUNGRY);
+    if (hunger <= HUNGER_VERY_HUNGRY)
     {
         if (verbose)
             mpr("You're too hungry to go berserk.");
@@ -715,6 +745,58 @@ bool player::can_go_berserk(bool intentional, bool potion, bool quiet) const
     return true;
 }
 
+bool player::can_jump(bool quiet) const
+{
+    if (duration[DUR_EXHAUSTED])
+    {
+        if (!quiet)
+            mpr("You're too exhausted to jump.");
+        return false;
+    }
+    if (in_water())
+    {
+        if (!quiet)
+            mpr("You can't jump while in water.");
+        return false;
+    }
+    if (in_lava())
+    {
+        if (!quiet)
+            mpr("You can't jump while standing in lava.");
+        return false;
+    }
+    if (liquefied_ground())
+    {
+        if (!quiet)
+            mpr("You can't jump while stuck in this mess.");
+        return false;
+    }
+    if (is_constricted())
+    {
+        if (!quiet)
+            mpr("You can't jump while being constricted.");
+        return false;
+    }
+    if (caught())
+    {
+        if (!quiet)
+            mprf("You can't jump while %s.", held_status());
+        return false;
+    }
+    if (form == TRAN_TREE || form == TRAN_WISP)
+    {
+        if (!quiet)
+            canned_msg(MSG_PRESENT_FORM);
+        return false;
+    }
+    return true;
+}
+
+bool player::can_jump() const
+{
+    return can_jump(false);
+}
+
 bool player::berserk() const
 {
     return duration[DUR_BERSERK];
@@ -722,7 +804,7 @@ bool player::berserk() const
 
 bool player::can_cling_to_walls() const
 {
-    return form == TRAN_SPIDER;
+    return form == TRAN_SPIDER || player_equip_unrand(UNRAND_SPIDER);
 }
 
 bool player::is_web_immune() const
