@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 2.7.12-4996.  */
+/* A Bison parser, made by GNU Bison 2.5.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.7.12-4996"
+#define YYBISON_VERSION "2.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -58,11 +58,14 @@
 /* Pull parsers.  */
 #define YYPULL 1
 
+/* Using locations.  */
+#define YYLSP_NEEDED 0
 
 
 
 /* Copy the first part of user declarations.  */
-/* Line 371 of yacc.c  */
+
+/* Line 268 of yacc.c  */
 #line 1 "levcomp.ypp"
 
 
@@ -95,16 +98,14 @@ static NORETURN void yyerror(const char *e)
 }
 
 
-/* Line 371 of yacc.c  */
-#line 100 "levcomp.tab.c"
 
-# ifndef YY_NULL
-#  if defined __cplusplus && 201103L <= __cplusplus
-#   define YY_NULL nullptr
-#  else
-#   define YY_NULL 0
-#  endif
-# endif
+/* Line 268 of yacc.c  */
+#line 104 "levcomp.tab.cc"
+
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
 
 /* Enabling verbose error messages.  */
 #ifdef YYERROR_VERBOSE
@@ -114,18 +115,12 @@ static NORETURN void yyerror(const char *e)
 # define YYERROR_VERBOSE 0
 #endif
 
-/* In a future release of Bison, this section will be replaced
-   by #include "levcomp.tab.h".  */
-#ifndef YY_YY_LEVCOMP_TAB_H_INCLUDED
-# define YY_YY_LEVCOMP_TAB_H_INCLUDED
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int yydebug;
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
 #endif
 
+
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
@@ -240,10 +235,12 @@ extern int yydebug;
 
 
 
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 387 of yacc.c  */
+
+/* Line 293 of yacc.c  */
 #line 34 "levcomp.ypp"
 
     int i;
@@ -252,36 +249,21 @@ typedef union YYSTYPE
     map_chance_pair chance;
 
 
-/* Line 387 of yacc.c  */
-#line 257 "levcomp.tab.c"
+
+/* Line 293 of yacc.c  */
+#line 255 "levcomp.tab.cc"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
-extern YYSTYPE yylval;
-
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int yyparse (void *YYPARSE_PARAM);
-#else
-int yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int yyparse (void);
-#else
-int yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
-
-#endif /* !YY_YY_LEVCOMP_TAB_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
 
-/* Line 390 of yacc.c  */
-#line 285 "levcomp.tab.c"
+
+/* Line 343 of yacc.c  */
+#line 267 "levcomp.tab.cc"
 
 #ifdef short
 # undef short
@@ -334,33 +316,24 @@ typedef short int yytype_int16;
 # if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
+#   define YY_(msgid) dgettext ("bison-runtime", msgid)
 #  endif
 # endif
 # ifndef YY_
-#  define YY_(Msgid) Msgid
-# endif
-#endif
-
-#ifndef __attribute__
-/* This feature is available in gcc versions 2.5 and later.  */
-# if (! defined __GNUC__ || __GNUC__ < 2 \
-      || (__GNUC__ == 2 && __GNUC_MINOR__ < 5))
-#  define __attribute__(Spec) /* empty */
+#  define YY_(msgid) msgid
 # endif
 #endif
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YYUSE(e) ((void) (e))
 #else
-# define YYUSE(E) /* empty */
+# define YYUSE(e) /* empty */
 #endif
 
-
 /* Identity function, used to suppress warnings about constant conditions.  */
 #ifndef lint
-# define YYID(N) (N)
+# define YYID(n) (n)
 #else
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
@@ -396,7 +369,6 @@ YYID (yyi)
 #    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS && (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 #     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
-      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
 #     ifndef EXIT_SUCCESS
 #      define EXIT_SUCCESS 0
 #     endif
@@ -488,20 +460,20 @@ union yyalloc
 #endif
 
 #if defined YYCOPY_NEEDED && YYCOPY_NEEDED
-/* Copy COUNT objects from SRC to DST.  The source and destination do
+/* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
 #  if defined __GNUC__ && 1 < __GNUC__
-#   define YYCOPY(Dst, Src, Count) \
-      __builtin_memcpy (Dst, Src, (Count) * sizeof (*(Src)))
+#   define YYCOPY(To, From, Count) \
+      __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
-#   define YYCOPY(Dst, Src, Count)              \
-      do                                        \
-        {                                       \
-          YYSIZE_T yyi;                         \
-          for (yyi = 0; yyi < (Count); yyi++)   \
-            (Dst)[yyi] = (Src)[yyi];            \
-        }                                       \
+#   define YYCOPY(To, From, Count)		\
+      do					\
+	{					\
+	  YYSIZE_T yyi;				\
+	  for (yyi = 0; yyi < (Count); yyi++)	\
+	    (To)[yyi] = (From)[yyi];		\
+	}					\
       while (YYID (0))
 #  endif
 # endif
@@ -662,7 +634,7 @@ static const yytype_uint16 yyrline[] =
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || 0
+#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
 /* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
@@ -694,7 +666,7 @@ static const char *const yytname[] =
   "depth", "chance", "chance_specifiers", "chance_roll", "chance_num",
   "chance_specifier", "weight", "weight_specifiers", "weight_specifier",
   "orientation", "welcome", "map_def", "map_lines", "map_line", "subvault",
-  "subvault_specifiers", "subvault_specifier", YY_NULL
+  "subvault_specifiers", "subvault_specifier", 0
 };
 #endif
 
@@ -867,10 +839,10 @@ static const yytype_uint8 yytable[] =
        0,   169
 };
 
-#define yypact_value_is_default(Yystate) \
-  (!!((Yystate) == (-132)))
+#define yypact_value_is_default(yystate) \
+  ((yystate) == (-132))
 
-#define yytable_value_is_error(Yytable_value) \
+#define yytable_value_is_error(yytable_value) \
   YYID (0)
 
 static const yytype_int16 yycheck[] =
@@ -945,35 +917,62 @@ static const yytype_uint8 yystos[] =
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
-#define YYBACKUP(Token, Value)                                  \
-do                                                              \
-  if (yychar == YYEMPTY)                                        \
-    {                                                           \
-      yychar = (Token);                                         \
-      yylval = (Value);                                         \
-      YYPOPSTACK (yylen);                                       \
-      yystate = *yyssp;                                         \
-      goto yybackup;                                            \
-    }                                                           \
-  else                                                          \
-    {                                                           \
+#define YYBACKUP(Token, Value)					\
+do								\
+  if (yychar == YYEMPTY && yylen == 1)				\
+    {								\
+      yychar = (Token);						\
+      yylval = (Value);						\
+      YYPOPSTACK (1);						\
+      goto yybackup;						\
+    }								\
+  else								\
+    {								\
       yyerror (YY_("syntax error: cannot back up")); \
       YYERROR;							\
     }								\
 while (YYID (0))
 
-/* Error token number */
+
 #define YYTERROR	1
 #define YYERRCODE	256
 
 
+/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
+   If N is 0, then set CURRENT to the empty location which ends
+   the previous symbol: RHS[0] (always defined).  */
+
+#define YYRHSLOC(Rhs, K) ((Rhs)[K])
+#ifndef YYLLOC_DEFAULT
+# define YYLLOC_DEFAULT(Current, Rhs, N)				\
+    do									\
+      if (YYID (N))                                                    \
+	{								\
+	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
+	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
+	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
+	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
+	}								\
+      else								\
+	{								\
+	  (Current).first_line   = (Current).last_line   =		\
+	    YYRHSLOC (Rhs, 0).last_line;				\
+	  (Current).first_column = (Current).last_column =		\
+	    YYRHSLOC (Rhs, 0).last_column;				\
+	}								\
+    while (YYID (0))
+#endif
+
+
 /* This macro is provided for backward compatibility. */
+
 #ifndef YY_LOCATION_PRINT
 # define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
+
 #ifdef YYLEX_PARAM
 # define YYLEX yylex (YYLEX_PARAM)
 #else
@@ -1023,8 +1022,6 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
     YYSTYPE const * const yyvaluep;
 #endif
 {
-  FILE *yyo = yyoutput;
-  YYUSE (yyo);
   if (!yyvaluep)
     return;
 # ifdef YYPRINT
@@ -1033,7 +1030,11 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
 # else
   YYUSE (yyoutput);
 # endif
-  YYUSE (yytype);
+  switch (yytype)
+    {
+      default:
+	break;
+    }
 }
 
 
@@ -1272,11 +1273,12 @@ static int
 yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                 yytype_int16 *yyssp, int yytoken)
 {
-  YYSIZE_T yysize0 = yytnamerr (YY_NULL, yytname[yytoken]);
+  YYSIZE_T yysize0 = yytnamerr (0, yytname[yytoken]);
   YYSIZE_T yysize = yysize0;
+  YYSIZE_T yysize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
-  const char *yyformat = YY_NULL;
+  const char *yyformat = 0;
   /* Arguments of yyformat. */
   char const *yyarg[YYERROR_VERBOSE_ARGS_MAXIMUM];
   /* Number of reported tokens (one for the "unexpected", one per
@@ -1336,13 +1338,11 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                     break;
                   }
                 yyarg[yycount++] = yytname[yyx];
-                {
-                  YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
-                  if (! (yysize <= yysize1
-                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-                    return 2;
-                  yysize = yysize1;
-                }
+                yysize1 = yysize + yytnamerr (0, yytname[yyx]);
+                if (! (yysize <= yysize1
+                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  return 2;
+                yysize = yysize1;
               }
         }
     }
@@ -1362,12 +1362,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 # undef YYCASE_
     }
 
-  {
-    YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
-    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-      return 2;
-    yysize = yysize1;
-  }
+  yysize1 = yysize + yystrlen (yyformat);
+  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    return 2;
+  yysize = yysize1;
 
   if (*yymsg_alloc < yysize)
     {
@@ -1423,26 +1421,36 @@ yydestruct (yymsg, yytype, yyvaluep)
     yymsg = "Deleting";
   YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);
 
-  YYUSE (yytype);
+  switch (yytype)
+    {
+
+      default:
+	break;
+    }
 }
 
 
+/* Prevent warnings from -Wmissing-prototypes.  */
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void *YYPARSE_PARAM);
+#else
+int yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void);
+#else
+int yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
 
 /* The lookahead symbol.  */
 int yychar;
 
-
-#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_END
-#endif
-#ifndef YY_INITIAL_VALUE
-# define YY_INITIAL_VALUE(Value) /* Nothing. */
-#endif
-
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE yylval YY_INITIAL_VALUE(yyval_default);
+YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
 int yynerrs;
@@ -1482,7 +1490,7 @@ yyparse ()
        `yyss': related to states.
        `yyvs': related to semantic values.
 
-       Refer to the stacks through separate pointers, to allow yyoverflow
+       Refer to the stacks thru separate pointers, to allow yyoverflow
        to reallocate them elsewhere.  */
 
     /* The state stack.  */
@@ -1500,7 +1508,7 @@ yyparse ()
   int yyn;
   int yyresult;
   /* Lookahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  int yytoken;
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE yyval;
@@ -1518,8 +1526,9 @@ yyparse ()
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yyssp = yyss = yyssa;
-  yyvsp = yyvs = yyvsa;
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
   yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1528,6 +1537,14 @@ yyparse ()
   yyerrstatus = 0;
   yynerrs = 0;
   yychar = YYEMPTY; /* Cause a token to be read.  */
+
+  /* Initialize stack pointers.
+     Waste one element of value and location stack
+     so that they stay on the same level as the state stack.
+     The wasted elements are never initialized.  */
+  yyssp = yyss;
+  yyvsp = yyvs;
+
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1668,9 +1685,7 @@ yybackup:
   yychar = YYEMPTY;
 
   yystate = yyn;
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
   goto yynewstate;
 
@@ -1707,37 +1722,43 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 60 "levcomp.ypp"
     { }
     break;
 
   case 3:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 63 "levcomp.ypp"
     {}
     break;
 
   case 4:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 64 "levcomp.ypp"
     {}
     break;
 
   case 5:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 67 "levcomp.ypp"
     {}
     break;
 
   case 6:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 68 "levcomp.ypp"
     {}
     break;
 
   case 7:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 72 "levcomp.ypp"
     {
                     yyerror("Unexpected character sequence.");
@@ -1745,19 +1766,22 @@ yyreduce:
     break;
 
   case 8:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 77 "levcomp.ypp"
     {}
     break;
 
   case 9:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 78 "levcomp.ypp"
     {}
     break;
 
   case 10:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 82 "levcomp.ypp"
     {
                     dgn_reset_default_depth();
@@ -1769,7 +1793,8 @@ yyreduce:
     break;
 
   case 11:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 92 "levcomp.ypp"
     {
                     lc_map.set_file(lc_desfile);
@@ -1792,31 +1817,36 @@ yyreduce:
     break;
 
   case 12:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 112 "levcomp.ypp"
     { }
     break;
 
   case 13:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 113 "levcomp.ypp"
     { }
     break;
 
   case 14:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 116 "levcomp.ypp"
     { }
     break;
 
   case 15:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 117 "levcomp.ypp"
     { }
     break;
 
   case 16:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 121 "levcomp.ypp"
     {
                     lc_map.init();
@@ -1842,31 +1872,36 @@ yyreduce:
     break;
 
   case 55:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 182 "levcomp.ypp"
     {}
     break;
 
   case 56:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 185 "levcomp.ypp"
     { }
     break;
 
   case 57:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 187 "levcomp.ypp"
     { }
     break;
 
   case 58:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 188 "levcomp.ypp"
     { }
     break;
 
   case 59:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 192 "levcomp.ypp"
     {
                     lc_global_prelude.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -1874,25 +1909,29 @@ yyreduce:
     break;
 
   case 60:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 196 "levcomp.ypp"
     { }
     break;
 
   case 61:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 198 "levcomp.ypp"
     { }
     break;
 
   case 62:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 199 "levcomp.ypp"
     { }
     break;
 
   case 63:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 203 "levcomp.ypp"
     {
                     lc_map.main.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -1900,25 +1939,29 @@ yyreduce:
     break;
 
   case 64:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 207 "levcomp.ypp"
     { }
     break;
 
   case 65:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 209 "levcomp.ypp"
     { }
     break;
 
   case 66:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 210 "levcomp.ypp"
     { }
     break;
 
   case 67:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 214 "levcomp.ypp"
     {
                     lc_map.validate.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -1926,25 +1969,29 @@ yyreduce:
     break;
 
   case 68:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 218 "levcomp.ypp"
     { }
     break;
 
   case 69:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 220 "levcomp.ypp"
     { }
     break;
 
   case 70:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 221 "levcomp.ypp"
     { }
     break;
 
   case 71:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 225 "levcomp.ypp"
     {
                     lc_map.veto.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -1952,25 +1999,29 @@ yyreduce:
     break;
 
   case 72:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 229 "levcomp.ypp"
     { }
     break;
 
   case 73:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 231 "levcomp.ypp"
     { }
     break;
 
   case 74:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 232 "levcomp.ypp"
     { }
     break;
 
   case 75:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 236 "levcomp.ypp"
     {
                     lc_map.prelude.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -1978,25 +2029,29 @@ yyreduce:
     break;
 
   case 76:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 240 "levcomp.ypp"
     { }
     break;
 
   case 77:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 242 "levcomp.ypp"
     { }
     break;
 
   case 78:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 243 "levcomp.ypp"
     { }
     break;
 
   case 79:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 247 "levcomp.ypp"
     {
                     lc_map.epilogue.add(yylineno, (yyvsp[(1) - (1)].text));
@@ -2004,13 +2059,15 @@ yyreduce:
     break;
 
   case 80:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 251 "levcomp.ypp"
     { }
     break;
 
   case 81:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 253 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2021,13 +2078,15 @@ yyreduce:
     break;
 
   case 82:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 260 "levcomp.ypp"
     { }
     break;
 
   case 83:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 262 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2038,13 +2097,15 @@ yyreduce:
     break;
 
   case 84:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 269 "levcomp.ypp"
     { }
     break;
 
   case 85:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 271 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2055,13 +2116,15 @@ yyreduce:
     break;
 
   case 86:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 278 "levcomp.ypp"
     { }
     break;
 
   case 87:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 280 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2072,13 +2135,15 @@ yyreduce:
     break;
 
   case 88:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 287 "levcomp.ypp"
     { }
     break;
 
   case 89:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 289 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2089,13 +2154,15 @@ yyreduce:
     break;
 
   case 90:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 296 "levcomp.ypp"
     {}
     break;
 
   case 93:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 304 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2106,7 +2173,8 @@ yyreduce:
     break;
 
   case 94:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 312 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2117,13 +2185,15 @@ yyreduce:
     break;
 
   case 95:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 320 "levcomp.ypp"
     {}
     break;
 
   case 98:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 328 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2134,7 +2204,8 @@ yyreduce:
     break;
 
   case 99:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 337 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2145,7 +2216,8 @@ yyreduce:
     break;
 
   case 100:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 346 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2156,7 +2228,8 @@ yyreduce:
     break;
 
   case 101:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 355 "levcomp.ypp"
     {
                     string key, arg;
@@ -2191,19 +2264,22 @@ yyreduce:
     break;
 
   case 102:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 387 "levcomp.ypp"
     { }
     break;
 
   case 103:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 390 "levcomp.ypp"
     { }
     break;
 
   case 104:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 392 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2214,13 +2290,15 @@ yyreduce:
     break;
 
   case 105:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 399 "levcomp.ypp"
     { }
     break;
 
   case 106:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 401 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2231,13 +2309,15 @@ yyreduce:
     break;
 
   case 107:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 408 "levcomp.ypp"
     { }
     break;
 
   case 108:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 410 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2248,13 +2328,15 @@ yyreduce:
     break;
 
   case 109:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 418 "levcomp.ypp"
     { }
     break;
 
   case 110:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 420 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2265,7 +2347,8 @@ yyreduce:
     break;
 
   case 114:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 435 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2276,7 +2359,8 @@ yyreduce:
     break;
 
   case 118:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 451 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2287,7 +2371,8 @@ yyreduce:
     break;
 
   case 122:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 467 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2298,7 +2383,8 @@ yyreduce:
     break;
 
   case 126:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 483 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2309,19 +2395,22 @@ yyreduce:
     break;
 
   case 127:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 492 "levcomp.ypp"
     { }
     break;
 
   case 128:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 493 "levcomp.ypp"
     { }
     break;
 
   case 129:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 497 "levcomp.ypp"
     {
                       lc_map.main.add(
@@ -2332,25 +2421,29 @@ yyreduce:
     break;
 
   case 130:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 505 "levcomp.ypp"
     { }
     break;
 
   case 131:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 508 "levcomp.ypp"
     { }
     break;
 
   case 132:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 509 "levcomp.ypp"
     { }
     break;
 
   case 133:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 513 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2361,13 +2454,15 @@ yyreduce:
     break;
 
   case 134:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 521 "levcomp.ypp"
     { }
     break;
 
   case 137:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 529 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2378,19 +2473,22 @@ yyreduce:
     break;
 
   case 138:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 537 "levcomp.ypp"
     {}
     break;
 
   case 139:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 538 "levcomp.ypp"
     {}
     break;
 
   case 142:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 546 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2401,19 +2499,22 @@ yyreduce:
     break;
 
   case 143:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 553 "levcomp.ypp"
     {}
     break;
 
   case 144:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 554 "levcomp.ypp"
     {}
     break;
 
   case 147:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 562 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2424,7 +2525,8 @@ yyreduce:
     break;
 
   case 148:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 571 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2435,7 +2537,8 @@ yyreduce:
     break;
 
   case 149:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 580 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2446,7 +2549,8 @@ yyreduce:
     break;
 
   case 150:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 589 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2456,13 +2560,15 @@ yyreduce:
     break;
 
   case 151:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 596 "levcomp.ypp"
     {}
     break;
 
   case 152:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 598 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2473,19 +2579,22 @@ yyreduce:
     break;
 
   case 153:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 606 "levcomp.ypp"
     { }
     break;
 
   case 154:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 607 "levcomp.ypp"
     { }
     break;
 
   case 157:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 614 "levcomp.ypp"
     {
                     (yyval.i) = (yyvsp[(1) - (2)].f) * 100;
@@ -2493,7 +2602,8 @@ yyreduce:
     break;
 
   case 158:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 618 "levcomp.ypp"
     {
                     (yyval.i) = (yyvsp[(1) - (1)].f);
@@ -2501,7 +2611,8 @@ yyreduce:
     break;
 
   case 159:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 623 "levcomp.ypp"
     {
                     (yyval.chance).priority = (yyvsp[(1) - (3)].f);
@@ -2510,7 +2621,8 @@ yyreduce:
     break;
 
   case 160:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 628 "levcomp.ypp"
     {
                     (yyval.chance).priority = DEFAULT_CHANCE_PRIORITY;
@@ -2519,7 +2631,8 @@ yyreduce:
     break;
 
   case 161:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 635 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2531,7 +2644,8 @@ yyreduce:
     break;
 
   case 162:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 643 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2542,7 +2656,8 @@ yyreduce:
     break;
 
   case 166:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 657 "levcomp.ypp"
     {
                      lc_map.main.add(
@@ -2553,7 +2668,8 @@ yyreduce:
     break;
 
   case 167:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 664 "levcomp.ypp"
     {
                      lc_map.main.add(
@@ -2563,13 +2679,15 @@ yyreduce:
     break;
 
   case 168:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 671 "levcomp.ypp"
     {}
     break;
 
   case 169:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 673 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2580,7 +2698,8 @@ yyreduce:
     break;
 
   case 170:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 682 "levcomp.ypp"
     {
                     lc_map.main.add(
@@ -2591,7 +2710,8 @@ yyreduce:
     break;
 
   case 174:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 698 "levcomp.ypp"
     {
                     lc_map.mapchunk.add(
@@ -2602,7 +2722,8 @@ yyreduce:
     break;
 
   case 178:
-/* Line 1787 of yacc.c  */
+
+/* Line 1806 of yacc.c  */
 #line 714 "levcomp.ypp"
     {
                        lc_map.main.add(
@@ -2613,8 +2734,9 @@ yyreduce:
     break;
 
 
-/* Line 1787 of yacc.c  */
-#line 2618 "levcomp.tab.c"
+
+/* Line 1806 of yacc.c  */
+#line 2740 "levcomp.tab.cc"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -2777,9 +2899,7 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
 
   /* Shift the error token.  */
@@ -2803,7 +2923,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#if !defined yyoverflow || YYERROR_VERBOSE
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2845,6 +2965,8 @@ yyreturn:
 }
 
 
-/* Line 2050 of yacc.c  */
+
+/* Line 2067 of yacc.c  */
 #line 722 "levcomp.ypp"
 
+
