@@ -54,7 +54,8 @@ void random_var::init()
     for (int v = start; v < end; ++v)
         total += weight(v);
     ASSERT(total > 0);
-    ASSERT(weight(start) > 0 && weight(end - 1) > 0);
+    ASSERT(weight(start) > 0);
+    ASSERT(weight(end - 1) > 0);
 }
 
 int random_var::roll2val(int r) const
@@ -78,7 +79,7 @@ int random_var::roll() const
 
 int random_var::max() const
 {
-    return (end - 1);
+    return end - 1;
 }
 
 int random_var::min() const
@@ -94,7 +95,6 @@ double random_var::expected() const
     return ev;
 }
 
-
 //////////////////////////////////
 
 random_var constant(int n)
@@ -129,7 +129,7 @@ random_var negate(const random_var& x)
 
 random_var operator-(const random_var& x, const random_var& y)
 {
-    return (x + ::negate(y));
+    return x + ::negate(y);
 }
 
 const random_var& operator+=(random_var& x, const random_var& y)
