@@ -48,7 +48,7 @@ public:
     static string get(const string &_key)
     {
         skill_op_map::const_iterator i = Skill_Op_Map.find(_key);
-        return (i == Skill_Op_Map.end()? string() : (i->second)());
+        return i == Skill_Op_Map.end()? string() : (i->second)();
     }
 private:
     const char *key;
@@ -157,7 +157,7 @@ int get_skill_progress(skill_type sk, int level, int points, int scale)
     // A scale as small as 92 would overflow with 31 bits if skill_rdiv()
     // is involved: needed can be 91985, skill_rdiv() multiplies by 256.
     const int64_t amt_done = points - prev_needed;
-    int prog = (amt_done * scale) / (needed - prev_needed);
+    int prog = amt_done * scale / (needed - prev_needed);
 
     ASSERT(prog >= 0);
 
@@ -176,14 +176,14 @@ int get_skill_percentage(const skill_type x)
 
 const char *skill_name(skill_type which_skill)
 {
-    return (skills[which_skill][0]);
+    return skills[which_skill][0];
 }
 
 skill_type str_to_skill(const string &skill)
 {
     for (int i = SK_FIRST_SKILL; i < NUM_SKILLS; ++i)
         if (skills[i][0] && skill == skills[i][0])
-            return (static_cast<skill_type>(i));
+            return static_cast<skill_type>(i);
 
     return SK_FIGHTING;
 }
@@ -206,17 +206,20 @@ static string _stk_genus_nocap()
 
 static string _stk_genus_short_cap()
 {
-    return (Skill_Species == SP_DEMIGOD ? "God" :
-            _stk_genus_cap());
+    return Skill_Species == SP_DEMIGOD ? "God" :
+           _stk_genus_cap();
 }
 
 static string _stk_walker()
 {
-    return (Skill_Species == SP_NAGA     ? "Slider" :
-            Skill_Species == SP_TENGU    ? "Glider" :
-            Skill_Species == SP_DJINNI   ? "Floater" :
-            Skill_Species == SP_OCTOPODE ? "Wriggler"
-                                         : "Walker");
+    return Skill_Species == SP_NAGA         ? "Slider"   :
+           Skill_Species == SP_TENGU        ? "Glider"   :
+#if TAG_MAJOR_VERSION == 34
+           Skill_Species == SP_DJINNI       ? "Floater"  :
+#endif
+           Skill_Species == SP_OCTOPODE     ? "Wriggler" :
+           Skill_Species == SP_VINE_STALKER ? "Stalker"
+                                            : "Walker";
 }
 
 static string _stk_weight()
@@ -296,11 +299,11 @@ static string _replace_skill_keys(const string &text)
 unsigned get_skill_rank(unsigned skill_lev)
 {
     // Translate skill level into skill ranking {dlb}:
-    return ((skill_lev <= 7)  ? 0 :
-                            (skill_lev <= 14) ? 1 :
-                            (skill_lev <= 20) ? 2 :
-                            (skill_lev <= 26) ? 3
-                            /* level 27 */    : 4);
+    return (skill_lev <= 7)  ? 0 :
+                           (skill_lev <= 14) ? 1 :
+                           (skill_lev <= 20) ? 2 :
+                           (skill_lev <= 26) ? 3
+                           /* level 27 */    : 4;
 }
 
 string skill_title_by_rank(skill_type best_skill, uint8_t skill_rank,
@@ -345,6 +348,15 @@ string skill_title_by_rank(skill_type best_skill, uint8_t skill_rank,
 
             break;
 
+        case SK_SHORT_BLADES:
+            if (player_genus(GENPC_ELVEN, static_cast<species_type>(species))
+                && skill_rank == 5)
+            {
+                result = "Blademaster";
+                break;
+            }
+            break;
+
         case SK_INVOCATIONS:
             if (god != GOD_NO_GOD)
                 result = god_title((god_type)god, (species_type)species, piety);
@@ -382,8 +394,8 @@ string skill_title_by_rank(skill_type best_skill, uint8_t skill_rank,
         result = _replace_skill_keys(result);
     }
 
-    return (result.empty() ? string("Invalid Title")
-                           : result);
+    return result.empty() ? string("Invalid Title")
+                          : result;
 }
 
 string skill_title(skill_type best_skill, uint8_t skill_lev,
@@ -475,27 +487,6 @@ void init_skill_order(void)
     }
 }
 
-void calc_hp()
-{
-    you.hp_max = get_real_hp(true, false);
-    if (you.species == SP_DJINNI)
-        you.hp_max += get_real_mp(true);
-    deflate_hp(you.hp_max, false);
-}
-
-void calc_mp()
-{
-    if (you.species == SP_DJINNI)
-    {
-        you.magic_points = you.max_magic_points = 0;
-        return calc_hp();
-    }
-
-    you.max_magic_points = get_real_mp(true);
-    you.magic_points = min(you.magic_points, you.max_magic_points);
-    you.redraw_magic_points = true;
-}
-
 bool is_useless_skill(skill_type skill)
 {
 #if TAG_MAJOR_VERSION == 34
@@ -537,7 +528,7 @@ int skill_bump(skill_type skill, int scale)
 
 static float _apt_to_factor(int apt)
 {
-    return (1 / exp(log(2) * apt / APT_DOUBLE));
+    return 1 / exp(log(2) * apt / APT_DOUBLE);
 }
 
 unsigned int skill_exp_needed(int lev, skill_type sk, species_type sp)
@@ -581,7 +572,7 @@ float species_apt_factor(skill_type sk, species_type sp)
     return _apt_to_factor(species_apt(sk, sp));
 }
 
-static vector<skill_type> _get_crosstrain_skills(skill_type sk)
+vector<skill_type> get_crosstrain_skills(skill_type sk)
 {
     vector<skill_type> ret;
 
@@ -614,14 +605,11 @@ static vector<skill_type> _get_crosstrain_skills(skill_type sk)
     }
 }
 
-// This threshold is in tenths of a skill point.
-#define CROSSTRAIN_THRESHOLD 1
-
 float crosstrain_bonus(skill_type sk)
 {
     int bonus = 1;
 
-    vector<skill_type> crosstrain_skills = _get_crosstrain_skills(sk);
+    vector<skill_type> crosstrain_skills = get_crosstrain_skills(sk);
 
     for (unsigned int i = 0; i < crosstrain_skills.size(); ++i)
         if (you.skill(crosstrain_skills[i], 10, true)
@@ -633,22 +621,7 @@ float crosstrain_bonus(skill_type sk)
     return bonus;
 }
 
-bool crosstrain_other(skill_type sk, bool show_zero)
-{
-    vector<skill_type> crosstrain_skills = _get_crosstrain_skills(sk);
-
-    for (unsigned int i = 0; i < crosstrain_skills.size(); ++i)
-        if (you.skill(crosstrain_skills[i], 10, true)
-            <= you.skill(sk, 10, true) - CROSSTRAIN_THRESHOLD
-           && (you.skills[crosstrain_skills[i]] > 0 || show_zero))
-        {
-            return true;
-        }
-
-    return false;
-}
-
-static skill_type _get_opposite(skill_type sk)
+skill_type opposite_skill(skill_type sk)
 {
     switch (sk)
     {
@@ -662,10 +635,10 @@ static skill_type _get_opposite(skill_type sk)
 
 static int _skill_elemental_preference(skill_type sk, int scale)
 {
-    const skill_type sk2 = _get_opposite(sk);
+    const skill_type sk2 = opposite_skill(sk);
     if (sk2 == SK_NONE)
         return 0;
-    return (you.skill(sk, scale) - you.skill(sk2, scale));
+    return you.skill(sk, scale) - you.skill(sk2, scale);
 }
 
 int elemental_preference(spell_type spell, int scale)
@@ -687,35 +660,25 @@ int elemental_preference(spell_type spell, int scale)
  * @param sk2 Second skill.
  * @return Whether first skill is higher than second skill.
  */
-static bool _compare_skills(skill_type sk1, skill_type sk2)
+bool compare_skills(skill_type sk1, skill_type sk2)
 {
     if (is_invalid_skill(sk1))
         return false;
     else if (is_invalid_skill(sk2))
         return true;
     else
-        return (you.skill(sk1, 10, true) > you.skill(sk2, 10, true)
-                || you.skill(sk1, 10, true) == you.skill(sk2, 10, true)
-                   && you.skill_order[sk1] < you.skill_order[sk2]);
+        return you.skill(sk1, 10, true) > you.skill(sk2, 10, true)
+               || you.skill(sk1, 10, true) == you.skill(sk2, 10, true)
+                  && you.skill_order[sk1] < you.skill_order[sk2];
 }
 
 bool is_antitrained(skill_type sk)
 {
-    skill_type opposite = _get_opposite(sk);
+    skill_type opposite = opposite_skill(sk);
     if (opposite == SK_NONE || you.skills[sk] >= 27)
         return false;
 
-    return _compare_skills(opposite, sk) && you.skills[opposite];
-}
-
-bool antitrain_other(skill_type sk, bool show_zero)
-{
-    skill_type opposite = _get_opposite(sk);
-    if (opposite == SK_NONE)
-        return false;
-
-    return ((you.skills[opposite] > 0 || show_zero) && you.skills[sk] > 0
-            && you.skills[opposite] < 27 && _compare_skills(sk, opposite));
+    return compare_skills(opposite, sk) && you.skills[opposite];
 }
 
 void dump_skills(string &text)
