@@ -3,7 +3,6 @@
  * @brief Spell casting functions.
 **/
 
-
 #ifndef SPL_CAST_H
 #define SPL_CAST_H
 
@@ -12,12 +11,12 @@
 enum spflag_type
 {
     SPFLAG_NONE                 = 0x000000,
-    SPFLAG_DIR_OR_TARGET        = 0x000001,      // use DIR_NONE targetting
-    SPFLAG_TARGET               = 0x000002,      // use DIR_TARGET targetting
-    SPFLAG_GRID                 = 0x000004,      // use DIR_GRID targetting
-    SPFLAG_DIR                  = 0x000008,      // use DIR_DIR targetting
+    SPFLAG_DIR_OR_TARGET        = 0x000001,      // use DIR_NONE targeting
+    SPFLAG_TARGET               = 0x000002,      // use DIR_TARGET targeting
+    SPFLAG_GRID                 = 0x000004,      // use DIR_GRID targeting
+    SPFLAG_DIR                  = 0x000008,      // use DIR_DIR targeting
     SPFLAG_TARG_OBJ             = 0x000010,      // use DIR_TARGET_OBJECT targ.
-    SPFLAG_TARGETTING_MASK      = 0x00001f,      // used to test for targetting
+    SPFLAG_TARGETING_MASK       = 0x00001f,      // used to test for targeting
     SPFLAG_HELPFUL              = 0x000020,      // TARG_FRIENDS used
     SPFLAG_NEUTRAL              = 0x000040,      // TARG_ANY used
     SPFLAG_NOT_SELF             = 0x000080,      // aborts on isMe
