@@ -3,7 +3,6 @@
  * @brief data handlers for player spell list
 **/
 
-
 #ifndef SPL_UTIL_H
 #define SPL_UTIL_H
 
@@ -112,7 +111,7 @@ void apply_area_cloud(cloud_func func, const coord_def& where,
                       int excl_rad = -1);
 
 bool spell_direction(dist &spelld, bolt &pbolt,
-                      targetting_type restrict = DIR_NONE,
+                      targeting_type restrict = DIR_NONE,
                       targ_mode_type mode = TARG_HOSTILE,
                       // pbolt.range if applicable, otherwise LOS_RADIUS
                       int range = 0,
