@@ -25,6 +25,7 @@
 #include "hints.h"
 #include "initfile.h"
 #include "itemname.h"
+#include "itemprop.h"
 #include "items.h"
 #include "libutil.h"
 #include "macro.h"
@@ -98,6 +99,7 @@ static void _initialize()
     info[0] = 0;
 
     reset_all_monsters();
+    init_anon();
 
     igrd.init(NON_ITEM);
     mgrd.init(NON_MONSTER);
@@ -144,11 +146,6 @@ static void _initialize()
     if (crawl_state.build_db)
         end(0);
 
-    if (!crawl_state.io_inited)
-        cio_init();
-
-    // System initialisation stuff.
-    textbackground(0);
 #ifdef USE_TILE_LOCAL
     if (!Options.tile_skip_title && crawl_state.title_screen)
     {
@@ -157,16 +154,22 @@ static void _initialize()
     }
 #endif
 
-    clrscr();
-
 #ifdef DEBUG_DIAGNOSTICS
     if (crawl_state.map_stat_gen)
     {
+        release_cli_signals();
         generate_map_stats();
         end(0, false);
     }
 #endif
 
+    if (!crawl_state.test_list)
+    {
+        if (!crawl_state.io_inited)
+            cio_init();
+        clrscr();
+    }
+
     if (crawl_state.test)
     {
 #if defined(DEBUG_TESTS) && !defined(DEBUG)
@@ -179,10 +182,8 @@ static void _initialize()
 #endif
         dgn_reset_level();
         crawl_state.show_more_prompt = false;
-        run_tests(true);
-        // Superfluous, just to make it clear that this is the end of
-        // the line.
-        end(0, false);
+        run_tests();
+        // doesn't return
 #else
         end(1, false, "Non-debug Crawl cannot run tests. "
             "Please use a debug build (defined FULLDEBUG, DEBUG_DIAGNOSTIC "
@@ -240,9 +241,11 @@ static void _post_init(bool newc)
     // Debug compiles display a lot of "hidden" information, so we auto-wiz.
     you.wizard = true;
 #endif
+#ifdef WIZARD
     // Save-less games are pointless except for tests.
     if (Options.no_save)
         you.wizard = true;
+#endif
 
     init_properties();
     burden_change();
@@ -308,6 +311,10 @@ static void _post_init(bool newc)
     }
 
     // This just puts the view up for the first turn.
+    you.redraw_title = true;
+    textcolor(LIGHTGREY);
+    set_redraw_status(REDRAW_LINE_2_MASK | REDRAW_LINE_3_MASK);
+    print_stats();
     viewwindow();
 
     activate_notes(true);
@@ -525,9 +532,9 @@ static int _find_save(const vector<player_save_info>& chars, const string& name)
 
 static bool _game_defined(const newgame_def& ng)
 {
-    return (ng.type != NUM_GAME_TYPE
-            && ng.species != SP_UNKNOWN
-            && ng.job != JOB_UNKNOWN);
+    return ng.type != NUM_GAME_TYPE
+           && ng.species != SP_UNKNOWN
+           && ng.job != JOB_UNKNOWN;
 }
 
 static const int SCROLLER_MARGIN_X  = 18;
@@ -916,7 +923,6 @@ bool startup_step()
     if (!SysEnv.crawl_name.empty())
         choice.name = SysEnv.crawl_name;
 
-
 #ifndef DGAMELAUNCH
     if (crawl_state.last_type == GAME_TYPE_TUTORIAL
         || crawl_state.last_type == GAME_TYPE_SPRINT)
