@@ -16,10 +16,10 @@ enum status_type
     STATUS_HUNGER,
     STATUS_REGENERATION,
     STATUS_ROT,
+    STATUS_DIG,
     STATUS_SICK,
     STATUS_SPEED,
     STATUS_CLINGING,
-    STATUS_HOVER,
     STATUS_SAGE,
     STATUS_STR_ZERO,
     STATUS_INT_ZERO,
@@ -30,7 +30,6 @@ enum status_type
     STATUS_CONSTRICTED,
     STATUS_MANUAL,
     STATUS_AUGMENTED,
-    STATUS_SUPPRESSED,
     STATUS_TERRAIN,
     STATUS_SILENCE,
     STATUS_MISSILES,
@@ -40,7 +39,10 @@ enum status_type
     STATUS_LIQUEFIED,
     STATUS_DRAINED,
     STATUS_RAY,
-    STATUS_LAST_STATUS = STATUS_RAY
+    STATUS_ELIXIR,
+    STATUS_INVISIBLE,
+    STATUS_MAGIC_SAPPED,
+    STATUS_LAST_STATUS = STATUS_MAGIC_SAPPED
 };
 
 struct status_info
