@@ -25,7 +25,6 @@ struct packed_cell
 
     bool is_bloody;
     bool is_silenced;
-    bool is_suppressed;
     char halo;
     bool is_moldy;
     bool glowing_mold;
@@ -44,18 +43,17 @@ struct packed_cell
     bool operator !=(const packed_cell &other) const { return !(*this == other); }
 
     packed_cell() : num_dngn_overlay(0), fg(0), bg(0), cloud(0), is_bloody(false),
-                    is_silenced(false), is_suppressed(false), halo(HALO_NONE),
-                    is_moldy(false), glowing_mold(false), is_sanctuary(false),
-                    is_liquefied(false), mangrove_water(false), orb_glow(0),
-                    blood_rotation(0), old_blood(false), travel_trail(0),
-                    quad_glow(false), disjunct(false), heat_aura(false) {}
+                    is_silenced(false), halo(HALO_NONE), is_moldy(false),
+                    glowing_mold(false), is_sanctuary(false), is_liquefied(false),
+                    mangrove_water(false), orb_glow(0), blood_rotation(0),
+                    old_blood(false), travel_trail(0), quad_glow(false),
+                    disjunct(false), heat_aura(false) {}
 
     packed_cell(const packed_cell* c) : num_dngn_overlay(c->num_dngn_overlay),
                                         fg(c->fg), bg(c->bg), flv(c->flv),
                                         cloud(c->cloud),
                                         is_bloody(c->is_bloody),
                                         is_silenced(c->is_silenced),
-                                        is_suppressed(c->is_suppressed),
                                         halo(c->halo),
                                         is_moldy(c->is_moldy),
                                         glowing_mold(c->glowing_mold),
