@@ -5,7 +5,7 @@
 #include "tilereg-abl.h"
 #include "process_desc.h"
 
-#include "abl-show.h"
+#include "ability.h"
 #include "cio.h"
 #include "libutil.h"
 #include "macro.h"
