@@ -3,6 +3,7 @@
 #include "tileview.h"
 
 #include "areas.h"
+#include "branch.h"
 #include "cloud.h"
 #include "colour.h"
 #include "coord.h"
@@ -15,7 +16,6 @@
 #include "mon-util.h"
 #include "options.h"
 #include "player.h"
-#include "showsymb.h"
 #include "state.h"
 #include "stuff.h"
 #include "terrain.h"
@@ -81,30 +81,35 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
 
     switch (br)
     {
-    case BRANCH_MAIN_DUNGEON:
+    case BRANCH_DUNGEON:
         flv.wall  = TILE_WALL_NORMAL;
         flv.floor = TILE_FLOOR_NORMAL;
         return;
 
+    case BRANCH_DEPTHS:
+        flv.wall  = TILE_WALL_NORMAL;
+        flv.floor = TILE_FLOOR_GREY_DIRT_B;
+        return;
+
     case BRANCH_VAULTS:
         flv.wall  = TILE_WALL_VAULT;
         flv.floor = TILE_FLOOR_VAULT;
         return;
 
-    case BRANCH_ECUMENICAL_TEMPLE:
+    case BRANCH_TEMPLE:
         flv.wall  = TILE_WALL_VINES;
         flv.floor = TILE_FLOOR_VINES;
         return;
 
 #if TAG_MAJOR_VERSION == 34
-    case BRANCH_DWARVEN_HALL:
+    case BRANCH_DWARF:
         flv.wall  = TILE_WALL_HALL;
         flv.floor = TILE_FLOOR_LIMESTONE;
         return;
 #endif
 
-    case BRANCH_ELVEN_HALLS:
-    case BRANCH_HALL_OF_BLADES:
+    case BRANCH_ELF:
+    case BRANCH_BLADE:
         flv.wall  = TILE_WALL_HALL;
         flv.floor = TILE_FLOOR_HALL;
         return;
@@ -124,7 +129,7 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         flv.floor = TILE_FLOOR_TOMB;
         return;
 
-    case BRANCH_VESTIBULE_OF_HELL:
+    case BRANCH_VESTIBULE:
         flv.wall  = TILE_WALL_HELL;
         flv.floor = TILE_FLOOR_CAGE;
         return;
@@ -144,7 +149,7 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         flv.floor = TILE_FLOOR_FROZEN;
         return;
 
-    case BRANCH_ORCISH_MINES:
+    case BRANCH_ORC:
         flv.wall  = TILE_WALL_ORC;
         flv.floor = TILE_FLOOR_ORC;
         return;
@@ -154,12 +159,12 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         flv.floor = TILE_FLOOR_LAIR;
         return;
 
-    case BRANCH_SLIME_PITS:
+    case BRANCH_SLIME:
         flv.wall  = TILE_WALL_SLIME;
         flv.floor = TILE_FLOOR_SLIME;
         return;
 
-    case BRANCH_SNAKE_PIT:
+    case BRANCH_SNAKE:
         flv.wall  = TILE_WALL_SNAKE;
         flv.floor = random_choose(TILE_FLOOR_SNAKE_A,
                                   TILE_FLOOR_SNAKE_C,
@@ -177,21 +182,22 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         flv.floor = TILE_FLOOR_SAND;
         return;
 
-    case BRANCH_SPIDER_NEST:
+    case BRANCH_SPIDER:
         flv.wall  = TILE_WALL_SPIDER;
         flv.floor = TILE_FLOOR_SPIDER;
         return;
 
-    case BRANCH_HALL_OF_ZOT:
+    case BRANCH_ZOT:
         flv.wall  = TILE_WALL_ZOT_YELLOW;
         flv.floor = TILE_FLOOR_TOMB;
         return;
 
+#if TAG_MAJOR_VERSION == 34
     case BRANCH_FOREST:
         flv.wall  = TILE_WALL_LAIR;
         flv.floor = TILE_FLOOR_GRASS;
         return;
-
+#endif
     case BRANCH_ABYSS:
         flv.floor = tile_dngn_coloured(TILE_FLOOR_NERVES, env.floor_colour);
         switch (random2(6))
@@ -272,7 +278,7 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         return;
 
     case BRANCH_ICE_CAVE:
-        flv.wall  = TILE_WALL_ZOT_CYAN;
+        flv.wall  = TILE_WALL_ICE_BLOCK;
         flv.floor = TILE_FLOOR_ICE;
         return;
 
@@ -286,9 +292,6 @@ void tile_default_flv(branch_type br, tile_flavour &flv)
         flv.floor = TILE_FLOOR_NORMAL;
         return;
 
-#if TAG_MAJOR_VERSION == 34
-    case BRANCH_UNUSED:
-#endif
     case NUM_BRANCHES:
         break;
     }
@@ -319,6 +322,8 @@ void tile_init_flavour()
         tile_init_flavour(*ri);
 }
 
+// 11111333333   55555555
+//   222222444444   6666666666
 static void _get_dungeon_wall_tiles_by_depth(int depth, vector<tileidx_t>& t)
 {
     if (crawl_state.game_is_sprint() || crawl_state.game_is_zotdef() || crawl_state.game_is_arena())
@@ -326,27 +331,33 @@ static void _get_dungeon_wall_tiles_by_depth(int depth, vector<tileidx_t>& t)
         t.push_back(TILE_WALL_CATACOMBS);
         return;
     }
-    if (depth <= 6)
+    if (depth <= 5)
         t.push_back(TILE_WALL_BRICK_DARK_1);
-    if (depth > 3 && depth <= 9)
+    if (depth > 2 && depth <= 8)
     {
         t.push_back(TILE_WALL_BRICK_DARK_2);
         t.push_back(TILE_WALL_BRICK_DARK_2_TORCH);
     }
-    if (depth > 6 && depth <= 14)
+    if (depth > 5 && depth <= 11)
         t.push_back(TILE_WALL_BRICK_DARK_3);
-    if (depth > 9 && depth <= 20)
+    if (depth > 8)
     {
         t.push_back(TILE_WALL_BRICK_DARK_4);
         t.push_back(TILE_WALL_BRICK_DARK_4_TORCH);
     }
-    if (depth > 14)
+    if (depth == brdepth[BRANCH_DUNGEON])
+        t.push_back(TILE_WALL_BRICK_DARK_4_TORCH);  // torches are more common on D:14...
+}
+
+static void _get_depths_wall_tiles_by_depth(int depth, vector<tileidx_t>& t)
+{
+    t.push_back(TILE_WALL_BRICK_DARK_6_TORCH);
+    if (depth <= 3)
         t.push_back(TILE_WALL_BRICK_DARK_5);
-    if (depth > 20)
-    {
+    if (depth > 3)
         t.push_back(TILE_WALL_BRICK_DARK_6);
-        t.push_back(TILE_WALL_BRICK_DARK_6_TORCH);
-    }
+    if (depth == brdepth[BRANCH_DEPTHS])
+        t.push_back(TILE_WALL_BRICK_DARK_6_TORCH);  // ...and on Depths:$
 }
 
 static tileidx_t _pick_random_dngn_tile(tileidx_t idx, int value = -1)
@@ -371,7 +382,7 @@ static tileidx_t _pick_random_dngn_tile(tileidx_t idx, int value = -1)
 
 static tileidx_t _pick_random_dngn_tile_multi(vector<tileidx_t> candidates, int value = -1)
 {
-    ASSERT(candidates.size() > 0);
+    ASSERT(!candidates.empty());
 
     int total = 0;
     for (unsigned int i = 0; i < candidates.size(); ++i)
@@ -418,15 +429,24 @@ void tile_init_flavour(const coord_def &gc)
             floor_base = tile_dngn_coloured(floor_base, colour);
         env.tile_flv(gc).floor = _pick_random_dngn_tile(floor_base);
     }
-    else
+    else if (env.tile_flv(gc).floor != TILE_HALO_GRASS
+             && env.tile_flv(gc).floor != TILE_HALO_GRASS2
+             && env.tile_flv(gc).floor != TILE_HALO_VAULT
+             && env.tile_flv(gc).floor != TILE_HALO_DIRT)
+    {
         env.tile_flv(gc).floor = _pick_random_dngn_tile(env.tile_flv(gc).floor);
+    }
 
     if (!env.tile_flv(gc).wall)
     {
-        if (player_in_branch(BRANCH_MAIN_DUNGEON) && env.tile_default.wall == TILE_WALL_NORMAL)
+        if ((player_in_branch(BRANCH_DUNGEON) || player_in_branch(BRANCH_DEPTHS))
+            && env.tile_default.wall == TILE_WALL_NORMAL)
         {
             vector<tileidx_t> tile_candidates;
-            _get_dungeon_wall_tiles_by_depth(you.depth, tile_candidates);
+            if (player_in_branch(BRANCH_DEPTHS))
+                _get_depths_wall_tiles_by_depth(you.depth, tile_candidates);
+            else
+                _get_dungeon_wall_tiles_by_depth(you.depth, tile_candidates);
             env.tile_flv(gc).wall = _pick_random_dngn_tile_multi(tile_candidates);
         }
         else
@@ -748,7 +768,7 @@ static tileidx_t _get_floor_bg(const coord_def& gc)
     {
         bg = tileidx_feature(gc);
 
-        if (is_unknown_stair(gc))
+        if (is_unknown_stair(gc) && (env.map_knowledge(gc).feat() != DNGN_ENTER_ZOT))
             bg |= TILE_FLAG_NEW_STAIR;
     }
 
@@ -959,7 +979,10 @@ struct tile_ray
     coord_def ep;
     aff_type in_range;
 };
-FixedVector<tile_ray, 40> tile_ray_vec;
+// Allow the rays to fill the whole visible area if necessary. The size is
+// about 4/pi times larger than it needs to be, but this way supports squarelos
+// as well.
+FixedVector<tile_ray, ENV_SHOW_DIAMETER * ENV_SHOW_DIAMETER> tile_ray_vec;
 
 void tile_place_ray(const coord_def &gc, aff_type in_range)
 {
@@ -967,19 +990,23 @@ void tile_place_ray(const coord_def &gc, aff_type in_range)
     // rays directly to the screen.  The tiles version doesn't have
     // (nor want) such direct access.  So, it batches up all of the
     // rays and applies them in viewwindow(...).
-    if (num_tile_rays < tile_ray_vec.size() - 1)
-    {
-        tile_ray_vec[num_tile_rays].in_range = in_range;
-        tile_ray_vec[num_tile_rays++].ep = grid2show(gc);
-    }
+    ASSERT(num_tile_rays < tile_ray_vec.size() - 1);
+    tile_ray_vec[num_tile_rays].in_range = in_range;
+    tile_ray_vec[num_tile_rays++].ep = grid2show(gc);
 }
 
 void tile_draw_rays(bool reset_count)
 {
+    tileidx_t flag = 0;
+
     for (unsigned int i = 0; i < num_tile_rays; i++)
     {
-        tileidx_t flag = tile_ray_vec[i].in_range > AFF_MAYBE ? TILE_FLAG_RAY
-                                                           : TILE_FLAG_RAY_OOR;
+        if (tile_ray_vec[i].in_range < AFF_YES)
+            flag = TILE_FLAG_RAY_OOR;
+        else if (tile_ray_vec[i].in_range == AFF_YES)
+            flag = TILE_FLAG_RAY;
+        else if (tile_ray_vec[i].in_range == AFF_LANDING)
+            flag = TILE_FLAG_LANDING;
         env.tile_bg(tile_ray_vec[i].ep) |= flag;
     }
 
@@ -1028,9 +1055,9 @@ void tile_wizmap_terrain(const coord_def &gc)
 
 static bool _is_torch(tileidx_t basetile)
 {
-    return (basetile == TILE_WALL_BRICK_DARK_2_TORCH
-            || basetile == TILE_WALL_BRICK_DARK_4_TORCH
-            || basetile == TILE_WALL_BRICK_DARK_6_TORCH);
+    return basetile == TILE_WALL_BRICK_DARK_2_TORCH
+           || basetile == TILE_WALL_BRICK_DARK_4_TORCH
+           || basetile == TILE_WALL_BRICK_DARK_6_TORCH;
 }
 
 // Updates the "flavour" of tiles that are animated.
@@ -1039,7 +1066,7 @@ void tile_apply_animations(tileidx_t bg, tile_flavour *flv)
 {
 #ifndef USE_TILE_WEB
     tileidx_t bg_idx = bg & TILE_FLAG_MASK;
-    if (bg_idx == TILE_DNGN_PORTAL_WIZARD_LAB)
+    if (bg_idx == TILE_DNGN_PORTAL_WIZARD_LAB && Options.tile_misc_anim)
         flv->special = (flv->special + 1) % tile_dngn_count(bg_idx);
     else if (bg_idx == TILE_DNGN_LAVA && Options.tile_water_anim)
     {
@@ -1049,12 +1076,17 @@ void tile_apply_animations(tileidx_t bg, tile_flavour *flv)
         flv->special = ((flv->special - ((flv->special % 4)))
                         + 4 + random2(4)) % tile_dngn_count(bg_idx);
     }
-    else if (bg_idx > TILE_DNGN_LAVA && bg_idx < TILE_BLOOD
+    else if (bg_idx > TILE_DNGN_LAVA && bg_idx < TILE_DNGN_ENTER_ZOT_CLOSED
              && Options.tile_water_anim)
     {
         flv->special = random2(256);
     }
-    else if (bg_idx == TILE_WALL_NORMAL)
+    else if (bg_idx >= TILE_DNGN_ENTER_ZOT_CLOSED && bg_idx < TILE_BLOOD
+             && Options.tile_misc_anim)
+    {
+        flv->special = random2(256);
+    }
+    else if (bg_idx == TILE_WALL_NORMAL && Options.tile_misc_anim)
     {
         tileidx_t basetile = tile_dngn_basetile(flv->wall);
         if (_is_torch(basetile))
@@ -1069,7 +1101,7 @@ static bool _suppress_blood(const map_cell& mc)
     if (feat_is_tree(feat))
         return true;
 
-    if (feat >= DNGN_FOUNTAIN_BLUE && feat <= DNGN_PERMADRY_FOUNTAIN)
+    if (feat == DNGN_DRY_FOUNTAIN)
         return true;
 
     if (feat_is_altar(feat))
@@ -1216,11 +1248,17 @@ void apply_variations(const tile_flavour &flv, tileidx_t *bg,
         if (orig == TILE_DNGN_STONE_WALL)
             orig = TILE_DNGN_STONE_WALL_BROWN;
     }
-    else if (player_in_branch(BRANCH_SLIME_PITS))
+    else if (player_in_branch(BRANCH_SLIME))
     {
         if (orig == TILE_DNGN_STONE_WALL)
             orig = TILE_STONE_WALL_SLIME;
     }
+    else if (player_in_branch(BRANCH_VAULTS))
+    {
+        if (orig == TILE_DNGN_STONE_WALL)
+            orig = TILE_STONE_WALL_VAULT;
+    }
+
 
     const bool mimic = monster_at(gc) && mons_is_feat_mimic(monster_at(gc)->type);
 
@@ -1268,9 +1306,9 @@ void apply_variations(const tile_flavour &flv, tileidx_t *bg,
 static bool _top_item_is_corpse(const map_cell& mc)
 {
     const item_info* item = mc.item();
-    return (item
-            && item->base_type == OBJ_CORPSES
-            && item->sub_type == CORPSE_BODY);
+    return item
+           && item->base_type == OBJ_CORPSES
+           && item->sub_type == CORPSE_BODY;
 }
 
 static uint8_t _get_direction_index(const coord_def& delta)
@@ -1354,9 +1392,6 @@ void tile_apply_properties(const coord_def &gc, packed_cell &cell)
     if (mc.flags & MAP_SILENCED)
         cell.is_silenced = true;
 
-    if (mc.flags & MAP_SUPPRESSED)
-        cell.is_suppressed = true;
-
     if (feat == DNGN_MANGROVE)
         cell.mangrove_water = true;
 
