@@ -59,6 +59,7 @@ static const char* form_names[] =
     "wisp",
     "jelly",
     "fungus",
+    "shadow",
 };
 
 const char* transform_name(transformation_type form)
@@ -70,8 +71,9 @@ const char* transform_name(transformation_type form)
 
 bool form_can_wield(transformation_type form)
 {
-    return (form == TRAN_NONE || form == TRAN_STATUE || form == TRAN_LICH
-            || form == TRAN_APPENDAGE || form == TRAN_TREE);
+    return form == TRAN_NONE || form == TRAN_STATUE || form == TRAN_LICH
+           || form == TRAN_APPENDAGE || form == TRAN_TREE
+           || form == TRAN_SHADOW;
 }
 
 bool form_can_wear(transformation_type form)
@@ -81,9 +83,11 @@ bool form_can_wear(transformation_type form)
 
 bool form_can_fly(transformation_type form)
 {
+    if (form == TRAN_TREE)
+        return false;
     if (you.racial_permanent_flight() && you.permanent_flight())
         return true;
-    return (form == TRAN_DRAGON || form == TRAN_BAT || form == TRAN_WISP);
+    return form == TRAN_DRAGON || form == TRAN_BAT || form == TRAN_WISP;
 }
 
 bool form_can_swim(transformation_type form)
@@ -92,8 +96,11 @@ bool form_can_swim(transformation_type form)
     if (form == TRAN_ICE_BEAST || form == TRAN_JELLY)
         return true;
 
-    if (you.species == SP_MERFOLK && !form_changed_physiology(form))
+    if ((you.species == SP_MERFOLK || you.species == SP_OCTOPODE)
+        && (!form_changed_physiology(form) || form == TRAN_LICH))
+    {
         return true;
+    }
 
     if (you.species == SP_OCTOPODE && !form_changed_physiology(form))
         return true;
@@ -102,46 +109,52 @@ bool form_can_swim(transformation_type form)
     if (size == SIZE_CHARACTER)
         size = you.body_size(PSIZE_BODY, true);
 
-    return (size >= SIZE_GIANT);
+    return size >= SIZE_GIANT;
 }
 
 bool form_likes_water(transformation_type form)
 {
-    return (form_can_swim(form) || you.species == SP_GREY_DRACONIAN
-                                   && !form_changed_physiology(form));
-}
-
-bool form_has_mouth(transformation_type form)
-{
-    return form != TRAN_TREE
-        && form != TRAN_WISP
-        && form != TRAN_JELLY
-        && form != TRAN_FUNGUS;
+    // Grey dracs can't swim, so can't statue form merfolk/octopodes
+    // -- yet they can still survive in water.
+    if (you.species == SP_GREY_DRACONIAN || you.species == SP_MERFOLK
+        || you.species == SP_OCTOPODE)
+    {
+        if (form == TRAN_NONE
+            || form == TRAN_BLADE_HANDS
+            || form == TRAN_APPENDAGE
+            || form == TRAN_LICH
+            || form == TRAN_STATUE)
+        {
+            return true;
+        }
+    }
+    return form_can_swim(form);
 }
 
 bool form_likes_lava(transformation_type form)
 {
     // Lava orcs can only swim in non-phys-change forms.
-    return (you.species == SP_LAVA_ORC
-            && !form_changed_physiology(form));
+    return you.species == SP_LAVA_ORC
+           && !form_changed_physiology(form);
 }
 
-bool form_can_butcher_barehanded(transformation_type form)
+bool form_can_butcher(transformation_type form)
 {
-    return (form == TRAN_BLADE_HANDS || form == TRAN_DRAGON
-            || form == TRAN_ICE_BEAST);
+    return form != TRAN_WISP
+        && form != TRAN_JELLY
+        && form != TRAN_FUNGUS;
 }
 
 // Used to mark transformations which override species intrinsics.
 bool form_changed_physiology(transformation_type form)
 {
-    return (form != TRAN_NONE && form != TRAN_APPENDAGE
-            && form != TRAN_BLADE_HANDS);
+    return form != TRAN_NONE && form != TRAN_APPENDAGE
+           && form != TRAN_BLADE_HANDS;
 }
 
 bool form_can_use_wand(transformation_type form)
 {
-    return (form_can_wield(form) || form == TRAN_DRAGON);
+    return form_can_wield(form) || form == TRAN_DRAGON;
 }
 
 bool form_can_wear_item(const item_def& item, transformation_type form)
@@ -155,7 +168,7 @@ bool form_can_wear_item(const item_def& item, transformation_type form)
         if (jewellery_is_amulet(item))
             return true;
         // Bats and pigs can't wear rings.
-        return (form != TRAN_BAT && form != TRAN_PIG);
+        return form != TRAN_BAT && form != TRAN_PIG;
     }
 
     // It's not jewellery, and it's worn, so it must be armour.
@@ -166,6 +179,7 @@ bool form_can_wear_item(const item_def& item, transformation_type form)
     // Some forms can wear everything.
     case TRAN_NONE:
     case TRAN_LICH:
+    case TRAN_SHADOW:
     case TRAN_APPENDAGE: // handled as mutations
         return true;
 
@@ -179,18 +193,18 @@ bool form_can_wear_item(const item_def& item, transformation_type form)
 
     // And some need more complicated logic.
     case TRAN_BLADE_HANDS:
-        return (eqslot != EQ_SHIELD && eqslot != EQ_GLOVES
-                && item.special != UNRAND_LEAR);
+        return eqslot != EQ_SHIELD && eqslot != EQ_GLOVES
+               && item.special != UNRAND_LEAR;
 
     case TRAN_STATUE:
-        return (eqslot == EQ_CLOAK || eqslot == EQ_HELMET
-                || eqslot == EQ_SHIELD);
+        return eqslot == EQ_CLOAK || eqslot == EQ_HELMET
+               || eqslot == EQ_SHIELD;
 
     case TRAN_FUNGUS:
-        return (eqslot == EQ_HELMET && !is_hard_helmet(item));
+        return eqslot == EQ_HELMET && !is_hard_helmet(item);
 
     case TRAN_TREE:
-        return (eqslot == EQ_SHIELD || eqslot == EQ_HELMET);
+        return eqslot == EQ_SHIELD || eqslot == EQ_HELMET;
 
     default:                // Bug-catcher.
         die("Unknown transformation type %d in form_can_wear_item", you.form);
@@ -206,6 +220,7 @@ bool form_keeps_mutations(transformation_type form)
     case TRAN_BLADE_HANDS:
     case TRAN_STATUE:
     case TRAN_LICH:
+    case TRAN_SHADOW:
     case TRAN_APPENDAGE:
         return true;
     default:
@@ -320,8 +335,9 @@ static bool _mutations_prevent_wearing(const item_def& item)
     if (eqslot == EQ_GLOVES && player_mutation_level(MUT_CLAWS) >= 3)
         return true;
 
-    if (eqslot == EQ_HELMET && (player_mutation_level(MUT_HORNS) == 3
-        || player_mutation_level(MUT_ANTENNAE) == 3))
+    if (eqslot == EQ_HELMET
+        && (player_mutation_level(MUT_HORNS) == 3
+            || player_mutation_level(MUT_ANTENNAE) == 3))
     {
         return true;
     }
@@ -470,6 +486,8 @@ monster_type transform_mons()
         return MONS_ANIMATED_TREE;
     case TRAN_WISP:
         return MONS_INSUBSTANTIAL_WISP;
+    case TRAN_SHADOW:
+        return MONS_SHADOW; // XXX: should get its own monster?
     case TRAN_BLADE_HANDS:
     case TRAN_APPENDAGE:
     case TRAN_NONE:
@@ -510,7 +528,7 @@ monster_type dragon_form_dragon_type()
         return MONS_STEAM_DRAGON;
     case SP_RED_DRACONIAN:
     default:
-        return MONS_DRAGON;
+        return MONS_FIRE_DRAGON;
     }
 }
 
@@ -534,6 +552,9 @@ int form_hp_mod()
 
 static bool _flying_in_new_form(transformation_type which_trans)
 {
+    if (which_trans == TRAN_TREE)
+        return false;
+
     // If our flight is uncancellable (or tenguish) then it's not from evoking
     if (you.attribute[ATTR_FLIGHT_UNCANCELLABLE]
         || you.permanent_flight() && you.racial_permanent_flight())
@@ -564,7 +585,7 @@ static bool _flying_in_new_form(transformation_type which_trans)
             sources_removed++;
     }
 
-    return (sources > sources_removed);
+    return sources > sources_removed;
 }
 
 bool feat_dangerous_for_form(transformation_type which_trans,
@@ -574,10 +595,6 @@ bool feat_dangerous_for_form(transformation_type which_trans,
     if (form_can_fly(which_trans) || _flying_in_new_form(which_trans))
         return false;
 
-    // ... or hover our butts up if need arises.
-    if (you.species == SP_DJINNI)
-        return false;
-
     // We can only cling for safety if we're already doing so.
     if (which_trans == TRAN_SPIDER && you.is_wall_clinging())
         return false;
@@ -586,7 +603,19 @@ bool feat_dangerous_for_form(transformation_type which_trans,
         return !form_likes_lava(which_trans);
 
     if (feat == DNGN_DEEP_WATER)
-        return (!form_likes_water(which_trans) && !beogh_water_walk());
+    {
+        if (beogh_water_walk()
+            || species_likes_water(you.species)
+            || you.racial_permanent_flight())
+        {
+            return false;
+        }
+        // Trees are ok with deep water, but you need means of escaping
+        // once the transformation expires.
+        if (which_trans == TRAN_TREE)
+            return !you.wearing_ego(EQ_ALL_ARMOUR, SPARM_FLYING);
+        return !form_likes_water(which_trans);
+    }
 
     return false;
 }
@@ -657,9 +686,11 @@ static bool _transformation_is_safe(transformation_type which_trans,
         if (cloud != EMPTY_CLOUD && is_damaging_cloud(env.cloud[cloud].type, false))
             return false;
     }
+#if TAG_MAJOR_VERSION == 34
 
     if (which_trans == TRAN_ICE_BEAST && you.species == SP_DJINNI)
         return false; // melting is fatal...
+#endif
 
     if (!feat_dangerous_for_form(which_trans, feat))
         return true;
@@ -684,6 +715,7 @@ static int _transform_duration(transformation_type which_trans, int pow)
     case TRAN_STATUE:
     case TRAN_DRAGON:
     case TRAN_LICH:
+    case TRAN_SHADOW:
     case TRAN_BAT:
         return min(20 + random2(pow) + random2(pow), 100);
     case TRAN_ICE_BEAST:
@@ -726,16 +758,13 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
 
     if (you.transform_uncancellable)
     {
-        // Jiyva's wrath-induced transformation is blocking the attempt.
-        // May need to be updated if transform_uncancellable is used for
-        // other uses.
-        if (!just_check)
+        if (!involuntary)
             mpr("You are stuck in your current form!");
         return false;
     }
 
     if (!_transformation_is_safe(which_trans, env.grid(you.pos()),
-        involuntary || just_check))
+        involuntary))
     {
         return false;
     }
@@ -776,6 +805,7 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         {
         case TRAN_STATUE:
         case TRAN_LICH:
+        case TRAN_SHADOW:
             break;
         default:
             skip_wielding = true;
@@ -787,6 +817,7 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
 
     // Catch some conditions which prevent transformation.
     if (you.is_undead
+        && which_trans != TRAN_SHADOW
         && (you.species != SP_VAMPIRE
             || which_trans != TRAN_BAT && you.hunger_state <= HS_SATIATED
             || which_trans == TRAN_LICH))
@@ -894,7 +925,8 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         mutation_type app = _beastly_appendage();
         if (app == NUM_MUTATIONS)
         {
-            mpr("You have no appropriate body parts free.");
+            if (!involuntary)
+                mpr("You have no appropriate body parts free.");
             return false;
         }
 
@@ -956,6 +988,11 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         msg      += "an insubstantial wisp of gas.";
         break;
 
+    case TRAN_SHADOW:
+        tran_name = "shadow";
+        msg      += "a swirling mass of dark shadows.";
+        break;
+
     case TRAN_NONE:
         tran_name = "null";
         msg += "your old self.";
@@ -1001,7 +1038,7 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         you.duration[DUR_STONESKIN] = 0;
 
     // Give the transformation message.
-    mpr(msg);
+    mprf("%s", msg.c_str());
 
     // Update your status.
     you.form = which_trans;
@@ -1037,7 +1074,7 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
             mpr("Your new body is particularly stony.");
         if (you.duration[DUR_ICY_ARMOUR])
         {
-            mpr("Your new body cracks your icy armour.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "Your new body cracks your icy armour.");
             you.duration[DUR_ICY_ARMOUR] = 0;
         }
         break;
@@ -1066,8 +1103,6 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         break;
 
     case TRAN_TREE:
-        if (you_worship(GOD_FEDHAS) && !player_under_penance())
-            simple_god_message(" makes you hardy against extreme temperatures.");
         // ignore hunger_state (but don't reset hunger)
         you.hunger_state = HS_SATIATED;
         set_redraw_status(REDRAW_HUNGER);
@@ -1104,7 +1139,7 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         // undead cannot regenerate -- bwr
         if (you.duration[DUR_REGENERATION])
         {
-            mpr("You stop regenerating.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You stop regenerating.");
             you.duration[DUR_REGENERATION] = 0;
         }
 
@@ -1128,6 +1163,14 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         set_redraw_status(REDRAW_HUNGER);
         break;
 
+    case TRAN_SHADOW:
+        drain_exp(true, 25, true);
+        if (you.invisible())
+            mpr("You fade into the shadows.");
+        else
+            mpr("You feel less conspicuous.");
+        break;
+
     default:
         break;
     }
@@ -1183,9 +1226,22 @@ bool transform(int pow, transformation_type which_trans, bool involuntary,
         move_player_to_grid(you.pos(), false, true);
     }
 
+    if (you.hp <= 0)
+    {
+        ouch(0, NON_MONSTER, KILLED_BY_FRAILTY,
+             make_stringf("gaining the %s transformation", form_names[which_trans]).c_str());
+    }
+
     return true;
 }
 
+/**
+ * End the player's transformation and return them to their normal
+ * form.
+ * @param skip_wielding  If true, don't swap the player's weapon back.
+ * @param skip_move      If true, skip any move that was in progress before
+ *                       the transformation ended.
+ */
 void untransform(bool skip_wielding, bool skip_move)
 {
     const flight_type old_flight = you.flight_mode();
@@ -1211,7 +1267,7 @@ void untransform(bool skip_wielding, bool skip_move)
     switch (old_form)
     {
     case TRAN_SPIDER:
-        mpr("Your transformation has ended.", MSGCH_DURATION);
+        mprf(MSGCH_DURATION, "Your transformation has ended.");
         notify_stat_change(STAT_DEX, -5, true,
                      "losing the spider transformation");
         if (!skip_move)
@@ -1219,7 +1275,7 @@ void untransform(bool skip_wielding, bool skip_move)
         break;
 
     case TRAN_BAT:
-        mpr("Your transformation has ended.", MSGCH_DURATION);
+        mprf(MSGCH_DURATION, "Your transformation has ended.");
         notify_stat_change(STAT_DEX, -5, true,
                      "losing the bat transformation");
         notify_stat_change(STAT_STR, 5, true,
@@ -1237,10 +1293,10 @@ void untransform(bool skip_wielding, bool skip_move)
         if (you.species == SP_LAVA_ORC && temperature_effect(LORC_STONESKIN)
             || you.species == SP_GARGOYLE)
         {
-            mpr("You revert to a slightly less stony form.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You revert to a slightly less stony form.");
         }
         else if (you.species != SP_LAVA_ORC)
-            mpr("You revert to your normal fleshy form.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You revert to your normal fleshy form.");
         notify_stat_change(STAT_DEX, 2, true,
                      "losing the statue transformation");
         notify_stat_change(STAT_STR, -2, true,
@@ -1254,9 +1310,9 @@ void untransform(bool skip_wielding, bool skip_move)
 
     case TRAN_ICE_BEAST:
         if (you.species == SP_LAVA_ORC && !temperature_effect(LORC_STONESKIN))
-            mpr("Your icy form melts away into molten rock.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "Your icy form melts away into molten rock.");
         else
-            mpr("You warm up again.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You warm up again.");
 
         // Note: if the core goes down, the combined effect soon disappears,
         // but the reverse isn't true. -- bwr
@@ -1265,7 +1321,7 @@ void untransform(bool skip_wielding, bool skip_move)
         break;
 
     case TRAN_DRAGON:
-        mpr("Your transformation has ended.", MSGCH_DURATION);
+        mprf(MSGCH_DURATION, "Your transformation has ended.");
         notify_stat_change(STAT_STR, -10, true,
                     "losing the dragon transformation");
         break;
@@ -1273,16 +1329,22 @@ void untransform(bool skip_wielding, bool skip_move)
     case TRAN_LICH:
         you.is_undead = get_undead_state(you.species);
         if (you.is_undead == US_ALIVE)
-            mpr("You feel yourself come back to life.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You feel yourself come back to life.");
         else
-            mpr("You feel your undeath return to normal.", MSGCH_DURATION);
+            mprf(MSGCH_DURATION, "You feel your undeath return to normal.");
         break;
 
     case TRAN_PIG:
     case TRAN_JELLY:
     case TRAN_PORCUPINE:
     case TRAN_WISP:
-        mpr("Your transformation has ended.", MSGCH_DURATION);
+        break;
+
+    case TRAN_SHADOW:
+        if (you.invisible())
+            mprf(MSGCH_DURATION, "You feel less shadowy.");
+        else
+            mprf(MSGCH_DURATION, "You emerge from the shadows.");
         break;
 
     case TRAN_APPENDAGE:
@@ -1299,20 +1361,29 @@ void untransform(bool skip_wielding, bool skip_move)
         break;
 
     case TRAN_FUNGUS:
-        mpr("You stop sporulating.", MSGCH_DURATION);
+        mprf(MSGCH_DURATION, "You stop sporulating.");
         you.set_duration(DUR_CONFUSING_TOUCH, 0);
         break;
     case TRAN_TREE:
-        mpr("You feel less woody.", MSGCH_DURATION);
-        if (grd(you.pos()) == DNGN_DEEP_WATER && you.species == SP_TENGU
-            && you.experience_level >= 5)
+        mprf(MSGCH_DURATION, "You feel less woody.");
+        if (grd(you.pos()) == DNGN_DEEP_WATER)
         {
-            // Flight was disabled, need to turn it back NOW.
-            if (you.experience_level >= 15)
+            if (beogh_water_walk() || species_likes_water(you.species))
+                ; // nothing to do
+            else if (you.racial_permanent_flight()
+                  || you.wearing_ego(EQ_ALL_ARMOUR, SPARM_FLYING))
+            {
                 you.attribute[ATTR_PERM_FLIGHT] = 1;
-            else
+                mpr("You fly out of the water.");
+            }
+            else if (you.species == SP_TENGU
+                     && you.experience_level >= 5)
+            {
                 you.increase_duration(DUR_FLIGHT, 50, 100);
-            mpr("You frantically escape the water.");
+                mpr("You frantically fly out of the water.");
+            }
+            else
+                fall_into_a_pool(you.pos(), true, DNGN_DEEP_WATER);
         }
         notify_stat_change(STAT_STR, -10, true,
                      "losing the tree transformation");
@@ -1347,9 +1418,6 @@ void untransform(bool skip_wielding, bool skip_move)
         init_player_doll();
 #endif
 
-    if (form_can_butcher_barehanded(old_form))
-        stop_butcher_delay();
-
     // If nagas wear boots while transformed, they fall off again afterwards:
     // I don't believe this is currently possible, and if it is we
     // probably need something better to cover all possibilities.  -bwr
@@ -1378,14 +1446,21 @@ void untransform(bool skip_wielding, bool skip_move)
 
     if (hp_downscale != 10 && you.hp != you.hp_max)
     {
-        you.hp = you.hp * 10 / hp_downscale;
-        if (you.hp < 1)
-            you.hp = 1;
-        else if (you.hp > you.hp_max)
-            you.hp = you.hp_max;
+        int hp = you.hp * 10 / hp_downscale;
+        if (hp < 1)
+            hp = 1;
+        else if (hp > you.hp_max)
+            hp = you.hp_max;
+        set_hp(hp);
     }
     calc_hp();
 
+    if (you.hp <= 0)
+    {
+        ouch(0, NON_MONSTER, KILLED_BY_FRAILTY,
+             make_stringf("losing the %s form", form_names[old_form]).c_str());
+    }
+
     // Stop being constricted if we are now too large.
     if (you.is_constricted())
     {
@@ -1395,7 +1470,7 @@ void untransform(bool skip_wielding, bool skip_move)
     }
 
     if (!skip_wielding)
-        handle_interrupted_swap(true, false);
+        handle_interrupted_swap();
 
     you.turn_is_over = true;
     if (you.transform_uncancellable)
