@@ -10,6 +10,7 @@
 #include <errno.h>
 
 #include "acquire.h"
+#include "act-iter.h"
 #include "art-enum.h"
 #include "artefact.h"
 #include "coordit.h"
@@ -17,7 +18,6 @@
 #include "cio.h"
 #include "dbg-util.h"
 #include "decks.h"
-#include "describe.h"
 #include "effects.h"
 #include "env.h"
 #include "godpassive.h"
@@ -28,9 +28,7 @@
 #include "makeitem.h"
 #include "mapdef.h"
 #include "misc.h"
-#include "mon-iter.h"
 #include "mon-stuff.h"
-#include "mon-util.h"
 #include "options.h"
 #include "output.h"
 #include "player-equip.h"
@@ -47,8 +45,7 @@ static void _make_all_books()
 {
     for (int i = 0; i < NUM_FIXED_BOOKS; ++i)
     {
-        int thing = items(0, OBJ_BOOKS, i, true, 0, MAKE_ITEM_NO_RACE,
-                          0, 0, AQ_WIZMODE);
+        int thing = items(0, OBJ_BOOKS, i, true, 0, 0, 0, 0, AQ_WIZMODE);
         if (thing == NON_ITEM)
             continue;
 
@@ -101,13 +98,10 @@ void wizard_create_spec_object()
 
     do
     {
-        mpr(") - weapons     ( - missiles  [ - armour  / - wands    ?  - scrolls",
-            MSGCH_PROMPT);
-        mpr("= - jewellery   ! - potions   : - books   | - staves   \\  - rods",
-            MSGCH_PROMPT);
-        mpr("} - miscellany  X - corpses   % - food    $ - gold     0  - the Orb",
-            MSGCH_PROMPT);
-        mpr("ESC - exit", MSGCH_PROMPT);
+        mprf(MSGCH_PROMPT, ") - weapons     ( - missiles  [ - armour  / - wands    ?  - scrolls");
+        mprf(MSGCH_PROMPT, "= - jewellery   ! - potions   : - books   | - staves   \\  - rods");
+        mprf(MSGCH_PROMPT, "} - miscellany  X - corpses   %% - food    $ - gold     0  - the Orb");
+        mprf(MSGCH_PROMPT, "ESC - exit");
 
         msgwin_prompt("What class of item? ");
 
@@ -246,7 +240,8 @@ void wizard_create_spec_object()
     }
 }
 
-static const char* _prop_name[] = {
+static const char* _prop_name[] =
+{
     "Brand",
     "AC",
     "EV",
@@ -289,13 +284,15 @@ static const char* _prop_name[] = {
     "+Fog",
 #endif
     "Regen",
+    "noupg",
 };
 
 #define ARTP_VAL_BOOL 0
 #define ARTP_VAL_POS  1
 #define ARTP_VAL_ANY  2
 
-static int8_t _prop_type[] = {
+static int8_t _prop_type[] =
+{
     ARTP_VAL_POS,  //BRAND
     ARTP_VAL_ANY,  //AC
     ARTP_VAL_ANY,  //EVASION
@@ -338,6 +335,7 @@ static int8_t _prop_type[] = {
     ARTP_VAL_BOOL, //FOG
 #endif
     ARTP_VAL_ANY,  //REGENERATION
+    ARTP_VAL_BOOL, //NO_UPGRADE
 };
 
 static void _tweak_randart(item_def &item)
@@ -347,8 +345,7 @@ static void _tweak_randart(item_def &item)
 
     if (item_is_equipped(item))
     {
-        mpr("You can't tweak the randart properties of an equipped item.",
-            MSGCH_PROMPT);
+        mprf(MSGCH_PROMPT, "You can't tweak the randart properties of an equipped item.");
         return;
     }
     else
@@ -390,9 +387,9 @@ static void _tweak_randart(item_def &item)
 
         choice_num++;
     }
-    mpr(prompt, MSGCH_PROMPT, 0);
+    mprf(MSGCH_PROMPT, "%s", prompt.c_str());
 
-    mpr("Change which field? ", MSGCH_PROMPT);
+    mprf(MSGCH_PROMPT, "Change which field? ");
 
     int keyin = toalower(get_ch());
     unsigned int  choice;
@@ -469,20 +466,13 @@ void wizard_tweak_object(void)
 
         while (true)
         {
-            mpr_nocap(you.inv[item].name(DESC_INVENTORY_EQUIP).c_str());
+            mprf_nocap("%s", you.inv[item].name(DESC_INVENTORY_EQUIP).c_str());
 
-            if (is_art)
-            {
-                mpr("a - plus  b - plus2  c - art props  d - quantity  "
-                    "e - flags  ESC - exit", MSGCH_PROMPT);
-            }
-            else
-            {
-                mpr("a - plus  b - plus2  c - special  d - quantity  "
-                    "e - flags  ESC - exit", MSGCH_PROMPT);
-            }
+            mprf(MSGCH_PROMPT, "a - plus  b - plus2  c - %s  "
+                               "d - quantity  e - flags  ESC - exit",
+                               is_art ? "art props" : "special");
 
-            mpr("Which field? ", MSGCH_PROMPT);
+            mprf(MSGCH_PROMPT, "Which field? ");
 
             keyin = toalower(get_ch());
 
@@ -559,8 +549,8 @@ void wizard_tweak_object(void)
 // Returns whether an item of this type can be an artefact.
 static bool _item_type_can_be_artefact(int type)
 {
-    return (type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
-            || type == OBJ_BOOKS);
+    return type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
+           || type == OBJ_BOOKS;
 }
 
 static bool _make_book_randart(item_def &book)
@@ -569,7 +559,7 @@ static bool _make_book_randart(item_def &book)
 
     do
     {
-        mpr("Make book fixed [t]heme or fixed [l]evel? ", MSGCH_PROMPT);
+        mprf(MSGCH_PROMPT, "Make book fixed [t]heme or fixed [l]evel? ");
         type = toalower(getchk());
     }
     while (type != 't' && type != 'l');
@@ -690,8 +680,7 @@ void wizard_make_object_randart()
         item.props.clear();
     }
 
-    mpr("Fake item as gift from which god (ENTER to leave alone): ",
-        MSGCH_PROMPT);
+    mprf(MSGCH_PROMPT, "Fake item as gift from which god (ENTER to leave alone): ");
     char name[80];
     if (!cancellable_get_line(name, sizeof(name)) && name[0])
     {
@@ -729,14 +718,14 @@ void wizard_make_object_randart()
     if (eq != EQ_NONE)
         equip_item(eq, invslot);
 
-    mpr_nocap(item.name(DESC_INVENTORY_EQUIP).c_str());
+    mprf_nocap("%s", item.name(DESC_INVENTORY_EQUIP).c_str());
 }
 
 // Returns whether an item of this type can be cursed.
 static bool _item_type_can_be_cursed(int type)
 {
-    return (type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
-            || type == OBJ_STAVES || type == OBJ_RODS);
+    return type == OBJ_WEAPONS || type == OBJ_ARMOUR || type == OBJ_JEWELLERY
+           || type == OBJ_STAVES || type == OBJ_RODS;
 }
 
 void wizard_uncurse_item()
@@ -788,9 +777,9 @@ void wizard_unidentify_pack()
     you.redraw_quiver = true;
 
     // Forget things that nearby monsters are carrying, as well.
-    // (For use with the "give monster an item" wizard targetting
+    // (For use with the "give monster an item" wizard targeting
     // command.)
-    for (monster_iterator mon(you.get_los()); mon; ++mon)
+    for (monster_near_iterator mon(&you); mon; ++mon)
     {
         for (int j = 0; j < NUM_MONSTER_SLOTS; ++j)
         {
@@ -882,8 +871,7 @@ static void _debug_acquirement_stats(FILE *ostat)
     mesclr();
     mpr("[a] Weapons [b] Armours [c] Jewellery      [d] Books");
     mpr("[e] Staves  [f] Wands   [g] Miscellaneous  [h] Food");
-    mpr("What kind of item would you like to get acquirement stats on? ",
-        MSGCH_PROMPT);
+    mprf(MSGCH_PROMPT, "What kind of item would you like to get acquirement stats on? ");
 
     object_class_type type;
     const int keyin = toalower(get_ch());
@@ -1028,7 +1016,8 @@ static void _debug_acquirement_stats(FILE *ostat)
         EQ_WEAPON, EQ_BODY_ARMOUR, EQ_SHIELD, EQ_HELMET, EQ_CLOAK,
         EQ_GLOVES, EQ_BOOTS, EQ_AMULET, EQ_RIGHT_RING, EQ_LEFT_RING,
         EQ_RING_ONE, EQ_RING_TWO, EQ_RING_THREE, EQ_RING_FOUR,
-        EQ_RING_FIVE, EQ_RING_SIX, EQ_RING_SEVEN, EQ_RING_EIGHT
+        EQ_RING_FIVE, EQ_RING_SIX, EQ_RING_SEVEN, EQ_RING_EIGHT,
+        EQ_RING_AMULET
     };
 
     bool naked = true;
@@ -1123,13 +1112,16 @@ static void _debug_acquirement_stats(FILE *ostat)
 
         fprintf(ostat, "Egos (including artefacts):\n");
 
-        const char* names[] = {
+        const char* names[] =
+        {
             "normal",
             "flaming",
             "freezing",
             "holy wrath",
             "electrocution",
+#if TAG_MAJOR_VERSION == 34
             "orc slaying",
+#endif
             "dragon slaying",
             "venom",
             "protection",
@@ -1142,16 +1134,25 @@ static void _debug_acquirement_stats(FILE *ostat)
             "pain",
             "antimagic",
             "distortion",
+#if TAG_MAJOR_VERSION == 34
             "reaching",
             "returning",
+#endif
             "chaos",
             "evasion",
+#if TAG_MAJOR_VERSION == 34
             "confusion",
+#endif
             "penetration",
             "reaping",
+            "INVALID",
             "acid",
+#if TAG_MAJOR_VERSION != 34
+            "confuse",
+#endif
             "debug randart",
         };
+        COMPILE_CHECK(ARRAYSZ(names) == NUM_SPECIAL_WEAPONS);
 
         for (int i = 0; i < NUM_SPECIAL_WEAPONS; ++i)
             if (ego_quants[i] > 0)
@@ -1170,7 +1171,8 @@ static void _debug_acquirement_stats(FILE *ostat)
 
         fprintf(ostat, "Egos (excluding artefacts):\n");
 
-        const char* names[] = {
+        const char* names[] =
+        {
             "normal",
             "running",
             "fire resistance",
@@ -1193,6 +1195,7 @@ static void _debug_acquirement_stats(FILE *ostat)
             "reflection",
             "spirit shield",
             "archery",
+            "jumping",
         };
 
         const int non_art = acq_calls - num_arts;
@@ -1212,7 +1215,8 @@ static void _debug_acquirement_stats(FILE *ostat)
         {
             fprintf(ostat, "Primary disciplines/levels of randart books:\n");
 
-            const char* names[] = {
+            const char* names[] =
+            {
                 "none",
                 "conjuration",
                 "enchantment",
@@ -1339,7 +1343,8 @@ static void _debug_rap_stats(FILE *ostat)
     }
 
     // -1 = always bad, 1 = always good, 0 = depends on value
-    const int good_or_bad[] = {
+    const int good_or_bad[] =
+    {
          1, //ARTP_BRAND
          0, //ARTP_AC
          0, //ARTP_EVASION
@@ -1355,10 +1360,11 @@ static void _debug_rap_stats(FILE *ostat)
          1, //ARTP_EYESIGHT
          1, //ARTP_INVISIBLE
          1, //ARTP_FLY
+#if TAG_MAJOR_VERSION != 34
+         1, //ARTP_FOG,
+#endif
          1, //ARTP_BLINK
-         1, //ARTP_CAN_TELEPORT
          1, //ARTP_BERSERK
-         1, //ARTP_UNUSED_1
         -1, //ARTP_NOISES
         -1, //ARTP_PREVENT_SPELLCASTING
         -1, //ARTP_CAUSE_TELEPORTATION
@@ -1377,10 +1383,13 @@ static void _debug_rap_stats(FILE *ostat)
          0, //ARTP_BASE_ACC
          0, //ARTP_BASE_DAM
          1, //ARTP_RMSL
+#if TAG_MAJOR_VERSION == 34
          1, //ARTP_FOG
+#endif
          1, //ARTP_REGENERATION
-         -1
+         0, //ARTP_NO_UPGRADE
     };
+    COMPILE_CHECK(ARRAYSZ(good_or_bad) == ARTP_NUM_PROPERTIES);
 
     // No bounds checking to speed things up a bit.
     int all_props[ARTP_NUM_PROPERTIES];
@@ -1489,7 +1498,8 @@ static void _debug_rap_stats(FILE *ostat)
             max_balance_props,
             (float) total_balance_props / (float) num_randarts);
 
-    const char* rap_names[] = {
+    const char* rap_names[] =
+    {
         "ARTP_BRAND",
         "ARTP_AC",
         "ARTP_EVASION",
@@ -1505,6 +1515,9 @@ static void _debug_rap_stats(FILE *ostat)
         "ARTP_EYESIGHT",
         "ARTP_INVISIBLE",
         "ARTP_FLY",
+#if TAG_MAJOR_VERSION != 34
+        "ARTP_FOG",
+#endif
         "ARTP_BLINK",
         "ARTP_BERSERK",
         "ARTP_NOISES",
@@ -1524,10 +1537,14 @@ static void _debug_rap_stats(FILE *ostat)
         "ARTP_CLARITY",
         "ARTP_BASE_ACC",
         "ARTP_BASE_DAM",
-        "ARTP_RMSL"
+        "ARTP_RMSL",
+#if TAG_MAJOR_VERSION == 34
         "ARTP_FOG",
+#endif
         "ARTP_REGENERATION",
+        "ARTP_NO_UPGRADE",
     };
+    COMPILE_CHECK(ARRAYSZ(rap_names) == ARTP_NUM_PROPERTIES);
 
     fprintf(ostat, "                            All    Good   Bad\n");
     fprintf(ostat, "                           --------------------\n");
@@ -1602,4 +1619,43 @@ void wizard_draw_card()
     if (!found_card)
         mpr("Unknown card.");
 }
+
+void wizard_identify_all_items()
+{
+    wizard_identify_pack();
+    for (int i = 0; i < MAX_ITEMS; ++i)
+    {
+        item_def& item = mitm[i];
+        if (item.defined())
+            set_ident_flags(item, ISFLAG_IDENT_MASK);
+    }
+    for (int ii = 0; ii < NUM_OBJECT_CLASSES; ii++)
+    {
+        object_class_type i = (object_class_type)ii;
+        if (!item_type_has_ids(i))
+            continue;
+        for (int j = 0; j < get_max_subtype(i); j++)
+            set_ident_type(i, j, ID_KNOWN_TYPE);
+    }
+}
+
+void wizard_unidentify_all_items()
+{
+    wizard_unidentify_pack();
+    for (int i = 0; i < MAX_ITEMS; ++i)
+    {
+        item_def& item = mitm[i];
+        if (item.defined())
+            unset_ident_flags(item, ISFLAG_IDENT_MASK);
+    }
+    for (int ii = 0; ii < NUM_OBJECT_CLASSES; ii++)
+    {
+        object_class_type i = (object_class_type)ii;
+        if (!item_type_has_ids(i))
+            continue;
+        for (int j = 0; j < get_max_subtype(i); j++)
+            set_ident_type(i, j, ID_UNKNOWN_TYPE);
+    }
+}
+
 #endif
